/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef synaps_arithm_inv_H
#define synaps_arithm_inv_H

#include <synaps/init.h>
#include <synaps/arithm/traits.h>
#include <synaps/arithm/pow.h>
//#include <synaps/arithm/RationalOf.h>

__BEGIN_NAMESPACE_SYNAPS

/// Generic inverse. The result is of type the rational type of @T@.
template<class C>
typename rationalof<C>::T  inv(const C& a)
{
  return typename rationalof<C>::T (C(1),a);
}

/// Generic power of @a@, including negative powers.
template<class C>
typename rationalof<C>::T powgen( const C& a, int i )
{
  typedef typename rationalof<C>::T rat_t; 
  return (i>0? rat_t(pow(a,i)): (i==0? rat_t(1): rat_t(rat_t(1)/pow(a,-i))));
}

__END_NAMESPACE_SYNAPS
#endif // synaps_arithm_inv_H

