/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): O. Ruatta, GALAAD, INRIA                              *
**********************************************************************
History: 
$Id: pow.h,v 1.1 2005/07/11 10:39:49 mourrain Exp $
**********************************************************************/

#ifndef SYNAPS_ARITHM_POW_H
#define SYNAPS_ARITHM_POW_H

#include <synaps/init.h>
#include <synaps/arithm/traits.h>
#include <synaps/arithm/inv.h>

__BEGIN_NAMESPACE_SYNAPS


template<class T>
T pow(const T& a, int i, const type::false_t& val)
{
  assert(i>=0);
  if(i == 1) return a;
  if(i > 0)
    {
      if (a == T(1))  return a;
      if (a == T(-1)) return (i % 2  == 0) ? T(1) : T(-1);
      T y(a);
      T z(1);
      unsigned int n = i;
      unsigned int m;
      while (n > 0) {
	m = n / 2;
	if (n %2 ) 
	  {
	    z *= y;
	  }
	y = y * y;
	n = m;
      }
      return z;
    }
  return T(1);
}

//--------------------------------------------------------------------
/// Generic power of @a@, for fields, including negative powers.
template<class C> inline
C pow( const C& a, int i, const type::true_t& val)
{
  typedef C rat_t;
  return (i>0? rat_t(pow(a,i,type::false_t())): (i==0? rat_t(1): rat_t(rat_t(1)/pow(a,-i,type::false_t()))));
}
//--------------------------------------------------------------------
/// Fast exponentiation of @a@, only for positive power.
template<class X> inline
X pow(const X& a, int i)
{
  typedef typename arithm::isfield<X>::T is_field_t;
  return pow(a,i, is_field_t());
}

__END_NAMESPACE_SYNAPS
#endif // SYNAPS_ARITHM_POW_H

