/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: rdw.h,v 1.1 2005/07/11 09:15:23 mourrain Exp $
 ********************************************************************/
#ifndef SYNAPS_NUMERICS_RDW_H
#define SYNAPS_NUMERICS_RDW_H
//--------------------------------------------------------------------
#include <synaps/arithm/rounding.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace numerics
{
  namespace rdw
  {
    template< typename T > inline  
    T upadd( const T& a, const T& b ) { T tmp = -a; tmp -= b; return -tmp;  }; 
    template< typename T > inline  
    T dwadd( const T& a, const T& b ) { return    a+b;    };
    template< typename T > inline  
    T upsub( const T& a, const T& b ) { T tmp =  b; tmp -= a; return -tmp;  };
    template< typename T > inline  
    T dwsub( const T& a, const T& b ) { return    a-b;    };
    template< typename T > inline  
    T upmul( const T& a, const T& b ) { T tmp = -a; tmp *= b; return -tmp;  };
    template< typename T > inline  
    T dwmul( const T& a, const T& b ) { return    a*b;    };
    template< typename T > inline  
    T updiv( const T& a, const T& b ) { T tmp = -a; tmp /= b; return -tmp;  };
    template< typename T > inline  
    T dwdiv( const T& a, const T& b ) { return    a/b;    };
    template< typename T > inline  
    T updet( const T& a, const T& b, const T& c, const T& d ) 
    { return upsub(upmul(a,d),dwmul(b,c)); };
    template< typename T > inline
    T dwdet( const T& a, const T& b, const T& c, const T& d ) 
    { return dwsub(dwmul(a,d),upmul(b,c)); };  
  };
 
  namespace rup
  {
    template< typename T > inline  
    T upadd( const T& a, const T& b ) { return    a+b;    };
    template< typename T > inline  
    T dwadd( const T& a, const T& b ) { T tmp = -a; tmp -= b; return -tmp;  }; 
    template< typename T > inline  
    T upsub( const T& a, const T& b ) { return    a-b;    };
    template< typename T > inline  
    T dwsub( const T& a, const T& b ) { T tmp =  b; tmp -= a; return -tmp;  };
    template< typename T > inline  
    T upmul( const T& a, const T& b ) { return    a*b;    };
    template< typename T > inline  
    T dwmul( const T& a, const T& b ) { T tmp = -a; tmp *= b; return -tmp;  };
    template< typename T > inline  
    T updiv( const T& a, const T& b ) { return    a/b;    };
    template< typename T > inline  
    T dwdiv( const T& a, const T& b ) { T tmp = -a; tmp /= b; return -tmp;  };
    template< typename T > inline  
    T updet( const T& a, const T& b, const T& c, const T& d ) 
    { return upsub(upmul(a,d),dwmul(b,c)); };
    template< typename T > inline
    T dwdet( const T& a, const T& b, const T& c, const T& d ) 
    { return dwsub(dwmul(a,d),upmul(b,c)); };
  };
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
