/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: fpu.h,v 1.1 2005/07/11 09:15:23 mourrain Exp $
 ********************************************************************/
#ifndef SYNAPS_NUMERICS_FPU_HPP
#define SYNAPS_NUMERICS_FPU_HPP
//--------------------------------------------------------------------
#include <fenv.h>
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace numerics
{
  inline int get_cw()  { return fegetround(); };
  inline int get_rnd() { return fegetround(); };
  inline void set_cw( int cw ) { fesetround(cw);};
  inline static int rnd_up() { return FE_UPWARD;};
  inline static int rnd_dw() { return FE_DOWNWARD;};
  inline static int rnd_nr() { return FE_TONEAREST;};
  inline static int rnd_z()  { return FE_TOWARDZERO;};
  template< typename T > struct LongVersion { typedef T result_t; };
  template<> struct LongVersion<float>  { typedef long double result_t; };
  template<> struct LongVersion<double> { typedef long double result_t; };
  template< typename T > struct fpu_rounding { 
    static const int id = 0; 
    typedef int rnd_t;
    inline static rnd_t  getrnd() { return 0; };
    inline static void   setrnd( rnd_t r ) { };
    inline static int rnd_up() { return 0; };
    inline static int rnd_dw() { return 0; };
    inline static int rnd_nr() { return 0; };
    inline static int rnd_z()  { return 0; };
    fpu_rounding( int ){}; 
  };
  template<> struct fpu_rounding<long double> 
  { 
    typedef int rnd_t;
    inline static rnd_t  getrnd() { return fegetround(); };
    inline static void   setrnd( rnd_t r ) { fesetround(r); };
    inline static int rnd_up() { return FE_UPWARD;};
    inline static int rnd_dw() { return FE_DOWNWARD;};
    inline static int rnd_nr() { return FE_TONEAREST;};
    inline static int rnd_z()  { return FE_TOWARDZERO;};
    static const int id = 1;
    rnd_t m_prev;
    bool  m_chg;
    fpu_rounding( rnd_t rnd ) 
    {
      m_chg = false;
      rnd_t m_prev = getrnd();
      if ( m_prev != rnd ) 
	{ 
	  m_chg = true;
	  setrnd(rnd); 
	};
    };
    ~fpu_rounding() { 
      if ( m_chg ) setrnd(m_prev);
    };
  };
  
  template< typename T >
  struct rounding : fpu_rounding< typename LongVersion< T >::result_t >  
  { rounding( int rnd ) : fpu_rounding< typename LongVersion< T >::result_t >( rnd ) {}; };
  
  template< typename T > inline T div_dw(const T& a, const T& b) { rounding<T> r(rnd_dw()); return a/b; };
  template< typename T > inline T div_up(const T& a, const T& b) { rounding<T> r(rnd_up()); return a/b; };
  template< typename T > inline T add_dw(const T& a, const T& b) { rounding<T> r(rnd_dw()); return a+b; };
  template< typename T > inline T add_up(const T& a, const T& b) { rounding<T> r(rnd_up()); return a+b; };
  template< typename T > inline T mul_dw(const T& a, const T& b) { rounding<T> r(rnd_dw()); return a*b; };
  template< typename T > inline T mul_up(const T& a, const T& b) { rounding<T> r(rnd_up()); return a*b; };
  template< typename T > inline T sub_dw(const T& a, const T& b) { rounding<T> r(rnd_dw()); return a-b; };
  template< typename T > inline T sub_up(const T& a, const T& b) { rounding<T> r(rnd_up()); return a-b; };
};

//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
