#ifndef SYNAPS_TEXP_STRUCTURE_PROTOTYPES_H
#define SYNAPS_TEXP_STRUCTURE_PROTOTYPES_H

#include <synaps/arithm/traits.h>

__BEGIN_NAMESPACE_SYNAPS

namespace arithm
{

  define_binary_operator(_add_,operator+,add);
  define_binary_operator(_sub_,operator-,sub);
  define_binary_operator(_mul_,operator*,mul);
  define_binary_operator(_div_,operator/,div);
  define_binary_operator(_pow_,operator^,pow);
  define_binary_operator(_mod_,operator%,mod);
  define_unary_operator(_neg_,operator-,neg);
  
  
  template<class X, class Y, class SX, class SY>
  struct binary_operator_structure_prototype<_add_,X,Y,SX,SY>
  {
    typedef typename ringof<X,Y>::T F;
    typedef                       F U;
    typedef                       F V;
  };
  
  template<class X, class Y, class SX, class SY>
  struct binary_operator_structure_prototype<_sub_,X,Y,SX,SY>
  {
    typedef typename ringof<X,Y>::T F;
    typedef                       F U;
    typedef                       F V;
  };
  
  
  template<class X, class Y, class SX, class SY>
  struct binary_operator_structure_prototype<_mul_,X,Y,SX,SY>
  {
    typedef typename ringof<X,Y>::T F;
    typedef                       F U;
    typedef                       F V;
  };
  

  template<class X, class Y, class SX, class SY>
  struct binary_operator_structure_prototype<_div_,X,Y,SX,SY>
  {
    typedef typename fieldof<X,Y>::T F;
    typedef                       F U;
    typedef                       F V;
  };
  
  
  // vector + vector = vector
  template<typename X, typename Y>
    struct binary_operator_structure_prototype<_add_,X,Y,structure::vector,structure::vector> 
  { 
    typedef typename ringof<typename X::value_type,typename Y::value_type>::T C;
    typedef typename instanceof<X,C>::T                                       F;
    typedef typename instanceof<X,C>::T                                       U;
    typedef typename instanceof<X,C>::T                                       V;
  };
  
  // vector - vector = vector
  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_sub_,X,Y,structure::vector,structure::vector> 
  { 
    typedef typename ringof<typename X::value_type,typename Y::value_type>::T C;
    typedef typename instanceof<X,C>::T                                       F;
    typedef typename instanceof<X,C>::T                                       U;
    typedef typename instanceof<X,C>::T                                       V;
  };
  
  // vector * vector = scalar
  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_mul_,X,Y,structure::vector,structure::vector>
  {
    typedef typename ringof<typename X::value_type,typename Y::value_type>::T C;
    typedef C                                                                 F;
    typedef typename instanceof<X,C>::T                                       U;
    typedef typename instanceof<X,C>::T                                       V;
  };
  
  // vector * scalar = vector
  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_mul_,X,Y,structure::vector,structure::scalar>
  {
    typedef typename ringof<typename X::value_type,Y>::T C;
    typedef typename instanceof<X,C>::T                                       F;
    typedef typename instanceof<X,C>::T                                       U;
    typedef C                                                                 V;
  };

  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_mul_,X,Y,structure::scalar,structure::vector>
  {
    typedef typename ringof<X,typename Y::value_type>::T C;
    typedef typename instanceof<Y,C>::T                                       F;
    typedef typename instanceof<Y,C>::T                                       V;
    typedef C                                                                 U;
  };

  // vector / scalar = vector
  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_div_,X,Y,structure::vector,structure::scalar>
  {
    typedef typename fieldof<typename X::value_type,Y>::T C;
    typedef typename instanceof<X,C>::T                   F;
    typedef typename instanceof<X,C>::T                   U;
    typedef C                                             V;
  };
  
  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_mul_,X,Y,structure::matrix,structure::vector>
  {
    typedef typename ringof<typename X::value_type,typename Y::value_type>::T C;
    typedef typename instanceof<Y,C>::T                       F;
    typedef typename instanceof<X,C>::T                       U;
    typedef typename instanceof<Y,C>::T                       V;
  };
  
  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_mul_,X,Y,structure::matrix,structure::scalar>
  {
    typedef typename  ringof<typename X::value_type,Y>::T      C;
    typedef typename  instanceof<X,C>::T                       F;
    typedef typename  instanceof<X,C>::T                       U;
    typedef C                                                  V;

    //    template<class E> 
    //    struct is_result_valid
    //    { 
    //       typedef typename type::and < typename type::equal<typename structureof<E>::T, structure::matrix >::T,
    //				   typename issupersetof<typename valuetypeof<E>::T, C>::T >::T T;
    //    };
    

  };

  template<typename X, typename Y>
  struct binary_operator_structure_prototype<_mul_,X,Y,structure::scalar,structure::matrix>
  {
    typedef typename ringof<X,typename Y::value_type>::T C;
    typedef typename instanceof<Y,C>::T                       F;
    typedef  C                                                U;
    typedef typename instanceof<Y,C>::T                       V;
  };
  
  template<typename M, typename S>
  struct binary_operator_structure_prototype<_div_,M,S,structure::matrix,structure::scalar>
  {
    typedef typename fieldof<typename M::value_type,S>::T C;
    typedef typename instanceof< M, C >::T                F;
    typedef F                                             U;
    typedef C                                             V;
  };
  
  template<class op, class X, class Y>
  std::ostream& operator<<( std::ostream & o, const binary_operator_prototype<op,X,Y> & b )
  {
    typedef binary_operator_prototype<op,X,Y> proto;
    o << " name = ";    
    print<proto>(o);
    o << std::endl;
    o << "\tdefault = ";
    print<typename proto::F>(o); 
    o << std::endl;
    o << "\tslot 0  = ";
    print<typename proto::U>(o); 
    o << std::endl;
    o << "\tslot 1  = ";
    print<typename proto::V>(o); 
    o << std::endl;
    return o;
  };
  
};

__END_NAMESPACE_SYNAPS

#endif
