#ifndef SYNAPS_ARITHM_TRAITS_ISSUBSETOF_H
#define SYNAPS_ARITHM_TRAITS_ISSUBSETOF_H

#include <synaps/init.h>
#include <synaps/arithm/traits/kernelof.h>

__BEGIN_NAMESPACE_SYNAPS

namespace arithm {
  /// X can be seen as an arithmetic subset of Y (as "Z in Q")
  template<typename X, typename Y> 
  struct issubsetof_  { typedef type::false_t T; };
  
  /// kernel version of issubsetof
  template<class K, class X, class Y>
  struct kernel_issubsetof { typedef false_t T; };
  template<class X, class Y>
  struct kernel_issubsetof<null_t,X,Y> { typedef null_t T; };
  
  template<class K>
  struct kernel_issubsetof< K, typename K::integer, algebraic<K>  > { typedef true_t T; };
  template<class K> 
  struct kernel_issubsetof< K, typename K::integer, typename K::rational > { typedef true_t T; };
  /// neglecting the overflow problem we assume that floating is superset of integer
  template<class K> 
  struct kernel_issubsetof< K, typename K::integer, typename K::floating > { typedef true_t T; };
  template<class K>
  struct kernel_issubsetof< K, double, typename K::floating > { typedef true_t T; };
  template<class K>
  struct kernel_issubsetof< K, typename K::integer, double > { typedef true_t T; };
  template<class K>
  struct kernel_issubsetof< K, typename K::floating, typename K::rational > { typedef true_t T; };
  template<typename Y>
  struct issubsetof_<int,Y>
  {
    typedef type::true_t T;
  };
  
  template< template<class> class R, class X, class Y> 
  struct issubsetof_< R<X>, R<Y> >
  { typedef typename issubsetof_<X,Y>::T T; };
  
  /// return type::true_t if all x in X are in Y.  
  /// specialization must be done on subsetof_
  template<typename X, typename Y>
  struct issubsetof
  {
    typedef typename ucstref < X>::T UX; typedef typename ucstref < Y>::T UY;
    typedef typename kernelof<UX>::T KX; typedef typename kernelof<UY>::T KY;
    typedef typename 
    sfirstnn<
      typename equal<UX,UY>::T,
      typename issubsetof_<UX,UY>::T,
      typename kernel_issubsetof<typename kernelof<UX>::T,UX,UY>::T,
      typename kernel_issubsetof<typename kernelof<UY>::T,UX,UY>::T >::T T;
  };

  template<typename X, typename TypeOrListOfTypes > struct issupersetof;
  template<typename X, typename Type>
  struct issupersetof { 
    typedef typename issubsetof<Type,X>::T T; 
  };
  
  template<typename X, typename A>
  struct issupersetof< X, type::tlist<A,type::null_t> >
  { 
    typedef typename issupersetof<X,A>::T T; 
  };
  
  template<typename X, typename A, typename Tail>
  struct issupersetof< X, type::tlist<A,Tail> >
  {
    typedef typename issubsetof<A,X>::T                                                        ok;
    typedef typename tselect<ok, typename issupersetof<X,Tail>::T, type::false_t >::T T;
  };
  
}
__END_NAMESPACE_SYNAPS
#endif
