#ifndef SYNAPS_ARITHM_TRAITS_REALOF_H
#define SYNAPS_ARITHM_TRAITS_REALOF_H

#include <complex>

__BEGIN_NAMESPACE_SYNAPS

namespace arithm {
  
  struct Real {};
  /**
     This class is a traits class used to specify the type corresponding
     to the algebraic closure. The complex type for a class @C@ is
     @algebraic_closure<C>::T@.
     By default the algebraic closure class of @C@ is @std::complex<C>@.
     We have the following specialisation:
     \arg  @algebraic_closure< std::complex<C> >::T = std:complex<C>@ for any @C@.
  **/
  template <class C> struct algebraic_closure {
    typedef std::complex<C> T;
  };
  
  template<class C> struct algebraic_closure<std::complex<C> > {
    typedef std::complex<C> T;
  };

  template < typename X >
  struct realof_ { typedef X T; };

  template < typename X > struct realof_ < std::complex<X> > { typedef X T; };
  
  template < typename X >
  struct realof  { typedef typename realof_< typename type::ucstref< X >::T >::T T; };

}

__END_NAMESPACE_SYNAPS

#endif
