/*********************************************************************
*      This file is part of the source code of AXEL library          *
*   Author(s): G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef SYNAPS_GEOMVIEW_OSTREAM_H_
#define SYNAPS_GEOMVIEW_OSTREAM_H_
/********************************************************************/
#include <synaps/init.h>
#include <iostream>
#include <fstream>
#include "Color.h"
#include "Width.h"
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
//! Container for the geomview domain
/*!
  Here, we have implemented the structures to store the results 
  we compute in the geomview format. 
 */
namespace geomview
{
  //------------------------------------------------------------------
  //! The ostream file structure 
  /*!
    Structure which allows us to specify the file in order
    to store the results.
   */
  struct ostream
  {
    /*! The file where we store the result.*/
    char *file;
    Color color;
    Width width;

    /*! The default constructor. */
    ostream() {}

    /*! The default destructor. */
    ~ostream() {}

    /*! 
      \param f a file.
      
      Construct an ostream with the file \c f and initialize
      the options of geomview in the file.
    */
    ostream(char *fic) 
    {
      file=fic;
	std::ofstream f(file,std::ios::out);
	if( !f )
	  {
	    std::cerr << "Creation refused" << std::endl;
	    exit(1);
	  }
	else
	  std::cout<< "Output in "<<file<<std::endl<<std::endl;

	f << "appearance { \n";
	f << "-edge \n";
	f << "+shading smooth \n";
	f << "}\n";
	
	f<< "LIST\n";

	f.close();
      }

    /*!
      Allows us to clean the file and reinitialize.
     */
    void init()
      {
	std::ofstream f(file,std::ios::out);
	if( !f )
	  {
	    std::cerr << "Creation refused" << std::endl;
	    exit(1);
	  }
	
	f << "appearance { \n";
	f << "-edge \n";
	f << "+shading smooth \n";
	f << "}\n";
	
	f<< "LIST\n";
	f.close();
      }

    /*!
      Allows us to execute geomview.
     */
    void view()
      {
	char cmd[1000];
	strcpy(cmd,"geomview \0");
	strcat(cmd,file);
	strcat(cmd," -b 1 1 1 &\0");
	system(cmd);
      }
  };

  inline std::ostream& operator<<(std::ostream& os, const ostream & g)
  {
    os <<"Output("<<g.file<<")"; return os;
  }

  inline ostream& operator<<(ostream & os, const Color & c)
  {
    os.color=c; return os;
  }

  inline ostream& operator<<(ostream & os, const Width & w)
  {
    os.width=w; return os;
  }


  inline ostream& operator<<(ostream & os, char* s)
  {
    std::ofstream f;
    f.open (os.file, std::ofstream::out | std::ofstream::app);
    f<<s;
    f.close();
    return os;
  }
}

/*********************************************************************/
__END_NAMESPACE_SYNAPS 
#endif // axel_viewer_geomview_mesh_H__
