#ifndef SYNAPS_UTIL_STRIDE_ITERATOR_H
#define SYNAPS_UTIL_STRIDE_ITERATOR_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS

template <class coef_t, class pointer_t= coef_t*, class stride_t=unsigned , class reference_t = coef_t& >
class stride_iterator {

  pointer_t my_rep;
  stride_t  my_stride;

public:
  
  typedef std::random_access_iterator_tag  iterator_category;
  typedef coef_t                           value_type;
  typedef int                              difference_type;
  typedef pointer_t                        pointer;
  typedef reference_t                      reference;
  
  typedef stride_iterator<coef_t,pointer_t,stride_t,reference_t> self_t;
  
  stride_iterator(): my_rep(0), my_stride(1) {}
  stride_iterator(pointer_t const ptr,  stride_t const stride)
    : my_rep(ptr), my_stride(stride)         {}
  
  self_t& operator++()
  {my_rep += my_stride; return *this;}
  
  self_t  operator+(int i)
  {self_t tmp(*this); tmp.my_rep+=my_stride*i; return tmp;}
  self_t  operator+(int i)  const
  {self_t tmp(*this); tmp.my_rep+=my_stride*i; return tmp;}
  
  self_t& operator=(const self_t & it)
  {my_rep = it.my_rep; my_stride= it.my_stride; return *this;}
  
  reference_t operator* () {return(*my_rep);}
  pointer_t   operator->() {return  my_rep;}
  
  bool operator!=( self_t const& rhs) const { return my_rep != rhs.my_rep; }
  bool operator<=( self_t const& rhs) const { return my_rep <= rhs.my_rep; }
  
  
  int operator-( const self_t & b  ) const 
  {
    assert( my_stride == b.my_stride );
    return (my_rep-b.my_rep)/my_stride;
  };
  
  
};


//----------------------------------------------------------------------
template <class T, class T_ptr, class T_stride, class T_ref>
stride_iterator<T,T_ptr,T_stride,T_ref>&
operator++(const stride_iterator<T,T_ptr,T_stride,T_ref> & it)   {return ++it;}
//----------------------------------------------------------------------

__END_NAMESPACE_SYNAPS

#endif //stride_iterator_H

