/*********************************************************************
*    This file is part of the source code of SYNAPS library.         *
*    Author(s): B. Mourrain, GALAAD, INRIA                           *
**********************************************************************/
#ifndef synaps_util_xml_H_
#define synaps_util_xml_H_
//--------------------------------------------------------------------
#include <iostream>
#include <complex>
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/// Namespace for parsing and printing in @xml@  format.
namespace xml {
  /** Skip the input stream until s is found. It returns true if @s@ is found.
   **/
  template<class ISTREAM>
  bool skip(ISTREAM& is, char* s) 
  {
    char c=(*s)+1; char* t=s;
    unsigned n= strlen(s),i=0;    
    while(c != EOF)
      {
	for(i=0; i<n && (c !=EOF) && (c=is.get()) == (*t); i++,t++);
	if(i==n) 
	  return true;
	else 
	  t=s;
      }
    return false;
  }

  template<class OSTREAM, typename X>
    OSTREAM& print(OSTREAM& os, const X& x, char *tag)
    {
      os<<"<"<<tag<<">"<<x<<"</"<<tag<<">";
      return os;
    }


  #define begin_polynomial "<polynomial>"
  #define end_polynomial "</polynomial>"

  /// Read a polynomial, enclosed in @polynomial@ tags.
  template<class ISTREAM, typename T>
  bool read_pol(ISTREAM& is, T& p)
  {
    if(skip(is,begin_polynomial))
      {
        std::string s; char c;
	while((c=is.get()) != EOF && c != '<') 
	  {
	    if(c != ' ' && c != '\n') s+=c;
	  }
	s+='\0';
	p = T(s.data());
	return true;
      }
    else 
      {
	p=T(0); return false;
      }
  }

  /// Read a polynomial, enclosed in @polynomial@ tags.
  template<class ISTREAM, typename T, class VAR>
  bool read_pol(ISTREAM& is, T& p, const VAR& V)
  {
    if(skip(is,begin_polynomial))
      {
        std::string s; char c;
	while((c=is.get()) != EOF && c != '<') 
	  {
	    if(c != ' ' && c != '\n') s+=c;
	  }
	s+='\0';
	p = T(s.data(),V);
	return true;
      }
    else 
      {
	p=T(0); return false;
      }
  }
}
//----------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
#endif //synaps_util_xml_H_
