<TeXmacs|1.0.6>

<style|generic>

<\body>
  <section|Geometric computations>

  The aim of this section is to show some tools for the construction and
  manipulation of rational surfaces on a bounded domain, allow us to compute
  their intersection and visualise the result. For this purpose, we will use
  \ the visualisation tool <samp|axel>.

  <subsection|Rational surfaces>

  A rational surface is defined by a map

  <\eqnarray*>
    <tformat|<table|<row|<cell|\<sigma\>:
    D>|<cell|\<rightarrow\>>|<cell|\<bbb-P\><rsup|3>>>|<row|<cell|(s,t)>|<cell|\<mapsto\>>|<cell|
    (f<rsub|0>(s,t): f<rsub|1>(s,t):f<rsub|2>(s,t):f<rsub|3>(s,t))>>>>
  </eqnarray*>

  where <with|mode|math|D> is a domain of <with|mode|math|\<bbb-R\><rsup|2>>
  (in geometric modeling applications, usually
  <with|mode|math|[0,1]<rsup|2>>) and <with|mode|math|f<rsub|0>,\<ldots\>,
  f<rsub|3>> are polynomials. The data structures for these rational maps is
  defined in

  <verbatim| #include \<less\>synaps/shape/surface/MPDrationalSurface.h\<gtr\>>

  It can be defined easily from strings, as illustrated below:

  File: <verbatim|step5-1.cc>

  <subsection|Visualisation>

  In order to visualised these objects, one can use the connection files for
  <samp|axel>:

  <verbatim| #include \<less\>synaps/base/io/axel.h\<gtr\>>

  This defines a <samp|axel ostream>, which can be used as follows:

  <\code>
    \ axel::ostream vw("tmp.axl");

    \ vw\<less\>\<less\> s1;

    \ vw\<less\>\<less\>Color(25,25,225);

    \ vw\<less\>\<less\>s2;

    \ vw.view();
  </code>

  In the construction of the viewer <samp|vw>, <samp|tmp.axl> is the name of
  the file where the data \ are stored. Then we send <samp|s1> to this view
  and visualise. We modify the color used to print the objects by using
  <verbatim|os\<less\>\<less\>Color(25,25,225);> The color is describe in
  (red,green,blue) format with a integer number between 0 and 255. The
  surface <samp|s2> is printed, with this color. The view is displayed by the
  command <samp|vw.view()>. It yields

  <\with|par-mode|center>
    <postscript|step5-fig1.ps|*5/8|*5/8||||>
  </with>

  File: <verbatim|step5-2.cc>

  <subsection|Intersection>

  Given two such surfaces <with|mode|math|s<rsub|1>> and
  <with|mode|math|s<rsub|2>>, given by the polynomials
  <with|mode|math|f<rsub|0>,\<ldots\>, f<rsub|3>> and
  <with|mode|math|g<rsub|0>,\<ldots\>, g<rsub|3>>, we want to compute their
  intersection points defined by the system:

  <\equation*>
    (1) <left|{><tabular|<tformat|<table|<row|<cell|g<rsub|1>(u,v)*f<rsub|0>(s,t)-f<rsub|1>(s,t)
    *g<rsub|0>(u,v)=0>>|<row|<cell|g<rsub|2>(u,v)*f<rsub|0>(s,t)-f<rsub|2>(s,t)
    *g<rsub|0>(u,v)=0>>|<row|<cell|g<rsub|3>(u,v)*f<rsub|0>(s,t)-f<rsub|3>(s,t)
    *g<rsub|0>(u,v)=0>>>>>
  </equation*>

  This defines a curve in <with|mode|math|\<bbb-R\><rsup|2> \<times\>
  \<bbb-R\><rsup|2>> which image by <with|mode|math|s<rsub|1>> or
  <with|mode|math|s<rsub|2>> in the intersection of the surfaces.

  In order to visualise this intersection curve, we are going to fixe the
  values of <with|mode|math|v\<in\>[0,1]> and to solve the corresponding
  system in <with|mode|math|s,t,u> and then compute the image of these points
  by <with|mode|math|s<rsub|1>>.

  We are going to use the function <verbatim|intersect>, which applies for
  anay paramteric surface. Its arguments are converted to pointers to
  <verbatim|IParametricSurface> type. The result <verbatim|r> is of type
  <verbatim|IShape>, from which we deduce a mesh approximation of type
  <verbatim|IGLGood> by the instruction <verbatim|r-\<gtr\>mesh()>. It
  corresponds to a piecewise linear approximation of the result.

  File: <verbatim|step5-3.cc>

  Here is what we otain for the previous surfaces <samp|s1>, <samp|s2>:\ 

  <\with|par-mode|center>
    <postscript|step5-fig2.ps|*5/8|*5/8||||>
  </with>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Geometric
      computation> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|Rational surfaces
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|Visualisation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|Intersection
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>