dnl @synopsis RX_LIB_SEARCH_MPSOLVE([ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]])
dnl
dnl
AC_DEFUN([RX_LIB_SEARCH_MPSOLVE], [dnl
AC_REQUIRE([RX_LIB_SEARCH_GMP])

ac_mpsolve_header_found=no
ac_mpsolve_lib_found=no

AC_ARG_WITH(mpsolve,
   AC_HELP_STRING([--with-mpsolve=<DIR>|yes|no],
		  [search for a MPSolve installation in the DIR directory. If
		   argument is yes, assume that the installation can be found
		   using the default compiler and linker serach paths. If
		   argument is no, no search for MPSolve is done.]),
	[if test "$withval" = no; then
	  ac_mpsolve_header_found=disable
	  ac_mpsolve_lib_found=disable
	 elif test "$withval" = yes; then
          LIBS="-lmps ${LIBS}"
	  ac_mpsolve_header_found=yes
	  ac_mpsolve_lib_found=yes
         else
	   MPSOLVE_PATH="$withval"
	 fi], [MPSOLVE_PATH="${DEFAULT_CHECKING_PATH}"])

if test "$ac_mpsolve_header_found" = disable; then
  AC_MSG_NOTICE([configuring without MPSolve library])
else
  AC_MSG_NOTICE([looking for MPSolve library in ${MPSOLVE_PATH}])
fi

mpsolve_save_CPPFLAGS="${CPPFLAGS}"
mpsolve_save_LDFLAGS="${LDFLAGS}"
mpsolve_save_LIBS="${LIBS}"

# Test for header directly in MPSOLVE_PATH
if test "$ac_mpsolve_header_found" = no; then
  MPSOLVE_INC_PATH="${MPSOLVE_PATH}"
  CPPFLAGS="-I${MPSOLVE_INC_PATH} ${CPPFLAGS}"
  unset ac_cv_header_mpc_h
  AC_CHECK_HEADER([mpc.h],
		  [ac_mpsolve_header_found=yes],
		  [ac_mpsolve_header_found=no])
  CPPFLAGS="${mpsolve_save_CPPFLAGS}"
fi

# Test for header directly in MPSOLVE_PATH/include 
if test "$ac_mpsolve_header_found" = no; then
  MPSOLVE_INC_PATH="${MPSOLVE_PATH}/include"
  CPPFLAGS="-I${MPSOLVE_INC_PATH} ${CPPFLAGS}"
  unset ac_cv_header_mpc_h
  AC_CHECK_HEADER([mpc.h],
		  [ac_mpsolve_header_found=yes],
		  [ac_mpsolve_header_found=no])
  CPPFLAGS="${mpsolve_save_CPPFLAGS}"
fi

# Test for header in MPSOLVE_PATH/include using include directives written
# as <mpsolve/mps.h>
if test -d mpsolve; then 
  LIBS="$LIBS -lmps -lxt"
fi;
if test "$ac_mpsolve_header_found" = no; then
  MPSOLVE_INC_PATH="${MPSOLVE_PATH}/include"
  CPPFLAGS="-I${MPSOLVE_INC_PATH} ${CPPFLAGS}"
  unset ac_cv_header_mpc_h
  AC_CHECK_HEADER([mpsolve/mpc.h],
		  [ac_mpsolve_header_found=yes],
		  [ac_mpsolve_header_found=no])
  CPPFLAGS="${mpsolve_save_CPPFLAGS}"
fi

if test "$ac_mpsolve_lib_found" = no; then
  unset ac_cv_lib_mps_mpc_add
  MPSOLVE_LIB_PATH="${MPSOLVE_PATH}"
  LDFLAGS="-L${MPSOLVE_LIB_PATH} ${LDFLAGS}"
  AC_CHECK_LIB(mps, mpc_add,
	       [ac_mpsolve_lib_found=yes], [ac_mpsolve_lib_found=no])
  LDFLAGS="${mpsolve_save_LDFLAGS}"
fi

if test "$ac_mpsolve_lib_found" = no; then
  unset ac_cv_lib_mps_mpc_add
  MPSOLVE_LIB_PATH="${MPSOLVE_PATH}/lib"
  LDFLAGS="-L${MPSOLVE_LIB_PATH} ${LDFLAGS}"
  AC_CHECK_LIB(mps, mpc_add,
	       [ac_mpsolve_lib_found=yes], [ac_mpsolve_lib_found=no])
  LDFLAGS="${mpsolve_save_LDFLAGS}"
fi



LIBS=${mpsolve_save_LIBS}
LDFLAGS=${mpsolve_save_LDFLAGS}
CPPFLAGS=${mpsolve_save_CPPFLAGS}

if test "$ac_mpsolve_header_found" = yes && test "$ac_mpsolve_lib_found" = yes
then
  AC_DEFINE([SYNAPS_WITH_MPSOLVE], 1, [Define to 1 if you have MPSolve])
  CPPFLAGS="-I${MPSOLVE_INC_PATH} $CPPFLAGS"
  LIBS="$LIBS"
  LDFLAGS="${LDFLAGS} -L${MPSOLVE_LIB_PATH}"
  AC_MSG_NOTICE([found MPSolve library in ${MPSOLVE_PATH}])
  $1
else
  if test "$withval" != no; then
    AC_MSG_NOTICE([failed to found MPSolve library in ${MPSOLVE_PATH}])
    $2
    :
  fi
fi
])dnl
