<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Dense vectors>

  Vectors are one-dimensional arrays in which all the coefficients are stored
  (even if they are zero). They provide classical arithmetic operations such
  as addition, substraction, multiplication by scalars . . . available in a
  vector space. An implementation is given by the class:

  <\quote-env>
    <\code>
      template\<less\>class C, class R=linalg::rep1d\<less\>C\<gtr\> \<gtr\>
      struct VectDse\<less\>C,R\<gtr\>;
    </code>
  </quote-env>

  The class depends on two parameters:

  <\itemize-dot>
    <item> <verbatim|C> which specifies the type of coefficients,

    <item> <verbatim|R> which specifies the internal representation or
    container type. The default is <verbatim|linalg::rep1d\<less\>C\<gtr\>>.
  </itemize-dot>

  Notice that this vector type differs from the container type
  <verbatim|std:vector\<less\>C\<gtr\>>, used for array storage.\ 

  <subsection|Containers>

  Any container type which provides the following operations can be used for
  the parameter <verbatim|R>:

  <\quote-env>
    <\code>
      typedef size_type; // type of the index.\ 

      typedef value_type; // type of the coefficients.\ 

      typedef iterator; // type of the iterator.\ 

      R(size_type n); R(size_type n, value_type* t);\ 

      R(iterator b,iterator e);\ 

      R& R::operator=(const R &);

      size_type R::size();

      void R::resize(size_type);\ 

      iterator \ \ \ \ \ \ R::begin();\ 

      const_iterator R::begin() const;\ 

      iterator \ \ \ \ \ \ R::end();\ 

      const_iterator R::end() const;\ 

      value_type & R::operator[] (size_type i);\ 

      value_type \ \ R::operator[] (size_type i) const;
    </code>
  </quote-env>

  For instance

  <\itemize>
    <item><verbatim|linal::rep1d\<less\>C\<gtr\>>

    <item><verbatim|std::vector\<less\>C\<gtr\>>
  </itemize>

  <subsection|Example>

  File: <verbatim|VectDse.cc>

  <subsection|Implementation>

  See: <verbatim|synaps/linalg/VectDse.h>

  <subsection|Generic tools for vectors>

  Generic implementations for vectors are available in the module
  <verbatim|VECTOR>.
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-3|<tuple|1.2|?>>
    <associate|toc-4|<tuple|1.3|?>>
    <associate|toc-5|<tuple|1.4|?>>
    <associate|toc-6|<tuple|1.5|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Dense
      vectors> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Containers
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>Generic tools for vectors
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>
    </associate>
  </collection>
</auxiliary>