/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
$Id: Rank.h,v 1.1 2005/07/11 08:33:34 mourrain Exp $
**********************************************************************/
/** \file                                                           **/
#ifndef SYNAPS_LINALG_RANK_H
#define SYNAPS_LINALG_RANK_H

#include <synaps/init.h>
#include <synaps/linalg/MATRIX.m>

__BEGIN_NAMESPACE_SYNAPS


//----------------------------------------------------------------------
namespace lapack {
  template<class R> struct rep;
}
//--------------------------------------------------------------------
/** 
    Compute the rank of @M@, using @MATRIX::rank(m.rep())@. 
**/
template<class R> inline
typename R::size_type Rank(const R & m) 
{
  using namespace MATRIX;
  return rank(m.rep());
}
//--------------------------------------------------------------------
/** 
    Compute the rank of @M@, using @MATRIX::rank(m.rep())@. 
**/
template<class R,class MTH> inline
typename R::size_type Rank(const R & m, const MTH& mth) 
{
  using MATRIX::rank;
  return rank(mth,m.rep());
}
//----------------------------------------------------------------------
/** Compute the rank of @M@, using @MATRIX::rank@. The columns (resp. row)
    indices of a permutation which put the non-zero minor in the upper
    corner are stored in @permut_col@ and @permut_row@. They have to be
    initialized before, to a size larger than @min(M.nbcol(), M.nbrow())@.
**/
template<class R,class V> 
typename R::size_type Rank(const R & M, 
			   V & I, V & J)
{
  lapack::rep<typename R::value_type> A(M.nbrow(),M.nbcol());
  assign(A,M);
  return rank(A,I.begin(), J.begin());
}
//----------------------------------------------------------------------
/** Extraction of a square submatrix of size rank from the matrix @M@.
    This is performed by keeping the columns whose index is one of the 
    first @rank@ elements of @jpvt@, 
    and the rows whose index is one of the first rank elements of @ipvt@,
    in the same orders.
**/
template<class R,class I> 
R Extract(R & M, const I & ipvt, const I & jpvt, 
		const typename R::size_type & rank)
{
   R SM(rank,rank);
   for (typename R::size_type i=0; i<rank; i++)
     for (typename R::size_type j = 0; j < rank; j++) 
       SM(i,j)=M(ipvt[i],jpvt[j]);
   return (SM);
}
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
#endif // SYNAPS_LINALG_RANK_H


