//#include "blaswrap.h"
//#include "f2c.h"

/* Subroutine */ 
template<typename T>
int zlarf_(char *side, integer *m, integer *n, complexdouble<T> 
	*v, integer *incv, complexdouble<T> *tau, complexdouble<T> *c__, 
	integer *ldc, complexdouble<T> *work)
{
/*  -- LAPACK auxiliary routine (version 3.0) --   
       Univ. of Tennessee, Univ. of California Berkeley, NAG Ltd.,   
       Courant Institute, Argonne National Lab, and Rice University   
       September 30, 1994   


    Purpose   
    =======   

    ZLARF applies a complex elementary reflector H to a complex M-by-N   
    matrix C, from either the left or the right. H is represented in the   
    form   

          H = I - tau * v * v'   

    where tau is a complex scalar and v is a complex vector.   

    If tau = 0, then H is taken to be the unit matrix.   

    To apply H' (the conjugate transpose of H), supply conjg(tau) instead   
    tau.   

    Arguments   
    =========   

    SIDE    (input) CHARACTER*1   
            = 'L': form  H * C   
            = 'R': form  C * H   

    M       (input) INTEGER   
            The number of rows of the matrix C.   

    N       (input) INTEGER   
            The number of columns of the matrix C.   

    V       (input) COMPLEX*16 array, dimension   
                       (1 + (M-1)*abs(INCV)) if SIDE = 'L'   
                    or (1 + (N-1)*abs(INCV)) if SIDE = 'R'   
            The vector v in the representation of H. V is not used if   
            TAU = 0.   

    INCV    (input) INTEGER   
            The increment between elements of v. INCV <> 0.   

    TAU     (input) COMPLEX*16   
            The value tau in the representation of H.   

    C       (input/output) COMPLEX*16 array, dimension (LDC,N)   
            On entry, the M-by-N matrix C.   
            On exit, C is overwritten by the matrix H * C if SIDE = 'L',   
            or C * H if SIDE = 'R'.   

    LDC     (input) INTEGER   
            The leading dimension of the array C. LDC >= max(1,M).   

    WORK    (workspace) COMPLEX*16 array, dimension   
                           (N) if SIDE = 'L'   
                        or (M) if SIDE = 'R'   

    =====================================================================   


       Parameter adjustments */
    /* Table of constant values */
    static complexdouble<T> c_b1 = {1.,0.};
    static complexdouble<T> c_b2 = {0.,0.};
    static integer c__1 = 1;
    
    /* System generated locals */
    integer c_dim1, c_offset;
    complexdouble<T> z__1;
    /* Local variables */
    //extern logical lsame_(char *, char *);
    //extern /* Subroutine */ int zgerc_(integer *, integer *,doublecomplex *, 
    //    doublecomplex *, integer *, doublecomplex *, integer *, 
    //    doublecomplex *, integer *), zgemv_(char *, integer *, integer *, 
    //    doublecomplex *, doublecomplex *, integer *, doublecomplex *, 
    //    integer *, doublecomplex *, doublecomplex *, integer *);


    --v;
    c_dim1 = *ldc;
    c_offset = 1 + c_dim1 * 1;
    c__ -= c_offset;
    --work;

    /* Function Body */
    if (lsame_(side, "L")) {

/*        Form  H * C */

	if (tau->r != (T)0. || tau->i != (T)0.) {

/*           w := C' * v */

	    zgemv_("Conjugate transpose", m, n, &c_b1, &c__[c_offset], ldc, &
		    v[1], incv, &c_b2, &work[1], &c__1);

/*           C := C - v * w' */

	    z__1.r = (T)-1* tau->r, z__1.i = (T)-1* tau->i;
	    zgerc_(m, n, &z__1, &v[1], incv, &work[1], &c__1, &c__[c_offset], 
		    ldc);
	}
    } else {

/*        Form  C * H */

	if (tau->r != (T)0. || tau->i != (T)0.) {

/*           w := C * v */

	    zgemv_("No transpose", m, n, &c_b1, &c__[c_offset], ldc, &v[1], 
		    incv, &c_b2, &work[1], &c__1);

/*           C := C - w * v' */

	    z__1.r = (T)-1* tau->r, z__1.i = (T)-1* tau->i;
	    zgerc_(m, n, &z__1, &work[1], &c__1, &v[1], incv, &c__[c_offset], 
		    ldc);
	}
    }
    return 0;

/*     End of ZLARF */

} /* zlarf_ */

