/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
$Id: hankel.C,v 1.1 2005/07/11 08:33:35 mourrain Exp $
**********************************************************************/

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS


//OPEN 
namespace linalg {
//----------------------------------------------------------------------
template <class C,class V>
void mult_vect(V & r, const hankel<C> & a, const V & b)
{  
  #ifdef VERBATIM 
	std::cout <<"Using fft for the multiplication of hankel<C>"<<std::endl;
  #endif
  unsigned long col = a.nbcol(), row = a.nbrow();
  unsigned long N = 1 << ((int)(1 + (int) (log(row+col-1)/log(2))));
  std::complex<typename realof<C>::TYPE>  w ;

  Root_Unity(w, N);
  
  std::complex<typename realof<C>::T>* XA = 
    new std::complex<typename realof<C>::T>[N];
  std::complex<typename realof<C>::TYPE>* XB = 
    new std::complex<typename realof<C>::T>[N];

  for (unsigned long i = 0; i < row ; i++) XA[i] = a(i,col-1);
  for (unsigned long i = N-col+1, k = 0; i < N ; i++, k++) XA[i] = a(0,k);

  // Bit reversal permutation, to allow Butterfly algorithm to calculate FFT
  // on Matrix and Vector values
      
  Permut(XA, N);
  FFT(N, w, XA,0);

  for (unsigned long i = 0; i < col ; i++) XB[i] = b[col-i-1];
      
  Permut(XB, N);
  FFT(N, w, XB, 0);
      
  // multiplication of FFT(Matrix) by FFT(Vector)
  for (unsigned long k = 0; k < N; k++)  XA[k] *= XB[k] ; 

  // result of this multiplication is also permuted for the same reason
  Permut(XA, N);

  // Backward FFT , to retrieve the result of multiplication 
  // of Matrix-Vector
  FFT(N, pow(w,-1), XA, 1);

  for (unsigned long k = 0; k < row; k++) r[k]=(XA[k].real()/N);

  delete [] XA;
  delete [] XB;
}
 
} //CLOSE linalg

__END_NAMESPACE_SYNAPS


