#ifndef LAPACK_SYM2D_H
#define LAPACK_SYM2D_H

//----------------------------------------------------------------------
#include <synaps/init.h>
#include "f2c.h"

#ifndef F77NAME
	#ifdef F77_FUNC
		#define F77NAME(x) F77_FUNC(x,x)
	#else
		#define F77NAME(x) x##_
	#endif
#endif


__BEGIN_NAMESPACE_SYNAPS

namespace lapack
{
  template <class C>
  struct sym2d: public linalg::sym2d<C> 
  {
    unsigned int lda_;
    sym2d(): linalg::sym2d<C>() {}
    sym2d(const sym2d<C> & v): 
      linalg::sym2d<C>(v), lda_(v.lda_){}
    sym2d(unsigned int i, unsigned int j=1): 
      linalg::sym2d<C>(i,j), lda_(i){}
    sym2d(unsigned int i, unsigned int j, C* t): 
      linalg::sym2d<C>(i,j,t), lda_(i){}
    sym2d(unsigned int i, unsigned int j, C* t, char c): 
       linalg::sym2d<C>(i,j,t,c), lda_(i) {}
    sym2d(unsigned int i, unsigned int j, unsigned int k, C* t): 
       linalg::sym2d<C>(i,j,t), lda_(i) {this->nbrow_=k;}
    sym2d(const char * nm): linalg::sym2d<C>(nm) {lda_ = this->nbrow_;}
    
    sym2d<C> & operator=(const sym2d<C> & v);
    
    
    void resize(unsigned int i, unsigned int j)
      {
	this->linalg::sym2d<C>::resize(i,j);
	lda_=i;           //XXX
      } 
  };
}

__END_NAMESPACE_SYNAPS

//====================================================================
#ifdef SYNAPS_WITH_LAPACK

__BEGIN_NAMESPACE_SYNAPS
namespace lapack
{
  integer decomp(const LU & lu, sym2d<double> & A);
  template<class C> void eignval(typename linalg::rep1d<C>& V, const sym2d<C> & A, Real);

}
__END_NAMESPACE_SYNAPS
//====================================================================
#ifndef SEPARATE_COMPILATION
#include "lapack_sym2d.C"
#endif
//====================================================================
#endif  //SYNAPS_WITH_LAPACK
//====================================================================
#endif // LAPACK_SYM2D_H

