#include "row-col-access.h"

__BEGIN_NAMESPACE_SYNAPS


template<class C1, class R1, 
	 class C2, class R2 > inline
void add( MatrDse<C1,R1> & r, const MatrDse<C2,R2> & a )
{
  using namespace MATRIX;
  add(r.rep(),a.rep());

};

/// matrix = matrix + matrix
template<class C1, class R1, 
	 class C2, class R2, 
	 class C3, class R3> inline
void add( MatrDse<C1,R1> & r, const MatrDse<C2,R2> & a, const MatrDse<C3,R3> & b )
{
  if ( &r == &a ) { add(r,b); return;};
  if ( &r == &b ) { add(r,a); return;};
  r.rep().resize(a.nbrow(),b.nbcol());
  using namespace MATRIX;
  add(r.rep(),a.rep(),b.rep());

};



/// matrix = matrix - matrix
template<class C1, class R1,
	 class C2, class R2, 
	 class C3, class R3> inline
void sub( MatrDse<C1,R1>& r, const MatrDse<C2,R2> & a, const MatrDse<C3,R3> & b )
{
  using MATRIX::sub;
  r.rep().resize(a.nbrow(),b.nbcol());
  sub(r.rep(),a.rep(),b.rep());
}

/// matrix = matrix * matrix
template<class C1, class R1,
	 class C2, class R2,
	 class C3, class R3> inline
void mul(MatrDse<C1,R1> & r, const MatrDse<C2,R2> & a, const MatrDse<C3,R3> & b )
{
  using namespace MATRIX;
  if ( &r == &a ) { mul(r,b); return; };
  if ( &r == &b ) { MatrDse<C3,R3> cb = b; mul(r,a,cb); return; };
  r.rep().resize(a.nbrow(),b.nbcol()); 
  init(r,C1(0));
  mul(r.rep(),a.rep(),b.rep());
}

/// matrix = matrix * matrix
template<class C1, class R1,
	 class C2, class R2 >
void mul(MatrDse<C1,R1> & r, const MatrDse<C2,R2> & a )
{
  using namespace MATRIX;
  MatrDse<C1,R1> tmp = r;
  mul(r,tmp,a);
}
  
/// vector = matrix * vector
template<class Cr, class Rr, 
	 class Cm, class Rm, 
	 class Cv, class Rv >
void mul( VectDse<Cr,Rr> & r, const MatrDse<Cm,Rm> & m, const VectDse<Cv,Rv> & v ) 
{
  {
    using namespace VECTOR;
    r.rep().resize( m.nbrow() );
    init(r.rep(),Cr(0));
  }
  using namespace MATRIX;
  mul_vect(r.rep(),m.rep(),v.rep());
};

/// matrix = matrix * scalar
template<class C1, class R1, 
	 class C2, class R2, 
	 class C3> inline
void mul( MatrDse<C1,R1> & r, const MatrDse<C2,R2> & a, const C3 & b )
{
  using namespace MATRIX;
  r.rep().resize( a.nbrow(),a.nbcol());
  mul_ext(r.rep(),a.rep(),b);
}

/// matrix = scalar * matrix
template<class C1, class R1, 
	 class C2, class R2, 
	 class C3> inline
void mul( MatrDse<C1,R1> & r,  const C3 & b, const MatrDse<C2,R2> & a)
{
  using namespace MATRIX;
  r.rep().resize( a.nbrow(),a.nbcol());
  mul_ext(r.rep(),a.rep(),b);
}

/// matrix *= scalar
template<class C1, class R1, class C2> inline
void mul( MatrDse<C1,R1> & r, const C2 & x )
{
  using namespace MATRIX;
  mul_ext(r.rep(),x);
};

/// matrix = matrix / scalar
template<class C1, class R1, 
	 class C2, class R2,
	 class C3> inline
void div( MatrDse<C1,R1> & r, const MatrDse<C2,R2> & a, const C3 & x )
{
  using namespace MATRIX;
  r.resize( a.nbrow(), a.nbcol() );
  div_ext(r.rep(),a.rep(),x);
};

/// matrix /= scalar 
template<class C1, class R1, class C2> inline
void div( MatrDse<C1,R1> & r, const C2 & x )
{
  using namespace MATRIX;
  div_ext(r.rep(),x);
};


template<class C, class R>
void submatrix( MatrDse<C,R> & sm,   const MatrDse<C,R> & m, const Range2d & rg )
{
  using namespace MATRIX;
  submatrix(sm.rep(),m.rep(),rg);
};

template<class Ca, class Ra, class Cb, class Rb>
void col(  MatrDse<Ca,Ra>& sm, const MatrDse<Cb,Rb> & m, const Range & rg )
{
  submatrix(sm,m,Range2d(0,m.nbrow()-1,rg.i1,rg.i2));
};

template<class Ca, class Ra, class Cb, class Rb>
void row(  MatrDse<Ca,Ra>& sm, const MatrDse<Cb,Rb> & m, const Range & rg )
{
  submatrix(sm,m,Range2d(rg.i1,rg.i2,0,m.nbcol()-1));
};

/// Row of index @i@ of a matrix (local view).
template <class Ca, class Ra, class Cb, class Rb>  inline 
void row( VectDse<Ca,Ra> & rt, const MatrDse<Cb,Rb> & a, int i)
{
  rt.resize( a.nbcol() );
  for ( int k = 0; k < rt.size(); k ++ ) rt[k] = a(i,k);
}

template<class C, class R> 
VectDse<C, row_access< MatrDse<C,R> > > 
row( const MatrDse<C,R> & m, int i )
{
  return VectDse<C, row_access< MatrDse<C,R> > >
    (row_access< MatrDse<C,R> >(m,i));
};

template<class C, class R>
VectDse<C, col_access< MatrDse<C,R> > >
col( const MatrDse<C,R> & m, int i )
{
  return VectDse<C, col_access< MatrDse<C,R> > >
    (col_access< MatrDse<C,R> >(m,i));
};

//----------------------------------------------------------------------
/// Column of index @i@ of a matrix (local view).
template <class Ca, class Ra, class Cb, class Rb>  inline 
void col( VectDse<Ca,Ra> & rt, const MatrDse<Cb,Rb> & a, int i)
{
  rt.resize( a.nbrow() );
  for ( int k = 0; k < rt.size(); k ++ ) rt[k] = a(k,i);
}



__END_NAMESPACE_SYNAPS
