/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
**********************************************************************
    $Id: rep3d.h,v 1.1 2005/07/11 08:33:35 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_LINALG_ARRAY3D_H
#define SYNAPS_LINALG_ARRAY3D_H

#include <synaps/init.h>
#include <synaps/arithm/Norm.h>
#include <synaps/linalg/MethodName.h>
#include <synaps/base/stride_iterator.h>
#include <synaps/linalg/rep1d.h>


__BEGIN_NAMESPACE_SYNAPS

namespace linalg {  

  typedef unsigned size_type;
 
  /**  Class of 3d tensors derived from @linalg::rep1d<C>@ including
      the number of rows  @nbrow_@ the number of columns @n2_@ and 
      the height of the matrix. The matrix is stored by column in the
      array  @linalg::rep1d<C>@.**/
  template < class C >
  struct rep3d : public rep1d<C> {
    
    typedef rep1d<C> rep1d;

    /** Row, column, heigth. **/
    size_type n1_, n2_, n3_;
    
    /** Default constructor. **/
    rep3d(): linalg::rep1d< C >(){}

    /** Copy constructor. **/
    rep3d( const rep3d< C > & v ):
      linalg::rep1d< C >( v ), n1_( v.n1_ ), n2_( v.n2_ ), n3_( v.n3_ ) {}
    
    /** Regular constructor. **/
    rep3d( size_type i, size_type j, size_type k ): 
      linalg::rep1d< C >( i*j*k ), n1_( i ), n2_( j ), n3_( k ) {}

    rep3d( size_type i, size_type j, size_type k, C* t): 
      linalg::rep1d< C >( i*j*k,t ), n1_( i ), n2_( j ), n3_( k ) {}

    rep3d( size_type i, size_type j, size_type k, C* t, char c );
    
    rep3d( const char * nm );
    
    rep3d< C > & operator=( const rep3d< C > & v);
    
    C & operator()( size_type i, size_type j, size_type k ) {
      assert( i < n1_ && j < n2_ && k < n3_ );
      return this->tab_[ i+n1_*j+n1_*n2_*k ];
    }
    
    const C & operator()( size_type i, size_type j, size_type k ) const {
      assert( i < n1_ && j < n2_ && k < n3_);
      return this->tab_[ i+n1_*j+n1_*n2_*k ];
    }
    
    /** Accessor. **/
    size_type nbrow() const { return n1_; }
    size_type nbcol() const { return n2_; }
    size_type nbhgt() const { return n3_; }
    
    void resize( size_type i, size_type j, size_type k ){
      this->linalg::rep1d< C >::resize( i*j*k );
      n1_ = i;
      n2_ = j;
      n3_ = k;
    }
    
  }; // end struct
  
  
  template < class C >
  rep3d< C >::rep3d(const char * nm){
    std::ifstream ins( nm, std::ios::in );
    ins >> n1_ >> n2_ >>n3_;
    this->size_ = n1_*n2_*n3_; this->tab_ = new C[this->size_];
    for( size_type i = 0; i < this->size_ ; i++ ) 
      ins >>this->tab_[i];
    ins.close();
  }


  template < class C >
  rep3d< C > & rep3d< C >::operator=( const rep3d< C > & v ){
    this->linalg::rep1d< C >::operator=( v );
    n1_ = v.nbrow();
    n2_ = v.nbcol();
    n3_ = v.nbhgt();
    return (*this);
  }

} // namespace linalg


__END_NAMESPACE_SYNAPS

#endif // SYNAPS_LINALG_ARRAY3D_H

