
#ifdef HAVE_CONFIG_H
	#include <synaps/config.h>
#endif


#ifdef HAVE_UNISTD_H
	#include <unistd.h>
#endif


#include <cmath>
#include <sys/types.h>
#include <sys/times.h>
#include <ctime>
#include <sys/time.h>
#include <cstdio>
#include <cstdlib>
#include <string>
#include <cassert>
#include <synaps/init.h>


__BEGIN_NAMESPACE_SYNAPS

namespace sparse {

using std::cout;
using std::cin;
using std::endl;
using std::cerr;

template<typename T>
rep2d<T>::rep2d(const int nrow,const int ncol)
{
  this->nrow=nrow;
  this->ncol=ncol;
  this->rowind=(int*)malloc(sizeof(int)*(this->ncol));
  this->colptr=(int*)malloc(sizeof(int)*(1+this->ncol));
  this->nzval=malloc(sizeof(T)*(this->ncol));
  for(int i=0;i<ncol;i++)
    new((T*)nzval+i) T;
  this->nnz=this->ncol;
  for(int i=0;i<this->ncol;i++)
    {
      this->rowind[i]=i;
      this->colptr[i]=i;
      ((T*)this->nzval)[i]=0;
    }
  this->colptr[this->ncol]=this->nnz;
}

template<typename T>
inline rep2d<T>::~rep2d()
{
  //destroystore();
}

template<typename T>
inline void rep2d<T>::destroystore()
{
  free(rowind);
  free(colptr);
  for(int i=0;i<nnz;i++)
    ((T*)nzval+i)->~T();
  free(nzval);
}

template<typename T>
T & rep2d<T>::operator() (const int  &i,const int &j)
{
  static T d=T(0);
  int colbegin=colptr[j]
    ,colend=(j+1<this->ncol)?colptr[j+1]:nnz;
  int t;
  for(t=colbegin;(t<colend)&&(i!=rowind[t]);t++);
  if (t>=colend) return d;
  return ((i==(rowind[t]))?(((T*)(nzval))[t]):d);
}

template<typename T>
T  rep2d<T>::operator() (const int  &i,const int &j) const
{
  static T d=0;
  int colbegin=colptr[j]
    ,colend=(j+1<this->ncol)?colptr[j+1]:nnz;
  int t;
  for(t=colbegin;(t<colend)&&(i!=rowind[t]);t++);
  if (t>=colend) return d;
  return ((i==(rowind[t]))?(((T*)(nzval))[t]):d);
}

template<typename T>
T  &rep2d<T>::operator() (const int &i,const int &j,const T &d)
{
  static T zero=0;
  int colbegin=colptr[j],colend=(j+1<this->ncol)?colptr[j+1]:nnz;
  int t,carry=0;;
  for(t=colbegin;(t<colend)&&(i!=(rowind[t]));t++);
  if (t>=colend) {t--;carry=1;};
  if (rowind[t]==i)
    {
      if (d!=(T)0)
	{
	  return ((T *)(nzval))[t]=d;
	}
      else
	{
	  if ((t==colbegin)&&(colend==colbegin+1))
	    return ((T *)(nzval))[t]=d;
	  else
	    {
	      ((T*)(nzval)+t)->~T();
	      memmove(((T*)(nzval)+t),((T*)(nzval))+t+1,(nnz-t-1)*sizeof(T));
	      memmove(((int*)(rowind)+t),((int*)(rowind))+t+1
		      ,(nnz-t-1)*sizeof(int));
	      nnz--;

	      for(int k=j+1;k<this->ncol;k++)
		colptr[k]--;

	      colptr[this->ncol]=nnz;

	      return zero;
	    }
	}
    }


  else
    if (d==(T)0) {return zero;}
    else
      {
	nnz++;
	t+=carry;
	T* tmpnzval=(T*)malloc(sizeof(T)*(nnz));
	int *tmprowind=(int*)malloc(sizeof(int)*nnz);
	memcpy(((T*)(tmpnzval)),((T*)(nzval)),(t)*sizeof(T));
	memcpy(tmprowind,(rowind),(t)*sizeof(int));
	//faire un new avec placement sur tmpnzval
	new(tmpnzval+t) T;
	tmpnzval[t]=d;
	tmprowind[t]=i;
	if (t==colbegin) colptr[j]=t;
	for (int i=j+1;i<this->ncol;i++)
	  colptr[i]++;
	memcpy(((T*)(tmpnzval))+t+1,((T*)(nzval))+t,(nnz-t-1)*sizeof(T));
	memcpy(tmprowind+t+1,rowind+t,(nnz-t-1)*sizeof(int));

	free((T*)(nzval));
	nzval=(void*)tmpnzval;
	free(rowind);
	rowind=tmprowind;
	colptr[this->ncol]=nnz;
	return (((T *)(nzval))[t]);
      }
}

template<typename typmat,typename coeff>
void allocstore(typmat &m,int nrow,int ncol,coeff val)
{
  //cout<<"ici alloctsore"<<endl;
  m.ncol=ncol;
  m.nrow=nrow;
  m.rowind=(int*)malloc(sizeof(int)*(m.ncol));
  m.colptr=(int*)malloc(sizeof(int)*(1+m.ncol));
  m.nzval=malloc(sizeof(coeff)*(m.ncol));
  for(int i=0;i<m.ncol;i++)
    new((coeff*)m.nzval+i) coeff;
  m.nnz=m.ncol;
  for (int i=0;i<m.ncol;i++)
     {
      m.rowind[i]=i;
      m.colptr[i]=i;
      ((coeff*)m.nzval)[i]=val;
    }
  m.colptr[m.ncol]=m.nnz;
  //  cout<<"fin alloc"<<endl;
  return;
}

template<class C>
std::ostream & print( std::ostream & os, const rep2d<C> & m)
{
  os<<m.nbrow()<<"x"<<m.nbcol()<<endl;
  for(int j=0;j<(int)m.nbcol();++j)
    {
      int colbegin=m.colptr[j]
	, colend=(j+1<m.ncol)?(m.colptr[j+1]):m.nnz;
      for(int t=colbegin;t<colend;t++)
	os<<"("<< (m.rowind[t]) <<","<<j<<") "<< ((C*)m.nzval)[t] <<" ";
    }

  return os;
}

}//End namespace sparse

__END_NAMESPACE_SYNAPS

#include "superLU/LUdecomp.C"

