/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
$Id: sym2d.h,v 1.1 2005/07/11 08:33:37 mourrain Exp $
**********************************************************************/

#ifndef SYNAPS_LINALG_SYM2D_H
#define SYNAPS_LINALG_SYM2D_H
//======================================================================
#include <synaps/init.h>
#include <synaps/arithm/Norm.h>
#include <synaps/linalg/MethodName.h>
#include <synaps/base/stride_iterator.h>
#include <synaps/linalg/rep1d.h>


__BEGIN_NAMESPACE_SYNAPS

//======================================================================
namespace linalg {  

//----------------------------------------------------------------------
  typedef unsigned size_type;
  unsigned sym2dsz(unsigned i, unsigned j)
  { 
    unsigned m = std::max(i,j);
    return ( (m*(m+1))/2 );
  }


  unsigned sym2dcf(unsigned i, unsigned j)
  { 

    return (i<j ? i + (j*(j+1))/2 : j + (i*(i+1))/2 );
  }   
 
  //----------------------------------------------------------------------
  /** 
      Class of mat_rep derived from @linalg::rep1d<C>@ including the 
      number of rows  @nbrow_@ and the number of columns @nbcol_@
      of the matrix. The matrix is stored by column in the array @linalg::rep1d<C>@.
  **/
  template <class C>
  struct sym2d : public linalg::rep1d<C> {
    
    typedef C*                              col_iterator;
    typedef stride_iterator<C,C*,size_type> row_iterator;
    typedef sym2d<C>                        self_t;
    
    size_type nbrow_, nbcol_;
    
    sym2d(): linalg::rep1d<C>(){}
    sym2d(const self_t & v): 
      linalg::rep1d<C>(v), nbrow_(v.nbrow_), nbcol_(v.nbcol_) {}
    sym2d(size_type i,size_type j): 
      linalg::rep1d<C>(sym2dsz(i,j)), nbrow_(i),nbcol_(j) {}
    sym2d(size_type i, size_type j, C* t): 
      linalg::rep1d<C>(sym2dsz(i,j),t), nbrow_(i), nbcol_(j) {}
    sym2d(size_type i, size_type j, C* t, char c);
    sym2d(const char * nm);

    sym2d<C> & operator=(const sym2d<C> & v)
      {
	this->linalg::rep1d<C>::operator=(v); 
	nbrow_ = v.nbrow(); nbcol_ = v.nbcol(); 
	return *this;
      }

    C &       operator()(size_type i, size_type j) 
      {return this->tab_[sym2dcf(i,j)];}
    const C & operator()(size_type i, size_type j) const 
      {return this->tab_[sym2dcf(i,j)];}
  
    size_type nbrow() const {return nbrow_;}
    size_type nbcol() const {return nbcol_;}


    self_t resize(size_type i,size_type j)
      {
	nbrow_=i; nbcol_=j; 
	this->linalg::rep1d<C>::resize(sym2dsz(i,j)); 
	return *this;
      }
    
  };
}

__END_NAMESPACE_SYNAPS

//----------------------------------------------------------------------
//#include "synaps/linalg/sym2d.C"
//----------------------------------------------------------------------

#endif // SYNAPS_LINALG_SYM2D_H

