/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History:
$Id: toeplitz.h,v 1.1 2005/07/11 08:33:37 mourrain Exp $
**********************************************************************/
#ifndef toeplitz_H
#define toeplitz_H
//----------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/arithm/GENERIC.h>
#include <synaps/linalg/rep1d.h>
#include <synaps/base/Range.h>
#include <synaps/linalg/FFT.h>


__BEGIN_NAMESPACE_SYNAPS


template<char O,class A, class B> struct OP;
//----------------------------------------------------------------------
//OPEN
namespace linalg {
//----------------------------------------------------------------------
/**
  Containers for Toeplitz matrices, as a subclass of @array1d@.
  Defined by its number of row @nbrow_@, its number of columns @nbcol_@.
  The elements of the first row and first column are stored, starting 
  from @(0,nbcol)@ up to @(nbrow,0)@.
  The product of a Toeplitz matrix by a vector is performed by FFT 
  computations (\ref{linalg/FFT.h}).   
**/
template<class C> 
struct toeplitz : public linalg::rep1d<C> 
{

  unsigned int nbrow_, nbcol_;

  typedef C                coeff_t;
  typedef typename linalg::rep1d<C>::size_type   size_type;
  typedef linalg::rep1d<C>   row_t;
  typedef linalg::rep1d<C>   col_t;

  toeplitz(): linalg::rep1d<C>(){}
  toeplitz(const toeplitz<C> & v):linalg::rep1d<C>(v),
    nbrow_(v.nbrow_),nbcol_(v.nbcol_){}
  toeplitz(unsigned int i, unsigned int j): 
    linalg::rep1d<C>(i+j-1),nbrow_(i),nbcol_(j) {}
  toeplitz(unsigned int i, unsigned int j, C* t): 
    linalg::rep1d<C>(i+j-1,t),nbrow_(i),nbcol_(j){}

  void resize(unsigned int i) {this->linalg::rep1d<C>::resize(i);}
  void resize(unsigned int i, unsigned int j);

  C   operator()(unsigned int i, unsigned int j) const 
        {return this->tab_[i-j+nbcol_-1];} 

  void transpose() ;
  
  size_type nbrow() const {return nbrow_;}
  size_type nbcol() const {return nbcol_;}
};
//----------------------------------------------------------------------
// template<class C>
// void assign(toeplitz<C> & r, const toeplitz<C>* v) 
// {
//   assign((linalg::rep1d<C> &)r, (linalg::rep1d<C>*) v);
//   r.nbrow_=v->nbrow_;
//   r.nbcol_=v->nbcol_;
// }
//----------------------------------------------------------------------
  template<class C, class M>
  void submatrix(toeplitz<C> & r, const M & m, const Range2d & rg)
  {
    typedef typename toeplitz<C>::size_type size_type;
    
    for(size_type i=rg.j1; i< rg.j2+1; i++)
      r.tab_[-i+rg.j2]=m(rg.i1,i);
    
    for(size_type i=rg.i1+1; i< rg.i2+1; i++) 
	r.tab_[i-rg.i1+rg.j2-rg.j1]=m(i,rg.j1);
  }
  
//---------------------------------------------------------------
// Assignment of a matrix to a container {\tt toeplitz}
template<class S, class C>
inline void assign(toeplitz<C> & r, const S & M)
{
  for(unsigned int i=0;i<M.nbcol();i++) r(0,i)=M(0,i);  
  for(unsigned int i=1;i<M.nbrow();i++) r(i,0)=M(i,0);  
}
//----------------------------------------------------------------------
} //CLOSE linalg


__END_NAMESPACE_SYNAPS

//======================================================================
#ifndef SEPARATE_COMPILATION
#include "synaps/linalg/toeplitz.C"
#endif


#endif //toeplitz_H


