#include <synaps/init.h>
#ifdef SYNAPS_WITH_LAPACK
#include "synaps/linalg/VectDse.h"  
#include "synaps/linalg/MatrDse.h"  
#include "synaps/linalg/lapack.h"  
#include "synaps/linalg/Svd.h"  
#include "synaps/linalg/Rank.h"  
#include "synaps/linalg/Decomp.h"  
#include "synaps/linalg/Det.h"  
#include "synaps/linalg/Eigen.h"  
#include "synaps/linalg/Uschur.h"  
//#include "synaps/linalg/maple.h"

typedef VectDse<double> vector_t; 
  // Vector definition.
typedef VectDse<std::complex<double> > VECTC; 
  // The same with complex coefficients.
typedef MatrDse<double,lapack::rep2d<double> > matrix_t;
  // Marix definition.
typedef MatrDse<std::complex<double>,
                lapack::rep2d<std::complex<double> > > MATC;
  // The same with complex coefficients.

int main(int argc, char** argv) {

  using std::cout;
  using std::cin;
  using std::endl;

  double v[]={1,-3,1,2,3,4,2,2,4,5,5,5,5,-2, 1,2,3,1, -2.8,-2.4,1,.2,5.8};
  matrix_t A(4,4,v, ByRow()), 
      B(4,4,"1 2 1 1 3 4 2 2 5 5 5 5 -2  1 2 3 1  .2 5.8", ByCol());

  cout<<"A="<<A<<", B="<<B<<endl;

  cout <<"Conversion: "<<let::convert(A,type::As<Seq<vector_t> >())<<endl;

  vector_t V(v+1,v+5);  cout<<"V="<<V<<endl;

  vector_t W(A*V);      cout<<"A*V="<<W<<endl;

  matrix_t Uo, Vo;

  W =Svd(A);        cout<<"Svd(A)   = "<<W<<endl;
  W =Svd(A,Uo);    
  W =Svd(A,Uo,Vo);   
  cout <<"Left  singular vectors:"<<endl;
  cout<<Uo<<endl;
  cout <<"Right singular vectors:"<<endl;
  cout<<Vo<<endl;
  
  cout <<"Conditionnement of A: "<< Cond(A)<<endl;
  
  cout<<"Rank(A)  = "<<Rank(A)<<endl;
  //Triangulation by Gauss method.
  B = Triang<LU>(A);    cout<<B<<endl;
  B = Triang<Bareiss>(A);  cout<<B<<endl;
  
  //Determinant computed by Bareiss method.			 
  cout<<"Det(A) = "<<Det(A)<<endl;
 

#ifdef SYNAPS_WITH_LAPACK
  //Eigenvalue and eigenvector computations.
  VECTC L=Eigenval(A);         cout<<"Eigenvalues of A: "<<L<<endl;
  MATC  E=Eigenvct(A);         cout<<"Eigenvectors of A:\n"<<E<<endl;
  vector_t Lr=Eigenval(A,Real());  cout<<"Real eigenvalues of A:\n"<<Lr<<endl;
  matrix_t Er=Eigenvct(A,Real());  cout<<"Real eigenvectors of A:\n"<<Er<<endl;
  
  L =Eigenval(A,B);            cout<<"Eigenvalues of (A,B): "<<L<<endl;
  E =Eigenvct(A,B);            cout<<"Eigenvectors of (A,B):\n"<<E<<endl;
  Lr=Eigenval(A,B,Real());     cout<<"Real eigenvalues of (A,B): "<<Lr<<endl;
  Er=Eigenvct(A,Real());       cout<<"Real eigenvectors of (A,B):\n"<<Er<<endl;

  B =USchur(A,2);  cout <<"2x2 Schur complement:\n"<<B<<endl;
#else
  cout <<"  Lapack not connected"<<endl;
#endif
  //#else return(73);
  //  cmaple<<"A:= "<<A<<endl;

};
#else
#include <iostream>
int main()
{
  std::cout <<"  Lapack is not connected"<<std::endl;
  return(77);
}
#endif
