/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*      Author(s): B. Mourrain, GALAAD, INRIA                         *
**********************************************************************
$Id: DegLex.h,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_MPOL_DEGLEX_H
#define SYNAPS_MPOL_DEGLEX_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS

/** 
Class specifying the degree inverse lexicographic order on the monomials
of type {\tt M}. $m_{1} < m_{2}$ if either the degree of
$degree(m_{1})<degree(m_{2})$ or $degree(m_{1})= degree(m_{2})$ and $m_{2}$
is greater than $m_{1}$ for the inverse lexicographic ordering.
**/
struct DegLex 
{
  template <class M>  static bool less (const M &, const M &);
  template <class M>  bool operator() (const M & m1, const M & m2) const 
    {return less(m1,m2);}
};
 
template <class M>
bool DegLex::less(const M & m1, const M & m2) {
  
   if (degree(m1) < degree(m2) ) return true;
   else if (degree(m1) > degree(m2)) return false;
   else {
     int n1 = m1.nvars(), n2 =m2.nvars();
     if(n1 <0 && n2>0) return false;
     if(n1 <0 && n2<0) return false;
     if(n2 <0 && n1>0) return true;
     int n = std::max(n1,n2); 
     int i = std::max(n,0); 
     while(i>=0 && m1[i] == m2[i]) i--;
     if(i <0) 
       return false;
     else 
       return (m1[i]> m2[i]);
   }
}

// template <class M>
// bool operator<(const M & m1, const M & m2) {return DegLex<M>::less(m1,m2);}

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_MPO_DEGLEX_H

