/*********************************************************************
*      This file is part of the source code of SYNAPS software.      *
*      (C) B. Mourrain, GALAAD, INRIA                                *
**********************************************************************
History: 
	07/2001 created I.Z. Emiris
	04/2002 updated I.Z. Emiris
**********************************************************************/
#ifndef SYNAPS_MPOL_ILEX_H
#define SYNAPS_MPOL_ILEX_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS

/**
Class specifying the lexicographic order on the monomials of type @M@,
of inverse order of the variables. 
Decide the order on the leftmost element which differs.
**/
struct Ilex 
{
  template <class M>
  static bool less (const M &, const M &);
  template <class M>
  bool operator() (const M & m1, const M & m2) const {return less(m1,m2);}
};
 
template <class M>
inline bool Ilex::less(const M & m1, const M & m2) {
   int n1 = m1.nvars(), n2 =m2.nvars();
   int i = 0, mx = std::max(n1,n2);
	
//    if(n1 <0 && n2>0) return true;
//    if(n2 <0 && n1>0) return false;

   while( m1[i] == m2[i] && i<=mx) i++;
   if(i >mx) 
     return false;     
   else 
     return (m1[i]< m2[i]);
}


__END_NAMESPACE_SYNAPS

#endif // SYNAPS_MPOL_ILEX_H

