/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
$Id: Vlex.h,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_MPOL_VLEX_H
#define SYNAPS_MPOL_VLEX_H

#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS


/**
 Class defining the following order on the monomials $m_{1}$ and $m_{2}$;
 $m_{1} < m_{2}$ if either the degree of $degree(m_{1})>degree(m_{2})$ or
 $degree(m_{1})= degree(m_{2})$, or they are equal and $m_{1}$ is greater for 
 the inverse lexicopgraphic ordering.
 In a sequence of decreasing monomial for this order, the degree are 
 increasing. 
**/
struct Vlex {
  template <class M> 
  static bool less(const M &, const M &);
  template <class M> 
  bool operator() (const M & m1, const M & m2) const {return less(m1,m2);}
};
 
template <class M>
bool Vlex::less(const M & m1, const M & m2) {
   if (Degree(m1) > Degree(m2) ) return true;
   else if (Degree(m1) < Degree(m2)) return false;
   else {
     int n1 = m1.nvars(), n2 =m2.nvars();
     if(n1 <0 && n2>0) return false;
     if(n2 <0 && n1>0) return true;
     int n = max(n1,n2); 
     int i = max(n,0); 
     while(i>=0 && m1[i] == m2[i]) i--;
     if(i <0) 
       return false;
     else 
       return (m1[i]> m2[i]);
   }
}


__END_NAMESPACE_SYNAPS

#endif // SYNAPS_MPOL_VLEX_H

