/*********************************************************************
*      This file is part of the source code of SYNAPS library.       *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
**********************************************************************/
#ifndef synaps_mpol_lisp_H
#define synaps_mpol_lisp_H
//--------------------------------------------------------------------
#include <iostream>
#include <synaps/base/lisp.h>
#include <synaps/mpol/Variables.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace MONOMIAL 
{     
  template<class OSTREAM, class MONOM> 
  OSTREAM& lisp(OSTREAM& os, 
		const MONOM & m,
		const Variables & V,
		int i=-1) 
  {
    //    std::cout <<"\nP: "<<i<<" "<<m<<" "<<m.rep().size()<<std::endl;
    if(i<int(m.rep().size()))
      {
	os<<"(* ";
	if(i<0)    
	  {
	    print_lisp(os,m.coeff())<<" ";
	    MONOMIAL::lisp(os,m,V,i+1);
	  }
	else
	  {
	  if(m[i] <=1)
	    {
	      os<<" "<<V[i]<<" ";
	    }
	  else if(m[i]>1)
	    {
	      os<<"(^ "<<V[i]<<" "<<m[i]<<") ";
	    }
	    MONOMIAL::lisp(os,m,V,i+1);
	  }
	os<<")";
      }
    else
      os<<" 1";
    return os;
  }
}
//--------------------------------------------------------------------
namespace MPOLDST
{
  template <class OSTREAM, class ITER> inline 
  OSTREAM& lisp(OSTREAM& os, ITER it,  const ITER &ed, 
		const Variables & V= Variables::default_) 
  {
    using namespace lisp;
    if(it==ed)
      return os<<" 0";
    else
      {
	os<<"(+ ";
	MONOMIAL::lisp(os,*it,V);
	os <<" ";
	MPOLDST::lisp(os,++it,ed,V);
	os<<")";
      } 
    return os;
  }
} 
//----------------------------------------------------------------------
template<typename C,typename O,typename R> struct MPol;
//----------------------------------------------------------------------
template<class OSTREAM, class C, class R,class O> 
OSTREAM& print_lisp(OSTREAM& os, const MPol<C,O,R>& p) 
{
  return MPOLDST::lisp(os,p.begin(),p.end());
}

template<class C, class R,class O> 
std::ostream& print_lisp(std::ostream& os, const MPol<C,O,R>& p) 
{
  return MPOLDST::lisp(os,p.begin(),p.end());
}
//----------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
#endif //synaps_mpol_lisp_h
