#ifndef SYNAPS_MPOLDSE_VIEW_C
#define SYNAPS_MPOLDSE_VIEW_C
#include <algorithm>
#include <synaps/arithm/pow.h>

__BEGIN_NAMESPACE_SYNAPS

template<class C, class R>
template<class X> inline
MPolDse<C,R>::MPolDse( const X& x ){ 
  using let::assign;
  assign(*this,x); 
};

template<class C,class R> inline
MPolDse<C,R> & MPolDse<C,R>::operator= ( const char *   name ) { 
    using let::assign;
    assign(*this,MPol<C>(name));
    return *this;
};

template<class C,class R>
template<class X> inline
MPolDse<C,R> & MPolDse<C,R>::operator= ( const X & x ) { 
    using let::assign;
    assign(*this,x);
    return *this;
};

template<class C, class R> inline
MPolDse<C, R>::MPolDse( int nvr, const int * szs, const int * vrs ) : data(nvr,szs,vrs) {};

template < class C, class R > inline 
MPolDse < C, R >::MPolDse (const C & x): data (x) {};
template < class C, class R >
template < class O, class E > inline 
MPolDse < C, R >::MPolDse (const MPol < C, O, E > &mpol) : data (mpol) {};
template < class C, class R > inline
MPolDse < C, R >::MPolDse (const char *s) : data(MPol<C>(s)) {};

template<class C, class R> inline const C& MPolDse<C,R>::operator[]( int i ) const { return rep()[i]; };
template<class C, class R> inline  C& MPolDse<C,R>::operator[]( int i ) { return rep()[i]; };
template < class C, class R > inline 
MPolDse < C, R > &MPolDse < C, R >::operator+= (const MPolDse < C, R > &mp)
{
  using namespace mpoldse;
  add(rep(),mp.rep());
  return *this;
};

template < class C, class R > inline
MPolDse < C, R > & MPolDse < C, R >::operator+= (const C & x)
{
  using namespace mpoldse;
  add (rep (), x);
  return *this;
};

template < class C, class R > inline
MPolDse < C, R > &MPolDse < C, R >::operator-= (const MPolDse < C, R > &mp)
{
  using namespace mpoldse;
  sub (rep (), mp.rep ());
  return *this;
};

template < class C, class R > inline
  MPolDse < C, R > &MPolDse < C, R >::operator-= (const C & x)
{
  using namespace mpoldse;
  sub (rep (), x);
  return *this;
};

template < class C, class R > inline
  MPolDse < C, R > &MPolDse < C, R >::operator*= (const MPolDse < C, R > &mp)
{
  using namespace mpoldse;
  mul (rep (), mp.rep ());
  return *this;
};

template < class C, class R > inline
  MPolDse < C, R > &MPolDse < C, R >::operator*= (const C & x)
{
  using namespace mpoldse;
  mul (rep (), x);
  return *this;
};

template < class C, class R > inline
bool MPolDse < C, R >::operator== (const MPolDse & mp) const 
{
  return rep () == mp.rep ();
};

template < class C, class R >
template < class O, class E > inline
MPolDse < C, R > &MPolDse < C, R >::operator= (const MPol < C, O, E > &mpol)
{
  using namespace mpoldse;
  this->~MPolDse();
  new (this) MPolDse(mpol);
  return *this;
};

// texp / arithmetic
template < class C, class R > inline 
void add (MPolDse < C, R > &r, const MPolDse < C, R > &a )
{
  using namespace mpoldse;
  add (r.rep (), a.rep () );
};
template < class C, class R > inline 
void add (MPolDse < C, R > &r, const MPolDse < C, R > &a, const MPolDse < C, R > &b)
{
  using namespace mpoldse;
  add (r.rep (), a.rep (), b.rep ());
};

template < class C, class R > inline void
add (MPolDse < C, R > &r, const MPolDse < C, R > &a, const C & b)
{
  using namespace mpoldse;
  add (r.rep (), a.rep (), b);
};

template < class C, class R > inline void
add (MPolDse < C, R > &r, const C & a, const MPolDse < C, R > &b)
{
  using namespace mpoldse;
  add (r, b, a);
};

template < class C, class R > inline void
sub (MPolDse < C, R > &r, const MPolDse < C, R > &a )

{
  using namespace mpoldse;
sub (r.rep (), a.rep () );
}
;
template < class C, class R > inline void
sub (MPolDse < C, R > &r, const MPolDse < C, R > &a, const MPolDse < C,
     R > &b)
{
  using namespace mpoldse;
  sub (r.rep (), a.rep (), b.rep ());
};

template < class C, class R > inline void
sub (MPolDse < C, R > &r, const C & a, const MPolDse < C, R > &b)
{
  using namespace mpoldse;
  sub (r.rep (), a, b.rep ());
};

template < class C, class R > inline void
sub (MPolDse < C, R > &r, const MPolDse < C, R > &a, const C & b)
{
  using namespace mpoldse;
  sub (r.rep (), a.rep (), b);
};

template < class C, class R > inline void
mul (MPolDse < C, R > &r, const MPolDse < C, R > &a )
{
  using namespace mpoldse;
  mul (r.rep (), a.rep () );
};

template < class C, class R > inline void
mul (MPolDse < C, R > &r, const MPolDse < C, R > &a, const MPolDse < C,
     R > &b)
{
  using namespace mpoldse;
  mul (r.rep (), a.rep (), b.rep ());
};

template < class C, class R > inline void
mul (MPolDse < C, R > &r, const MPolDse < C, R > &a, const C & b)
{
  using namespace mpoldse;
  mul (r.rep (), a.rep (), b);
};

template < class C, class R > inline void
mul (MPolDse < C, R > &r, const C & a, const MPolDse < C, R > &b)
{
  using namespace mpoldse;
  mul (r.rep (), b.rep (), a);
};

/*
template < class C, class R >
template < class S >
MPolDse < C, R > &MPolDse < C, R >::operator= (const texp < S > &exp)
{
  eval (*this, exp);
  return *this;
};

template < class C, class R >
template < class S >
MPolDse < C, R > &MPolDse < C, R >::operator+= (const texp < S > &exp)
{
  MPolDse<C,R> tmp;
  eval(tmp,exp);
  add(*this,tmp);
  return *this;
};


template < class C, class R >
template < class S >
MPolDse < C, R > &MPolDse < C, R >::operator-= (const texp < S > &exp)
{
  MPolDse<C,R> tmp;
  eval(tmp,exp);
  sub(*this,tmp);
  return *this;
};

template < class C, class R >
template < class S >
MPolDse < C, R > &MPolDse < C, R >::operator*= (const texp < S > &exp)
{
  MPolDse<C,R> tmp;
  eval(tmp,exp);
  mul(*this,tmp);
  return *this;
};

*/
namespace arithm
{
  template < class C, class R > struct sup_ <C, MPolDse < C, R > >
  { typedef MPolDse < C, R > T; };

  template < class C, class R > struct sup_ <MPolDse < C, R >, C >
  { typedef MPolDse < C, R > T; };

  template<class Op, class Ca, class Ra, class Cb, class Rb> 
  struct binary_operator_prototype< Op, MPolDse<Ca,Ra>, MPolDse<Cb,Rb> >
  {
    typedef typename ringof<Ca,Cb>::T C;
    typedef typename instanceof<Ra,C>::T R;
    typedef MPolDse<C,R> F;
    typedef MPolDse<C,R> U;
    typedef MPolDse<C,R> V;
  };
  
  template<class Op, class Ca, class Ra, class Cb>
  struct binary_operator_prototype< Op, MPolDse<Ca,Ra>, Cb >
  {
    typedef typename ringof<Ca,Cb>::T C;
    typedef typename instanceof<Ra,C>::T R;
    typedef MPolDse<C,R> F;
    typedef MPolDse<C,R> U;
    typedef C V;
  };

  template<class Op, class Ca, class Ra, class Cb>
  struct binary_operator_prototype< Op, Cb,  MPolDse<Ca,Ra> >
  {
    typedef binary_operator_prototype<Op, MPolDse<Ca,Ra>, Cb > X;
    typedef typename X::F F;
    typedef typename X::U V;
    typedef typename X::V U;
  };
  
  #define head template<class Ca,class Ra, class Cb, class Rb>
  #define parm0 MPolDse<Ca,Ra>
  #define parm1 MPolDse<Cb,Rb>
  declare_binary_operator(head,parm0,parm1,_add_,operator+);
  declare_binary_operator(head,parm0,parm1,_sub_,operator-);
  declare_binary_operator(head,parm0,parm1,_mul_,operator*);
#undef head
#define head template<class Ca,class Ra>
#define parm0 MPolDse<Ca,Ra>
#undef parm1
#define parm1 Ca
  declare_binary_operator(head,parm0,parm1,_mul_,operator*);
  declare_binary_operator(head,parm1,parm0,_mul_,operator*);
#undef head
#undef parm0
#undef parm1
};

template < class C, class R >
std::ostream & operator<<(std::ostream & o, const MPolDse < C, R > &mp)
{
  print (o, mp.rep());
  return o;
};

template < class C, class R >
  MPolDse < C, R > operator^(const MPolDse < C, R > &a, unsigned n)
{
  return pow (a, n);
};

template < class C, class R > bool MPolDse < C, R >::operator== (const C & c) const 
{
  return rep () == c;
};

template < class C, class R >
  bool operator== (const C & c, const MPolDse < C, R > &mp)
{
  return mp.rep () == c;
};

template<class C, class R > inline 
int MPolDse<C,R>::nvars() const { return rep().nvr(); };


template<class C, class R> inline int MPolDse<C,R>::varid( int i )  const { 
  return rep().env.vrs()[i];
};



/** 
    Multivariate Polynomial Differentiation.
    Compute the partial derivative of @polynom@ w.r.t @v@ the vth variable of @polynom@.
    The result is stored in @derivative@.
 **/
template<class Coeff, class Rep> inline
void diff( MPolDse<Coeff,Rep>& derivative, const MPolDse<Coeff,Rep>& polynom, int v )
{
  diff(derivative.rep(),polynom.rep(),v);
};

/**
   Multivariate Polynomial Evaluation.
   The type {\tt Prm } must a container type, @parameters@ corresponds to the $0..nvr()$ coordinates.
   The computation is done in the {\tt Sup Type} of the {\tt Contained Type } of {\tt Prm} and {\tt Coeff}.
   The type {\tt Result } is the type of the result, it is assumed to have an operator= on the computation type.
**/
template<class Result, class Coeff, class Rep, class Parameters> inline
void eval( Result& result, const MPolDse<Coeff,Rep>& polynom, const Parameters& parameters ) 
{
  eval( result, polynom.rep(), parameters );
};

__END_NAMESPACE_SYNAPS
#endif
