#ifndef SYNAPS_MPOLDSE_EENV_H
#define SYNAPS_MPOLDSE_EENV_H
#include <synaps/init.h>
#include <synaps/mpol/MPol.h>

__BEGIN_NAMESPACE_SYNAPS

namespace mpoldse
{
  struct eenv
  { 
    void construct( int nvr_, int const * szs_, int const * vrs_, int * mem );
    static int msz( int nvr_ );
    int * szs();
    int * str();
    int * vrs();
    void  cstr();
    void  cvrs();
    int * alloc( int nvr_ );
    void  dealloc();
    static eenv& copy(eenv& e, const eenv& b );
    eenv& cdup();
    /*
    struct vr_t 
    { 
    int v,r; bool operator<( const vr_t& gv ) const { return v < gv.v; }; 
    bool operator==( int gv ) const { return v == gv; }; 
    vr_t() {};
    vr_t( int _v, int _r ) :v(_v), r(_r) {};
    };
    vr_t * gvr();
    const 
    vr_t * gvr() const;
    */
    
    int  * data;
    
  public:
    /// number of variables
    int         nvr()  const;
    /// point to an array representing the order for each variable
    int const * szs()  const;
    /// point to an array representing the stride for each variable
    int const * str()  const;
    /// point to an array representing the global id for each variable
    int const * vrs()  const;
    /// return the total number of coefficients of the dense representation
    int         sz ()  const;
    /// return the order for variable v
    int sz ( int  v ) const;
    /// return the stride for variable v
    int st ( int  v ) const;
    /// return the global id for variable lv
    int vr ( int lv ) const;
    /// return true if gv is in the variable set 
    /// lv is the corresponding local name.
    bool hasvar( int& lv, int gv ) const;
    /// memory size used by the eenv representation in bytes.
    int  msz()          const;
    int localv( int i ) const { return i; };
int mxvr() const { return vrs()[nvr()-1]; };
  int mnvr() const { return vrs()[0]; };
  

    /// constructors
    eenv() : data(0) {};
    template<class C, class O, class R> eenv( const MPol<C,O,R>& mpl );
    eenv( int * data );
    /// create an allocated but uninitialized environement
    /// sizes and names given by szs_ and vrs_
    eenv( int nvr_, int const * szs_ = 0, int const * vrs_  = 0, int * mem = 0 );
    /// copy constructors
    eenv( const eenv& e );
    eenv& operator=( const eenv& e );
    void swap( eenv& e );
    ~eenv();
    static eenv  mrgvrs( const eenv& a, const eenv& b );
    /* new environement that can represent a and b  */
    static eenv  common( const eenv& a, const eenv& b ); 
    /* new environment (can represent a * b) */
    static eenv  mul   ( const eenv& a, const eenv& b );
    /* new environment that can represent. diff(b,lv) */
    static eenv  diff( const eenv& b, int lv );
    bool operator<( const eenv& b ) const;
    
    /// variable names translation from environment i to environment o
    static bool vmap( int * vmap, const eenv& o, const eenv&  i );
    /// addresses translation from environment i to environment o
    static bool oaddress( const eenv&  oenv, unsigned * osupp,
			  const eenv&  ienv, unsigned * isupp = 0 ,unsigned nsp = 0 );
    friend std::ostream& operator<<( std::ostream& o, const eenv& env );
    //friend std::ostream& operator<<( std::ostream& o, const vr_t& v );
        static bool equiv( const eenv& a, const eenv& b );
    static bool equal( const eenv& a, const eenv& b );
    bool operator==( const eenv& a ) const;
    bool operator!=( const eenv& a ) const;
    static bool subset( const eenv& a, const eenv& b );
    static eenv elevation( const eenv& a, const eenv& b );
    void vswap( int * perm );
    static eenv rewrite( const eenv&o, const eenv& i );
  };
}

__END_NAMESPACE_SYNAPS

#include "eenv.C"
#include "eenv-loops.h"

#endif
