#ifndef SYNAPS_MPOLDSE_MONOMIAL_C
#define SYNAPS_MPOLDSE_MONOMIAL_C

#include "monomial.h"

#include "MPOLDSE.h"

__BEGIN_NAMESPACE_SYNAPS 

namespace mpoldse
{
  template < class C > inline
  monomial < C >::monomial (const eenv & e):env (e), data (e.sz ()) {};
  template < class C > inline
  monomial < C >::monomial (const eenv & e, const C & c):env(e),data (e.sz (), c) {};
  template < class C > inline
  monomial < C >::monomial (int nvr, const int *szs, const int *vrs):env (nvr, szs, vrs),data (env.sz(),0) {};
  template < class C > inline
  monomial < C >::monomial (const C & x)
  { int szs = 1; new (this) monomial (1, &szs); data[0] = x; }
  
  template < class C > 
  void realloc (monomial < C > &mpl, const eenv & nenv)
  {
    if (!eenv::equal (mpl.env, nenv ))
      {
	monomial< C > tmp (nenv);
	mpl.swap (tmp);
      };
  };

  template < class C >  inline
  void clear ( monomial < C >& monoms ) { std::fill(monoms.begin(),monoms.end(),C(0)); };
  template < class C >  inline
  void clear ( monomial < C > &mpl, const eenv & nenv)
  {
    if (mpl.env != nenv)
      {
	monomial < C > tmp (nenv, C (0));
	mpl.swap (tmp);
      }
    else clear(mpl);
  };

  template < class C > 
  bool varindex (int &lv, const monomial<C>& mpl, int gv) 
  {
    return mpl.env.hasvar (lv, gv);
  }

  template < class C > bool monomial < C >::operator== (const monomial & mpl) const
  {
    if (!eenv::equal (env, mpl.env))
      return false;
    unsigned *oadd = new unsigned[esz ()];
    eenv::oaddress (env, oadd, mpl.env, 0, 0);
    for (unsigned i = 0; i < esz (); i++)
      if (data[i] != mpl.data[oadd[i]])
	{
	  delete[]oadd;
	  return false;
	};
    delete[]oadd;
    return true;
  };

  template < class Result, class Coeff, class Parameters > inline 
  void eval (Result & result, const monomial < Coeff > & monoms , const Parameters & parameters )
  { 
    levalm( result, monoms.begin (), monoms.env, parameters ); 
  };
  
  template < class Result, class Coeff, class Parameters > inline 
  void subs0 (Result *result, const monomial < Coeff > & monoms , const Parameters & parameters )
  { 
    subs0m( result, monoms.begin (), monoms.env, parameters ); 
  };

  template < class C > 
  void extend (monomial < C > &mpl, const eenv & nenv)
  {
    if ( mpl.env != nenv )
      {
	unsigned *oadd = new unsigned[mpl.esz ()];
	eenv::oaddress (nenv, oadd, mpl.env, 0, 0);
	monomial < C > tmp (nenv);
	vct::icopy (tmp.begin (), oadd, mpl.esz (), mpl.begin ());
	delete[]oadd;
	mpl.swap (tmp);
      };
  };

  // addition optimisable ...(mais pas maintenant)
  template < class X, class Y >
  void waddm (monomial < X > &mpla, const monomial < Y > &mplb)
  {
    assert (eenv::subset (mplb.env, mpla.env));
    unsigned *oadd = new unsigned[mplb.esz ()];
    eenv::oaddress (mpla.env, oadd, mplb.env, 0, 0);
    vct::ipadd (mpla.begin (), oadd, mplb.esz (), mplb.begin ());
    delete[]oadd;
  };


  template < class C > inline
  void add (monomial < C > &r, const monomial < C > &a)
  {
    extend (r, eenv::common (r.env, a.env));
    waddm (r, a);
  }

  template < class C > inline
  void add (monomial < C > &r, const monomial < C > &a, const monomial < C > &b)
  {
    if ( &r == &a  ) {  add(r,b);  return;  };
    if ( &r  == &b ) {  add(r,a); return; };
    realloc(r,eenv::common(a.env,b.env));
    unsigned *oadd = new unsigned[std::max(a.esz (),b.esz())];
    eenv::oaddress (r.env, oadd,a.env, 0, 0);
    vct::icopy(r.begin(),oadd,a.esz(),a.begin());
    eenv::oaddress(r.env,oadd,b.env,0,0);
    vct::ipadd (r.begin (), oadd,b.esz (), b.begin ());
  };
  
  template < class C > inline
  void sub (monomial < C > &r, const monomial < C > &a, const monomial < C > &b)
  {
    if ( &r == &a  ) {  sub(r,b);  return;  };
    if ( &r == &b  ) {  monomial<C> tmp = b; sub(r,a,tmp); return; };
    
    realloc(r,eenv::common(a.env,b.env));
    unsigned *oadd = new unsigned[std::max(a.esz (),b.esz())];
    eenv::oaddress (r.env, oadd,a.env, 0, 0);
    vct::icopy(r.begin(),oadd,a.esz(),a.begin());
    eenv::oaddress(r.env,oadd,b.env,0,0);
    vct::ipsub (r.begin (), oadd,b.esz (), b.begin ());
  };

  template < class C > inline void add (monomial < C > &a, const C & x)
  {
    a[0] += x;
  }

  template < class X, class Y >
  void wsubm (monomial < X > &mpla, const monomial < Y > &mplb)
  {
    assert (eenv::subset (mplb.env, mpla.env));
    unsigned *oadd = new unsigned[mplb.esz ()];
    eenv::oaddress (mpla.env, oadd, mplb.env, 0, 0);
    vct::ipsub (mpla.begin (), oadd, mplb.esz (), mplb.begin ());
    delete[]oadd;
  };

  template < class X, class Y > inline
  void sub (monomial < X > &r, const monomial < Y > &a)
  {
    extend (r, eenv::common (r.env, a.env)), wsubm (r, a);
  }

  template < class C > inline void sub (monomial < C > &a, const C & x)
  {
    a[0] -= x;
  }

  template < class C > void mul (monomial < C > &r, const C & c)
  {
    vct::scmul (r.begin (), c, r.esz (), 1);
  };

  template < class C > void mul (monomial < C > &r, const monomial < C > &a)
  {
    MPOLDSE::conv(r,a);
  };
  
  
  template < class X, class Y, class Z > inline
  void mul (monomial < X > &r, const monomial < Y > &a, const monomial < Z > &b)
  {
    MPOLDSE::conv(r,a,b);
  };

  template < class C > inline
  void mul (monomial < C > &a, const monomial < C > &b, const C & c)
  {
    mul (a = b, c);
  };


  template < class C >
    std::ostream & operator<< (std::ostream & o, const monomial < C > &mpl)
  {
    MPol < C > tmp;
    convert(tmp,mpl);
    return o << tmp;
  };
  template<class C> 
  void print( std::ostream& o, const monomial<C>& mpl ) { o << mpl; };

  template<class Coeff> inline
  void diff( monomial<Coeff>& res, const monomial<Coeff>& src, int v )
  {
    clear( res, eenv::diff(src.env,v) );
    mdiff( res.begin(), src.env, src.begin(), v );
  };

  template < class C > 
  template < class Q, class O, class R >
  monomial < C >::monomial (const MPol < Q, O, R > &imp): env (imp), data (env.sz (), 0)
  {
    const int *vr = this->vrs ();
    const int *st = this->str ();
    const int  nvr = this->nvr();
    C *data = this->begin ();
    mpolfill(this->begin(),imp,env);
  };
  
  template < class C, class O, class R >
  void convert (monomial < C > &mpl, const MPol < C, O, R > &imp)
  {
    mpl. ~ monomial < C > ();
    new (&mpl) monomial<C>(imp);
  };
  

template < class C, class O, class R >
void convert (MPol < C, O, R > &mpol, const monomial < C > &mpl)
{
  typedef typename MPol < C, O, R >::monom_t monom_t;
  typedef typename monom_t::container_t container_t;
  mpol = C(0);
  container_t tmp (mpl.env.mxvr()+1);
  const int *szs = mpl.szs ();
  const int *vrs = mpl.vrs ();
  for (int i = 0; i < mpl.esz (); i++)
    if (mpl[i] != 0)
      {
	int a, v;
	for (a = i, v = mpl.nvr () - 1; a; a /= szs[v], v--)
	  tmp[vrs[v]] = a % szs[v];
	mpol += monom_t (mpl[i], tmp);
      };
};
};

__END_NAMESPACE_SYNAPS
#endif	/* SYNAPS_MPOLDSE_MONOMIAL_C */
