#include <synaps/base/Clock.h>
#include <synaps/arithm/gmp.h>
#include <synaps/mpol/MPol.h>
#include <synaps/mpol/MPolDse.h>
#include <synaps/mpol/xml.h>
#include <fstream>



typedef QQ      Coeff;
typedef MPol<Coeff>                             MPoly;
typedef MPolDse<Coeff, mpoldse::monomial<Coeff> >  Monomials;
typedef MPolDse<Coeff, mpoldse::bernstein<Coeff> > Bernstein;

template<class MP>
void mptest( const MP& a, const MP& b, const MP& c,  bool verbose  )
{
  if ( verbose ) {
    std::cout << a << "\n" << b << "\n" << c << "\n";
  };
  MP result;
  result  = (a*b);
  if ( verbose )
    {
      std::cout << result << std::endl;
    };
  result = (a+b);
  //std::cout << "add\n";
  if ( verbose )
    {
      std::cout << result << std::endl;
    };
  result = a-b;
  //  std::cout << "sub\n";
  if ( verbose)
    {
      std::cout << result << std::endl;
    };
  //  std::cout << "big expr\n";
  std::cout.flush();
  //result = (a*b+c)-a*c;
  //  result = (a*b+c);//-a*c;
  //  std::cout << "big expr end\n";
  if ( verbose )
    {
      std::cout << result << std::endl;
    };
};


int main()
{
  std::ifstream in("test-mpl.xml");
  Clock cl;
  
  MPoly a,b,c;
  
  if ( ! ( xml::read(in,a) && 
	   xml::read(in,b) &&
	   xml::read(in,c) ) ) return 1;

  std::cout << "----------------------------\n";
  std::cout << " Monomial Sparse Arithmetic \n";
  std::cout << "----------------------------\n";
  
  cl.start();
  for ( int i = 0; i < 50; i ++ ) mptest(a,b,c, i==0);
  cl.stop();
  
  std::cout << std::endl;
  std::cout << " time = " << cl.time() << std::endl;
  std::cout << std::endl;
  
  Monomials mna(a), mnb(b), mnc(c);

  std::cout << "----------------------------\n";
  std::cout << " Monomial Dense  Arithmetic \n";
  std::cout << "----------------------------\n";

  cl.start();
  for ( int i = 0; i < 50; i ++ ) mptest(mna,mnb,mnc,i==0);
  cl.stop();

  Bernstein bra(a), brb(b), brc(c);  
  
  std::cout << std::endl;
  std::cout << " time = " << cl.time() << std::endl;
  std::cout << std::endl;
  
  std::cout << "----------------------------\n";
  std::cout << " Bernstein    Arithmetic    \n";
  std::cout << "----------------------------\n";
  
  cl.start();
  for ( int i = 0; i < 50; i ++ ) mptest(bra,brb,brc,i==0);
  cl.stop();
  
  std::cout << std::endl;
  std::cout << " time = " << cl.time() << std::endl;
  std::cout << std::endl;

  // DIFF 
  {
    Bernstein mpb("x^2+y^2+z^2+1");
    
    /* differentiation using the local names of variables */
    {
      Bernstein tmp;
      diff(tmp,mpb,0);
      std::cout << tmp << std::endl;
      diff(tmp,mpb,1);
      std::cout << tmp << std::endl;
      diff(tmp,mpb,2);
      std::cout << tmp << std::endl;
    };
    
    /* using "real names" */
    /*    {
      Bernstein tmp;
      gdiff(tmp,mpb,0);
      std::cout << tmp << std::endl;
      gdiff(tmp,mpb,1);
      std::cout << tmp << std::endl;
      gdiff(tmp,mpb,2);
      std::cout << tmp << std::endl;
    };
    */
    /* which corresponds to  */
    /*    {
      Bernstein tmp;
      gdiff(tmp,mpb,0);
      std::cout << tmp << std::endl;
      gdiff(tmp,mpb,1);
      std::cout << tmp << std::endl;
      gdiff(tmp,mpb,2);
      std::cout << tmp << std::endl;
    };
    */
    /* you want to be sure that the local order of variables agree with lexicographical order ? */
    
    VECTOR::print(std::cout,mpb.rep().data);
    std::cout << std::endl;
    //    mpb.lexsort();
    VECTOR::print(std::cout,mpb.rep().data);
    std::cout << std::endl;
    {
      Bernstein tmp;
      diff(tmp,mpb,0);
      std::cout << tmp << std::endl;
      diff(tmp,mpb,1);
      std::cout << tmp << std::endl;
      diff(tmp,mpb,2);
      std::cout << tmp << std::endl;
    };
    
    // SPLIT 
    std::cout << "* polynom = " << mpb << std::endl;
    {
      MPolDse<QQ,mpoldse::bernstein<QQ> > tmp;
      std::cout << "split\n";
      for ( int i = 0; i < 3; i ++ )
	{
	  MPolDse<QQ,mpoldse::bernstein<QQ> > tmp2 = mpb;
	  mpoldse::split(tmp.rep(),tmp2.rep(),i); // split( a, b, v ) ==> a = b(t/2), b = b(1/2+t/2)
	  std::cout << tmp  << " AND " << std::cout << tmp2 << std::endl;
	};
    };
    // RESTRICT 
    std::cout << "* polynom = " << mpb << std::endl;
    {
      MPolDse<QQ,mpoldse::bernstein<QQ> > tmp = mpb;
      std::cout << "restrict\n";
      for ( int i = 0; i < 3; i ++ )
	{
	  mpoldse::restrict(tmp.rep(),i,QQ("1/3"),QQ("2/3")); // restrict(p,v,a,b) ==> p = p((1-t)*a+t*b)
	  std::cout << tmp << std::endl;
	};
    }
    {
      MPolDse<QQ, mpoldse::bernstein<QQ> > tmp("2*x+3*y");
      MPolDse<QQ, mpoldse::bernstein<QQ> > tmp2;
      MPolDse<QQ, mpoldse::bernstein<QQ> > tmp3;
      std::cout << tmp << std::endl;

      
      face( tmp2.rep(), tmp.rep(), 0, 0 );
      face( tmp3.rep(), tmp.rep(), 0, 1 );

      VECTOR::print( std::cout,tmp.rep().data );
      std::cout << std::endl;
      VECTOR::print( std::cout,tmp2.rep().data );
      std::cout << std::endl;
      VECTOR::print( std::cout,tmp.rep().data );
      std::cout << std::endl;
      VECTOR::print( std::cout,tmp3.rep().data );
      std::cout << tmp3 << std::endl;
      
            
      face( tmp2.rep(), tmp.rep(), 1, 0 );
      face( tmp3.rep(), tmp.rep(), 1, 1 );

      VECTOR::print( std::cout,tmp.rep().data );
      std::cout << std::endl;
      VECTOR::print( std::cout,tmp2.rep().data );
      std::cout << std::endl;
      VECTOR::print( std::cout,tmp.rep().data );
      std::cout << std::endl;
      VECTOR::print( std::cout,tmp3.rep().data );
      std::cout << tmp3 << std::endl;

    };
    
  }


  Coeff tmp[ mna.nvr() ];
  for ( int i = 0; i < mna.nvr(); i ++ ) tmp[i] = i;
  Coeff va;
  std::cout << mna << std::endl;
  eval(va,mna,&tmp[0]);
  std::cout << " A = " << va << std::endl;
    std::cout << bra << std::endl;
  eval(va,bra,&tmp[0]);
  std::cout << " B = " << va << std::endl;
  return 0;
}
