#include <synaps/init.h>
#ifdef SYNAPS_WITH_GMP
#include <synaps/mpol.h>
#include <synaps/mpol/xml.h>
#include <synaps/base/Seq.h>
#include <synaps/arithm/gmp.h>
#include <synaps/mpol/xml.h>
#include <synaps/msolve/Newmac.h>
#include <fstream>

typedef MPol<RR> mpol_t;
void usage (char* cmd)
{
  std::cout<<"usage:\n"<<cmd<<" file [-hpsv] ..."<<std::endl;
  std::cout<<"\n  The file should contain the m equations in n variables (m >= n)"<<std::endl;
  std::cout<<"  in the following format:\n  <polynomial>...</polynomial>\n  <polynomial>...</polynomial>\n  ..."<<std::endl;
  std::cout<<"   (example: <polynomial>x^2+2*y^2-2</polynomial><polynomial>x*y-1</polynomial>)\n"<<std::endl;
  std::cout<<"  -s <double> : the threshold used to detect the zeroes"<<std::endl;
  std::cout<<"  -p <int>    : the number of digist used during the computation"<<std::endl;
  std::cout<<"  -v          : verbatim mode"<<std::endl;
}
int main (int argc, char **argv)
{
//   std::cout << binary_operator_prototype<_div_, MatrDse< std::complex<long double> >, std::complex<long double> >() << 
//     std::endl;
//   MatrDse< std::complex<long double> > tmp;
//   return 0;
// }

  using std::cout;
  using std::endl;
  typedef std::string   str_t;

  if(argc<2)
    {
      usage(argv[0]);
      return 0;
    }
  
  std::fstream in(argv[1],std::ios::in);
  if(!in.good())
    {
      if(str_t(argv[1])!=str_t("-h"))
	{
	  std::cerr<<argv[0]<<": file not found"<<std::endl;
	  return 1;
	}
      usage(argv[0]);
      return 0;
    }
  int N=100;
  bool verbatim=false;
  for(unsigned c=2;c<argc;c++)
    {
      if(str_t(argv[c])==str_t("-p"))
	{
	  c++; N=atoi(argv[c]);
	  mpf_set_default_prec(N);
	  std::cout<< std::setprecision(N);
	  std::cout<<"precision: "<<N<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-s"))
	{
	  c++; 
	  SEUIL=atof(argv[c]);
	  std::cout<<"threshold: "<<SEUIL<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-v"))
	{
	  verbatim=true;
	}
    }

  mpol_t p;
  std::list<mpol_t> I; 
  unsigned u=0;
  while(xml::read(in,p))
    {
      std::cout<<"pol["<<u<<"]: deg "<<degree(p)<<" sz "<<p.size() <<std::endl;
      u++;
      if(verbatim)
	std::cout<<"  <polynomial>"<<p<<"</polynomial>"<<endl;
      I.push_back(p);
    }
  std::cout <<"Solving..."<<std::endl;

  Seq<VectDse<std::complex<long double> > > sol= solve(I, Newmac<RR>());

  if(sol.size())
    {  
      std::cout <<"Solution(s): "<<std::endl;
      for(unsigned i=0;i<sol.size();i++)
	std::cout<<"sol["<<i<<"]: "<<sol[i]<<endl;
    }
  else
      std::cout <<"No solution found."<<std::endl;
}
#else
#include <iostream>
int main()
{
  std::cout <<"  Gmp is not connected"<<std::endl;
  return(77);
}
#endif //SYNAPS_WITH_GMP

