#ifndef SYNAPS_MSOLVE_ACS_KERNEL_H_
#define SYNAPS_MSOLVE_ACS_KERNEL_H_
#include <synaps/init.h>
#include <algorithm>
#include <synaps/linalg/VectDse.h>
#include <synaps/base/Seq.h>
#include <synaps/mpol/MPoly_2_2.h>
#include <synaps/msolve/BSolver.h>
#include <synaps/arithm/gmp.h>


__BEGIN_NAMESPACE_SYNAPS

struct ACS_AK_Ring
{
  typedef ZZ              Ring;
  typedef Algebraic<Ring> SOLVER;
  
  typedef UPolDse<Ring>                        Polynomial_1;
  typedef Algebraic<ZZ>::RO_t                  Solution_1; 
  typedef Seq < Solution_1 >                   SolutionSet_1;
  
  typedef MPol<Ring>                           Polynomial_2;
  typedef std::pair< Solution_1, Solution_1 >  Solution_2;
  typedef Seq< Solution_2 >                    SolutionSet_2;

  static void init( Polynomial_1 & p, const char * formula );
  static void init( Polynomial_2 & p, const char * formula );
  
  // sign values   
  static int sign_at   ( const Polynomial_1 & p, const Solution_1 & s );
  static int sign_at   ( const Polynomial_2 & p, const Solution_2 & s );
  static int sign_at_x ( const Polynomial_1 & p, const Solution_2 & s );
  static int sign_at_y ( const Polynomial_1 & p, const Solution_2 & s );
  
  // comparisons of solution (algebraic numbers)
  static int compare   ( const Solution_1   & a, const Solution_1 & b );
  static int compare_x ( const Solution_2   & a, const Solution_2 & b );
  static int compare_y ( const Solution_2   & a, const Solution_2 & b );
  static int compare_xy( const Solution_2   & a, const Solution_2 & b );
  static int compare_yx( const Solution_2   & a, const Solution_2 & b );
    
  // resolution of univariate system
  static SolutionSet_1 solve( const Polynomial_1 & p );

  // resolution of bivariate system
  static SolutionSet_2 solve( const Polynomial_2 & p, const Polynomial_2 & q );
  
  static void approx( double & x, const Solution_1 & s );
  static void approx( double & x, double & y , const Solution_2 & s ); 

}; 

__END_NAMESPACE_SYNAPS

#endif
