/********************************************************************
 *  This file is part of the source code of SYNAPS library
 *  Author(s): B. Mourrain, GALAAD, INRIA               
 *  History: 
 *    2000/03/07 Add solve (., Compagnon) ;
 *  $Id: EigenMult.H,v 1.1 2005/07/11 10:03:50 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_EIGEN_MULT_H
#define SYNAPS_SOLVE_EIGEN_MULT_H
//--------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/arithm/AlgClos.h>
#include <synaps/linalg/MATRIX.m>
#include <synaps/linalg/Eigen.h>
#include <synaps/linalg/lapack.h>
#include <synaps/linalg/MethodName.h>
#include <synaps/linalg/MATRIXPOL.h>


__BEGIN_NAMESPACE_SYNAPS


//--------------------------------------------------------------------
/**
Compute the simple roots from the first $n+1$ first coordinates of the 
eigenvectors of the matrix of multiplication @A@. It is assumed that the
 roots are simple and that the first $n+1$ elements of the basis of the
 quotient ring are $1, x_{1}, \ldots, x_{n}$.
\begin{xxx}
 E =solve(M,n,Eigen());
\end{xxx}
where \c M is a matrix of multiplication and \c n the number 
of variable. The matrix  \c E will be the matrix of complex coordinates
of the (simple) roots.
\index{solve, Eigen}\index{Eigenvector}
**/
template <class MAT> 
MatrDse<typename AlgClos<typename MAT::value_type>::TYPE, lapack::rep2d<typename AlgClos<typename MAT::value_type>::TYPE > >
solve(MAT & A, int n, Eigen mth) {
  typedef typename MAT::rep_type rep_t;
  typedef typename AlgClos<typename MAT::value_type>::TYPE CC;
  typedef MatrDse<CC, lapack::rep2d<CC> > E;
  E e(A.nbrow(),A.nbcol());
  E x(n,A.nbcol());
  using namespace MATRIX; leigvct(e.rep(),A.rep());
  for(unsigned int j=0; j<A.nbcol(); j++){
    CC c = (e)(0,j);
    for(int i=0; i<n; i++)
      (x)(i,j)=((e)(i+1,j)/c);
  }
  return x;
}
//----------------------------------------------------------------------
/**
   Compute the simple roots from $n+1$ coordinates of the 
   eigenvectors of the matrix of multiplication @A@. It is assumed 
   that the roots are simple. The array @t@ is the array of positions
   of the monomials $1,x_{1},\ldots,x_{n}$.
\help{solve, Eigen}
**/
template <class MAT> inline 
MatrDse<typename AlgClos<typename MAT::value_type>::TYPE, 
        lapack::rep2d<typename AlgClos<typename MAT::value_type>::TYPE > >
solve(MAT & A, const linalg::rep1d<typename MAT::size_type> & t, 
      Eigen method) {
  typedef typename MAT::rep_type rep_t;
  typedef typename MAT::size_type size_type;
  typedef typename AlgClos<typename MAT::value_type>::TYPE CC;
  typedef MatrDse<CC,lapack::rep2d<CC> > E;
  E e(A.nbrow(),A.nbcol());
  E x(t.size()-1,A.nbcol());
  using namespace MATRIX; leigvct(e.rep(),A.rep());
  for(size_type j=0; j<A.nbcol(); j++){
    CC c = (e)(t[0],j);
    for(size_type i=1; i<t.size(); i++)
      (x)(i-1,j)=((*e)(t[i],j)/c);
  }
  return (x);
}


__END_NAMESPACE_SYNAPS

#endif // SYNAPS_SOLVE_EIGEN_MULT_H

