/*********************************************************************
 *  This file is part of the source code of the SYNAPS library.
 *  Author(s): B. Mourrain, GALAAD, INRIA                 
 *  History: 
 *    2000/03/07 Add solve (.Compagnon) 
 *  $Id: EigenRslt.H,v 1.1 2005/07/11 10:03:51 jppavone Exp $
 *********************************************************************/
#ifndef SYNAPS_SOLVE_EIGENRSLT_H
#define SYNAPS_SOLVE_EIGENRSLT_H
//--------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/linalg.h>
#include <synaps/mpol.h>
#include <synaps/resultant.h>
#include <synaps/msolve/projection/EigenMult.h>


__BEGIN_NAMESPACE_SYNAPS

//--------------------------------------------------------------------
template<class T=MatrDse<double,lapack::rep2d<double> > >
struct Projectiv {typedef T matrix_type;};

/** solve the system of equations @l@, by computing the tranposed matrix of
multiplication by @f0@ modulo @l@ as the Schur complement of the Macaulay
matrix of the $n+1$ polynomials.  It is assumed that {\em the monomials
$1,x_{1},...,x_{n}$ are smaller than any of the other monomials of degree
$\ge 2$}.  The type @Mat@ specifies the type of the intermediate matrices,
used for constructing the resultant matrix and the Schur complement.
\index{solve, Resultant}
**/
template<class LPol, class Mat>
MatrDse<typename AlgClos<typename Mat::value_type>::TYPE, lapack::rep2d<typename AlgClos<typename Mat::value_type>::TYPE > >
solve(const LPol & l, Projectiv<Mat> method)
{
  typedef typename LPol::iterator iterator;
  typedef typename Mat::value_type value_type;

  int d=1, n=l.size()-1;
  iterator it=l.begin(); it++;
  for(; it != l.end();it++) d*=Degree(*it);

  Mat S= macaulay<Mat>(l);
  Mat M= USchur(S,d);

  linalg::rep1d<unsigned> I(n+1);
  I[0]=0; for(int i=1;i<=n;i++) I[i]=n+1-i;

  return solve(M,I,Eigen());
}


__END_NAMESPACE_SYNAPS

/********************************************************************/
#endif // SYNAPS_SOLVE_EIGENRSLT_H

