/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: upoldse.hpp,v 1.1 2005/07/11 10:03:56 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_SBDSLV_LOOPS_UPOLDSE_HPP
#define SYNAPS_SOLVE_SBDSLV_LOOPS_UPOLDSE_HPP
//--------------------------------------------------------------------
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace upoldse /* univariate dense polynomial loops */
  {
    template<typename real_t> inline /* differentiation */
    void diff( real_t * dst, real_t const * const src, unsigned sz, int st )
    { for ( unsigned i = 0; i < sz-1; dst[i*st] = (i+1)*src[(i+1)*st],  i ++ ); };
    template<typename coeff_t,typename parm_t> inline /* évaluation */
    void horner( parm_t& res, coeff_t const * const mnms, unsigned sz, const parm_t& t, int st = 1  )
    { res = 0.0; for ( int p = (sz-1)*st; p != 0; res +=  mnms[p], res *= t, p -= st ); res+=mnms[0]; };
    template<typename real_t> inline  /* évaluation */
    real_t horner( real_t const * const mnms, unsigned sz, const real_t& t, int st = 1  )
    { real_t res; horner(res,mnms,sz,t,st); return res; };
    template<typename real_t> inline  /* A FAIRE: utiliser st ! */
    void dhorner( real_t& p, real_t& dp, real_t const * const mnms, unsigned sz, const real_t& t )/*, int st = 1 )*/
    { int n = sz-1; p = mnms[n], dp  = 0.0; for ( int j = n-1; j>=0; dp=dp*t+p, p=p*t+mnms[j], j-- ); };

//     template<typename real_t> inline  /* x <- x+c */
//     void shift( real_t * d, unsigned sz, int st, const real_t& c  )
//     { 
//       int j,k; 
//       for ( j = 0; j <= sz-2; j++ )
// 	for( k =n-2; k >= j; d[k*st] += c*d[(k+1)*st], k--); 
//     };
  };
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
