/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: support.hpp,v 1.1 2005/07/11 10:03:55 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_SBDSLV_SUPPORT_HPP
#define SYNAPS_SOLVE_SBDSLV_SUPPORT_HPP
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace sbdslv
{
  template<class V>
  struct system_ctrl
  {
    typedef int sz_t;
    V& m_v;
    system_ctrl( V& v ): m_v(v) {};
    template< class _interval_ > static inline
    bool check( _interval_ const * const dmns, int nvars )  { return true; };
    template< class _interval_  > inline
    void output(  _interval_ const * const dmns, sz_t nvars  ) 
    {
      //      std::cout << "OUTPUT\n";
      //      vctops::print(dmns,nvars);
      //      std::cout << std::endl;
      for ( sz_t i = 0; i < nvars; i ++ )
	m_v.push_back( dmns[i].lower() ), 
	  m_v.push_back( dmns[i].upper() );
    };
  };
 
  template< class real_t >
  unsigned clean_result( real_t * isols, int nvars, int nsols, const real_t& prec )
  {
    int a,c,n,i;
    real_t cd;
    if ( nsols == 0 ) return 0;
    for ( a = 0; a < nsols*nvars; isols[a] = (isols[2*a]+isols[2*a+1])/2.0, a ++ );

    for ( n = 1, c = 0, a = 0; a < nsols*nvars; a += nvars )
      {
	for ( cd = 0, i = 0; i < nvars; cd = std::max(cd,std::abs((isols+c)[i]-(isols+a)[i])), i ++ );
	if ( cd < prec )
	{
	  double fa = ((double)n)/(n+1);
	  double fb = ((double)1.0)/(n+1);
	  for ( int i = 0; i < nvars; i ++ ) (isols+c)[i] = fa*(isols+c)[i]+fb*(isols+a)[i];
	  n ++;
	}
	else
	  {
	    c += nvars;
	    std::copy(isols+a,isols+a+nvars,isols+c);
	    n  = 1;
	  };
      };
    return (c+nvars)/nvars;
  };
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
