
#include <synaps/msolve/acs-kernel.h>
#include <iostream>

__BEGIN_NAMESPACE_SYNAPS

void ACS_AK_Ring::init( Polynomial_1 & p, const char * formula )
{ 
  p = formula; 
};

void ACS_AK_Ring::init( Polynomial_2 & p, const char * formula ) 
{ 
  p = formula; 
  assert(NbVar(p)==2);
};

int ACS_AK_Ring::sign_at( const Polynomial_1 & p, const Solution_1 & s ) 
{ 
  return ALGEBRAIC::sign_at( p, s ); 
};

int ACS_AK_Ring::sign_at( const Polynomial_2 & p, const Solution_2 & s )
{ 
  return ALGEBRAIC::sign_at(p, s.first, s.second );
};

int ACS_AK_Ring::sign_at_x( const Polynomial_1 & p, const Solution_2 & s )
{ 
  return ALGEBRAIC::sign_at( p, s.first ); 
};

int ACS_AK_Ring::sign_at_y( const Polynomial_1 & p, const Solution_2 & s )
{ 
  return ALGEBRAIC::sign_at( p, s.second ); 
};

int ACS_AK_Ring::compare ( const Solution_1 & a, const Solution_1 & b )
{ 
  return ALGEBRAIC::compare( a, b ); 
};

int ACS_AK_Ring::compare_x( const Solution_2 & a, const Solution_2 & b ) 
{ return ALGEBRAIC::compare(a.first,b.first);  };

int ACS_AK_Ring::compare_y( const Solution_2 & a, const Solution_2 & b )
{ return ALGEBRAIC::compare(a.second,b.second);};

int ACS_AK_Ring::compare_xy( const Solution_2 & a, const Solution_2 & b )
{
  int c = compare_x(a,b) ;
  if ( c == 0 ) return compare_y(a,b);
  return c;
};

int ACS_AK_Ring::compare_yx( const Solution_2 & a, const Solution_2 & b )
{
  int c = compare_y(a,b) ;
  if ( c == 0 ) return compare_x(a,b);
  return c;
};

ACS_AK_Ring::SolutionSet_2 ACS_AK_Ring::solve( const Polynomial_2 & p, const Polynomial_2 & q )
{ 
  // Solve_with_subres marche pas ?
  return ALGEBRAIC::Solve_with_sign(p,q, SOLVER() ); 
};

ACS_AK_Ring::SolutionSet_1 ACS_AK_Ring::solve( const Polynomial_1 & p )
{
  return ::solve(p, SOLVER());
};

void ACS_AK_Ring::approx( double & x, const Solution_1 & s )
{ x = to_double( s ); };

void ACS_AK_Ring::approx( double & x, double & y , 
			  const Solution_2 & s )
{ 
  x = to_double( s.first );
  y = to_double( s.second); 
};
  
__END_NAMESPACE_SYNAPS
