#include <synaps/msolve/acs-kernel.h>


template<class Kernel>
struct revlex_compare
{
  typedef typename Kernel::Solution_2 T;
  bool operator()( const T & a, const T & b ) const { return  Kernel::compare_yx(a,b); };
};

template<class Kernel>
void print( const typename Kernel::SolutionSet_1 & s )
{
  typedef typename Kernel::SolutionSet_1 SolutionSet_1;
  double tmp;
  for ( typename SolutionSet_1::const_iterator it = s.begin(); 
	it != s.end(); 
	it++ )
    {
      Kernel::approx(tmp,*it);
      std::cout << tmp << ", ";
    };
  std::cout << std::endl;
};

template<class Kernel>
void print( const typename Kernel::SolutionSet_2 & s )
{
  typedef typename Kernel::SolutionSet_2 SolutionSet_2;
  double x,y;
  for ( typename SolutionSet_2::const_iterator it = s.begin(); 
	it != s.end(); 
	it++ )
      {
	Kernel::approx(x,y,(*it));
	std::cout << "[" << x << "," << y << "], ";
      };
    std::cout << std::endl;
};


int main()
{
  typedef ACS_AK_Ring AK;

  // bivariate solve example

  typedef ACS_AK_Ring::Polynomial_2  Polynomial_2;
  typedef ACS_AK_Ring::SolutionSet_2 SolutionSet_2;

  Polynomial_2  A;
  Polynomial_2  B;
  SolutionSet_2 I_AB;
  
  AK::init(A,"x^2+y^2-1"); 
  AK::init(B,"x^2-2*x+y^2");
  
  I_AB = AK::solve(A,B);     
  
  std::cout << "\tsolutions of A^B : "; // write the solutions to stdout
  print<AK>(I_AB);                      //
  
  // check that signs of A and B are zeros on A^B
  std::cout << "\tcheck sign of A and B on A^B: ";
  for ( SolutionSet_2::const_iterator it = I_AB.begin();
	it != I_AB.end();
	it ++ )
    {
      std::cout << "[" << AK::sign_at( A, *it );
      std::cout << "," << AK::sign_at( B, *it ) << "],";
    };
  std::cout << std::endl;
  
  // univariate solve example 
  
  typedef ACS_AK_Ring::Polynomial_1  Polynomial_1;
  typedef ACS_AK_Ring::SolutionSet_1 SolutionSet_1;
  
  Polynomial_1  P;
  SolutionSet_1 I_P;
  
  AK::init(P,"x^2-1");
  I_P  = AK::solve(P);
  std::cout << "\tP=0, solutions : "; // write the solutions to stdout
  print<AK>(I_P);
  
  // taking signs of P on the abcissae of A ^ B.
  std::cout << "\tsign of P on abscissae of A^B: ";
  
  for ( SolutionSet_2::const_iterator it = I_AB.begin();
	it != I_AB.end();
	it ++ )
    std::cout << AK::sign_at_x( P, *it ) << ", ";
  
  std::cout << std::endl;
  
  // approximate the mean value of A ^ B (why not?)
  double sx(0.0), sy(0.0), n(0);
  
  for ( SolutionSet_2::const_iterator it = I_AB.begin();
	it != I_AB.end();
	it ++, n = n + 1.0 )
    {
      double x, y;
      AK::approx( x, y, *it );
      sx = sx + x;
      sy = sy + y;
    };
  
  sx /= n;
  sy /= n;
  
  std::cout << "\tmean of A ^ B is more or less " << "( " << sx << ", " << sy << " )" << std::endl;
  
  // comparisons (revlex sort)

  typedef ACS_AK_Ring::Solution_2 Solution_2;
  
  // we pass trough a copy (SolutionSet_2 has not necessary write-access)
  Seq< Solution_2 > V_IAB(I_AB.size());
  std::copy(I_AB.begin(),I_AB.end(),V_IAB.begin());
  std::sort(V_IAB.begin(),V_IAB.end(),revlex_compare<ACS_AK_Ring>());
  
  std::cout << "\trevlex sort of A ^ B : ";
  for ( Seq< Solution_2 >::const_iterator it = V_IAB.begin(); it != V_IAB.end(); it ++ )
    {
      double x,y;
      AK::approx(x,y,*it);
      std::cout << "[ " << x << "," << y << "] ";
    };
  std::cout << std::endl;
  return 0;
};
