#ifndef SYNAPS_SHAPE_SISL_CURVEBASE_H
#define SYNAPS_SHAPE_SISL_CURVEBASE_H

#include <synaps/init.h>
#include <sisl/sisl.h>
#include <sisl/GoReadWrite.h>
#include <synaps/shape/base/fxv.h>

__BEGIN_NAMESPACE_SYNAPS

struct SISLCurveBase
{ 
  SISLCurve       * sisl_data_; 
  SISLCurveBase() : sisl_data_(0) {};
  SISLCurveBase & operator=( const SISLCurveBase & rep );
  ~SISLCurveBase() { if ( sisl_data_ ) freeCurve(sisl_data_); }
  inline const SISLCurve * sisl_data() const { assert(sisl_data_); return sisl_data_; };  
  inline       SISLCurve * sisl_data()       { assert(sisl_data_); return sisl_data_; };
  int ord() const  { return sisl_data()->ik; };
  int nbp() const  { return sisl_data()->in; };
  int nbk() const  { return nbp()+ord();    };
  int dim() const  { return sisl_data()->idim; };
  int type()const  { return sisl_data()->ikind;}; 
};

inline SISLCurveBase & SISLCurveBase::operator=( const SISLCurveBase & rep )
{ 
  if ( sisl_data_ )     freeCurve(sisl_data_);
  sisl_data_ = 0;
  if ( rep.sisl_data_ ) sisl_data_ = copyCurve(rep.sisl_data_); 
};

inline std::istream& operator>>( std::istream & ins, SISLCurveBase & rep ) 
{ 
  rep.~SISLCurveBase();
  rep.sisl_data_ = readGoCurve(ins); 
};

inline void domain( double & tmin, double & tmax, const SISLCurveBase & c )
{
  int stat;
  s1363(c.sisl_data_,&tmin,&tmax,&stat);
};

template<class C>
inline void domain( C & tmin, C & tmax, const SISLCurveBase& c )
{
  double tmin_, tmax_;
  domain(tmin_,tmax_,c);
  tmin = tmin_;
  tmax = tmax_;
};

inline void eval( double * p, const SISLCurveBase & c, double t )
{
  int stat;
  int leftk;
  s1227(c.sisl_data_,0,t,&leftk,p,&stat);
};

template<class R, unsigned N> 
inline void eval( fxv<double,N> & p, const SISLCurveBase &c, double t ) { eval(&p[0],c,t); };

template<class Point> 
inline void eval( Point & p, const SISLCurveBase &c , const typename Point::value_type & t )
{
  double ptmp[c.dim()];
  double t_ = t;
  eval(&ptmp[0],c,t_);
  for ( int i = 0; i < c.dim(); i ++ ) p[i] = ptmp[i];
};

inline double length( const SISLCurveBase & c, double eps )
{
  double result;
  int stat;
  s1240(c.sisl_data_,eps,&result,&stat);
  return result;
};

__END_NAMESPACE_SYNAPS
#endif

