#ifndef GMV_H
#define GMV_H


template<class Point>
void GmvSurfaceUVSample( const char * file_name, Point * src, unsigned w, unsigned h )
{
#define LIN(i,j) ((i)*w+j)
  unsigned i,j;
  FILE * out = fopen(file_name,"w");
  fprintf(out,"OFF\n");
  fprintf( out, "%i %i %i\n", w*h, (w-1)*(h-1) );
  for ( i = 0; i < w*h; i++ )
	fprintf(out, "%f %f %f\n",  src[i][0], src[i][1], src[i][2] );
  for ( i = 0; i < h-1; i++ )
    for ( j = 0; j < w-1; j++ )
      fprintf(out, "%i %i %i %i %i\n", 4, LIN(i,j), LIN(i+1,j), LIN(i+1,j+1), LIN(i,j+1) );
  fclose(out);
#undef LIN
};



template<class Point2>
void GmvCurves2dPairs( const char * file_name, Point2 ** left, Point2 ** right,
		       unsigned * sizes, unsigned nbcurves, 
		       bool show_end = false , bool use_palette = false )
{
  std::ofstream out(file_name);
  out << "LIST\n";
  out << "{\n";
  
  out << "VECT\n";
  
  unsigned nbvertices = 0;
  unsigned i;
  
  unsigned nbcurves2 = nbcurves;
  for ( i = 0; i < nbcurves; i++ ) 
    {
      if ( sizes[i] == 0 ) { nbcurves2--; } nbvertices += sizes[i];
    };
  
  out << 2*nbcurves2 << " " << 2*nbvertices << " " << 2*nbcurves2 << std::endl;
  for ( i = 0; i < nbcurves; i++ )
    if ( sizes[i] != 0 )
      out <<  sizes[i] << " " << sizes[i] << " ";
  out << std::endl;
  for ( i = 0; i < 2*nbcurves2; i++ )
    out << 1 << " ";
  out << std::endl;
  
  for ( i = 0; i < nbcurves; i++ )
    {
      unsigned k,d;
      for ( k = 0; k < sizes[i]; k++ )
	{
	  for ( d = 0; d < 2; d++ )
	    out << left[i][k][d] << " ";
	  out << 0;
	  out << std::endl;
	};

      for ( k = 0; k < sizes[i]; k++ )
	{
	  for ( d = 0; d < 2; d++ )
	    out << right[i][k][d] << " ";
	  out << 0;
	  out << std::endl;
	};
    };

  float pal[6][3] = { 
    { 0, 0, 0.5 }, {0,0.5,0.5}, {0.5,0,0}, 
    {0.5, 0, 0.5 }, { 0.5, 0.5, 0 }, {0, 0, 0 } };

  if ( use_palette )
    {
      for ( i = 0; i < nbcurves2; i++ )
	{
	  out << pal[i%6][0] << " " << pal[i%6][1] << " " 
	      << pal[i%6][2] << " " << 1.0 << std::endl;
	  out << pal[i%6][0] << " " << pal[i%6][1] << " " 
	      << pal[i%6][2] << " " << 1.0 << std::endl;
	};
      out << "}\n";
    }
  else 
    {
      for ( i = 0; i < nbcurves2; i++ )
	{
	  out << 0.2 << " " << 0.2 << " " << 0.2 << " " << 1 << std::endl;
	  out << 0.2 << " " << 0.2 << " " << 0.2 << " " << 1 << std::endl;
	};
      out << "}\n";
    };
  
  if ( show_end )
    {
      for ( unsigned i = 0; i < nbcurves; i++ )
	{
	  if ( sizes[i] )
	    {
	      Point2 tmp[] = { left[i][0], left[i][sizes[i]-1] };
	      out << "{\n";
	      out << "SPHERE\n";
	      out << "0.01  " <<tmp[0][0] << " " << tmp[0][1] << " " 
		  << 0 << std::endl;
	      out << "}\n";
	      out << "{\n";
	      out << "SPHERE\n";
	      out << "0.01  " <<tmp[1][0] << " " << tmp[1][1] << " " 
		  << 0 << std::endl;
	      out << "}\n";
	    };
	};
      
      for ( unsigned i = 0; i < nbcurves; i++ )
	{
	  if ( sizes[i] )
	    {
	      Point2 tmp[] = { right[i][0], right[i][sizes[i]-1] };
	      out << "{\n";
	      out << "SPHERE\n";
	      out << "0.01  " << tmp[0][0] << " " << tmp[0][1] << " " 
		  << 0 << std::endl;
	      out << "}\n";
	      out << "{\n";
	      out << "SPHERE\n";
	      out << "0.01  " <<tmp[1][0] << " " << tmp[1][1] << " " 
		  << 0 << std::endl;
	      out << "}\n";
	    }
	};
    }
  
};

template<class Point3>
void GmvSpaceCurves( const char * file_name, Point3 ** spaces, 
		     unsigned * sizes, unsigned nbcurves, 
		     bool use_palette = false )
{

  std::ofstream out(file_name);
  
  out << "LIST\n";
  out << "{ ";
  out << "VECT\n";
  unsigned nbvertices = 0;
  unsigned i;
  
  for ( i = 0; i < nbcurves; i++ ) nbvertices += sizes[i];
  out << nbcurves << " " << nbvertices << " " << nbcurves << std::endl;
  for ( i = 0; i < nbcurves; i++ )
    out <<  sizes[i] << " ";
  out << std::endl;
  for ( i = 0; i < nbcurves; i++ )
    out << 1 << " ";
  out << std::endl;
  
  for ( i = 0; i < nbcurves; i++ )
    {
      unsigned k,d;
      for ( k = 0; k < sizes[i]; k++ )
	{
	  for ( d = 0; d < 3; d++ )
	    out << spaces[i][k][d] << " ";
	  out << std::endl;
	};
    };
  
  float pal[6][3] = { { 0, 0, 0.5 }, {0,0.5,0.5}, {0.5,0,0}, {0.5, 0, 0.5 }, { 0.5, 0.5, 0 }, {0, 0, 0 } };
  
  for ( i = 0; i < nbcurves; i++ )
    {
      if ( use_palette )
	{
	  out << pal[i%6][0] << " " << pal[i%6][1] << " " << pal[i%6][2] << 
	    " " << 1.0 << std::endl;
	  
	}
      else 
	{
	  out << 1 << " " << 0 << " " << 0 << " " << 1 << std::endl;
	};
      
    };
  out << "} \n";
  
};

#endif
