#ifndef SYNAPS_SHAPE_GAIA_LSEGMENT_H
#define SYNAPS_SHAPE_GAIA_LSEGMENT_H

#include <synaps/shape/gaia/sample.h>

__BEGIN_NAMESPACE_SYNAPS

namespace gaia
{
  struct mark_t
  {
    rid_t     head;
    coord_t   j;
    vcode_t code;
    mark_t  * next;
    inline coord_t a() { return j;             };
    inline coord_t b() { return (this+1)->a(); };
  };
  
  struct lsegment : sample
  {
    typedef coord_t bounds_t[2];
    
    struct region_t
    {
      coord_t _umin, _umax;
      vcode_t _code; 
      void  * data;
      bool operator<( const region_t& b ) const { return _umin < b._umin;         };
      coord_t min   ( coord_t u         ) const { return ((bounds_t*)data)[u][0]; };
      coord_t max   ( coord_t u         ) const { return ((bounds_t*)data)[u][1]; };
      coord_t umin  ()                    const { return _umin; };
      coord_t umax  ()                    const { return _umax; };
      inline bool inside( coord_t u, coord_t v )
      {
	if  ( u >= umin() && u < umax() )
	  if ( v >= min(u) && v < max(u) ) return true;
	return false;
      };
      vcode_t code() const { return _code; };
      friend class lsegment_t;
    };
    
    mark_t   *  marks;  
    unsigned    _size;  
    unsigned *  lines;
    
    
    unsigned nmarks  () const { return _size; };
    void lines_changes();
  public:
    
    void rfree( region_t& r )
    {
      free((void*)(((bounds_t*)(r.data)) + r._umin));
    };
    
    
    
    mark_t * begin( unsigned i ) { return &(marks[lines[i]]); };
    mark_t * end  ( unsigned i ) { return &(marks[lines[i+1]-1]);};
    
    std::vector< region_t > regions;
    graph_t * grp;
    void pushm( vcode_t code, coord_t j );
    void pushr( coord_t i, mark_t * p );
    void addneighbors( mark_t * m0, mark_t * m1 );
    void promote( region_t& r );
    void convert_regions();
    void find_regions();
  public:
    double _lt;

    lsegment( const IParametricSurface * s , unsigned _w, unsigned _h );
    unsigned size() const { return regions.size(); };
    region_t& operator[]( unsigned i ) { return regions[i]; };

    ~lsegment();
  };
};
__END_NAMESPACE_SYNAPS
#endif
