#ifndef S223_H
#define S223_H
#include "math.h"
template<class Point2, class Point3>
void spc2prms( Point2& pp0, Point2& pp1, 
	       Point3& sp0, Point3& sp1, 
	       Point3& base, const Point3& bu, const Point3& bv )
{
  /* T(u,v) = base + u*bu +v*bv
     => 
     spc[0] - base[0] = delta[0]  = / bu[0] bv[0]\  / u \
     spc[1] - base[1] = delta[1]  = | bu[1] bv[1]|  |   |
     spc[2] - base[2] = delta[2]  = \ bu[2] bv[2]/  \ v /
  */
    
  typedef typename Point3::value_type number;
  
  number det01 = det22(bu[0],bv[0],
		       bu[1],bv[1]);
  number det02 = det22(bu[0],bv[0],
		       bu[2],bv[2]);
  number det12 = det22(bu[1],bv[1],
		       bu[2],bv[2]);
  
  number idet;
  unsigned i,j;
  unsigned k;
  // choose of system (0,1) (0,2) or (1,2)
  if ( abs(det01) < abs(det02) )
    {
      if ( abs(det02) < abs(det12) ) 
	idet = 1.0/det12, i = 1, j = 2;
      else                           
	idet = 1.0/det02, i = 0, j = 2;
    }
  else
    {
      if ( abs(det01) < abs(det12) ) 
	idet = 1.0/det12, i = 1, j = 2;
      else                           
	idet = 1.0/det01, i = 0, j = 1;
    }
  Point3 delta;

  // solve system for sp0
  for ( k = 0; k < 3; k++ ) delta[k] = sp0[k] - base[k];
  
  /* delta[i] = bu[i] bv[i] | u                       */
  /* delta[j] = bu[j] bv[j] | v                       */
  /*                                                  */
  /*    1    |  bv[j]   -bv[i] | | spc[i] - base[i] | */
  /*   ---   |                 | |                  | */
  /*   detij | -bu[j]    bu[i] | | spc[j] - base[j] | */

  pp0[0] = idet*(  bv[j]*delta[i] - bv[i]*delta[j] );
  pp0[1] = idet*( -bu[j]*delta[i] + bu[i]*delta[j] );

  for ( k = 0; k < 3; k++ ) delta[k] = sp1[k] - base[k];

  pp1[0] = idet*(  bv[j]*delta[i] - bv[i]*delta[j] );
  pp1[1] = idet*( -bu[j]*delta[i] + bu[i]*delta[j] );

};

#endif
