#ifndef SYNAPS_SHAPE_GEOMVIEW_STREAM_H
#define SYNAPS_SHAPE_GEOMVIEW_STREAM_H

#include <stdio.h>
#include <assert.h>
#include <synaps/init.h>
#include <algorithm>

__BEGIN_NAMESPACE_SYNAPS

namespace gmv
{
  void swab4( unsigned *, unsigned );
  void swab4( unsigned *, unsigned *, unsigned );
  struct stream
  {
    FILE * m_fd;
    float  m_lcolor  [4];
    bool   m_binary;
    //float  m_matcolor[4];
    stream( const char * fname, bool binary = true );
    ~stream();
    void flush();
    void line_color( float * color );
    inline void matcolor( float * color )
    { fprintf(m_fd,"appearance {\n material{\n diffuse %f %f %f\n}\n}\n",color[0],color[1],color[2]); };
    
    void polyline(  const float * src, int nv, float * colors = 0, int nc = 1 );
    template<class C> inline
    void polyline( const C * src, int nv, float * colors = 0, int nc = 1 )
    {
      float * tmp = new float[nv*3];
      std::copy(src,src+3*nv,tmp);
      polyline(tmp,nv,colors,nc);
      delete[] tmp;
    };
    void uvsmp ( const float * src, unsigned m, unsigned n, float * colors = 0 );
    template<class C> inline
    void uvsmp ( const C * src, unsigned m, unsigned n, float * colors = 0 )
    {
      float * tmp = new float[m*n*3];
      std::copy(src,src+3*m*n,tmp);
      uvsmp(tmp,m,n,colors);
      delete[] tmp;
    };
    void quads ( const float * src, unsigned nq, float * colors = 0 );
    template<class C> inline
    void quads ( const C * src, unsigned nq, float * colors = 0 )
    {
      float * tmp = new float[nq*4*3];
      std::copy(src,src+nq*4*3,tmp);
      quads(tmp,nq,colors);
      delete[] tmp;
    };
    
    void points( const float * src, unsigned nv );
    template<class C> inline
    void points( const C * src, unsigned nv )
    {
      float * tmp = new float[nv*3];
      std::copy(src,src+nv*3,tmp);
      points(tmp,nv);
      delete[] tmp;
    };
    
    void sphere( float * p, float r );
    template<class C> inline
    void sphere( C * p, float r )
    {
      float tmp[3];
      std::copy(p,p+3,tmp);
      sphere(tmp,r);
    };
    
    void ellipse( float * prms ); // incomplet
    void spheres( float * centers, float r, unsigned n, float * color = 0, int dice = 2 );
    void graph( float * z, unsigned m, unsigned n, float * colors = 0 );
    //    void thickpolyline( float * pts, unsigned n, float thickness );
    void stick( float * pts, unsigned m, float thick );
  };
}

__END_NAMESPACE_SYNAPS

#endif
