#ifndef SYNAPS_SHAPE_PARAMETRIC_SURFACE_H
#define SYNAPS_SHAPE_PARAMETRIC_SURFACE_H

#include <synaps/shape/Surface.h>

__BEGIN_NAMESPACE_SYNAPS

VIEW_DCL(Surface,ParametricSurface) ;

template<class C, class Surface> inline
void domain( C & umin, C & umax, C & vmin, C & vmax, const Surface& s )
{
  domain(umin,umax,vmin,vmax,view(s));
};

template<class C, class R> inline
void domain( C & umin, C & umax, C & vmin, C & vmax, const ParametricSurface<R> & rep  )
{
  umin = vmin = 0;
  umax = vmax = 1;
};


template<class Point, class R> inline
void eval( Point * samples, 
	   const ParametricSurface<R> & s, 
	   const typename Point::value_type * uv, int n )
{
  for ( Point * s = samples; s != samples + n; eval( *s, repr(s), uv[0], uv[1] ), s++, uv += 2 );
};

//template<class Point, class Surface > inline
//void eval( Point * samples, const Surface & s, const typename Point::value_type * uv, int n )
//{
//  eval( samples, view(s), uv, n );
//};


template<class Point, class Surface >
void eval( Point & p, const Surface & s, 
	   const typename Point::value_type & u, 
	   const typename Point::value_type & v )
{
  eval( p, view(s), u, v );
};

template<class Point, class Surface >
void sample( Point * smp, 
	     const Surface & s, 
	     const typename Point::value_type * u, 
	     const typename Point::value_type * v, 
	     int m, 
	     int n )
{
  sample(smp,view(s),u,v,m,n);
};

template<class Point, class R >
void sample( Point * smp, 
	     const ParametricSurface<R> & s, 
	     const typename Point::value_type * u, 
	     const typename Point::value_type * v, 
 	     int m, 
	     int n )

{
  typedef typename Point::value_type C;
  for ( const C * p = u; p != u + m; p ++ )
    for ( const C * q = v; q != v + n; q ++, smp ++ )
      eval( *smp, repr(s), *p, *q );
};

template<class Point, class Surface >
void sample( Point * smp, 
	     typename Point::value_type * u, 
	     typename Point::value_type * v, 
	     const Surface & s, 
	     int m, 
	     int n )

{
  sample(smp,u,v,view(s),m,n);
};

template<class Point, class R> 
void sample( Point * smp, 
	     typename Point::value_type * u, 
	     typename Point::value_type * v, 
	     const ParametricSurface<R> & s, 
	     int m, 
	     int n )

{
  typename Point::value_type umin,umax,vmin,vmax;
  domain(umin,umax,vmin,vmax,repr(s));
  tsample(u,umin,umax,m);
  tsample(v,vmin,vmax,n);
  sample(smp,repr(s),u,v,m,n);
};


template<class C>
struct ParametricSurfaceSample
{
  typedef fxv<C,3> point_t;
  std::vector< point_t > m_svals;
  std::vector< C > m_uvals;
  std::vector< C > m_vvals;
  
  
  const point_t & operator[]( int i ) const 
  { return m_svals[i]; };
  const C & vval( int i ) const { return m_vvals[i]; };
  const C & uval( int i ) const { return m_uvals[i]; };

  int sizeu() const { return m_uvals.size(); };
  int sizev() const { return m_vvals.size(); };

  template<class Surface>
  ParametricSurfaceSample( const Surface & c, int m, int n  ) : m_svals(m*n), m_uvals(m), m_vvals(n) 
  {
    sample(&m_svals[0],&m_uvals[0],&m_vvals[0],repr(c),m,n);
  };
};

template<class C> inline
gmv::stream & operator<<( gmv::stream & o, const ParametricSurfaceSample<C> & psmp )
{
  o.uvsmp( (const C*)&(psmp.m_svals[0]), psmp.sizeu(), psmp.sizev() );
  return o;
};

template<class R> inline
gmv::stream & operator<<( gmv::stream & o, const ParametricSurface<R> & psmp )
{
  
};

  
__END_NAMESPACE_SYNAPS

#endif
