#ifndef SYNAPS_SHAPE_POINT_SET_H
#define SYNAPS_SHAPE_POINT_SET_H
#include <synaps/shape/base/fxv.h>
#include <synaps/base/chrono.h>
template<class C, unsigned N>
struct point_set 
{
  typedef fxv<C,N> point;
  struct node { 
    point v;
    int d;
    int f;
    int l;
    int r;
  };
  
  std::vector< node > nodes;

  int free;
  int new_node( const point &  v, int d, int f, int l, int r )
  {
    if ( free == -1 )
      {
	nodes.push_back( node() );
	int tmp = nodes.size()-1;
	nodes[tmp].v = v;
	nodes[tmp].d = d;
	nodes[tmp].f = f;
	nodes[tmp].l = l;
	nodes[tmp].r = r;
	return nodes.size()-1;
      };
    int tmp = free;
    free = nodes[free].l;
    nodes[tmp].v = v;
    nodes[tmp].d = d;
    nodes[tmp].f = f;
    nodes[tmp].l = l;
    nodes[tmp].r = r;
    return tmp;
  };
  
  void free_node( int n )
  {
    nodes[n].l = free;
    nodes[n].d = -1;
    free = n;
  };

  int root;
  
  point_set() : root(-1), free(-1) {
  };
  
  void remove( int n )
  {
    std::cout << " n = " << n << std::endl;
    int c = leftmost(n,nodes[n].d);
    if ( nodes[nodes[c].f].l == c ) nodes[nodes[c].f].l = -1;
    if ( nodes[nodes[c].f].r == c ) nodes[nodes[c].f].r = -1;
    nodes[c].l = nodes[n].l;
    nodes[c].r = nodes[n].r;
    nodes[c].f = nodes[n].f;
    if ( nodes[n].f != -1 )
      {
	if ( nodes[nodes[n].f].l == n ) nodes[nodes[n].f].l = c;
	if ( nodes[nodes[n].f].r == n ) nodes[nodes[n].f].r = c;
      };
    free_node(n);
  };
  
  inline
  int leftmost( int a, int b, int d )
  { 
    if ( a == -1 ) return b;
    if ( b == -1 ) return a;
    if ( nodes[a].v[d] < nodes[b].v[d] ) return a; else return b; 
  };

  int leftmost( int n, int d )
  {
    if ( n == -1 ) return n;
    if ( nodes[n].r == -1 && nodes[n].l == -1 ) return n;
    if ( nodes[n].d == d ) return leftmost(nodes[n].l,d);
    else return leftmost(leftmost(nodes[n].l,d),leftmost(nodes[n].r,d),d);
  };
  
  int  search( const point&  p  )
  {
    int c = root;
    int d = 0;
    int b;
    do
      {
	if ( p == nodes[c].v ) return c;
	bool b = p[d]>=nodes[c].v[d];
	++d %= 4;
	c = *(&(nodes[c].l)+b);
      }
    while( c != -1 );
    return c;
  };
  
  void insert( const point &    p )
  {
    //    std::cout << p << std::endl;
    if ( root == -1 ) root = new_node(p,0,-1,-1,-1);
    int c = root;
    int f;
    int d = 0;
    int b;
    do
      {
	f = c;
	b = (p[d]>=nodes[c].v[d]);
	++d %= N;
	c =  *(&(nodes[c].l)+b);
      }
    while( c != -1 );
    *(&(nodes[f].l)+b) = new_node(p,d,-1,-1,-1);
  };
  
  void remove( const point&  p )
  {
    assert(root!= -1);
    int tmp = search(p);
    if ( tmp != -1 ) remove(tmp);
  };
};

void genpts( fxv<double,3> * p, int n )
{
  for ( int i = 0; i < n; i ++ )
    {
      urand(p[i],0.0,1.0);
    };

};

int main()
{
  point_set<double,3> ps;
  fxv<double,3> * tmp = new fxv<double,3>[100000];
  genpts(tmp,100000);
  chrono tm;
  tm.start();
  for ( int i = 0; i < 100000; i ++ )
    {
      ps.insert( tmp[i] );
    };
  tm.stop();
  tm.write( std::cout );
  tm.start();
  for ( int i = 0; i < 100000; i ++ )
    ps.remove( tmp[i] );

  tm.stop();
  tm.write( std::cout );
};


#endif
