#ifndef TREES_KDTREE
#define TREES_KDTREE

__BEGIN_NAMESPACE_SYNAPS


struct SSI
{
  struct dim_cmp
  {
    unsigned m_d;
    dim_cmp( unsigned d ) : m_d(d) {};
    inline dim_cmp next () const { return dim_cmp((m_d+1)%P::_dimension_); };
    inline bool operator()( const P&  a, const P& b ) const { return __rep(a)[m_d] < __rep(b)[m_d]; };
  };
  
  static node * build( P * first, P * last, const dim_cmp& cmp  )
  {
    if ( last-first >= 1 ) {
      if ( last-first > 1 ) {
	unsigned m = (last-first)/2;
	std::nth_element(first,first+m,last,cmp);
	return new node( first+m, 
			 mkkd< node, Iterator, Cmp >( first, first+m, cmp.next() ),
			 mkkd< node, Iterator, Cmp >( first+m+1,last, cmp.next() ) ); 
      };
      return new node( first, 0, 0 );
    };
    return 0;
  };
  
  struct knode
  {
    P m_data;
    knode * m_l;
    knode * m_r;
    knode( P* p, r it, knode * l, knode * r ) : m_data(*it), m_l(l), m_r(r), m_f(0) {};
    const typename P::value_type & operator[]( int i ) const { return m_data[i]; };
    ~knode()
    { 
      if ( m_l != 0 )  delete m_l; 
      if ( m_r != 0 )  delete m_r; 
    };
  };
  
  bool isleaf( knode const  * const n ) { return n->m_l == n->m_r; };
  
  KDStaticSearch( P * begin, P * end ) { m_root = build(begin,end,dim_cmp(0)); };
  
  void share_closest( P ** a, P ** lba, P **lbb )
  {
    double sv;
    double d = 1e6;
    P **   closest;
    for ( P ** c = lba; c != lbb; c ++ )
      if ( (sv=distance(**a,**c)) < d ) {
	d = sv;
	closest = c;
      };
    if ( d < TOL ) 
      {
      };
  };
  For ( int i = 0; i < ssi.nbpts; i ++ )
  {
    
  };
  
  void ksearch( result& c, node * n, const Point& query, const real_t eps, unsigned d = 0 )
  {
    bool skipleft   = query[d] > (*n)[d] + eps;
    if ( skipleft ) { 
      if ( !isleaf( n ) )  ksearch( c, right( n ), query, eps, (d+1) % P::_dimension_ ); return; 
    };
    bool skipright  = query[d] < __rep(n->data())[d] - eps;
    if ( skipright) 
      { 
	if ( !isleaf(n) ) ksearch( c,  left( n ), query, eps, (d+1) % N); return; 
      };    
    if ( distance( n->m_data, query) < eps )
      {
      };
    ksearch( c, left( n ), query, eps, (d+1) % N );
    ksearch( c, right( n ), query, eps, (d+1) % N );
  };
  
  template<class P>
  void LevelDump( std::ostream& out, knode<P> * n, unsigned l, unsigned d, unsigned c = 0 )
  {
    if ( (c != l+1) && n != 0  )
      {
	for ( unsigned i = 0; i < 4*c; i ++ )
	  std::cout << " ";
	std::cout << "[ ";
	for ( unsigned i = 0; i < d; i ++ )
	  {
	    std::cout << __rep( n->data() )[i];
	    if ( i != d-1 ) std::cout << ",";
	  };
	std::cout << " ]\n";
	LevelDump( out, left( n ), l, d, c + 1 );
	LevelDump( out, right( n ), l, d, c + 1 );
      };
  };
};

__END_NAMESPACE_SYNAPS
#endif
