#ifndef SYNAPS_SHAPE_SISL_SURFACEBASE_H
#define SYNAPS_SHAPE_SISL_SURFACEBASE_H

#include <synaps/init.h>
#include <sisl/sisl.h>
#include <sisl/GoReadWrite.h>
#include <synaps/shape/base/fxv.h>

__BEGIN_NAMESPACE_SYNAPS

namespace SISL
{
  struct SurfaceBase
  { 
    SISLSurf       * sisl_data_; 
    SurfaceBase() : sisl_data_(0) {};
    SurfaceBase & operator=( const SurfaceBase & rep );
    ~SurfaceBase() { if ( sisl_data_ ) freeSurf(sisl_data_); }
    inline const SISLSurf * sisl_data() const { assert(sisl_data_); return sisl_data_; };  
    inline       SISLSurf * sisl_data()       { assert(sisl_data_); return sisl_data_; };
    int ordu() const  { return sisl_data()->ik1;  };
    int ordv() const  { return sisl_data()->ik2;  };
    int nbpu() const  { return sisl_data()->in1;  };
    int nbpv() const  { return sisl_data()->in2;  };
    int nbku() const  { return nbpu()+ordu();     };
    int nbkv() const  { return nbpv()+ordv();     };
    int nbp()  const  { return nbpu()*nbpv();     };
    int dim()  const  { return sisl_data()->idim; };
    int type() const  { return sisl_data()->ikind;}; 
  };
  
  inline SurfaceBase & SurfaceBase::operator=( const SurfaceBase & rep )
  { 
    if ( sisl_data_ )     freeSurf(sisl_data_);
    sisl_data_ = 0;
    if ( rep.sisl_data_ ) sisl_data_ = copySurface(rep.sisl_data_); 
  };
  
  inline std::istream& operator>>( std::istream & ins, SurfaceBase & rep ) 
  { 
    rep.~SurfaceBase();
    rep.sisl_data_ = readGoSurface(ins); 
  };
  
  inline void range( double & umin, double & umax, 
		     double & vmin, double & vmax,
		     const SurfaceBase & s )
  {
    int stat;
    s1603(s.sisl_data_,&umin,&vmin,&umax,&vmax,&stat);
  };
  
  
  template<class SISLRep>
  inline void range( double & umin, double & umax, double & vmin, double & vmax, 
		     const SISLRep & s )
  {
    range(umin,umax,vmin,vmax,(const SurfaceBase&)s);
  };
  template<class C>
  inline void range( C & umin, C & umax, C & vmin, C & vmax,
		     const SurfaceBase & s )
  {
    double umin_,umax_,vmin_,vmax_;
    range(umin_,umax_,vmin_,vmax_,s);
    using let::assign;
    assign(umin,umin_);
    assign(umax,umax_);
    assign(vmin,vmin_);
    assign(vmax,vmax_);
  };
  
  
  inline void eval( fxv<double,3> & img, 
		    const SurfaceBase & s, 
		    double u, double v )
  {
    double prm[] = { u, v };
    int stat;
    int lk1,lk2;
    s1424(s.sisl_data_,0,0,prm,&lk1,&lk2,&img[0],&stat);
  };
  
  template<class SISLRep>
  inline void eval( fxv<double,3> & img, 
		    const SISLRep & s, 
		    double u, double v )
  {
    eval(img,(const SurfaceBase&)s,u,v);
  };

  inline bool eval( fxv<double,3> * smp, const SurfaceBase & s,
  		    const double * u, const double * v, int m, int n ) 
    { 
     int stat; 
     fxv<double,3> * tmp = new fxv<double,3>[m*n];    
     s1506(s.sisl_data_,0,m,(double*)u,n,(double*)v,(double*)tmp,0,&stat); 

     for ( int i = 0; i < m; i++ )
       for ( int j = 0; j < n; j ++ )
	 for ( int k = 0; k < 3; k ++ ) smp[i*n+j][k] = tmp[j*m+i][k];
     delete[] tmp;
     return false;
    }; 
  
   template<class SISLRep> 
   inline bool eval( fxv<double,3> * smp, const SISLRep & s,  
		     const double * u, const double * v, int m, int n ) 
   {  
     return eval( smp, (const SurfaceBase&)s,u,v,m,n);  
   }; 

};

__END_NAMESPACE_SYNAPS
#endif

