#ifndef SYNAPS_SHAPE_WBSPLINECURVE_H
#define SYNAPS_SHAPE_WBSPLINECURVE_H

#include <synaps/shape/wrappers/WParametricCurve.h>
#include <synaps/shape/functions/FBSplineCurve.h>

__BEGIN_NAMESPACE_SYNAPS

namespace BSplineCurve
{
  template<class R, class C, class Point> inline
  void setDefinition( R& c, int nbpt, int order, const C * knots, const Point * controls )
  {
    using let::assign;
    setDefinition( c, nbpt, order );
    for ( typename R::cp_iterator p = c.cp_begin(); p != c.cp_end(); assign(*p,*controls), p++, controls++ );
    for ( typename R::kn_iterator p = c.kn_begin(); p != c.kn_end(); assign(*p,*knots), p++, knots++ );
  };
  
  template<class C, class R> inline
  void domain( C & tmin, C & tmax, const R  & c )
  {
    using namespace let;
    assign(tmin,c.knot(c.ord()-1));
    assign(tmax,c.knot(c.nbk()-c.ord()));
  };
  
};

template<class C, class R> inline
void domain( C & tmin, C & tmax, const wrapper<R,IBSplineCurve> & c )
{
  using namespace BSplineCurve;
  domain(tmin,tmax,c.rep());
};

template<class Point, class R, class C>
void setDefinition( wrapper<R,IBSplineCurve> & c, int nbpt, int order, const C * knots, const Point * controls )
{
  using namespace BSplineCurve;
  setDefinition( c.rep(), nbpt, order, knots, controls );
};



#define IBSplineCurveWrap \
void setDefinition( int nbpt, int order, const double * knots, const double * coeffs  ) \
{ ::setDefinition(this->rep(),nbpt,order,(const double*)knots,(const fxv<double,3>*)coeffs); };\

template<class R>
struct wrapper<R,IBSplineCurve> :  wrapper<R,IBSplineCurve::base_t>
{
  template<class C, class Point>
  void setDefinition( int nbpt, int order, const C * knots, const Point * controls )
  { ::setDefinition(this->rep(),nbpt,order,knots,controls); };
  IBSplineCurveWrap;
};

__END_NAMESPACE_SYNAPS

#endif
