/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define CRV_CRV_TANG

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
static
void
   c_c_t_s9corr(double [],double,double,double,double,double,double);
static
void
   c_c_t_s9dir(double *,double *,double *,double [],double [],double [],
		  double [],double [], int);
#else
static void c_c_t_s9corr();
static void c_c_t_s9dir();
#endif

#if defined(SISLNEEDPROTOTYPES)
void 
   crv_crv_tang(SISLCurve *pc1, SISLCurve *pc2, double aepsge,
	   double guess_par[], double iter_par[], int *jstat)
#else
     void crv_crv_tang(pc1, pc2, aepsge, guess_par, iter_par, jstat)
     SISLCurve   *pc1;
     SISLCurve   *pc2;
     double aepsge;
     double guess_par[];
     double iter_par[];
     int    *jstat;
#endif
                       
{                        
  int kstat = 0;            
  int kpos = 0;             
  int kleft1=0;             
  int kleft2=0;             
  int kder=1;               
  int kdim;                 
  int knbit;                
  int kdir;                 
  double tdelta[2];         
  double tdist;             
  double td[2],t1[2],tdn[2];
  double tprev;             
  double *sval =SISL_NULL;        
  double *sdiff;            
  double enext[2];          
  double snext[2];          
  
  double estart[2];         
  double eend[2];           
  double zero_pnt[2];       
  
  
  
  if (pc1->idim != 2 || pc2->idim != 2) goto err106;
  
  kdim = 2;
  
  
  
  estart[0] = pc1->et[pc1->ik - 1];
  estart[1] = pc2->et[pc2->ik - 1];
  eend[0] = pc1->et[pc1->in];
  eend[1] = pc2->et[pc2->in];
  zero_pnt[0] = (double)0.;
  zero_pnt[1] = (double)0.;
  enext[0] = guess_par[0];
  enext[1] = guess_par[1];
  
  
  
  tdelta[0] = eend[0] - estart[0];
  tdelta[1] = eend[1] - estart[1];
  
  
  
  sval = newarray(4*kdim,double);
  if (sval == SISL_NULL) goto err101;
  
  sdiff = sval + 3*kdim;
  
  
  
  tprev = (double)HUGE;
  
  
  
  eval_2_crv(pc1, pc2, kder, enext, &kleft1, &kleft2,
	     sval, &kstat);
  if (kstat < 0) goto error;
  
  
  
  c_c_t_s9dir(&tdist,td,td+1,sdiff,zero_pnt,sval,sval+kdim,sval+kdim+kdim,kdim);
  
  
  
  t1[0] = td[0];
  t1[1] = td[1];
  c_c_t_s9corr(t1,enext[0],enext[1],estart[0],eend[0],estart[1],eend[1]);
  
  
  
  for (knbit = 0; knbit < 50; knbit++)
    {
      
      
      snext[0] = enext[0] + t1[0];
      snext[1] = enext[1] + t1[1];
      
      eval_2_crv(pc1, pc2, kder, snext, &kleft1, &kleft2,
    	         sval, &kstat);
      if (kstat < 0) goto error;
      
      
      
      c_c_t_s9dir(&tdist,tdn,tdn+1,sdiff,zero_pnt,
	    sval,sval+kdim,sval+kdim+kdim,kdim);
      
      
      
      kdir = (s6scpr(td,tdn,2) >= DZERO);     
      
      
      
      if (tdist < tprev/(double)2 || kdir)
	{
          enext[0] += t1[0];
          enext[1] += t1[1];
	  
          td[0] = t1[0] = tdn[0];
          td[1] = t1[1] = tdn[1];
	  
	  
	  
	  c_c_t_s9corr(t1,enext[0],enext[1],estart[0],eend[0],estart[1],eend[1]);
	  
          if ( (fabs(t1[0]/tdelta[0]) <= REL_COMP_RES) &&
	      (fabs(t1[1]/tdelta[1]) <= REL_COMP_RES)) break;
	  
          tprev = tdist;
	}
      
      
      
      else
	{
          t1[0] /= (double)2;
          t1[1] /= (double)2;
          knbit--;
	}
    }
  
  
  
  if (tdist <= aepsge)
    *jstat = 1;
  else
    *jstat = 2;
  
  iter_par[0] = enext[0];
  iter_par[1] = enext[1];
  
  
  
  goto out;
  
  
  
 err101: *jstat = -101;
  s6err("crv_crv_tang",*jstat,kpos);
  goto out;                  
  
  
  
 err106: *jstat = -106;
  s6err("crv_crv_tang",*jstat,kpos);
  goto out;                  
  
  
  
  error : *jstat = kstat;
  s6err("crv_crv_tang",*jstat,kpos);
  goto out;                  
  
 out:    if (sval != SISL_NULL) freearray(sval);
}

#if defined(SISLNEEDPROTOTYPES)
static
void
   c_c_t_s9corr(double gd[],double acoef1,double acoef2,
		   double astart1,double aend1,double astart2,double aend2)
#else
static void c_c_t_s9corr(gd,acoef1,acoef2,astart1,aend1,astart2,aend2)
     double gd[];
     double acoef1;
     double acoef2;
     double astart1;
     double aend1;
     double astart2;
     double aend2;
#endif
                       
{
  if (acoef1 + gd[0] < astart1)  gd[0] = astart1 - acoef1;
  else if (acoef1 + gd[0] > aend1) gd[0] = aend1 - acoef1;
  
  if (acoef2 + gd[1] < astart2)  gd[1] = astart2 - acoef2;
  else if (acoef2 + gd[1] > aend2) gd[1] = aend2 - acoef2;
}

#if defined(SISLNEEDPROTOTYPES)
static
void
   c_c_t_s9dir(double *cdist,double *cdiff1,double *cdiff2,
		  double gdiff[],double eval1[],double eval2[],
		  double eder1[],double eder2[], int idim)
#else
static void c_c_t_s9dir(cdist,cdiff1,cdiff2,gdiff,eval1,eval2,eder1,eder2,idim)
     double *cdist;
     double *cdiff1;
     double *cdiff2;
     double gdiff[];
     double eval1[];
     double eval2[];
     double eder1[];
     double eder2[];
     int    idim;
#endif
                       
{                        
  int kstat;		          
  register double tdet;		  
  register double t1,t2,t3,t4,t5; 
  
  
  
  s6diff(eval1,eval2,idim,gdiff);
    
  
  
  *cdist = s6length(gdiff,idim,&kstat);
  
  t1 =  s6scpr(eder1,eder1,idim);
  t2 =  s6scpr(eder1,eder2,idim);
  t3 =  s6scpr(eder2,eder2,idim);
  t4 =  s6scpr(gdiff,eder1,idim);
  t5 =  s6scpr(gdiff,eder2,idim);
  
  
  
  tdet = t1*t3 - t2*t2;
  
  if (DEQUAL(tdet,DZERO))
    {
      *cdiff1 = DZERO;
      *cdiff2 = DZERO;
    }
  else 
    {
      
      
      *cdiff1 =  (t4*t3-t5*t2)/tdet;
      *cdiff2 =  (t1*t5-t2*t4)/tdet;
    }
}

