/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define EVAL_2_CRV

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
   eval_2_crv(SISLCurve *pc1,SISLCurve *pc2,int ider,double epar[],int *ilfs,
	   int *ilft,double eder[],int *jstat)
#else
void eval_2_crv(pc1,pc2,ider,epar,ilfs,ilft,eder,jstat)
     SISLCurve *pc1;
     SISLCurve *pc2;
     int       ider;
     double    epar[];
     int       *ilfs;
     int       *ilft;
     double    eder[];
     int       *jstat;
#endif
                                     
{
  int kstat=0;         
  int kpos=0;          
  int kder = ider + 1;  
  double crv1[8];      
  double crv2[8];      
  double diffvec[2];   
  
  
  
  if (pc1->idim != 2 || pc2->idim != 2) goto err102;
  if (ider > 2) goto err103;
  
  
  
  s1221(pc1, kder, epar[0], ilfs, crv1, &kstat);
  if (kstat < 0) goto error;

  s1221(pc2, kder, epar[1], ilft, crv2, &kstat);
  if (kstat < 0) goto error;
  
  diffvec[0] = crv2[0] - crv1[0];
  diffvec[1] = crv2[1] - crv1[1];
  
  
  
  eder[0] = diffvec[1]*crv1[2] - diffvec[0]*crv1[3];
  eder[1] = diffvec[1]*crv2[2] - diffvec[0]*crv2[3];
  
  
  
  if (ider > 0)
    {
       eder[2] = diffvec[1]*crv1[4] - diffvec[0]*crv1[5];
       eder[3] = crv1[2]*crv2[3] - crv1[3]*crv2[2];
       eder[4] = crv1[2]*crv2[3] - crv1[3]*crv2[2];
       eder[5] = diffvec[1]*crv2[4] - diffvec[0]*crv2[5];
    }
  
  
  
  if (ider > 1)
    {
       eder[6] = crv1[2]*crv1[5] - crv1[3]*crv1[4]
	  + diffvec[1]*crv1[6] - diffvec[0]*crv1[7];
       eder[7] = crv1[4]*crv2[3] - crv1[5]*crv2[2];
       eder[8] = crv1[4]*crv2[3] - crv1[5]*crv2[2];
       eder[9] = crv1[2]*crv2[5] - crv1[3]*crv2[4];
       eder[10] = crv1[2]*crv2[5] - crv1[3]*crv2[4];
       eder[11] = crv2[3]*crv2[4] - crv2[2]*crv2[5]
	  + diffvec[1]*crv2[6] - diffvec[0]*crv2[7];
    }
	  
  
  
  
  *jstat = 0;
  goto out;
  
  
  
  error : *jstat = kstat;
  s6err("eval_2_crv",*jstat,kpos);
  goto out;
  
  
  
  err102: *jstat = -102;
  s6err("eval_2_crv",*jstat,kpos);

  
  
  err103: *jstat = -103;
  s6err("eval_2_crv",*jstat,kpos);

  out:
  
  return;
}


