/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define EVAL_CRV_ARC

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
   eval_crv_arc(SISLCurve *pc1,double center[],double radius,int ider,
	   double epar[],int *ilfs,double eder[],int *jstat)
#else
void eval_crv_arc(pc1,center,radius,ider,epar,ilfs,eder,jstat)
     SISLCurve *pc1;
     double    center[];
     double    radius;
     int       ider;
     double    epar[];
     int       *ilfs;
     double    eder[];
     int       *jstat;
#endif
                                     
{
  int kstat=0;         
  int kpos=0;          
  int kder = ider + 1;  
  double crv1[8];      
  double crv2[8];      
  double diffvec[2];   
  double ang_cos;      
  double ang_sin;      
  
  
  
  if (pc1->idim != 2) goto err102;
  if (ider > 2) goto err103;
  
  
  
  s1221(pc1, kder, epar[0], ilfs, crv1, &kstat);
  if (kstat < 0) goto error;
  
  

  ang_cos = cos(epar[1]);
  ang_sin = sin(epar[1]);
  
  crv2[0] = center[0] + radius*ang_cos;
  crv2[1] = center[1] + radius*ang_sin;
  crv2[2] = - radius*ang_sin;
  crv2[3] = radius*ang_cos;
  crv2[4] = - radius*ang_cos;
  crv2[5] = - radius*ang_sin;
  crv2[6] = radius*ang_sin;
  crv2[7] = - radius*ang_cos;
  
  diffvec[0] = crv2[0] - crv1[0];
  diffvec[1] = crv2[1] - crv1[1];
  
  
  
  eder[0] = diffvec[1]*crv1[2] - diffvec[0]*crv1[3];
  eder[1] = diffvec[1]*crv2[2] - diffvec[0]*crv2[3];
  
  
  
  if (ider > 0)
    {
       eder[2] = diffvec[1]*crv1[4] - diffvec[0]*crv1[5];
       eder[3] = crv1[2]*crv2[3] - crv1[3]*crv2[2];
       eder[4] = crv1[2]*crv2[3] - crv1[3]*crv2[2];
       eder[5] = diffvec[1]*crv2[4] - diffvec[0]*crv2[5];
    }
  
  
  
  if (ider > 1)
    {
       eder[6] = crv1[2]*crv1[5] - crv1[3]*crv1[4]
	  + diffvec[1]*crv1[6] - diffvec[0]*crv1[7];
       eder[7] = crv1[4]*crv2[3] - crv1[5]*crv2[2];
       eder[8] = crv1[4]*crv2[3] - crv1[5]*crv2[2];
       eder[9] = crv1[2]*crv2[5] - crv1[3]*crv2[4];
       eder[10] = crv1[2]*crv2[5] - crv1[3]*crv2[4];
       eder[11] = crv2[3]*crv2[4] - crv2[2]*crv2[5]
	  + diffvec[1]*crv2[6] - diffvec[0]*crv2[7];
    }
	  
  
  
  
  *jstat = 0;
  goto out;
  
  
  
  error : *jstat = kstat;
  s6err("eval_crv_arc",*jstat,kpos);
  goto out;
  
  
  
  err102: *jstat = -102;
  s6err("eval_crv_arc",*jstat,kpos);
  goto out;

  
  
  err103: *jstat = -103;
  s6err("eval_crv_arc",*jstat,kpos);

  out:
  
  return;
}


