/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1172

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void s1172_s9corr(double *,double,double,double);
static void s1172_s9dir(double *,double []);
#else
static void s1172_s9corr();
static void s1172_s9dir();
#endif

#if defined(SISLNEEDPROTOTYPES)
void 
s1172(SISLCurve *pcurve,double astart,
     double aend, double anext, double *cpos,int *jstat)
#else
void s1172(pcurve,astart,aend,anext,cpos,jstat)
     SISLCurve        *pcurve;
     double       astart;
     double       aend;
     double       anext;
     double       *cpos;
     int          *jstat;
#endif
                       
{                        
  int kstat = 0;            
  int kpos = 0;             
  int kleft=0;              
  int kder=3;               
  int knbit;                
  int kdir;                 
  double tdelta;            
  double tdist;             
  double tprev;             
  double td,t1,tdn;         
  double sval[4];            
  double tnext;             
  double tol = (double)1000.0*REL_COMP_RES; 
  
  
  
  if (pcurve->idim != 1) goto err106;
  
  
  
  tdelta = pcurve->et[pcurve->in] - pcurve->et[pcurve->ik - 1];
  
  
  s1221(pcurve,kder,anext,&kleft,sval,&kstat);
  if (kstat < 0) goto error;

  
  tprev = fabs(sval[1]);
  
  
  s1172_s9dir(&td,sval);
  
  
  t1 = td;
  s1172_s9corr(&t1,anext,astart,aend);
  
  
  
  for (knbit = 0; knbit < 50; knbit++)
    {
      
      
      tnext = anext + t1;
      
      s1221(pcurve,kder,tnext,&kleft,sval,&kstat);
      if (kstat < 0) goto error;

      
      tdist = fabs(sval[1]);
  
      
      s1172_s9dir(&tdn,sval);
      
      
      
      kdir = (td*tdn >= DZERO);     
      
      if (tdist <= tprev || kdir)
	{
	  
      
          anext += t1;

          td = t1 = tdn;
	  
	  
	  s1172_s9corr(&t1,anext,astart,aend);
	  
	  
          if (fabs(t1/tdelta) <= REL_COMP_RES)
	    {
	      anext += t1;
	      break;
	    }
	  
          tprev = tdist;
	}
      
      else
	{
	  
      
          t1 /= (double)2;
	  
	}
    }
  
  
  
  if (tdist <= tol)
    *jstat = 1;
  else
    *jstat = 0;

 
  
  if (fabs(anext - pcurve->et[kleft])/tdelta < tol)
    anext = pcurve->et[kleft];
  else if (fabs(anext - pcurve->et[kleft+1])/tdelta < tol)
    anext = pcurve->et[kleft+1];

  
  *cpos = anext;
  
  
  goto out;
  
  
  
 err106: *jstat = -106;
  s6err("s1172",*jstat,kpos);
  goto out;                  
  
  
  error : *jstat = kstat;
  s6err("s1172",*jstat,kpos);
  goto out;                  
  
 out:;
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1172_s9corr(double *cd, double acoef,double astart,double aend)
#else
static void s1172_s9corr(cd,acoef,astart,aend)
     double *cd;
     double acoef;
     double astart;
     double aend;
#endif
                       
{
  if (acoef + *cd < astart)  *cd = astart - acoef;
  else if (acoef + *cd > aend) *cd = aend - acoef;  
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1172_s9dir(double *cdiff,double evals[])
#else
static void s1172_s9dir(cdiff,evals)
     double *cdiff;
     double evals[];
#endif
                       
{                        
   double a,b,c,d,d1,d2;

   a = evals[3];
   b = evals[2];
   c = b*b - 2.0*a*evals[1];
   
   if (fabs(b) > DZERO)  d = -evals[1]/b;
   else                  d = 0.0;
   
   
   if (c < DZERO)                    *cdiff = d;
   else if (fabs(a) > DZERO)
   {
      c = sqrt(c);
      d1 = (-b + c)/a;
      d2 = (-b - c)/a;
      if (DEQUAL(b,c))               *cdiff = d;
      else
	if (fabs(d1-d) < fabs(d2-d)) *cdiff = d1;
      else                           *cdiff = d2;
   }
   else                              *cdiff = d;
}


