/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1190

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1190(SISLObject *po1, double *cmax, double aepsge,int *jstat)
#else
void s1190(po1,cmax,aepsge,jstat)
     SISLObject *po1;
     double        *cmax;
     double        aepsge;
     int           *jstat;
#endif
                                     
{
  int kstat = 0;        
  int kpos = 0;         
  int kcorn = 0;        
  int li[4];	        
  int kbez = 0;         
  int kdim;	        
  int in1,in2;	        
  int i1;	        
  int kmax;             
  double *tmin1,*tmax1; 
  double scorn[4];      
  
  
  *jstat = 0;  
  
  
  
  if (po1->iobj == SISLPOINT)
    {
      kcorn = 0;
      
      
      if((kdim = po1->p1->idim) != 1) goto err105;;
      
      
      
      
      if (po1->p1->pbox == SISL_NULL)
	{
	  
	  
	  s1192(po1,aepsge,&kstat);
	  if (kstat<0) goto error;
	}
      
      
      
      kmax  = po1->p1->pbox->imax;      
      tmax1 = po1->p1->pbox->emax;
      tmin1 = po1->p1->pbox->emin;
    }
  else
    if (po1->iobj == SISLCURVE)
      {
	
	
	if((kdim = po1->c1->idim) != 1) goto err105;;
	
	
	
	kcorn = 2;
	li[0] = 0;
	li[1] = po1->c1->in - 1;
	scorn[0] = po1->c1->ecoef[li[0]];
	scorn[1] = po1->c1->ecoef[li[1]];
	
	
	
	if (po1->c1->in == po1->c1->ik) kbez = 1;
	
	
	
	if (po1->c1->pbox == SISL_NULL)
	  {
	    
	    
	    s1192(po1,aepsge,&kstat);
	    if (kstat<0) goto error;
	  }
	
	
	kmax  = po1->c1->pbox->imax;      	
	tmax1 = po1->c1->pbox->emax;
	tmin1 = po1->c1->pbox->emin;
      }
    else
      if (po1->iobj == SISLSURFACE)
	{
	  
	  
	  if((kdim = po1->s1->idim) != 1) goto err105;;
	  
	  
	  kcorn = 4;
	  in1   = po1->s1->in1;
	  in2   = po1->s1->in2;
	  li[0] = 0;
	  li[1] = in1 - 1;
	  li[2] = in1*(in2 - 1);
	  li[3] = in1*in2-1;
	  scorn[0] = po1->s1->ecoef[li[0]];
	  scorn[1] = po1->s1->ecoef[li[1]];
	  scorn[2] = po1->s1->ecoef[li[2]];
	  scorn[3] = po1->s1->ecoef[li[3]];
	  
	  
	  
	  if (po1->s1->in1 == po1->s1->ik1 &&
	      po1->s1->in2 == po1->s1->ik2)    kbez = 1;
	  
	  
	  
	  if (po1->s1->pbox == SISL_NULL)
	    {
	      
	      
	      s1192(po1,aepsge,&kstat);
	      if (kstat<0) goto error;
	    }
	  
	  
	  kmax  = po1->s1->pbox->imax;	  
	  tmax1 = po1->s1->pbox->emax;
	  tmin1 = po1->s1->pbox->emin;
	}
      else  goto err121;
  
  
  
  
  if (*cmax - *tmax1 > aepsge)
    *jstat = 1;         
  
  else if (*tmax1 - *tmin1 < aepsge)
    *jstat = 2;          
  
  else
    
    {
      
      for (i1=0; i1<kcorn; i1++)
	if (fabs(scorn[i1] - *tmax1) < aepsge)
	  {
	    
	    *jstat = 3;         
	    break;
	    
	  }
    }
  
  
       
    
  
  
  goto out;

  
  
 err105: *jstat = -105;
  s6err("s1190",*jstat,kpos);
  goto out;
  
  
  
 err121: *jstat = -121;
  s6err("s1190",*jstat,kpos);
  goto out;
  
  
  
 error:  *jstat = kstat;
  s6err("s1190",*jstat,kpos);
  goto out;
  
 out:	return;
}






