/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1245

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1245(double coef[], int ik, int dim, double point[], 
      double local_tol, int depth, double weight[], double *area, 
      double *moment, int *stat)
#else
void s1245(coef, ik, dim, point, local_tol, depth, weight, area, moment, stat)
     double     coef[];
     int        ik;
     int        dim;
     double     point[];
     double     local_tol;
     int        depth;
     double     weight[];
     double     *area;
     double     *moment;
     int        *stat;
#endif
{
   int ki, kj;                
   int pos = 0;               
   int index;                 
   double add_area;           
   double add_moment;         
   double left_area;          
   double right_area;         
   double left_moment;        
   double right_moment;       
   double vec1[2];            
   double vec2[2];            
   double vec3[2];            
   double vec4[2];            
   double left_weight[2];     
   double right_weight[2];    
   double* left_coef = SISL_NULL;  
   double* right_coef = SISL_NULL; 


   

   if (dim != 2)
      goto err106;
   
   

   if (ik < 3)
   {
      
      
      
      for (ki = 0; ki < 2; ki++)
      {
	 vec1[ki] = coef[ki] - point[ki];
         vec2[ki] = coef[2*(ik - 1) + ki] - point[ki];
	 vec3[ki] = (coef[ki] + coef[2*(ik - 1) + ki] + point[ki])/3.;
	 vec4[ki] = coef[2*(ik - 1) + ki] - coef[ki]; 
      }

      *area = (vec1[0]*vec2[1] - vec1[1]*vec2[0])/2.;
      *moment = ((vec1[0]*vec1[0] + vec1[1]*vec1[1])/4. +
		 (vec1[0]*vec4[0] + vec1[1]*vec4[1])/4. +
		 (vec4[0]*vec4[0] + vec4[1]*vec4[1])/12.)/
	         fabs(vec1[0]*vec4[1] - vec1[1]*vec4[0]);
      if (*area < 0)
	 *moment = - (*moment);
      
      weight[0] = (*area)*vec3[0];
      weight[1] = (*area)*vec3[1];
   }
   else
   {
      
      
      
      *area = 0.0;
      *moment = 0.0;
      weight[0] = 0.0;
      weight[1] = 0.0;

      

      for (ki = 1; ki < ik; ki++)
      {

	 for (kj = 0; kj < 2; kj++)
         {
	    vec1[kj] = coef[2*(ki - 1) + kj] - point[kj];
            vec2[kj] = coef[2*ki + kj] - point[kj];
    	    vec3[kj] = (coef[2*(ki - 1) + kj] + coef[2*ki + kj] +
			point[kj])/3.;
	    vec4[kj] = coef[2*ki + kj] - coef[2*(ki - 1) + kj]; 
         }

         add_area = (vec1[0]*vec2[1] - vec1[1]*vec2[0]);
         add_moment = ((vec1[0]*vec1[0] + vec1[1]*vec1[1])/4. +
		 (vec1[0]*vec4[0] + vec1[1]*vec4[1])/4. +
		 (vec4[0]*vec4[0] + vec4[1]*vec4[1])/12.)/
	         fabs(vec1[0]*vec4[1] - vec1[1]*vec4[0]);
         if (add_area < 0)
	    add_moment = - add_moment;
      
         weight[0] += add_area*vec3[0];
         weight[1] += add_area*vec3[1];	
	 *area += add_area;
	 *moment += add_moment;
      }
      
      
      
      for (kj = 0; kj < 2; kj++)
      {
	 vec1[kj] = coef[kj] - point[kj];
         vec2[kj] = coef[2*(ik - 1) + kj] - point[kj];
    	 vec3[kj] = (coef[kj] + coef[2*(ik - 1) + kj] + point[kj])/3.;
	 vec4[kj] = coef[2*(ik - 1) + kj] - coef[kj]; 
      }

      add_area = (vec1[0]*vec2[1] - vec1[1]*vec2[0]);
      add_moment = ((vec1[0]*vec1[0] + vec1[1]*vec1[1])/4. +
		 (vec1[0]*vec4[0] + vec1[1]*vec4[1])/4. +
		 (vec4[0]*vec4[0] + vec4[1]*vec4[1])/12.)/
	         fabs(vec1[0]*vec4[1] - vec1[1]*vec4[0]);
      if (add_area < 0)
	 add_moment = - add_moment;
      
      weight[0] += add_area*vec3[0];
      weight[1] += add_area*vec3[1];	
      *area += add_area;
      *moment += add_moment;  
      
      
      
      if (fabs(*area) < REL_COMP_RES)
      {
	 
	 
	 
	 weight[0] = 0.0;
	 weight[1] = 0.0;
	 *area = 0.0;
	 *moment = 0.0;
      }
      
      else if (fabs(2*add_area - *area)/fabs(*area) < local_tol || depth > 20)
      {
	 
	 
	 
	 weight[0] /= 4.;
	 weight[1] /= 4.;
	 *area /= 4.;
	 *moment /= 2.;
      }
      
      else
      {
	 
	 
	 
	 left_coef = newarray(2*ik, double);
	 right_coef = newarray(2*ik, double);
	 
	 for (ki = 0; ki < 2*ik; ki++)
	 {
	    left_coef[ki] = coef[ki]; 
	    right_coef[ki] = coef[ki];
	 }
	 
	 for (ki = 1; ki < ik; ki++)
	 {
	    for (kj = ki; kj < ik; kj++)
	    {
	       index = 2*(ik - kj + ki - 1);
	       left_coef[index] = (left_coef[index] + left_coef[index - 2])/2.;
	       index++;
	       left_coef[index] = (left_coef[index] + left_coef[index - 2])/2.;
	    }
	 }
	 
	 for (ki = 1; ki < ik; ki++)
	 {
	    for (kj = 0 ; kj < (ik - ki); kj++)
	    {
	       index = 2*kj;
	       right_coef[index] = (right_coef[index] + 
				    right_coef[index + 2])/2.;
	       index++;
	       right_coef[index] = (right_coef[index] + 
				    right_coef[index + 2])/2.;
	    }
	 }
	 
	 
	 
	 s1245(left_coef, ik, dim, point, local_tol, (depth + 1), left_weight, 
	       &left_area, &left_moment, stat);
	 if (*stat < 0) goto error;
	 
	 s1245(right_coef, ik, dim, point, local_tol, (depth + 1), right_weight, 
	       &right_area, &right_moment, stat);
	 if (*stat < 0) goto error;
	 
	 weight[0] = left_weight[0] + right_weight[0];
	 weight[1] = left_weight[1] + right_weight[1];
	 *area = left_area + right_area;
	 *moment = (left_moment + right_moment)/4.;
	 
	 if (left_coef != SISL_NULL) freearray(left_coef);
	 if (right_coef != SISL_NULL) freearray(right_coef);
      }
      
   }

   goto out;
     
   
  
  err106: 
   *stat = -106;
   s6err("s1245",*stat,pos);
   goto out;     
     
   
     
  error:
   s6err("s1245", *stat, pos);
   goto out;
     
  out:
   return;
}




