/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1330

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1330(double epar11[],double epar12[],double epar21[],double epar22[],
	   double eval11[],double eval12[],double eval21[],double eval22[],
	   int *jbound,double gpar1[],double gpar2[],int *jstat)
#else
void s1330(epar11,epar12,epar21,epar22,eval11,eval12,eval21,eval22,
           jbound,gpar1,gpar2,jstat)
     double epar11[];
     double epar12[];
     double epar21[];
     double epar22[];
     double eval11[];
     double eval12[];
     double eval21[];
     double eval22[];
     int    *jbound;
     double gpar1[];
     double gpar2[];
     int    *jstat;
#endif
{
  int    kstat1,kstat2;  
  int    kstat;          
  int    kpos=0;         
  int    kins1;          
  int    kins2;          
  int    kbound1;        
  int    kbound2;        
  
  double tdom;           
  double tfak1,tfak2;    
  double spar11[2];      
  double spar12[2];      
  double spar21[2];      
  double spar22[2];      
  
  *jbound = 0;
  
  
  
  kins1 = kins2 = 0; 
  
  if (eval11[0] <= epar11[0] && epar11[0] <= eval11[1] &&
      eval12[0] <= epar11[1] && epar11[1] <= eval12[1] &&
      eval21[0] <= epar12[0] && epar12[0] <= eval21[1] &&
      eval22[0] <= epar12[1] && epar12[1] <= eval22[1]) kins1 = 1;
  
  if (eval11[0] <= epar21[0] && epar21[0] <= eval11[1] &&
      eval12[0] <= epar21[1] && epar21[1] <= eval12[1] &&
      eval21[0] <= epar22[0] && epar22[0] <= eval21[1] &&
      eval22[0] <= epar22[1] && epar22[1] <= eval22[1]) kins2 = 1;
  
  
  
  
  if ((eval11[0] == epar11[0] && epar21[0] < eval11[0]) ||
      (epar11[0] == eval11[1] && eval11[1] < epar21[0]) ||
      (eval12[0] == epar11[1] && epar21[1] < eval12[0]) ||
      (epar11[1] == eval12[1] && eval12[1] < epar21[1]) ||
      (eval21[0] == epar12[0] && epar22[0] < eval21[0]) ||
      (epar12[0] == eval21[1] && eval21[1] < epar22[0]) ||
      (eval22[0] == epar12[1] && epar22[1] < eval22[0]) ||
      (epar12[1] == eval22[1] && eval22[1] < epar22[1])) goto war04;
  
  if (kins1==1 && kins2==1) goto war01;
  
  
  
  if ( (epar11[0]  < eval11[0] && epar21[0]  < eval11[0]) ||
      (eval11[1] < epar11[0]  && eval11[1] < epar21[0] ) ||
      (epar11[1]  < eval12[0] && epar21[1]  < eval12[0]) ||
      (eval12[1] < epar11[1]  && eval12[1] < epar21[1] ) ||
      (epar12[0]  < eval21[0] && epar22[0]  < eval21[0]) ||
      (eval21[1] < epar12[0]  && eval21[1] < epar22[0] ) ||
      (epar12[1]  < eval22[0] && epar22[1]  < eval22[0]) ||
      (eval22[1] < epar12[1]  && eval22[1] < epar22[1] )   ) goto war00;
  
  
  
                              
  
  s1305(epar11,epar21,eval11,eval12,&kbound1,spar11,&kstat);
  
  if (kstat<0) goto error;
  kstat1 = kstat;
  if (kstat1==0) goto war00;
  
  
  
  if (kstat1==2 || kstat1==3)
    {
      tfak1 = fabs(spar11[0]-epar11[0]) + fabs(spar11[1]-epar11[1]);
      tfak2 = fabs(epar21[0]-spar11[0]) + fabs(epar21[1]-spar11[1]);
      tdom = tfak1 + tfak2;
      if (DNEQUAL(tdom,DZERO))
        {
	  spar12[0] = (tfak2*epar12[0] + tfak1*epar22[0])/tdom;
	  spar12[1] = (tfak2*epar12[1] + tfak1*epar22[1])/tdom;
	  
	  
	  
	  if (spar12[0]<eval21[0] || eval21[1]<spar12[0] ||
	      spar12[1]<eval22[0] || eval22[1]<spar12[1])
            {
	      
	      
	      kbound1 = 0;
            }
        }
      else
        {
	  
	  
	  kbound1 = 0;
        }
    }
  else if (kstat1==4 && kins1==1)
    {
      
      goto war04;
    }
  
  
  
  s1305(epar12,epar22,eval21,eval22,&kbound2,spar22,&kstat);
  
  if (kstat<0) goto error;
  kstat2 = kstat;
  if (kstat2==0) goto war00;
  
  if (kstat1==1 && kstat2==1) goto war01;
  
  
  
  
  if (kstat2==2 || kstat2==3)
    {
      tfak1 = fabs(spar22[0]-epar12[0]) + fabs(spar22[1]-epar12[1]);
      tfak2 = fabs(epar22[0]-spar22[0]) + fabs(epar22[1]-spar22[1]);
      tdom = tfak1 + tfak2;
      if (DNEQUAL(tdom,DZERO))
        {
	  spar21[0] = (tfak2*epar11[0] + tfak1*epar21[0])/tdom;
	  spar21[1] = (tfak2*epar11[1] + tfak1*epar21[1])/tdom;
	  
	  
	  
	  if (spar21[0]<eval11[0] || eval11[1]<spar21[0] ||
	      spar21[1]<eval12[0] || eval12[1]<spar21[1])
            {
	      
	      
	      kbound2 = 0;
            }
        }
      else
        {
	  
	  
	  kbound2 = 0;
        }
    }
  else if (kstat2==4 && kins1==1)
    {
      
      goto war04;
    }
  
  
  
  
  
  
  if (kbound1!=0 && kbound2!=0)
    {
 
      double t1,t2,t3,t4; 

      t1 = s6dist(spar11,epar11,2);
      t2 = s6dist(spar12,epar12,2);
      t3 = s6dist(spar21,epar11,2);
      t4 = s6dist(spar22,epar12,2);
      
      if ((t1*t1+t2*t2) < (t3*t3+t4*t4) )
        kbound2 = 0;
      else
        kbound1 = 0;
    }
  
  if (kbound1==0 && kbound2 ==0)
    {
      
      goto war00;
    }
  else if (kbound1!=0 && kbound2==0)
    {
      
      memcopy(gpar1,spar11,2,DOUBLE);
      memcopy(gpar2,spar12,2,DOUBLE);
      *jbound = kbound1;
    }
  else if (kbound1==0 && kbound2!=0)
    {
      
      memcopy(gpar1,spar21,2,DOUBLE);
      memcopy(gpar2,spar22,2,DOUBLE);
      *jbound = kbound2+4;
    }
  
  if (kins1 == 1)
    {
      if (eval11[0] == epar11[0] || epar11[0] == eval11[1] ||
	  eval12[0] == epar11[1] || epar11[1] == eval12[1] ||
	  eval21[0] == epar12[0] || epar12[0] == eval21[1] ||
	  eval22[0] == epar12[1] || epar12[1] == eval22[1])
        {
	  goto war04;
	}
      else
	{
	  goto war02; 
	}
    }
  
  if (kins2 == 1) goto war03; 
  
  goto war05;
  
  
  
 war00:
  *jstat = 0;
  goto out;                                                                      
  
                            
  
 war01:
  *jstat = 1;
  goto out;
  
  
 war02:
  *jstat = 2;
  goto out;
  
  
 war03:
  *jstat = 3;
  goto out;
  
  
 war04:
  *jstat = 4;
  goto out;
  
  
 war05:
  *jstat = 5;
  goto out;
  
  
 error:
  *jstat = kstat;
  s6err("s1330",*jstat,kpos);
  goto out;
  
  
 out:
  return;
}

