/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1333_COUNT
#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1333_count(int inbcrv,SISLCurve *vpcurv[],int *jcont,int *jstat)
#else
void s1333_count(inbcrv,vpcurv,jcont,jstat)
     int    	inbcrv;
     SISLCurve  *vpcurv[];
     int        *jcont;
     int        *jstat;
#endif
{
  int kmult1,kmult2,kmult;   
  int kcont=0;               
  int kpos=0;
  int kleft = 0;
  int kstat;
  int ki;
  SISLCurve *curve=SISL_NULL;     

  *jcont = -1;
  

  for (ki=0 ; ki<inbcrv ; ki++)
    {
       curve = vpcurv[ki];
       kmult1 = s6knotmult(curve->et,curve->ik,curve->in,&kleft,
                           curve->et[curve->ik-1],&kstat);
       if (kstat<0)goto error;
       
       kmult2 = s6knotmult(curve->et,curve->ik,curve->in,&kleft,
                           curve->et[curve->in],&kstat);
       if (kstat<0)goto error;
       kmult = MAX(kmult1,kmult2);
       kmult = MIN(kmult,curve->ik);
       
       if (ki==0)
	 {
	   
	   kcont = curve->ik - kmult - 1;
	 }
       else
	 {   
           kcont = MIN(kcont, curve->ik - kmult - 1); 
	 }
     }
  

  
  
  *jcont = kcont;
  
  *jstat = 0;
  goto out; 
  
  

  error : 
    *jstat = kstat;     
  s6err("s1333_count",*jstat,kpos);
  goto out;
 out:
   
  return;
}

