/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1333_CYCLIC

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1333_cyclic(SISLSurf *vsurf,int icont,int *jstat)
#else
void s1333_cyclic(vsurf,icont,jstat)
     SISLSurf   *vsurf;
     int        icont;
     int    	*jstat;
#endif
{
  double *scycl=SISL_NULL;                    
  double *smatrix=SISL_NULL;                   
  double *salloc=SISL_NULL;                    
  double *salfa=SISL_NULL;                     
  double *spek=SISL_NULL;                      
  double *scoef=SISL_NULL;                     
  double *sb=SISL_NULL;                        
  double *sfrom,*sto;
  double *sp;                             
  double *st1=SISL_NULL;                       
  double *stx=SISL_NULL;                       

  int    kdim = vsurf->idim;
  int    kk1 = vsurf->ik1;
  int    kn1 = vsurf->in1;
  int    kn2 = vsurf->in2;
  int    knumb = kdim*kn1;

  int    kcont;                           
  int    kmult;                           
  int    ki,kj,kl;
  int    kperlength;
  int    kant;
  int    kleft=0;                         
  int    kpl,kfi,kla;                     
  int    kstat;
  int    *mpiv=SISL_NULL;                      
  int    kpos = 0;
  int    knst1;                           
  int    knstx;                           




  

  if (icont < 0) goto finished;
  kcont = icont;
  if (icont >= kk1-2) icont = kk1-2;

  
  kmult = kk1 - kcont - 1;

  

  kant = kk1-kmult;


  

  mpiv = new0array(2*kk1,INT);
  if (mpiv == SISL_NULL) goto err101;

  salloc = new0array(3*kn1+9*kk1+4*kk1*kk1+kdim*kn1*kn2,DOUBLE);
  if (salloc == SISL_NULL) goto err101;
  scycl = salloc;                  
  smatrix = scycl + kn1 + kk1;  
  salfa = smatrix + 4*kk1*kk1;     
  scoef = salfa + kk1;           
  sb    = scoef + kdim*kn1*kn2;    
  sp    = sb + 2*kk1;              
  st1   = sp + kk1;                
  stx   = st1 + kn1 + 2*kk1;       



  

  memcopy(scoef,vsurf->ecoef,kdim*kn1*kn2,DOUBLE);



  


  

  memcopy(scycl,vsurf->et1,kn1+kk1,DOUBLE);

  

  kperlength = kn1 - kk1 + kmult;

  

  for (ki=kk1-kmult-1 ; 0<=ki ; ki--)
    {
      scycl[ki] = scycl[kk1-1] - (scycl[kn1] - scycl[ki+kperlength]);
    }


  

  for (ki=kmult ; ki < kk1 ; ki++)
    {
      scycl[kn1+ki] = scycl[kn1] + (scycl[kk1+ki-kmult] - scycl[kk1-1]);

    }
      

  memcopy(st1,scycl,kant,DOUBLE);
  memcopy(st1+kant,vsurf->et1,kn1+kk1,DOUBLE);
  memcopy(st1+kant+kk1+kn1,scycl+kn1+kk1-kant,kant,DOUBLE);
  knst1 = kn1 + 2*kant;

  memcopy(stx,scycl,kn1,DOUBLE);
  memcopy(stx+kn1,st1+kant+kn1,kk1+kant,DOUBLE);
  knstx = kn1 + kant;

  


  


  


  

  for (ki=kant,spek=smatrix ; ki <kk1+kant ; ki++, spek+=kk1)
    {
      

      s1219(stx,kk1,kn1,&kleft,st1[ki],&kstat);
      if (kstat<0) goto error;

      s1701(ki,kleft,kk1,knstx,&kpl,&kfi,&kla,st1,stx,sp,salfa,&kstat);
      if(kstat<0) goto error;

      

      memcopy(spek+kfi,salfa+kpl+kfi,kla-kfi+1,DOUBLE);
    }



  

  s6lufacp(smatrix,mpiv,kk1,&kstat);
  if (kstat<0) goto error;

  


  for (ki=0 ; ki<kn2 ; ki++)
    for (kl=0 ; kl<kdim ; kl++)
      {
	for (kj=0, sfrom=(vsurf->ecoef)+ki*knumb+kl,sto=sb ;
	     kj<kk1 ; kj++,sfrom+=kdim,sto++)
	  *sto = *sfrom;

	

	s6lusolp(smatrix,sb,mpiv,kk1,&kstat);
	if (kstat<0) goto error;

	

	for (kj=0, sto=scoef+ki*knumb+kl,sfrom=sb ;
	     kj<kk1 ; kj++,sfrom++,sto+=kdim)
	  *sto = *sfrom;
      }


  


  for (ki=0,spek=smatrix ; ki<kk1*kk1 ; ki++,spek++) *spek = DZERO;


  for (ki=kn1-kk1 ,spek=smatrix ; ki <kn1 ; ki++, spek+=kk1)
    {
      s1219(scycl,kk1,kn1,&kleft,stx[ki],&kstat);
      if (kstat<0) goto error;

      s1701(ki,kleft,kk1,kn1,&kpl,&kfi,&kla,stx,scycl,sp,salfa,&kstat);
      if(kstat<0) goto error;

      

      memcopy(spek+kfi-(kn1-kk1),salfa+kpl+kfi,kla-kfi+1,DOUBLE);
    }



  

  s6lufacp(smatrix,mpiv,kk1,&kstat);
  if (kstat<0) goto error;

  

  for (ki=0 ; ki<kn2 ; ki++)
    for (kl=0 ; kl<kdim ; kl++)
      {
	for (kj=0, sfrom=scoef+ki*knumb+kdim*(kn1-kk1)+kl,sto=sb ;
	     kj<kk1 ; kj++,sfrom+=kdim,sto++)
	  *sto = *sfrom;

	

	s6lusolp(smatrix,sb,mpiv,kk1,&kstat);
	if (kstat<0) goto error;

	

	for (kj=0, sto=scoef+ki*knumb+kdim*(kn1-kk1)+kl,sfrom=sb ;
	 kj<kk1 ; kj++,sto+=kdim,sfrom++)
	  *sto = *sfrom;
      }


  

  memcopy(vsurf->ecoef,scoef,kdim*kn1*kn2,DOUBLE);
  memcopy(vsurf->et1,scycl,kn1+kk1,DOUBLE);

  
  vsurf->cuopen_1 = SISL_SURF_PERIODIC;


  

 finished:

  *jstat = 0;
  goto out;

  

 err101:
  *jstat = -101;
  s6err("s1333_cyclic",*jstat,kpos);
  goto out;



  

  error :
    *jstat = kstat;
  s6err("s1333_cyclic",*jstat,kpos);
  goto out;
 out:

  
  if (salloc != SISL_NULL) freearray(salloc);
  if (mpiv != SISL_NULL) freearray(mpiv);

  return;

}

