/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1334

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1334(double epoint[],int inbpnt,int idim,double nptyp[],
      int icnsta,int icnend,int iopen,int ik,double astpar,
      double *cendpar,SISLCurve **rc,double **gpar,int *jnbpar,int *jstat)
#else
void s1334(epoint,inbpnt,idim,nptyp,icnsta,icnend,iopen,ik,astpar,
           cendpar,rc,gpar,jnbpar,jstat)
     double epoint[];
     int    inbpnt;
     int    idim;
     double    nptyp[];
     int    icnsta;
     int    icnend;
     int    iopen;
     int    ik;
     double astpar;
     double *cendpar;
     SISLCurve  **rc;
     double **gpar;
     int    *jnbpar;
     int    *jstat;
#endif
{
  int kpos = 0;
  int kstat = 0;
  int *ltype = SISL_NULL;		
  int knpt;			
  double *lcond = SISL_NULL;		

  int *ityp = SISL_NULL;
  int ki;

  

  if (iopen==SISL_CRV_CLOSED) iopen = SISL_CRV_PERIODIC;

  

  ityp = newarray (inbpnt, INT);
  if (ityp == SISL_NULL)
    goto err101;

  for (ki=0; ki < inbpnt; ki++) ityp[ki] = (int)nptyp[ki];

  

  *jstat = 0;

  

  s1906 (epoint, ityp, icnsta, icnend, inbpnt, idim, &lcond,
	 &ltype, &knpt, &kstat);

  if (kstat < 0)
    goto error;

  

  s1901(s1909, s1902, lcond, ltype, knpt, astpar, ik, idim, iopen,
	cendpar, rc, gpar, jnbpar, &kstat);  
  if (kstat < 0) goto error;

  *jstat = 0;
  goto out;
  
  
 error: *jstat = kstat;
  s6err("s1334",*jstat,kpos);
  goto out;

  
 err101: *jstat = -101;
  s6err("s1334",*jstat,kpos);
  goto out;
  
 out:
  if (ltype != SISL_NULL)
    freearray (ltype);

  if (ityp != SISL_NULL)
    freearray (ityp);

  if (lcond != SISL_NULL)
    freearray (lcond);
    
  return;
}

