/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1340

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1340(SISLCurve *oldcurve, double eps[], int startfix, int endfix,
      double epsco, int itmax, SISLCurve **newcurve, double maxerr[],
      int *stat)
#else
void
s1340(oldcurve, eps, startfix, endfix, epsco, itmax, newcurve,
      maxerr, stat)
    SISLCurve 	*oldcurve;
    double	eps[];
    int		startfix;
    int		endfix;
    double	epsco;
    int		itmax;
    SISLCurve	**newcurve;
    double	maxerr[];
    int		*stat;
#endif
{
  char ready, big;              
  int k = oldcurve->ik;         
  int dim = oldcurve->idim;
  double *d = oldcurve->ecoef;
  int itcount=0;                
  int n1 = oldcurve->in;        
  int n2;
  int i, mini, maxi, indx;      
  double *local_err = SISL_NULL;     
  double *l2err = SISL_NULL;
  double *lepsco = SISL_NULL;
  double *temp_err = SISL_NULL;
  SISLCurve *tempcurve = SISL_NULL;  
  SISLCurve  *helpcurve = SISL_NULL;
  rank_info ranking;            
  int lstat=0;                  
  int pos=0;			
  SISLCurve *qc_kreg = SISL_NULL;    


  
  ranking.prio = SISL_NULL;
  ranking.groups = SISL_NULL;

  

  for (i=0; i<dim; i++) maxerr[i] = 0.;

  

  if (n1 == k)
  {
    *newcurve = newCurve(n1, k, oldcurve->et, oldcurve->ecoef,
			 oldcurve->ikind, oldcurve->idim, 1);
    if (*newcurve == SISL_NULL)  goto err101;

    *stat = 0;
    goto out;
  }

  

  if (oldcurve->cuopen == SISL_CRV_PERIODIC)
  {
    make_cv_kreg(oldcurve, &qc_kreg, &lstat);
    if (lstat < 0) goto error;

    

    startfix = MAX(startfix, 2);
    endfix = MAX(endfix, 2);
  }
  else
    qc_kreg = oldcurve;

  

  temp_err = newarray(dim, double);
  if (temp_err == SISL_NULL) goto err101;

  lepsco = newarray(dim, double);
  if (lepsco == SISL_NULL) goto err101;

  

  ranking.prio = newarray(MAX(n1-k,1), int);
  if (ranking.prio == SISL_NULL) goto err101;

  ranking.groups = newarray(MAX(n1-k,1), int);
  if (ranking.groups == SISL_NULL) goto err101;

  

  for (i=0; i<dim; i++) lepsco[i] = fabs(d[i]);

  indx = 0;
  for (i=1; i<n1*dim; i++)
  {
    lepsco[indx++] += fabs(d[i]);
    if (indx == dim) indx = 0;
  }

  

  for (i=0; i<dim; i++)
    lepsco[i] = MIN(lepsco[i]*epsco/n1, eps[i]);

  

  

  mini = 0;
  maxi = n1 - k + 1;

  

  s1353(qc_kreg, eps, &ranking, &lstat);
  if (lstat < 0) goto error;

  

  s1354(qc_kreg, qc_kreg, &ranking, eps, lepsco, startfix, endfix,
	mini, maxi, newcurve, maxerr, &lstat);
  if (lstat < 0) goto error;

  

  n2 = (*newcurve)->in;

  

  itcount = 1;
  big = 0;

  

  ready = (n1 == n2) || (n2 == k) || (itcount == itmax);
  while (!ready)
  {

    

    s1353(*newcurve, eps, &ranking, &lstat);
    if (lstat < 0) goto error;

    if (!big)
    {

      

      mini = 0; maxi = (*newcurve)->in - k + 1;
      s1354(*newcurve, *newcurve, &ranking, eps, lepsco, startfix,
	    endfix, mini, maxi, &helpcurve, temp_err, &lstat);
      if (lstat < 0) goto error;

      

      if (local_err != SISL_NULL) freearray(local_err);

      sh1365(qc_kreg, helpcurve->et, k, helpcurve->in,
	     startfix, endfix, &tempcurve,
	     &local_err, &l2err, &lstat);
      if (lstat < 0) goto error;

      

      freearray(l2err);
      freeCurve(helpcurve);
      helpcurve = SISL_NULL;

      
      i = 0;
      while (!big && i<dim)
      {
	big = local_err[i] > eps[i];
	i++;
      }
    }
    else

      

      tempcurve = *newcurve;

    if (big)
    {

      

      mini= 0; maxi = tempcurve->in - k + 1;
      s1354(qc_kreg, *newcurve, &ranking, eps, lepsco,
	    startfix, endfix, mini, maxi, &helpcurve, maxerr, &lstat);
      if (lstat < 0) goto error;

      if (*newcurve != SISL_NULL && *newcurve != tempcurve)
      {
	freeCurve(*newcurve);
	*newcurve = SISL_NULL;
      }

      if (tempcurve != SISL_NULL)
      {
	freeCurve(tempcurve);
	tempcurve = SISL_NULL;
      }
      *newcurve = helpcurve;
      helpcurve = SISL_NULL;
    }
    else
    {

      

      for (i=0; i<dim; i++) maxerr[i] = local_err[i];

      if (*newcurve != SISL_NULL) freeCurve(*newcurve);
      *newcurve = tempcurve;
      tempcurve = SISL_NULL;
    }

    

    n1 = n2;
    n2 = (*newcurve)->in;
    ++itcount;
    ready = (n1 == n2) || (n2 == k) || (itcount == itmax);
  }

  

  if (oldcurve->cuopen == SISL_CRV_CLOSED ||
      oldcurve->cuopen == SISL_CRV_PERIODIC)
    (*newcurve)->cuopen = SISL_CRV_CLOSED;

  

  *stat = 0;
  goto out;

  

err101:
  *stat = -101;
  s6err("s1340", *stat, pos);
  goto out;

  

error:
  *stat = lstat;
  s6err("s1340", *stat, pos);
  goto out;

  

out:
  if (temp_err != SISL_NULL) freearray(temp_err);
  if (local_err != SISL_NULL) freearray(local_err);
  if (l2err != SISL_NULL) freearray(l2err);
  if (lepsco != SISL_NULL) freearray(lepsco);
  if (ranking.prio != SISL_NULL) freearray(ranking.prio);
  if (ranking.groups != SISL_NULL) freearray(ranking.groups);
  if (qc_kreg != SISL_NULL && qc_kreg != oldcurve) freeCurve(qc_kreg);
  if (helpcurve != SISL_NULL && helpcurve != (*newcurve)) freeCurve(helpcurve);
  if (tempcurve != SISL_NULL && tempcurve != (*newcurve)) freeCurve(tempcurve);

  return;
}

