/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1358

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1358(double epoint[],int inbpnt,int idim,double ntype[],double epar[],
	   int icnsta,int icnend,int iopen,int ik,double astpar,
	   double *cendpar,SISLCurve **rc,double **gpar,int *jnbpar,int *jstat)
#else
void s1358(epoint,inbpnt,idim,ntype,epar,icnsta,icnend,iopen,ik,astpar,
           cendpar,rc,gpar,jnbpar,jstat)
     double epoint[];
     int    inbpnt;
     int    idim;
     double    ntype[];
     double epar[];
     int    icnsta;
     int    icnend;
     int    iopen;
     int    ik;
     double astpar;
     double *cendpar;
     SISLCurve  **rc;
     double **gpar;
     int    *jnbpar;
     int    *jstat;
#endif
{
  int kpos = 0;
  SISLCurve *qc = SISL_NULL;		
  SISLCurve *qc2 = SISL_NULL;
  int *ltype = SISL_NULL;		
  int *ltype2 = SISL_NULL;		
  int *sder = SISL_NULL;		
  int knpt;			
  int kordr;			
  int kstat;			
  int kn;			
  int ki;			
  int kright = 1;		
  int knlr = 0;			
  int knrc = 0;			
  double *lpar = SISL_NULL;		
  double *lcond = SISL_NULL;		
  double *sknot = SISL_NULL;		
  double *spar = SISL_NULL;		
  double *scond = SISL_NULL;		
  double *scoef = SISL_NULL;		
  int *ityp = SISL_NULL;



  if (iopen==SISL_CRV_CLOSED) iopen = SISL_CRV_PERIODIC;



  ityp = newarray (inbpnt, INT);
  if (ityp == SISL_NULL) goto err101;

  for (ki=0; ki < inbpnt; ki++) ityp[ki] = (int)ntype[ki];
  *jstat = 0;

  

  s1907 (epoint, ityp, epar, iopen, icnsta, icnend, inbpnt,
	 idim, &lcond, &ltype, &lpar,&knpt, &kstat);
  if (kstat < 0) goto error;

  

  s1908 (lcond, ltype, lpar, knpt, ik, idim, iopen, &scond, &ltype2,
	 &spar, &knpt, &kstat);
  if (kstat < 0) goto error;

  

  sder = newarray (knpt, INT);
  if (sder == SISL_NULL) goto err101;

  for (ki = 0; ki < knpt; ki++)
    sder[ki] = abs (ltype2[ki]);

  kordr = MIN (ik, knpt);

  if (!(iopen == SISL_CRV_OPEN))
    {
      knlr = kordr / 2;
      knrc = kordr - knlr - 1;
      knpt--;
    }

  

  s1902 (lpar, knpt, kordr, iopen, &sknot, &kstat);
  if (kstat < 0) goto error;

  

  s1891 (spar, scond, idim, knpt, kright, sder, iopen, sknot,
	 &scoef, &kn, kordr, knlr, knrc, &kstat);
  if (kstat < 0) goto error;

  

  qc = newCurve (kn, kordr, sknot, scoef, 1, idim, 1);
  if (qc == SISL_NULL) goto err101;
  qc->cuopen = (iopen == SISL_CRV_OPEN) ? iopen : SISL_CRV_PERIODIC;
  
  if (iopen == SISL_CRV_CLOSED)
    {
      

      s1713 (qc, sknot[kordr - 1], sknot[kn], &qc2, &kstat);
      if (kstat < 0) goto error;

      if (qc != SISL_NULL) freeCurve (qc);
      qc = qc2;
    }

  if (kordr < ik)
    {
      

      qc2 = SISL_NULL;
      s1750 (qc, ik, &qc2, &kstat);
      if (kstat < 0) goto error;
	
      if (qc != SISL_NULL) freeCurve (qc);
      qc = qc2;
    }

  

  qc->cuopen = iopen;

  
  
  *cendpar = *(qc->et + qc->in);

  

  

  *gpar = lpar;

  

  *jnbpar = 0;

   
  
  
 
  for (ki = 1; ki<knpt; ki++)
    {
      if (spar[ki - 1] < spar[ki])
	 
      

	(*gpar)[(*jnbpar)++] = spar[ki-1];
    }

  

  (*gpar)[(*jnbpar)++] = spar[ki-1];
  
  *rc = qc;
  goto out;


  

  err101:
    *jstat = -101;
    s6err ("s1358", *jstat, kpos);
    goto out;

  

  error:
    *jstat = kstat;
    s6err ("s1358", *jstat, kpos);
    goto out;

  out:
  

    if (scond != SISL_NULL)    freearray (scond);
    if (scoef != SISL_NULL)    freearray (scoef);
    if (sder != SISL_NULL)     freearray (sder);
    if (ltype != SISL_NULL)    freearray (ltype);

    if (ityp != SISL_NULL)     freearray (ityp);

    if (ltype2 != SISL_NULL)   freearray (ltype2);
    if (lcond != SISL_NULL)    freearray (lcond);
    if (sknot != SISL_NULL)    freearray (sknot);
    if (spar != SISL_NULL)     freearray (spar);

    return;
}


