/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1362

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1362(SISLCurve *pc1,double aoffset,double enorm[],int idim,
	   int ider,double ax,int *ileft,double eder[],int *jstat)
#else
void s1362(pc1,aoffset,enorm,idim,ider,ax,ileft,eder,jstat)
     SISLCurve *pc1;
     double       aoffset;
     double       enorm[];
     int          idim;
     int          ider;
     double       ax;
     int          *ileft;
     double       eder[];
     int          *jstat;
#endif
                                     
{
  int kstat=0;        
  int kpos=0;         
  int kder=ider+1;    
  int ki,kj;          
  double sder[12];    
  double snorm[3];    
  double sdnorm[3];   
  double sddnorm[3];  
  double tderl;       
  double tl;          
  double *spnt;       
  double *start;      
  
  if (idim !=2 && idim != 3) goto err105;
  
  if (DEQUAL(aoffset,DZERO))
    {
      s1221(pc1,ider,ax,ileft,eder,&kstat);
      if (kstat<0) goto error;
    }
  else
    {
      s1221(pc1,kder,ax,ileft,sder,&kstat);
      if (kstat<0) goto error;
      
      tderl = s6length(sder+idim,idim,&kstat);
      if (DEQUAL(tderl,DZERO)) tderl = (double)1.0;
      
      
      
      
      for (ki=1,start=sder+idim ; ki<=kder ; ki++,start+=idim)
        {
	  for (kj=0,spnt=start ; kj<idim*(kder+1-ki) ; kj++,spnt++)
            {
	      *spnt /= tderl;
            }
        }
      
      
      if (idim==2)
        {
	  snorm[0]   = -sder[3];
	  snorm[1]   =  sder[2];
	  if(ider>=1)
	    {                                                           
	      sdnorm[0]  = -sder[5];
	      sdnorm[1]  =  sder[4];
	      if(ider>=2)
		{
		  sddnorm[0] = -sder[7];
		  sddnorm[1] =  sder[6];
		}
	    }
        }
      else
        {
	  s6crss(sder+idim,enorm,snorm);
	  if (ider>=1)
	    {
	      s6crss(sder+2*idim,enorm,sdnorm);
	      if (ider>=2)
		s6crss(sder+3*idim,enorm,sddnorm);
	    }
        }
      
      tl = s6length(snorm,idim,&kstat);
      if (DEQUAL(tl,DZERO)) tl = (double)1.0;
      
      
      
      for (ki=0;ki<idim;ki++)
        eder[ki] = sder[ki] + aoffset*snorm[ki]/tl;
      
      if (ider>=1)
        {
	  double tpndpn;         
	  double tl3=tl*tl*tl;   
	  
	  tpndpn = s6scpr(snorm,sdnorm,idim);
	  
	  
	  
	  for (ki=0;ki<idim;ki++)
            {
	      eder[idim+ki] = sder[idim+ki] + aoffset*(sdnorm[ki]/tl
			    - snorm[ki]*tpndpn/tl3);
            }
	  if (ider>=2)
            {
	      double tl5= tl3*tl*tl; 
	      double tpnddpn;         
	      double tdpndpn;         
	      
	      tpnddpn = s6scpr(snorm,sddnorm,idim);
	      tdpndpn = s6scpr(sdnorm,sdnorm,idim); 
	      
	      
	      
	      for (ki=0;ki<idim;ki++)
                {
		  eder[2*idim+ki] = sder[2*idim+ki] + aoffset*(sddnorm[ki]/tl
				  - (double)2.0*sdnorm[ki]*tpndpn/tl3
				  - snorm[ki]*(tdpndpn+tpnddpn)/tl3
				  + (double)3.0*snorm[ki]*tpndpn*tpndpn/tl5);
                }
            }
        }
      
      
      
      
      for (ki=1,start=eder+idim ; ki<kder ; ki++,start+=idim)
        {
	  for (kj=0,spnt=start ; kj<idim*(kder-ki) ; kj++,spnt++)
            {
	      *spnt *= tderl;
            }
        }
    }
  
  
  *jstat = 0;
  goto out;
  
  
  
 err105: *jstat = -105;
  s6err("s1362",*jstat,kpos);
  goto out;
  
  
  
  
 error:  *jstat = kstat;
  s6err("s1362",*jstat,kpos);
  goto out;
  
 out: return;
}

