/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1504
#include "sislP.h"



#if defined(SISLNEEDPROTOTYPES)
void
s1504(double *et,int ik,int in,
	   double *ax,int im,int ider,double ebder[],int ileft[],int *jstat)
#else
void s1504(et,ik,in,ax,im,ider,ebder,ileft,jstat)
     double *et;
     int    ik;
     int    in;
     double *ax;
     int    im;
     int    ider;
     double ebder[];
     int ileft[];
     int    *jstat;
#endif
{
  int kstat=0;        
  int kpos=0;         
  int kleft=0;        
  int j,k,kk;         
  double *eder;       
  int size;           

  if (ider < 0) goto err178;

  size = (ider + 1) * ik;
  eder = newarray(size,double);
  if (eder == SISL_NULL) goto err101;

  kk = 0;
  for(k=0; k<im; k++)
  {
    s1220(et,ik,in,&kleft,ax[k],ider,eder,&kstat);
    if (kstat < 0) goto error;

    ileft[k] = kleft;
    for(j=0; j<size; j++)
    {
      ebder[kk+j] = eder[j];
    }
    kk += size;
  }

  *jstat = 0;
  goto out;

  
 err101: *jstat = -101;
  s6err("s1504",*jstat,kpos);
  goto out;

  
 err178: *jstat = -178;
  s6err("s1504",*jstat,kpos);
  goto out;

  

 error:  *jstat = kstat;
  s6err("s1504",*jstat,kpos);
  goto out;

 out: return;
}

