/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */



#include "copyright.h"



#define S1517

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1517(double ep[],double ev[],double epar[],int im, double mu,
           double **evnew,int *jstat)
#else
void s1517(ep,ev,epar,im,mu,evnew,jstat)
     double ep[];
     double ev[];
     double epar[];
     int    im;
     double mu;
     double **evnew;
     int    *jstat;
#endif
{
  int ki;             
  int kpos=0;         
  double *evtemp;
  double mu3;
  double knotint1;
  double knotint2;




  

  if (im < 2) goto err102;
  if(mu < 0.0 || mu >= 1.0) goto err102;

  mu3 = 3.0 * mu;

  

  evtemp    = newarray(im,DOUBLE);
  if (evtemp == SISL_NULL) goto err101;

  


  knotint2 = epar[1] - epar[0];

  if(ev[0] < - mu3 * ep[0] / knotint2)
  {
      evtemp[0] = - mu3 * ep[0] / knotint2;
  }
  
  else if (mu == DZERO)
  {
     evtemp[0] = mu;
  }
  
  else
  {
      evtemp[0] = ev[0];
  }


  for(ki=1; ki<im-1; ki++)
  {
      knotint1 = epar[ki] - epar[ki-1];
      knotint2 = epar[ki+1] - epar[ki];

      if(ev[ki] >  mu3 * ep[ki] / knotint1)
      {
          evtemp[ki] = mu3 * ep[ki] / knotint1;
      }
      else if(ev[ki] < - mu3 * ep[ki] / knotint2)
      {
          evtemp[ki] = - mu3 * ep[ki] / knotint2;
      }
      else
      {
          evtemp[ki] = ev[ki];
      }
  }

  knotint1 = epar[im-1] - epar[im-2];

  if(ev[im-1] > mu3 * ep[im-1] / knotint1)
  {
      evtemp[im-1] = mu3 * ep[im-1] / knotint1;
  }
  
  else if (mu == DZERO)
  {
     evtemp[im-1] = mu;
  }
  
  else
  {
      evtemp[im-1] = ev[im-1];
  }


  

  

  (*evnew) = evtemp;

  *jstat = 0;
  goto out;



  

 err101: *jstat = -101;
  s6err("s1517",*jstat,kpos);
  goto out;


  

 err102: *jstat = -102;
  s6err("s1517",*jstat,kpos);
  goto out;


 out:

  return;
}

