/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1520

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1520 (SISLCurve * pc, double angle, double ep[], double eaxis[],
       SISLSurf ** rs, int *jstat)
#else
void
s1520 (pc, angle, ep, eaxis, rs, jstat)
     SISLCurve *pc;
     double angle;
     double ep[];
     double eaxis[];
     SISLSurf **rs;
     int *jstat;
#endif
{
  double *st1;			
  double *scoef1;		
  double *rcoef1;		
  int kn1;			
  int kk1;			
  double *st2;			
  double *scoef2;		
  double *rcoef2;		
  int kn2;			
  int kk2;			
  int kdim;			
  int ksurfdim;			
  int ki;			
  int kj;			
  int kl;			
  int kn;			
  int kk;			
  double *st;			
  double *scoef;		
  int kind;			
  int kcopy;			
  double tangle;		
  int quadrant;			
  double resang;		
  double spar;			
  double epar;			
  SISLCurve *totcurve=SISL_NULL;	
  SISLCurve *pnorm;		
  double weight = (double) 1. / sqrt ((double) 2.);	
  double tfac;			
  double *sucof = SISL_NULL;		
  double smat[16];		
  int kstat;			
  double *srow;			
  double *scirc;		
  int nbvec = 1;		

  int kpos = 1;			



  

  *rs = SISL_NULL;


  

  st2      = pc->et;
  kn2      = pc->in;
  kk2      = pc->ik;
  scoef2   = pc->ecoef;
  rcoef2   = pc->rcoef;
  kdim     = pc->idim;
  ksurfdim = kdim + 1;

  

  if (kdim != 3)
    goto err104;

  

  kn = 9;
  kk = 3;
  st = newarray (kn + kk, DOUBLE);
  scoef = newarray (kn * ksurfdim, DOUBLE);
  kind = 4;
  kcopy = 2;
  st[0] = (double) 0.;
  for (ki = 1; ki < kk; ki++)
    {
      st[ki]     = (double) 0.;
      st[2 + ki] = PIHALF;
      st[4 + ki] = PI;
      st[6 + ki] = THREEPIHALF;
      st[8 + ki] = TWOPI;
    }
  st[11] = TWOPI;
  for (ki = 0; ki < 36; ki++)
    {
      switch (ki)
	{
	case 1:
	case 2:
	case 6:
	case 8:
	case 10:
	case 14:
	case 17:
	case 18:
	case 22:
	case 24:
	case 26:
	case 30:
	case 33:
	case 34:
	  scoef[ki] = (double) 0.0;
	  break;
	case 0:
	case 3:
	case 9:
	case 11:
	case 19:
	case 27:
	case 32:
	case 35:
	  scoef[ki] = (double) 1.0;
	  break;
	case 16:
	case 25:
	  scoef[ki] = (double) -1;
	  break;
	case 4:
	case 5:
	case 7:
	case 13:
	case 15:
	case 23:
	case 28:
	case 31:
	  scoef[ki] = weight;
	  break;
	case 12:
	case 20:
	case 21:
	case 29:
	  scoef[ki] = -weight;
	  break;
	}
    }

  totcurve = newCurve (kn, kk, st, scoef, kind, kdim, kcopy);
  if(totcurve == SISL_NULL) goto err101;

  

  tangle = fabs (angle);
  if (tangle > TWOPI)
    tangle = TWOPI;
  quadrant = (int) floor (tangle / PIHALF);
  resang = tangle - quadrant * PIHALF;
  spar = (double) 0.;
  epar = PIHALF * (quadrant + ((double) 1. + (sqrt ((double) 2.) + (double) 1.) * tan ((resang - PI / (double) 4.) / (double) 2.)) / (double) 2.);
  s1713 (totcurve, spar, epar, &pnorm, &kstat);
  if (kstat < 0)
    goto error;

  

  st1    = pnorm->et;
  scoef1 = pnorm->ecoef;
  rcoef1 = pnorm->rcoef;
  kn1    = pnorm->in;
  kk1    = pnorm->ik;

  if (angle < DZERO)
    for (ki = 0; ki < kn1; ki++)
      scoef1[kdim * ki + 1] = -scoef1[kdim * ki + 1];

  

  sucof = newarray (kn1 * kn2 * ksurfdim, DOUBLE);
  if (sucof == SISL_NULL)
    goto err101;

  

  for (ki = 0; ki < kn2; ki++)
    {

      

      if (pc->ikind == 2 || pc->ikind == 4)
	tfac = rcoef2[(ki + 1) * ksurfdim - 1];
      else
	tfac = (double) 1.;

      

      s6rotax (ep, eaxis, &scoef2[ki * kdim], smat, &kstat);
      if (kstat < 0)
	goto error;

      

      for (kj = 0; kj < kn1; kj++)
	{
	  srow  = sucof + ki * (kn1 * ksurfdim) + kj * ksurfdim;
	  scirc = scoef1 + kj * kdim;
	  s6mvec (smat, scirc, nbvec, srow);
	  weight = rcoef1[(kj + 1) * ksurfdim - 1] * tfac;
	  for (kl = 0; kl < kdim; kl++)
	    srow[kl] *= weight;
	  srow[kdim] = weight;
	}
    }

  

  *rs = newSurf (kn1, kn2, kk1, kk2, st1, st2, sucof, 2, kdim, 1);
  if ( *rs == SISL_NULL )  goto err101;

  if ( tangle >= TWOPI )
  {
    

    (*rs)->cuopen_1 = SISL_SURF_CLOSED;
  }

  *jstat = 0;
  goto out;

  

  err101:*jstat = -101;
    s6err ("s1520", *jstat, kpos);
    goto out;

  

  err104:*jstat = -104;
    s6err ("s1520", *jstat, kpos);
    goto out;

  

  error:*jstat = kstat;
    s6err ("s1520", *jstat, kpos);
    goto out;

  out:
    

    if (sucof != SISL_NULL) freearray (sucof);
    if (totcurve != SISL_NULL) freeCurve (totcurve);
    if (pnorm != SISL_NULL) freeCurve (pnorm);
    return;
}

