/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1904

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1904 (double epar[], int in, int ik, int cuopen, double *eknots[], int *jstat)

#else
void
s1904 (epar, in, ik, cuopen, eknots, jstat)
     double epar[];
     int in;
     int ik;
     int cuopen;
     double *eknots[];
     int *jstat;
#endif
{
  int ki;			
  int kpar1, kpar2;		
  int k1, k2;			
  int kstop;			
  int kmult;			
  double tdegree;		
  double tprev;			
  double curr;			
  double tval1;			
  double tval2;			
  double tparint;		
  int kpos = 0;

  *jstat = 0;


  if (cuopen)
    {
      

      *eknots = newarray (in +ik, DOUBLE);
      if (*eknots == SISL_NULL)
	goto err101;

      tdegree = (double) (ik - 1);
      tval1 = epar[0];
      tval2 = epar[in -1];

      

      for (ki = 0; ki < ik; ki++)
	(*eknots)[ki] = tval1;

      if (in -3 * ik + 4 < 0)
	{
	  

	  for (; ki < in; ki++)
	    (*eknots)[ki] = (*eknots)[ki - 1]
	      + ((double) (2 * ik - ki - 1) * (epar[ki - ik + 1] - epar[ki - ik])
		 + (double) (ki - in +ik - 1) *(epar[ki - 1] - epar[ki - 2])
		 + epar[ki - 2] - epar[ki - ik + 1]) / tdegree;
	}
      else
	{

	  

	  for (; ki < 2 * ik - 2; ki++)
	    (*eknots)[ki] = (*eknots)[ki - 1]
	      + ((double) (2 * ik - ki - 1) * (epar[ki - ik + 1] - epar[ki - ik])
		 + epar[ki - 1] - epar[ki - ik + 1]) / tdegree;

	  

	  for (; ki < in -ik + 2; ki++)
	    (*eknots)[ki] = (*eknots)[ki - 1]
	      + (epar[ki - 1] - epar[ki - ik]) / tdegree;

	  

	  for (; ki < in; ki++)
	    (*eknots)[ki] = (*eknots)[ki - 1]
	      + ((double) (ki - in +ik - 1) *(epar[ki - 1] - epar[ki - 2])
		 + epar[ki - 2] - epar[ki - ik]) / tdegree;
	}

      

      for (ki = 0; ki < ik; ki++)
	(*eknots)[in +ki] = tval2;
    }
  else
    {
      

      *eknots = newarray (in +2 * ik, DOUBLE);
      if (*eknots == SISL_NULL)
	goto err101;

      kstop = in +2 * ik - 1;
      tparint = epar[in] -epar[0];
      tdegree = (double) (ik - 1);

      

      (*eknots)[ik - 1] = (ik % 2 == 0) ? epar[0] : (double) 0.5 *(epar[0] + epar[1]);

      

      for (ki = ik, kpar1 = ki - 1, kpar2 = ki - ik, k1 = k2 = 0;
	   ki < kstop; ki++, kpar1++, kpar2++)
	{
	  

	  while (kpar1 < 0)
	    kpar1 += in, k1--;
	  while (kpar1 > in)
	    kpar1 -= in, k1++;
	  while (kpar2 < 0)
	    kpar2 += in, k2++;
	  while (kpar2 > in)
	    kpar2 -= in, k2--;

	  

	  (*eknots)[ki] = (*eknots)[ki - 1]
	    + (epar[kpar1] - epar[kpar2] + (double) (k1 + k2) * tparint) / tdegree;
	}

      

      for (ki = ik - 1, kpar1 = ki - 1, kpar2 = ki - ik, k1 = k2 = 0;
	   ki > 0; ki--, kpar1--, kpar2--)
	{
	  

	  while (kpar1 < 0)
	    kpar1 += in, k1--;
	  while (kpar1 > in)
	    kpar1 -= in, k1++;
	  while (kpar2 < 0)
	    kpar2 += in, k2++;
	  while (kpar2 > in)
	    kpar2 -= in, k2--;

	  

	  (*eknots)[ki - 1] = (*eknots)[ki]
	    - (epar[kpar1] - epar[kpar2] + (double) (k1 + k2) * tparint) / tdegree;
	}
    }


  

  if (cuopen)
    kstop = in +ik;

  for (ki = 1, tprev = (*eknots)[0], kmult = 0; ki < kstop; ki++, tprev = curr)
    {
       curr = (*eknots)[ki];
      kmult++;
      if (tprev > curr)
	goto err112;		
      if (tprev < curr)
	kmult = 1;
      if (kmult > ik)
	goto err112;		
    }

  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1904", *jstat, kpos);
  goto out;

  

err112:
  *jstat = -112;
  s6err ("s1904", *jstat, kpos);
  goto out;

out:
  return;
}

