/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1907

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1907(double *epoint, int *ntype, double *epar, int iopen, int icnsta,
	   int icnend, int inbpnt, int idim, double *opoint[], int *otype[],
	   double *opar[], int *knbpnt, int *jstat)
#else
void s1907(epoint, ntype, epar, iopen, icnsta, icnend, inbpnt, idim,
	   opoint, otype, opar, knbpnt, jstat)
     double *epoint;
     int *ntype;
     double *epar;
     int iopen;
     int icnsta;
     int icnend;
     int inbpnt;
     int idim;
     double *opoint[];
     int *otype[];
     double *opar[];
     int *knbpnt;
     int *jstat;
#endif
{
  int kpos = 0;
  int count1, count2;		
  int dummy;
  int start, start2;
  int cpv;			

  *jstat = 0;


  

  if (icnsta && icnend)
    *knbpnt = inbpnt + 2;
  else if (icnsta || icnend)
    *knbpnt = inbpnt + 1;
  else
    *knbpnt = inbpnt;

  *opoint = newarray ((*knbpnt) * idim, DOUBLE);
  if (*opoint == SISL_NULL)
    goto err101;
  *otype = newarray (*knbpnt, INT);
  if (*otype == SISL_NULL)
    goto err101;

  if (iopen == SISL_CRV_OPEN)
    *opar = newarray (*knbpnt, DOUBLE);
  else
    *opar = newarray (*knbpnt + 1, DOUBLE);
  if (*opar == SISL_NULL)
    goto err101;


  

  if (icnsta != 0)
    {
      for (count1 = 0; count1 < idim; count1++)
	(*opoint)[count1] = (double) 0.0;

      (*otype)[0] = -2;
      (*opar)[0] = epar[0];
    }

  if (icnend != 0)
    {
      dummy = (*knbpnt) * idim;
      for (count1 = ((*knbpnt) - 1) * idim; count1 < dummy; count1++)
	(*opoint)[count1] = (double) 0.0;

      (*otype)[(*knbpnt) - 1] = 2;
    }

  

  if (icnsta != 0)
    start = 1;
  else
    start = 0;

  cpv = -1;

  for (count1 = 0; count1 < inbpnt; count1++)
    {
      

      if (ntype[count1] == 13)
	{
	  start2 = (count1 + start) * idim;
	  for (count2 = 0; count2 < idim; count2++)
	    {
	      (*opoint)[start2 + count2] = epoint[(count1 + 1) * idim + count2] -
		epoint[count1 * idim + count2];
	    }
	}
      else if (ntype[count1] == 14)
	{
	  start2 = (count1 + start) * idim;
	  for (count2 = 0; count2 < idim; count2++)
	    {
	      (*opoint)[start2 + count2] = epoint[count1 * idim + count2] -
		epoint[(count1 - 1) * idim + count2];
	    }
	}
      else
	{
	  start2 = (count1 + start) * idim;
	  for (count2 = 0; count2 < idim; count2++)
	    {
	      (*opoint)[start2 + count2] = epoint[count1 * idim + count2];
	    }
	}

      

      if (ntype[count1] == 1 || ntype[count1] == 2)
	{
	  (*otype)[count1 + start] = 0;
	  cpv++;
	  (*opar)[count1 + start] = epar[cpv];
	}
      else if (ntype[count1] == 3)
	{
	  (*otype)[count1 + start] = -1;
	  (*opar)[count1 + start] = epar[cpv + 1];
	}
      else if (ntype[count1] == 4)
	{
	  (*otype)[count1 + start] = 1;
	  (*opar)[count1 + start] = epar[cpv];
	}
      else if (ntype[count1] == 5)
	{
	  (*otype)[count1 + start] = -2;
	  (*opar)[count1 + start] = epar[cpv + 1];
	}
      else if (ntype[count1] == 6)
	{
	  (*otype)[count1 + start] = 2;
	  (*opar)[count1 + start] = epar[cpv];
	}
      else if (ntype[count1] == 13)
	{
	  (*otype)[count1 + start] = -1;
	  (*opar)[count1 + start] = epar[cpv + 1];
	}
      else if (ntype[count1] == 14)
	{
	  (*otype)[count1 + start] = 1;
	  (*opar)[count1 + start] = epar[cpv];
	}
    }

  if (icnend != 0)
    (*opar)[(*knbpnt) - 1] = epar[cpv];

  if (!(iopen == SISL_CRV_OPEN))
    {
     

     cpv++;
     (*opar)[*knbpnt] = epar[cpv];     
   }

  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1907", *jstat, kpos);
  goto out;

out:
  return;
}

