/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1910

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1910 (double econd[], int ntype[], int inpt, int idim, int iopen,
       double astpar, double *cendpar, double *epar1[],
       double *epar2[], int *jstat)
#else
void
s1910 (econd, ntype, inpt, idim, iopen, astpar, cendpar, epar1, epar2, jstat)
     double econd[];
     int ntype[];
     int inpt;
     int idim;
     int iopen;
     double astpar;
     double *cendpar;
     double *epar1[];
     double *epar2[];
     int *jstat;
#endif
{
  int count1, count2;	
  int prev;
  int knpt;			
  double sum, cord;
  int kpos = 0;

  *jstat = 0;

  if (iopen != SISL_CRV_OPEN)
    knpt = inpt + 1;
  else
    knpt = inpt;


  

  *epar1 = newarray (knpt, DOUBLE);
  if (*epar1 == SISL_NULL)
    goto err101;

  *epar2 = newarray (knpt, DOUBLE);
  if (*epar2 == SISL_NULL)
    goto err101;


  

  sum = (double) 0.0;
  prev = -1;
  for (count1 = 0, count2 = 0; count1 < inpt; count1++)
    {
      if (ntype[count1] == 0)
	{
	  if (prev >= 0)
	    sum += s6dist (&econd[count1 * idim], &econd[prev * idim],
			   idim);
	  prev = count1;
	  count2++;
	}
    }
  cord = sum / (double) (count2 -  1.0);


  for (count1 = 0; count1 < inpt; count1++)
    {
      if (ntype[count1] > 0)
	(*epar1)[count1] = astpar - cord;
      else if (ntype[count1] < 0)
	(*epar1)[count1] = astpar;
      else
	{
	  (*epar1)[count1] = astpar;
	  astpar += cord;
	}
    }

  if (iopen != SISL_CRV_OPEN)
    (*epar1)[inpt] = astpar;

  *cendpar = (*epar1)[knpt - 1];


  

  count2 = 1;
  (*epar2)[0] = (*epar1)[0];

  for (count1 = 1; count1 < knpt; count1++)
    {
      if ((*epar1)[count1 - 1] < (*epar1)[count1])
	{
	  (*epar2)[count2] = (*epar1)[count1];
	  count2++;
	}
    }

  *epar2 = increasearray (*epar2, count2, DOUBLE);
  if (*epar2 == SISL_NULL)
    goto err101;


  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1910", *jstat, kpos);
  goto out;

out:

  return;
}

