/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1932

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void
s1932 (int inbcrv, SISLCurve ** crvarr, double start, double stop, double *et,
       int in, int iordr, double **iright, int *jstat)
#else
void
s1932 (inbcrv, crvarr, start, stop, et, in, iordr, iright, jstat)
     int inbcrv;
     SISLCurve **crvarr;
     double start;
     double stop;
     double *et;
     int in;
     int iordr;
     double **iright;
     int *jstat;

#endif
{
  int ki, kj, kp;		
  int kpos = 0;			
  int kordr;			
  int idim;			
  int kstat;			
  SISLCurve *crv = SISL_NULL;	
  double *kdumcf = SISL_NULL;	


  *jstat = 0;

  

  idim = crvarr[0]->idim;
  kordr = 0;

  for (ki = 0; ki < inbcrv; ki++)
    if ((crvarr[ki]->in <crvarr[ki]->ik) ||(crvarr[ki]->ik < 1))
      goto err112;


  

  for (ki = 0; ki < inbcrv; ki++)
    kordr = MAX (kordr, crvarr[ki]->ik);

  if (kordr > iordr)
    goto err151;


  

  

  kdumcf = newarray (idim * in, DOUBLE);
  if (kdumcf == SISL_NULL)
    goto err101;
  *iright = newarray (in *idim * inbcrv, DOUBLE);
  if (*iright == SISL_NULL)
    goto err101;

  kp = 0;

  for (ki = 0; ki < inbcrv; ki++)
    {
      

      s1750 (crvarr[ki], iordr, &crv, &kstat);
      if (kstat < 0)
	goto error;


      

      s1934 (crv->et, crv->in, crv->ik, start, stop, &kstat);
      if (kstat < 0)
	goto error;


      

      s1936 (crv, et, in, kdumcf, &kstat);
      if (kstat < 0)
	goto error;

      if (crv != SISL_NULL)
	freeCurve (crv);
      crv = SISL_NULL;	


      

      for (kj = 0; kj < (in *idim); kj++)
	(*iright)[kj + kp] = kdumcf[kj];
      kp += in *idim;
    }
  goto out;

  

  err101:
    *jstat = -101;
    s6err ("s1932", *jstat, kpos);
    goto out;

   
  
  err112:
     *jstat = -112;
     s6err("s1932",*jstat,kpos);
     goto out;
 
  

  err151:  
    *jstat = -151;
    s6err ("s1932", *jstat, kpos);
    goto out;

  

  error:
    *jstat = kstat;
    s6err ("s1932", *jstat, kpos);
    goto out;

  out:
   
   
   if (kdumcf != SISL_NULL) freearray(kdumcf);

   return;
}

