/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1933

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void
s1933 (int inbcrv, SISLCurve * crvarr[], double start, double stop,
       double **it, int *in, int *iordr, int *jstat)
#else
void
s1933 (inbcrv, crvarr, start, stop, it, in, iordr, jstat)
     int inbcrv;
     SISLCurve *crvarr[];
     double start;
     double stop;
     double **it;
     int *in;
     int *iordr;
     int *jstat;

#endif
{
  int ki, kj, kl, kr, kp;	
  int ktell;
  int kfirst;			
  int klast;			
  int kn;			
  int kbgn;			
  int kend;			
  int kmax;			
  int kdum1;			
  int kdum2;
  int kdiff;
  int kdf;			
  int knumb;			
  int kpos = 0;			
  int kstat;
  double tepsco;		
  double dum;
  double tlast;			
  double tval;
  double tincre;
  double tincr2;		
  double tmin, tmax;
  double tinmin, tinmax;
  double tbgn, tend;		
  double *kt = SISL_NULL;		
  double *knot = SISL_NULL;		
  double *incknt = SISL_NULL;	
  SISLCurve *curve = SISL_NULL;

  *jstat = 0;


  

  *in = 0;
  *iordr = 0;


  

  if (inbcrv < 2)
    goto err179;
  for (ki = 0; ki < inbcrv; ki++)
    if (crvarr[ki]->in <crvarr[ki]->ik || crvarr[ki]->ik < 1)
      goto err112;


  

  for (ki = 0; ki < inbcrv; ki++)
    *iordr = MAX (*iordr, crvarr[ki]->ik);

  kn = *iordr;


  

  for (ki = 0; ki < inbcrv; ki++)
    {
      curve = crvarr[ki];
      s1934 (curve->et, curve->in, curve->ik, start, stop, &kstat);
      if (kstat < 0)
	goto error;
    }

  

  kt = newarray (*iordr * 2, double);
  if (kt == SISL_NULL)
    goto err101;
  for (ki = 0; ki < *iordr; ki++)
    {
      kt[ki] = start;
      kt[*iordr + ki] = stop;
    }

  

  for (ki = 0; ki < inbcrv; ki++)
    {
      

      curve = crvarr[ki];
      s1754 (curve->et, curve->in, curve->ik, *iordr,
	     &incknt, &knumb, &kstat);
      if (kstat < 0)
	goto error;


      

      s1935 (kt, kn, incknt, knumb, &knot, &kn, *iordr, &kstat);
      if (kstat < 0)
	goto error;

      if (incknt != SISL_NULL)  freearray(incknt);  

      if (kt != SISL_NULL)
	freearray (kt);
      kt = knot;
      knot = SISL_NULL;  
    }

  

  kend = kn - 1;
  tbgn = kt[*iordr - 1];
  tend = kt[kn];

  

  for (kbgn = *iordr; (tbgn >= kt[kbgn]) && (kbgn < kend); kbgn++) ;

  

  for (; (tend <= kt[kend]) && (kbgn < kend); kend--) ;

  

  tepsco = (double) 0.00000000000001 * (stop - start);

  

  for (ki = kbgn; ki < kend; ki = kl + 1)
    {
      

      kl = ki - 1;
      do
	{
	  kl++;
	  dum = MAX (fabs(kt[kl + 1]), fabs(kt[ki]))/stop;
	  if (dum == (double) 0.0)
	    dum = (double) 1.0;
	}
      while ((kl < kend) && ((fabs (kt[kl + 1] - kt[ki]) / dum) <= tepsco));


      

      if (ki == kl)
	continue;


      

      tmin = kt[ki];
      tmax = MIN (kt[kl] + tepsco*dum, kt[kend]);


      

      if (tmin >= tmax)
	break;


      

      tinmin = tmin;
      tinmax = tmax;
      kmax = 0;
      for (kj = 0; kj < inbcrv; kj++)
	{
	  curve = crvarr[kj];
	  ktell = 0;
	  tlast = (double) 0.0;

	  for (kr = *iordr - 1; kr <= curve->in; kr++)
	    if ((tmin <= curve->et[kr]) && (curve->et[kr] <= tmax))
	      {
		if (ktell == 0)
		  ktell = 1;
		else if (curve->et[kr] > tlast)
		  ktell++;
		tlast = curve->et[kr];
		tinmin = MIN (tinmin, tlast);
		tinmax = MAX (tinmax, tlast);
	      }
	  kmax = MAX (kmax, ktell);
	}

      

      if (kmax > 1)
	tincre = (tinmax - tinmin) / (double) (kmax - 1);
      else
	tincre = (double) 0.0;
      tincr2 = tincre / (double) 2.0;

      

      if (tinmin >= tinmax)
	continue;

      for (kj = 0; kj < inbcrv; kj++)
	{
	  curve = crvarr[kj];
	  ktell = 0;
	  for (kr = *iordr - 1; kr <= curve->in; kr++)
	    if ((tinmin <= curve->et[kr]) && (curve->et[kr] <= tinmax))
	      {
		if (ktell == 0)
		  {
		    ktell = 1;
		    kfirst = kr;
		  }
		else if (curve->et[kr] >= tlast)
		  {
		    ktell++;
		    klast = kr;
		  }
		tlast = curve->et[kr];
	      }

	  

	  if (ktell == 1)
	    {

	      

	      if ((kmax == 1) || (tincre == (double) 0.0))
		tval = tinmin;
	      else
		{
		  kdum1 = (int)((curve->et[kfirst] - tinmin + tincr2) / tincre);
		  tval = tinmin + (double)kdum1 * tincre;
		}
	      tlast = curve->et[kfirst];
	      do
		{
		  curve->et[kfirst] = tval;
		  kfirst++;
	      } while (curve->et[kfirst] == tlast);
	      continue;
	    }

	  if ((ktell <= 1) || (tincre <= (double) 0.0))
	    continue;


	  

	  kdum1 = (int)((curve->et[kfirst] - tinmin + tincr2) / tincre);
	  kdum2 = (int)((curve->et[klast] - tinmin + tincr2) / tincre);
	  kdiff = ktell - kdum2 + kdum1 - 1;
	  if (kdiff > 0)
	    {
	      

	      kdf = kdiff / 2;
	      kdum1 = MAX (0, kdum1 - kdf);
	      kdum2 = MIN (kmax - 1, kdum2 + kdiff - kdf);
	      kdiff = ktell - kdum2 + kdum1 - 1;

	      if (kdiff > 0)
		{
		  if (kdum1 == 0)
		    kdum2 += kdiff;
		  else
		    {
		      if (kdum2 != ktell)
			goto err170;
		      kdum1 -= kdiff;
		    }
		}
	    }

	  if ((kdum1 < 0) || (kdum2 > ktell))
	    goto err170;


	  

	  tval = tinmin + kdum1 * tincre;
	  kr = kfirst;
	  tlast = curve->et[kr];
	  for (kp = 0; kp < ktell; kp++, kr++)
	    if (curve->et[kr] == tlast)
	      curve->et[kr] = tval;
	    else
	      {
		if (curve->et[kr] <= tlast)
		  goto err170;
		tval += tincre;
		tlast = curve->et[kr];
		curve->et[kr] = tval;
	      }
	}
    }

  

  if (kt != SISL_NULL)  freearray(kt);  

  kt = newarray (*iordr * 2, double);
  if (kt == SISL_NULL)
    goto err101;

  for (ki = 0; ki < *iordr; ki++)
    {
      kt[ki] = start;
      kt[*iordr + ki] = stop;
    }

  kn = *iordr;

  

  for (ki = 0; ki < inbcrv; ki++)
    {
      

      curve = crvarr[ki];
      s1754 (curve->et, curve->in, curve->ik, *iordr, &incknt, &knumb, &kstat);
      if (kstat < 0)
	goto error;


      

      s1935 (kt, kn, incknt, knumb, &knot, &kn, *iordr, &kstat);
      if (kstat < 0)
	goto error;

      if (incknt != SISL_NULL)  freearray(incknt);  

      if (kt != SISL_NULL)
	freearray (kt);
      kt = knot;
      knot = SISL_NULL;  
    }

  

  *in = kn;
  *it = kt;
  kt = SISL_NULL;  
  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1933", *jstat, kpos);
  goto out;

  

err112:
  *jstat = -112;
  s6err ("s1933", *jstat, kpos);
  goto out;

  

err179:
  *jstat = -179;
  s6err ("s1933", *jstat, kpos);
  goto out;

  

error:
  *jstat = kstat;
  s6err ("s1933", *jstat, kpos);
  goto out;

  

err170:
  *jstat = -170;
  s6err ("s1933", *jstat, kpos);
  goto out;

out:
  if (kt != SISL_NULL)  freearray(kt);  
  if (knot != SISL_NULL)  freearray(knot);  
  if (incknt != SISL_NULL)  freearray(incknt);  
  return;
}

