/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1934

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void
s1934 (double *et, int in, int ik, double start, double end, int *jstat)
#else
void
s1934 (et, in, ik, start, end, jstat)
     double *et;
     int in;
     int ik;
     double start;
     double end;
     int *jstat;

#endif
{
  int ii, stop;			
  int kpos = 0;			
  double store1;		
  double fac;			

  *jstat = 0;


  

  if ((ik < 1) || (in <ik))
    goto err112;

  if (start == end)
    goto err124;


  

  store1 = et[ik - 1];
  fac = (end - start) / (et[in] -store1);
  stop = in +ik;

  for (ii=0; ii<ik; ii++) et[ii] = start;
  
  for (ii = ik; ii < in; ii++)
    et[ii] = fac * (et[ii] - store1) + start;

  for (ii = in; ii < stop; ii++) et[ii] = end;


  

  goto out;


  

err112:
  *jstat = -112;
  s6err ("s1934", *jstat, kpos);
  goto out;

  

err124:
  *jstat = -124;
  s6err ("s1934", *jstat, kpos);
  goto out;

out:
  return;
}

