/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1936

#include "sislP.h"
#define MAX_SIZE  50



#if defined(SISLNEEDPROTOTYPES)
void
s1936 (SISLCurve * crv, double etd[], int ind, double *curvd, int *jstat)
#else
void
s1936 (crv, etd, ind, curvd, jstat)
     SISLCurve *crv;
     double etd[];
     int ind;
     double *curvd;
     int *jstat;

#endif
{
  int kpos = 0;			
  int ki, kj, kr, low;		
  int nu;			
  int ins;			
  int iordr;			
  int idim;			
  double *knt = SISL_NULL;		
  double *coef = SISL_NULL;		
  double sum;			
  double sarray[MAX_SIZE];
  int alloc_needed=FALSE;
  double *alfa = SISL_NULL;		

  *jstat = 0;


  

  knt = crv->et;
  ins = crv->in;
  iordr = crv->ik;
  idim = crv->idim;
  coef = crv->ecoef;


  

  if (iordr < 1)
    goto err110;
  if (ins < iordr || ind < iordr)
    goto err111;
  if (idim < 1)
    goto err102;


  

  if (iordr > MAX_SIZE)
    {
       if ((alfa = newarray (iordr, DOUBLE)) == SISL_NULL)
	 goto err101;
       alloc_needed = TRUE;
    }
  else
    alfa = sarray;

  

  kj = 0;

  for (ki = 0; kj < ins; ki++)
    {
      if (ki >= ind)
	goto err111;
      if ( DEQUAL(knt[kj], etd[ki]) )  
      {
	kj++;
	continue;
      }
      if (knt[kj] > etd[ki])
	continue;
      if (knt[kj] < etd[ki])
	goto err112;
    }

  

  nu = 1;
  for (kj = 0; kj < ind; kj++)
    {

      

       
       for (; (((knt[nu - 1] > (double)0.5*(etd[kj]+etd[kj+1])) 
		 || ((double)0.5*(etd[kj]+etd[kj+1]) >= knt[nu])) && (nu != ins)); nu++) ; 

      

      s1937 (knt, iordr, kj + 1, nu, alfa, etd);


      

      low = nu - iordr;
      for (ki = 0; ki < idim; ki++)
	{
	  sum = (double) 0.0;
	  for (kr = MAX (0, low); kr < nu; kr++)
	    sum += alfa[kr - low] * coef[kr * idim + ki];
	  curvd[kj * idim + ki] = sum;
	}
    }

  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1936", *jstat, kpos);
  goto out;

  

err102:
  *jstat = -102;
  s6err ("s1936", *jstat, kpos);
  goto out;

  

err110:
  *jstat = -110;
  s6err ("s1936", *jstat, kpos);
  goto out;

  

err111:
  *jstat = -111;
  s6err ("s1936", *jstat, kpos);
  goto out;

  

err112:
  *jstat = -112;
  s6err ("s1936", *jstat, kpos);

out:
  if (alloc_needed)
    freearray (alfa);

  return;
}
#undef MAX_SIZE

