/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1959

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1959(SISLPoint *ppoint, SISLCurve *pcurve, double *gpos, int *jstat)
#else
void s1959(ppoint,pcurve,gpos,jstat)
     SISLPoint        *ppoint;
     SISLCurve         *pcurve;
     double       *gpos;
     int          *jstat;
#endif
{
  int kstat = 0;            
  int kpos = 0;             
  int i;                    
  int iind;                 
  int kk;                
  int nk;                
  int dim;                  
  double *et;              
  double *lpoint;           
  double *scoef;            
  double tdist;             
  double tmin;              
  double vec1[3],vec2[3];   
  double vecp[3];           
  double lqua1=DZERO;
  double lqua2=DZERO;
                            
  double lprj1=DZERO;
  double lprj2=DZERO;
                            
  double svals,svale;       
 

  
  if (ppoint->idim != pcurve->idim || pcurve->idim <= 1) goto err106;

  
  kk  = pcurve->ik;
  nk  = pcurve->in;
  et = pcurve->et;
  scoef = pcurve->ecoef;
  dim = pcurve->idim;
  lpoint = ppoint->ecoef;

  
  tdist=s6dist(scoef,lpoint,dim);
  tmin=tdist;
  iind = 0;
  for (i=0; i<nk; i++)
  {
        tdist=s6dist(scoef,lpoint,dim);
        if (tdist<tmin)
        {
           tmin=tdist;
           iind = i;
        }
        scoef+=dim;  
  }

  
  scoef = pcurve->ecoef;

  if (iind < (nk-1))
     s6diff(&(scoef[(iind+1)*dim]),
            &(scoef[iind*dim]),dim,vec1);
  if (iind > 0)
     s6diff(&(scoef[(iind-1)*dim]),
            &(scoef[iind*dim]),dim,vec2);

  
  s6diff(lpoint,&(scoef[iind*dim]),dim,vecp);

  
  if (iind < (nk-1)) lqua1 = s6length(vec1,dim,&kstat);
  if (iind > 0) lqua2 = s6length(vec2,dim,&kstat);

  
  if (iind < (nk-1)) lprj1 = s6lprj(vecp,vec1,dim);
  if (iind > 0) lprj2 = s6lprj(vecp,vec2,dim);

  
  if (iind == 0)
  {
     

     
     svals = s6schoen(et,kk,iind);
     svale = s6schoen(et,kk,iind+1);

     
     if (lqua1 != DZERO) (*gpos) = svals + (lprj1/lqua1)*(svale-svals);
     else (*gpos) = svals;
  }
  else if (iind == (nk-1))
  {
     

     
     svals = s6schoen(et,kk,iind-1);
     svale = s6schoen(et,kk,iind);

     
     if (lqua2 != DZERO) (*gpos) = svals + ((lqua2-lprj2)/lqua2)*(svale-svals);
     else (*gpos) = svals;
  }
  else if (iind > 0 && iind < (nk-1))
  {
     

      if (lprj1 > lprj2)
      {
           

           
           svals = s6schoen(et,kk,iind);
           svale = s6schoen(et,kk,iind+1);

           
           if (lqua1 != DZERO) (*gpos) = svals + (lprj1/lqua1)*(svale-svals);
           else (*gpos) = svals;
      }
      else if (lprj2 > lprj1)
      {
           

           
           svals = s6schoen(et,kk,iind-1);
           svale = s6schoen(et,kk,iind);

           
           if (lqua2 != DZERO) (*gpos) = svals + ((lqua2-lprj2)/lqua2)*(svale-svals);
           else (*gpos) = svals;
      }
      else
      {
           
           

           goto usvert;
      }
  }
  else
  {
     
     goto usvert;
  }

  
  if ((*gpos)<et[kk-1]) (*gpos)=et[kk-1];
  else if ((*gpos)>et[nk]) (*gpos)=et[nk];
  *jstat = 0;

  
  goto out;

  
  usvert: *jstat = 1;

  
  (*gpos) = s6schoen(et,kk,iind);

  
  if ((*gpos)<et[kk-1]) (*gpos)=et[kk-1];
  else if ((*gpos)>et[nk]) (*gpos)=et[nk];

  goto out;

 
  
 err106: *jstat = -106;
  s6err("s1959",*jstat,kpos);
  goto out;

 out:
    return;
}

