/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1960

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1960(SISLPoint *ppoint, SISLSurf *psurf, double gpos[], int *jstat)
#else
void s1960(ppoint,psurf,gpos,jstat)
     SISLPoint        *ppoint;
     SISLSurf         *psurf;
     double       gpos[];
     int          *jstat;
#endif
                       
{  
  int kstat = 0;            
  int kpos = 0;             
  int i,j;                  
  int iind,jind;            
  int k1;                   
  int k2;                   
  int n1;                   
  int n2;                   
  int dim;                  
  double *et1;              
  double *et2;              
  double *lpoint;           
  double *scoef;            
  double tdist;             
  double tmin;              
  double vec1[3],vec2[3],vec3[3],vec4[3];
                            
  double vecp[3];           
  double lqua1=DZERO;
  double lqua2=DZERO;
  double lqua3=DZERO;
  double lqua4=DZERO;
                            
  double lprj1=DZERO;
  double lprj2=DZERO;
  double lprj3=DZERO;
  double lprj4=DZERO;
                            
  double svals,svale,tvals,tvale;
                            
  
  
  
  if (ppoint->idim != psurf->idim || psurf->idim <= 1) goto err106;

  
  k1  = psurf->ik1;
  k2  = psurf->ik2;
  n1  = psurf->in1;
  n2  = psurf->in2;
  et1 = psurf->et1;
  et2 = psurf->et2;
  scoef = psurf->ecoef;
  dim = psurf->idim;
  lpoint = ppoint->ecoef;
   
  
  tdist=s6dist(scoef,lpoint,dim);
  tmin=tdist;
  iind = 0;
  jind = 0;
  for (i=0; i<n1; i++)
  {
     for (j=0; j<n2; j++)
     {
        tdist=s6dist(scoef,lpoint,dim);
        if (tdist<tmin)
        {
           tmin=tdist;
           iind = i;
           jind = j;
        }
        scoef+=3;
     }
  }
  
  
  scoef = psurf->ecoef;
  
  if (iind < (n1-1))
     s6diff(&(scoef[(iind+1)*dim+jind*dim*n1]),
            &(scoef[iind*dim+jind*dim*n1]),dim,vec1);
  if (jind < (n2-1))
     s6diff(&(scoef[iind*dim+(jind+1)*dim*n1]),
            &(scoef[iind*dim+jind*dim*n1]),dim,vec2);
  if (iind > 0)
     s6diff(&(scoef[(iind-1)*dim+jind*dim*n1]),
            &(scoef[iind*dim+jind*dim*n1]),dim,vec3);
  if (jind > 0)
     s6diff(&(scoef[iind*dim+(jind-1)*dim*n1]),
            &(scoef[iind*dim+jind*dim*n1]),dim,vec4);
  
  
  s6diff(lpoint,&(scoef[iind*dim+jind*dim*n1]),dim,vecp);
     
  
  if (iind < (n1-1)) lqua1 = s6length(vec1,dim,&kstat);
  if (jind < (n2-1)) lqua2 = s6length(vec2,dim,&kstat);
  if (iind > 0) lqua3 = s6length(vec3,dim,&kstat);
  if (jind > 0) lqua4 = s6length(vec4,dim,&kstat);

  
  if (iind < (n1-1)) lprj1 = s6lprj(vecp,vec1,dim);
  if (jind < (n2-1)) lprj2 = s6lprj(vecp,vec2,dim);
  if (iind > 0) lprj3 = s6lprj(vecp,vec3,dim);
  if (jind > 0) lprj4 = s6lprj(vecp,vec4,dim);

  
  if (iind == 0 && jind == 0)
  {
     
     
     
     svals = s6schoen(et1,k1,iind);
     svale = s6schoen(et1,k1,iind+1);
     tvals = s6schoen(et2,k2,jind);
     tvale = s6schoen(et2,k2,jind+1);
     
     
     if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
     else gpos[0] = svals;
     if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
     else gpos[1] = tvals;
  }
  else if (iind == (n1-1) && jind == 0)
  {
     
     
     
     svals = s6schoen(et1,k1,iind-1);
     svale = s6schoen(et1,k1,iind);
     tvals = s6schoen(et2,k2,jind);
     tvale = s6schoen(et2,k2,jind+1);
     
     
     if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
     else gpos[0] = svals;
     if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
     else gpos[1] = tvals;
  }
  else if (iind == (n1-1) && jind == (n2-1))
  {
     
     
     
     svals = s6schoen(et1,k1,iind-1);
     svale = s6schoen(et1,k1,iind);
     tvals = s6schoen(et2,k2,jind-1);
     tvale = s6schoen(et2,k2,jind);
     
     
     if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
     else gpos[0] = svals;
     if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
     else gpos[1] = tvals;
  }
  else if (iind == 0 && jind == (n2-1))
  {
     
     
     
     svals = s6schoen(et1,k1,iind);
     svale = s6schoen(et1,k1,iind+1);
     tvals = s6schoen(et2,k2,jind-1);
     tvale = s6schoen(et2,k2,jind);
     
     
     if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
     else gpos[0] = svals;
     if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
     else gpos[1] = tvals;
  }
  else if (iind > 0 && iind < (n1-1) && jind == 0)
  {
     
     
     if (lprj1 > lprj3)
        {
           
           
           
           svals = s6schoen(et1,k1,iind);
           svale = s6schoen(et1,k1,iind+1);
           tvals = s6schoen(et2,k2,jind);
           tvale = s6schoen(et2,k2,jind+1);
           
           
           if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
           else gpos[0] = svals;
           if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else if (lprj3 > lprj1)
        {
           
           
           
           svals = s6schoen(et1,k1,iind-1);
           svale = s6schoen(et1,k1,iind);
           tvals = s6schoen(et2,k2,jind);
           tvale = s6schoen(et2,k2,jind+1);
           
           
           if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
           else gpos[0] = svals;
           if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else
        {
           
           
           goto usvert;
        }
  }
  else if (iind == (n1-1) && jind > 0 && jind < (n2-1))
  {
     
     
     if (lprj2 > lprj4)
        {
           
           
           
           svals = s6schoen(et1,k1,iind-1);
           svale = s6schoen(et1,k1,iind);
           tvals = s6schoen(et2,k2,jind);
           tvale = s6schoen(et2,k2,jind+1);
           
           
           if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
           else gpos[0] = svals;
           if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else if (lprj4 > lprj2)
        {
           
           
           
           svals = s6schoen(et1,k1,iind-1);
           svale = s6schoen(et1,k1,iind);
           tvals = s6schoen(et2,k2,jind-1);
           tvale = s6schoen(et2,k2,jind);
           
           
           if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
           else gpos[0] = svals;
           if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else
        {
           
           
           goto usvert;
        }
  }
  else if (iind > 0 && iind < (n1-1) && jind == (n2-1))
  {
     
     
     if (lprj3 > lprj1)
        {
           
           
           
           svals = s6schoen(et1,k1,iind-1);
           svale = s6schoen(et1,k1,iind);
           tvals = s6schoen(et2,k2,jind-1);
           tvale = s6schoen(et2,k2,jind);
           
           
           if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
           else gpos[0] = svals;
           if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else if (lprj1 > lprj3)
        {
           
           
           
           svals = s6schoen(et1,k1,iind);
           svale = s6schoen(et1,k1,iind+1);
           tvals = s6schoen(et2,k2,jind-1);
           tvale = s6schoen(et2,k2,jind);
           
           
           if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
           else gpos[0] = svals;
           if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else
        {
           
           
           goto usvert;
        }
  }
  else if (iind == 0 && jind > 0 && jind < (n2-1))
  {
     
     
     if (lprj2 > lprj4)
        {
           
           
           
           svals = s6schoen(et1,k1,iind);
           svale = s6schoen(et1,k1,iind+1);
           tvals = s6schoen(et2,k2,jind);
           tvale = s6schoen(et2,k2,jind+1);
           
           
           if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
           else gpos[0] = svals;
           if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else if (lprj4 > lprj2)
        {
           
           
           
           svals = s6schoen(et1,k1,iind);
           svale = s6schoen(et1,k1,iind+1);
           tvals = s6schoen(et2,k2,jind-1);
           tvale = s6schoen(et2,k2,jind);
           
           
           if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
           else gpos[0] = svals;
           if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else
        {
           
           
           goto usvert;
        }
  }
  else if (iind > 0 && iind < (n1-1) && jind > 0 && jind < (n2-1))
  {
     
     
     if (lprj1 > lprj3)
        {
           
           
           
           svals = s6schoen(et1,k1,iind);
           svale = s6schoen(et1,k1,iind+1);
           
           
           if (lqua1 != DZERO) gpos[0] = svals + (lprj1/lqua1)*(svale-svals);
           else gpos[0] = svals;
        }
        else if (lprj3 > lprj1)
        {
           
           
           
           svals = s6schoen(et1,k1,iind-1);
           svale = s6schoen(et1,k1,iind);
           
           
           if (lqua3 != DZERO) gpos[0] = svals + ((lqua3-lprj3)/lqua3)*(svale-svals);
           else gpos[0] = svals;
        }
        else
        {
           
           
           gpos[0] = s6schoen(et1,k1,iind);
        }


        if (lprj2 > lprj4)
        {
           
           
           
           tvals = s6schoen(et2,k2,jind);
           tvale = s6schoen(et2,k2,jind+1);
           
           if (lqua2 != DZERO) gpos[1] = tvals + (lprj2/lqua2)*(tvale-tvals);
           else gpos[1] = tvals;
        }


        else if (lprj4 > lprj2)
        {
           
           
           
           tvals = s6schoen(et2,k2,jind-1);
           tvale = s6schoen(et2,k2,jind);
           
           if (lqua4 != DZERO) gpos[1] = tvals + ((lqua4-lprj4)/lqua4)*(tvale-tvals);
           else gpos[1] = tvals;
        }
        else
        {
           
           
           gpos[1] = s6schoen(et2,k2,jind);
        }
  }
  else
  {
     
     goto usvert;
  }
  
  
  if (gpos[0]<et1[k1-1]) gpos[0]=et1[k1-1];
  else if (gpos[0]>et1[n1]) gpos[0]=et1[n1];
  if (gpos[1]<et2[k2-1]) gpos[1]=et2[k2-1];
  else if (gpos[1]>et2[n2]) gpos[1]=et2[n2];
  *jstat = 0;
  
  
  goto out;
  
  
  usvert: *jstat = 1;
           
  
  gpos[0] = s6schoen(et1,k1,iind);
  gpos[1] = s6schoen(et2,k2,jind);

  
  if (gpos[0]<et1[k1-1]) gpos[0]=et1[k1-1];
  else if (gpos[0]>et1[n1]) gpos[0]=et1[n1];
  if (gpos[1]<et2[k2-1]) gpos[1]=et2[k2-1];
  else if (gpos[1]>et2[n2]) gpos[1]=et2[n2];
  goto out;                  
  
  
  
 err106: *jstat = -106;
  s6err("s1960",*jstat,kpos);
  goto out;   
  
 out:
    return;
}


