/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2507

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s2507(SISLSurf *surf, int ider, double derive[], double normal[],
      double *totalCurvature, int *jstat)
#else
 void s2507(surf, ider, derive, normal, totalCurvature, jstat)
      SISLSurf *surf;
      int ider;
      double derive[];
      double normal[];
      double *totalCurvature;
      int *jstat;
#endif
{
   double fundform[6]; 
   double gc;          
   double mc;          
   double k1,k2;       



   if (surf->idim == 1 || surf->idim == 3) 
   {
      s2513(surf, ider, 2, 0, derive, normal, fundform, jstat);
      if (*jstat < 0) goto error;
      
      gc = (fundform[3]*fundform[5]-fundform[4]*fundform[4])
	 /((fundform[0]*fundform[2] - fundform[1]*fundform[1])*
	   (fundform[0]*fundform[2] - fundform[1]*fundform[1]));
      mc = 0.5*(fundform[3]*fundform[2] - 2*fundform[4]*fundform[1] 
			    + fundform[5]*fundform[0])
	 /((fundform[0]*fundform[2] - fundform[1]*fundform[1])
	   *sqrt(fundform[0]*fundform[2] - fundform[1]*fundform[1]));

     k1 = mc + sqrt(fabs(mc*mc - gc));
     k2 = mc - sqrt(fabs(mc*mc - gc));

     *totalCurvature = k1*k1 + k2*k2;
   }
  else if (surf->idim == 2) 
  {
    

    *totalCurvature = 0.0;
  }
  else 
  {
    goto err105;
  }

  

  *jstat = 0;
  goto out;


   
err105:
  *jstat = -105;
  s6err("s2507", *jstat, 0);
  goto out;
  
error:
  s6err("s2507",*jstat,0);
  goto out;

out:

  return;

}

