/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2513

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
   s2513(SISLSurf *surf, int ider, int type, int normalized, double derive[], 
	 double normal[], double fundform[], int *stat)
#else
   void s2513(surf, ider, type, normalized, derive, normal, fundform, stat)
      SISLSurf *surf;
      int ider;
      int type;
      int normalized;
      double derive[];
      double normal[];
      double fundform[];
      int *stat;
#endif
{
   double norm_scale; 
      
   if (ider != 0) goto err178;
   if (type < 1 || type > 3) goto err178;
      
   if (surf->idim == 1) 
   {
	 
      
	 
      if (normalized == 0)
         norm_scale = 1.;
      else
	 norm_scale = sqrt(1. + derive[1]*derive[1] + derive[2]*derive[2]);
	 
      fundform[0] = 1. + derive[1]*derive[1];
      fundform[1] = derive[1]*derive[2];
      fundform[2] = 1. + derive[2]*derive[2];
	 
      if (type > 1)
      {
	 fundform[3] = derive[3]/norm_scale;
	 fundform[4] = derive[4]/norm_scale;
	 fundform[5] = derive[5]/norm_scale;
      }
      if (type > 2)
      {
	 fundform[6] = derive[6]/norm_scale;
	 fundform[7] = derive[7]/norm_scale;
	 fundform[8] = derive[8]/norm_scale;
	 fundform[9] = derive[9]/norm_scale;
      }
   }
      
   else if (surf->idim == 3) 
   {
	 
      
	 
      if (normalized == 0)
         norm_scale = 1.;
      else
	 norm_scale = sqrt(normal[0]*normal[0] + normal[1]*normal[1] +
			   normal[2]*normal[2]);
	 
      fundform[0] = s6scpr(&derive[3], &derive[3], 3);
      fundform[1] = s6scpr(&derive[3], &derive[6], 3);
      fundform[2] = s6scpr(&derive[6], &derive[6], 3);
	 
      if (type > 1)
      {
	 fundform[3] = (s6scpr(normal, &derive[9], 3))/norm_scale;
	 fundform[4] = (s6scpr(normal, &derive[12], 3))/norm_scale;
	 fundform[5] = (s6scpr(normal, &derive[15], 3))/norm_scale;
      }
      
      if (type > 2)
      {
	 fundform[6] = (s6scpr(normal, &derive[18], 3))/norm_scale;
	 fundform[7] = (s6scpr(normal, &derive[21], 3))/norm_scale;
	 fundform[8] = (s6scpr(normal, &derive[24], 3))/norm_scale;
	 fundform[9] = (s6scpr(normal, &derive[27], 3))/norm_scale;
      }
   }
   else 
   {
      
	 
      goto err105;
   }
      
            
   
      
   *stat = 0;
   goto out;
      
      
   
err105:
   *stat = -105;
   s6err("s2513",*stat,0);
   goto out;
      
   
err178:
   *stat = -178;
   s6err("s2513",*stat,0);
   goto out;
      
out:	 
   return;
      
}

