/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2514

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
   s2514(SISLSurf *surf, int ider, double derive[], double normal[],
      double gaussian[], int *stat)
#else
 void s2514(surf, ider, derive, normal, gaussian, stat)
      SISLSurf *surf;
      int ider;
      double derive[];
      double normal[];
      double gaussian[];
      int *stat;
#endif
{
   double fundform[6]; 

   if (ider != 0) goto err178;

   if (surf->idim == 1 || surf->idim == 3) 
   {
      s2513(surf, ider, 2, 0, derive, normal, fundform, stat);
      if (*stat < 0) goto error;
      
      gaussian[0] = fundform[3]*fundform[5]-fundform[4]*fundform[4];
      gaussian[1] = (fundform[0]*fundform[2] - fundform[1]*fundform[1])*
	   (fundform[0]*fundform[2] - fundform[1]*fundform[1]);
  }

  else if (surf->idim == 2) 
  {
    

    gaussian[0] = 0.0;
    gaussian[1] = 1.0;
  }
  else 
  {
    goto err105;
  }

  

  *stat = 0;
  goto out;


   
err105:
  *stat = -105;
  s6err("s2514",*stat,0);
  goto out;

  
err178:
  *stat = -178;
  s6err("s2514",*stat,0);
  goto out;
  
error:
  s6err("s2514",*stat,0);
  goto out;

out:

  return;

}

