/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S2535

#include "sislP.h"



#if defined(SISLNEEDPROTOTYPES)
void
   s2535(SISLSurf *surf, int u_continuity, int v_continuity, int *u_surfnumb, 
	 int *v_surfnumb, SISLSurf ***patches, int *stat)
#else
void s2535(surf, u_continuity, v_continuity, u_surfnumb, v_surfnumb, patches,
	   stat)
     SISLSurf *surf;
     int      u_continuity;
     int      v_continuity;
     int      *u_surfnumb;
     int      *v_surfnumb;
     SISLSurf ***patches;
     int      *stat;
#endif
{
   int ki, kj;        
   int mult;          
   int u_mult;        
   int v_mult;        
   double *u_splitpar = SISL_NULL; 
   double *v_splitpar = SISL_NULL; 
   SISLSurf *u_surf;  
   SISLSurf *u_lsurf; 
   SISLSurf *u_rsurf; 
   SISLSurf *v_surf;  
   SISLSurf *v_lsurf; 
   SISLSurf *v_rsurf; 
   
   
   
   if (surf == SISL_NULL || u_continuity < 0 || v_continuity < 0) 
      goto err150;
   
   
   
   u_mult = max(surf->ik1 - u_continuity, 1);
   v_mult = max(surf->ik2 - v_continuity, 1);
   
   if ((u_splitpar = newarray(((int) floor(surf->in1/u_mult)) - 1, DOUBLE)) 
       == SISL_NULL) goto err101;
   if ((v_splitpar = newarray(((int) floor(surf->in2/v_mult)) - 1, DOUBLE)) 
       == SISL_NULL) goto err101;
   
   *u_surfnumb = 0;
   *v_surfnumb = 0;
   
   
   
   ki = surf->ik1;
   while(ki < surf->in1)
   {
      mult = 1;
      kj = ki;
      while (DEQUAL(surf->et1[kj], surf->et1[ki]))
      {
	 mult++;
	 kj++;
      }
      if (mult > u_mult)
      {
	 u_splitpar[*u_surfnumb] = surf->et1[ki];
	 *u_surfnumb += 1;
      }
      ki = kj;
   }
   
   ki = surf->ik2;
   while(ki < surf->in2)
   {
      mult = 1;
      kj = ki;
      while (DEQUAL(surf->et2[kj], surf->et2[ki]))
      {
	 mult++;
	 kj++;
      }
      if (mult > v_mult)
      {
	 v_splitpar[*v_surfnumb] = surf->et2[ki];
	 *v_surfnumb += 1;
      }
      ki = kj;
   }   


   *u_surfnumb += 1;
   *v_surfnumb += 1;
   
   if (*u_surfnumb == 1 && *v_surfnumb == 1)
      goto out;
   
   
   
   if ((*patches = newarray((*u_surfnumb)*(*v_surfnumb), SISLSurf*)) 
       == SISL_NULL) goto err101;
   
   
   
   v_surf = surf;
   for (ki = 0; ki < (*v_surfnumb - 1); ki++)
   {
      
      
      
      s1711(v_surf, 2, v_splitpar[ki], &v_lsurf, &v_rsurf, stat);
      if (*stat < 0) goto error;
      
      if (v_surf != surf && v_surf != SISL_NULL) freeSurf(v_surf);
      v_surf = v_rsurf;
      
      
      
      u_surf = v_lsurf;
      for (kj = 0; kj < (*u_surfnumb - 1); kj++)
      {
	 s1711(u_surf, 1, u_splitpar[kj], &u_lsurf, &u_rsurf, stat);
	 if (*stat < 0) goto error;
	 
	 if (u_surf != SISL_NULL) freeSurf(u_surf);
         u_surf = u_rsurf;
	 
	 (*patches)[ki*(*u_surfnumb) + kj] = u_lsurf;
      }
      
      
      
      (*patches)[ki*(*u_surfnumb) + (*u_surfnumb - 1)] = u_surf;
   }
   
   
   
   u_surf = v_surf;
   for (kj = 0; kj < (*u_surfnumb - 1); kj++)
   {
      s1711(u_surf, 1, u_splitpar[kj], &u_lsurf, &u_rsurf, stat);
      if (*stat < 0) goto error;
      
      if (u_surf != surf && u_surf != SISL_NULL) freeSurf(u_surf);
      u_surf = u_rsurf;
      
      (*patches)[(*v_surfnumb - 1)*(*u_surfnumb) + kj] = u_lsurf;
   }
   
   
   
   (*patches)[(*v_surfnumb)*(*u_surfnumb) - 1] = u_surf;

   goto out;



   

   
   
 err101: 
   *stat = -101;
   s6err("s2535",*stat,0);
   goto out;

   
   
 err150:
   *stat = -150;
   s6err("s2535", *stat, 0);
   goto out;
   
   
   
 error:
   s6err("s2535", *stat, 0);
   goto out;

   

 out:
   if (u_splitpar != SISL_NULL) freearray(u_splitpar);
   if (v_splitpar != SISL_NULL) freearray(v_splitpar);
   return;

}

