/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S2551

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s2551( SISLCurve *curve,
       double     parvalue,
       int       *leftknot,
       double     derive[],
       double    *curvature,
       int       *jstat )
#else
void s2551( curve, parvalue, leftknot, derive, curvature, jstat )
     SISLCurve  *curve;
     double      parvalue;
     int        *leftknot;
     double      derive[];
     double     *curvature;
     int        *jstat;
#endif
{
  int kdim = curve -> idim;   
  int kstat = 0;              
  int kpos = 0;               
  double *egeo = SISL_NULL;        



  

  egeo = newarray( 3*kdim+1, DOUBLE );

  if ( egeo == SISL_NULL )      goto err101;


  

  s1221( curve, 2, parvalue, leftknot, derive, &kstat );

  if ( kstat < 0 ) goto error;


  

  s1307( derive, kdim, egeo, &kstat );

  if ( kstat < 0 ) goto error;


  

  *curvature = s6length( egeo+2*kdim, kdim, &kstat );




 

  *jstat = 0;
  goto out;

  
 err101:
  *jstat = -101;
  s6err( "s2551",*jstat,kpos);

  

 error:  *jstat = kstat;
  s6err( "s2551", *jstat, kpos );
  goto out;


 out:
  

 if ( egeo != SISL_NULL ) freearray( egeo );

 return;

}

