/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6ADDCURVE

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void s6addcurve_s9moveknots(double [],int,double,double,
				   double [],int *); 
#else
static void s6addcurve_s9moveknots(); 
#endif

#if defined(SISLNEEDPROTOTYPES)
void s6addcurve(SISLCurve *pc1,SISLCurve *pc2,int isign,
		SISLCurve **rcurve,int *jstat)
#else
void s6addcurve(pc1,pc2,isign,rcurve,jstat)
     SISLCurve *pc1;
     SISLCurve *pc2;
     int isign;
     SISLCurve **rcurve;
     int *jstat;
#endif   
{
  int kstat = 0;         
  int ki;                
  int kdim = pc1->idim;  
  int knbcrv = 2;        
  int kknot1 = pc1->in + pc1->ik;  
  int kknot2 = pc2->in + pc2->ik;  
  int korder;            
  int ktau;              
  int kcont;             
  int kopen;             
  int kkind = pc1->ikind;  
  int kcopy = 1;         
  double tmin1,tmax1;    
  double tmin2,tmax2;    
  double *st = SISL_NULL;     
  double *stau = SISL_NULL;   
  double *scoef = SISL_NULL;  
  SISLCurve *ucurves[2];     
  
  ucurves[0] = ucurves[1] = SISL_NULL;
  
  

  if (kdim != pc2->idim) goto err106;
  
  

  make_cv_kreg(pc1,ucurves,&kstat);
  if (kstat < 0) goto error;
 
  make_cv_kreg(pc2,ucurves+1,&kstat);
  if (kstat < 0) goto error;
  
  

  if ((st = newarray(kknot2,DOUBLE)) == SISL_NULL) goto err101;
  
  
   
  s1363(ucurves[0],&tmin1,&tmax1,&kstat);
  if (kstat < 0) goto error;
  
  s1363(ucurves[1],&tmin2,&tmax2,&kstat);
  if (kstat < 0) goto error;
  
  if (DNEQUAL(tmin1,tmin2) || DNEQUAL(tmax1,tmax2))
    {
      

       s6addcurve_s9moveknots(ucurves[1]->et,kknot2,tmin1,tmax1,st,&kstat);
      if (kstat < 0) goto error;
    }
  else memcopy(st,ucurves[1]->et,kknot2,DOUBLE);
  
  

  s6takeunion(ucurves[0]->et,kknot1,st,kknot2,&stau,&ktau,&kstat);
  if (kstat < 0) goto error;
  
  korder = MAX(ucurves[0]->ik,ucurves[1]->ik);
  ktau -= korder;
  
  

  s1932(2,ucurves,tmin1,tmax1,stau,ktau,korder,&scoef,&kstat);
  if (kstat < 0) goto error;
  
  

  for (ki=0; ki<ktau*kdim; ki++)
    scoef[ki] += (double)isign*scoef[ktau*kdim+ki];
  
  

  if ((*rcurve = newCurve(ktau,korder,stau,scoef,kkind,kdim,kcopy)) == SISL_NULL)
    goto err101;
  
  (*rcurve)->cuopen = kopen = MAX(pc1->cuopen,pc2->cuopen);
  if (kopen == SISL_CRV_PERIODIC)
  {
     
     
     s1333_count(knbcrv,ucurves,&kcont,&kstat);
     if (kstat < 0) goto error;
     
     make_cv_cyclic(*rcurve,kcont,&kstat);
     if (kstat < 0) goto error;
  }
  
  

  *jstat = 0;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  err106 :
    *jstat = -106;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :

    

    if (st != SISL_NULL) freearray(st);
  if (stau != SISL_NULL) freearray(stau);
  if (scoef != SISL_NULL) freearray(scoef);
  if (ucurves[0] != SISL_NULL) freeCurve(ucurves[0]);		     
  if (ucurves[1] != SISL_NULL) freeCurve(ucurves[1]);		     
  
  return;
}

#if defined(SISLNEEDPROTOTYPES)
static void
  s6addcurve_s9moveknots(double et[],int inmbknot,double astart,
			 double aend,double etmoved[],int *jstat)
#else
static void s6addcurve_s9moveknots(et,inmbknot,astart,aend,etmoved,jstat)
     double et[];
     int inmbknot;
     double astart;
     double aend;
     double etmoved[];
     int *jstat;
#endif     
{
  int kwarn = 0;             
  double tstart = et[0];     
  double tend = et[inmbknot-1];   
  double tfac = (aend - astart)/(tend - tstart);  
  double *s1,*s2,*s3;        
  
  

  if (DNEQUAL(aend-astart,tend-tstart)) kwarn = 1;
  
  

  for (s1=et,s2=et+inmbknot,s3=etmoved; s1<s2; s1++,s3++)
    *s3 = astart + (*s1 - tstart)*tfac;
  
  *jstat = kwarn;
  return;
}



