/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6HERM

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s6herm(double *pt,double *uknots,double *vknots,int unum,int vnum,
	 int dim,int uindex,int vindex,double herminfo[],int *jstat)
#else
void
s6herm(pt,uknots,vknots,unum,vnum,dim,uindex,vindex,herminfo,jstat)
     double *pt;
     double *uknots;
     double *vknots;
     int    unum;
     int    vnum;
     int    dim;
     int    uindex;
     int    vindex;
     double herminfo[];
     int    *jstat;
#endif
{
  int kpos = 0;      
  int ki,kj,kk;       
  double Lu[3];         
  double Lv[3];         
  double Ldu[3];        
  double Ldv[3];        
  double temp[3],temp2[3]; 
  double upar,vpar;        
  double diff[3];     
  double kdiff[3];    
  int uind,vind;   
  int i00;                  
  int udeg,vdeg; 
  int ind;  

  

  if(dim < 1 || dim > 3) goto err105;
  if(unum < 2  || vnum < 2) goto err105;
  if(uindex < 0  || uindex > unum) goto err105;
  if(vindex < 0  || vindex > vnum) goto err105;

  

  upar = uknots[uindex];
  vpar = vknots[vindex];

  

  udeg = (unum > 2 ? 2 : 1);
  vdeg = (vnum > 2 ? 2 : 1);

  

  if(udeg == 2)
  {
      if(uindex == 0) uind = 0;
      if(uindex > 0) uind = uindex - 1;
      if(uindex == unum-1) uind = uindex - 2;
  }
  else
  {
      uind = 0;
  }

  if(vdeg == 2)
  {
      if(vindex == 0) vind = 0;
      if(vindex > 0) vind = vindex - 1;
      if(vindex == vnum-1) vind = vindex - 2;
  }
  else
  {
      vind = 0;
  }


  

  if(udeg == 2)
  {
      diff[0] = upar-uknots[uind];
      diff[1] = upar-uknots[uind+1];
      diff[2] = upar-uknots[uind+2];

      kdiff[0] = uknots[uind]-uknots[uind+1];
      kdiff[1] = uknots[uind]-uknots[uind+2];
      kdiff[2] = uknots[uind+1]-uknots[uind+2];

      Lu[0] =   diff[1] * diff[2] / (kdiff[0] * kdiff[1]);
      Lu[1] = - diff[0] * diff[2] / (kdiff[0] * kdiff[2]);
      Lu[2] =   diff[0] * diff[1] / (kdiff[1] * kdiff[2]);

      Ldu[0] =   (diff[1] + diff[2]) / (kdiff[0] * kdiff[1]);
      Ldu[1] = - (diff[0] + diff[2]) / (kdiff[0] * kdiff[2]);
      Ldu[2] =   (diff[0] + diff[1]) / (kdiff[1] * kdiff[2]);
  }
  else
  {
      diff[1] = upar-uknots[uind];
      diff[2] = upar-uknots[uind+1];

      kdiff[2] = uknots[uind]-uknots[uind+1];

      Lu[0] =   diff[2]  / kdiff[2];
      Lu[1] = - diff[1]  / kdiff[2];
      Lu[2] =   DZERO;

      Ldu[0] =   (double)1.0  / kdiff[2];
      Ldu[1] = - (double)1.0  / kdiff[2];
      Ldu[2] = DZERO;
  }


  

  if(vdeg == 2)
  {
      diff[0] = vpar-vknots[vind];
      diff[1] = vpar-vknots[vind+1];
      diff[2] = vpar-vknots[vind+2];

      kdiff[0] = vknots[vind]-vknots[vind+1];
      kdiff[1] = vknots[vind]-vknots[vind+2];
      kdiff[2] = vknots[vind+1]-vknots[vind+2];

      Lv[0] =   diff[1] * diff[2] / (kdiff[0] * kdiff[1]);
      Lv[1] = - diff[0] * diff[2] / (kdiff[0] * kdiff[2]);
      Lv[2] =   diff[0] * diff[1] / (kdiff[1] * kdiff[2]);

      Ldv[0] =   (diff[1] + diff[2]) / (kdiff[0] * kdiff[1]);
      Ldv[1] = - (diff[0] + diff[2]) / (kdiff[0] * kdiff[2]);
      Ldv[2] =   (diff[0] + diff[1]) / (kdiff[1] * kdiff[2]);
  }
  else
  {
      diff[1] = vpar-vknots[vind];
      diff[2] = vpar-vknots[vind+1];

      kdiff[2] = vknots[vind]-vknots[vind+1];

      Lv[0] =   diff[2]  / kdiff[2];
      Lv[1] = - diff[1]  / kdiff[2];
      Lv[2] =   DZERO;

      Ldv[0] =   (double)1.0  / kdiff[2];
      Ldv[1] = - (double)1.0  / kdiff[2];
      Ldv[2] = DZERO;
  }

  

  i00 = (vind*unum+uind)*dim;

  for(ki=0; ki<dim; ki++)
  {
      ind = i00+ki;

      for(kj=0; kj<=vdeg; kj++)
      {
	  temp[kj] = 0.0;
	  temp2[kj] = 0.0;

          for(kk=0; kk<=udeg; kk++,ind+=dim)
          {
	      temp[kj] += Lu[kk] * pt[ind];
	      temp2[kj] += Ldu[kk] * pt[ind];
	  }

	  ind += (unum - udeg - 1) * dim;
      }

      herminfo[ki] = 0.0;
      herminfo[dim+ki] = 0.0;
      herminfo[dim+dim+ki] = 0.0;

      for(kj=0; kj<=vdeg; kj++)
      {
	  

	  herminfo[ki] += Lv[kj] * temp2[kj];

	  

	  herminfo[dim+ki] += Ldv[kj] * temp[kj];

	  

	  herminfo[dim+dim+ki] += Ldv[kj] * temp2[kj];
      }
  }



  

  *jstat = 0;
  goto out;

  

  err105: *jstat = -105;
  s6err("s6herm",*jstat,kpos);
  goto out;

  out: return;
}

