/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6MVEC

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s6mvec(double emat[],double evec1[],int inbvec,double evec2[])
#else
void s6mvec(emat,evec1,inbvec,evec2)
     double emat[];
     double evec1[];
     int    inbvec;
     double evec2[];
#endif
{
  double svec2[3];           
  double *svec1;             
  double *svec3;             
  register double tdum;      
  register int ki,kj,kl,kp;  
  int kstop;                 
  
  
  
  kstop = 3*inbvec;
  
  for (kl=0;kl<kstop;kl=kl+3)
    {
      
      
      svec1 = evec1 + kl;
      svec3 = evec2 + kl;
      
      for (ki=0;ki<3;ki++)
        {
	  kp = ki;
	  
	  tdum = DZERO;
	  for (kj=0;kj<3;kj++)
            {
	      tdum += emat[kp]*svec1[kj];
	      kp += 4;
            }
	  
	  
	  svec2[ki]  = tdum + emat[kp];
        }
                           
      
      if (DNEQUAL(emat[3],DZERO) || DNEQUAL(emat[7],DZERO) ||
	  DNEQUAL(emat[11],DZERO) || DNEQUAL(emat[15],(double)1.0))
        {
	  
	  
	  tdum = evec1[0]*emat[3] + evec1[1]*emat[7] + evec1[2]*emat[11];
	  if (DNEQUAL(tdum,DZERO))
            {
	      for (ki=0;ki<3;ki++)
		svec2[ki] /= tdum;
            }
	}
      svec3[0] = svec2[0];    
      svec3[1] = svec2[1];
      svec3[2] = svec2[2];
    }
  
  return;
}

