/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S9CLIPIMP

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s9clipimp(double epar1[],double epar2[],SISLSurf *psurf1,double eimpli[],
	       int ideg,double euval[],double evval[],double aepsge,
	       double gpnt1[],double gpar1[],int *jstat)
#else
void s9clipimp(epar1,epar2,psurf1,eimpli,ideg,euval,evval,
               aepsge,gpnt1,gpar1,jstat)
     double epar1[];
     double epar2[];
     SISLSurf   *psurf1;
     double eimpli[];
     int    ideg;
     double euval[];
     double evval[];
     double aepsge;
     double gpnt1[];
     double gpar1[];
     int    *jstat;
#endif
{
  int kpos=0;                   
  int klfu=0;                   
  int klfv=0;                   
  int kder=2;                   
  int kstat;                    
  int kdir;                     
  int kcross;                   
  int knbit;                    
  int krem;                     
  int kbound;                   
  int ksize;                    
  int ksizem3;                  
  double tpar;                  
  double spnt1[33];             
  double spar1[2],spar2[2];     
  double spar3[2];              
  
  
  
  
  
  if (ideg==1003 || ideg==1004 || ideg==1005)
    {
      kder = 3;
      ksize = 33;
    }
  else
    {
      ksize = 21;
      kder =2;
    }
  ksizem3 = ksize -3;
  
  
  
  
  memcopy(spar1,epar1,2,DOUBLE);                        
  memcopy(spar2,epar2,2,DOUBLE);
  
  kcross = 1;
  knbit  = 0;
  
  while(kcross && knbit<4)
    {
      
      
      
      s1305(spar1,spar2,euval,evval,&kbound,spar3,&kstat);
      if (kstat<0) goto error;
      
      
      krem = kstat;
      
      if (kstat<2 || kbound == 0)
        {
	  
	  kcross = 0;
        }
      else
        {
	  
	  
	  
	  s1421(psurf1,kder,spar3,&klfu,&klfv,spnt1,spnt1+ksizem3,&kstat); 
	  if (kstat<0) goto error;
	  
	  if (kbound==1)
            {
	      kdir = 1;
	      tpar = euval[0];
            }
	  else if (kbound==2)
            {
	      kdir = 2;
	      tpar = evval[1];
            }
	  else if (kbound==3)
            {
	      kdir = 1;                                   
	      tpar = euval[1];
            }
	  else if (kbound==4)
            {
	      kdir = 2;
	      tpar = evval[0];
            }
	  
	  
	  
	  
	  s9boundimp(spnt1,gpar1,psurf1,eimpli,ideg,tpar,kdir,aepsge,
		     gpnt1,gpar1,&kstat);
	  if (kstat<0) goto error;
	  if (kstat==2) goto war02;
	  
	  
	  
	  
	  if (krem == 2)
            {
	      
	      memcopy(spar2,gpar1,2,double);
            }
	  else
            {
	      
	      memcopy(spar1,gpar1,2,double);
            }
	  
	  
	  
	  knbit++;
        }
    }
  
  
  
  
  
  if (kcross==0 && knbit ==0)
    {
      
      *jstat = 0;
    }
  else if (kcross==0 && knbit > 0)
    {
      
      
      if (spar1[0] == epar1[0] && spar1[1] == epar1[1])
        {
	  memcopy(gpar1,spar2,2,double);
        }
      else
        {
	  memcopy(gpar1,spar1,2,double);
	  
	  
	  
	  s1421(psurf1,kder,gpar1,&klfu,&klfv,gpnt1,gpnt1+ksizem3,&kstat); 
	  if (kstat<0) goto error;
        }
      
      *jstat = 1;
    }
  else
    {
      
      goto war02;
    }
  goto out;
  
  
  
 war02: *jstat = 2;
  goto out;
  
  
  
  error : *jstat = kstat;
  s6err("s9clipimp",*jstat,kpos);
  goto out;
  
 out:
  return;                                                                        
  
}


