/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define SH1260

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
      sh1260(double aconst,SISLCurve *vcurve[],int icurve,int *jstat)
#else	 
void sh1260(aconst,vcurve,icurve,jstat)
     double aconst;
     SISLCurve *vcurve[];
     int icurve;
     int *jstat;
#endif     
{
  int kstat = 0;      
  int ki;             
  int kder = 1;       
  int kleft = 0;      
  int kdim = vcurve[0]->idim;   
  double tpar1 = *(vcurve[0]->et + vcurve[0]->ik - 1);   
  double tpar2 = *(vcurve[0]->et + vcurve[0]->in);       
  double *sder1 = SISL_NULL;  
  double *sder2 = SISL_NULL;  
  double *sdiff = SISL_NULL;  
  double tdiff;          
  double t1,t2;          
  double tscal;          
  double tnewend;        
  double *s1;              
  double *s2;            
  SISLCurve *qcpt;           
  
  

  if (icurve < 1) goto err110;
  for (ki=1; ki<icurve; ki++)
    {
      qcpt = vcurve[ki];
      if (qcpt->idim != kdim) goto err106;
      if (*(qcpt->et+qcpt->ik-1) != tpar1) goto err112;
      if (*(qcpt->et+qcpt->in) != tpar2) goto err112;
    }
      
  

  if ((sder1 = newarray(2*kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((sder2 = newarray(2*kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((sdiff = newarray(kdim,DOUBLE)) == SISL_NULL) goto err101;

  

  s1221(vcurve[0],kder,tpar1,&kleft,sder1,&kstat);
  if (kstat < 0) goto error;
  
  s1221(vcurve[0],kder,tpar2,&kleft,sder2,&kstat);
  if (kstat < 0) goto error;

  

  s6diff(sder2,sder1,kdim,sdiff);
  
  

  tdiff = s6length(sdiff,kdim,&kstat);
  
  

  t1 = s6scpr(sder1+kdim,sdiff,kdim)/(tdiff*tdiff);   

  
  

  t2 = s6scpr(sder2+kdim,sdiff,kdim)/(tdiff*tdiff);  
  
  
  

  if (MAX(t1,t2) > aconst)
    {
      

      tscal = MAX(t1,t2)/aconst;
      
      

      tnewend = tscal*(tpar2-tpar1) + tpar1;

      for (ki=0; ki<3; ki++)
	{
	  qcpt = vcurve[ki];
	  
	  

	  for (s1=qcpt->et,s2=qcpt->et+qcpt->in+qcpt->ik; s1<s2; s1++)
	    *s1 = tpar1 + (*s1 - tpar1)*(tnewend - tpar1)/(tpar2 - tpar1);
	}
      
    }
 
  

  *jstat = 0;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  err110 :
    *jstat = -110;
  goto out;
  
  

  err106 :
    *jstat = -106;
  goto out;
  
  

  err112 :
    *jstat = -112;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :
    
    

    if (sder1 != SISL_NULL) freearray(sder1);
  if (sder2 != SISL_NULL) freearray(sder2);
  if (sdiff != SISL_NULL) freearray(sdiff);
  
  return;
}     

