/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define SH1261

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void sh1261_s9evalbez(double [],int,double,double,double,
			     double *,int *);
#else
static void sh1261_s9evalbez();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
      sh1261(SISLCurve *pcurve1,SISLCurve *pcurve2,double ecoef1[],
	     int ik1,double ecoef2[],int ik2,SISLCurve **rcrtanc,int *jstat)
#else
void sh1261(pcurve1,pcurve2,ecoef1,ik1,ecoef2,ik2,rcrtanc,jstat) 
   SISLCurve *pcurve1,*pcurve2,**rcrtanc; 
     double ecoef1[],ecoef2[]; 
     int ik1,ik2,*jstat; 
#endif     
{
  int kstat = 0;       
  int ki,kj;           
  int kk;              
  int kleft = 0;       
  int korder;          
  int knunion;         
  int knvert;          
  int kn;              
  int kder = 0;        
  int kind = pcurve1->ikind; 
  int kdim = pcurve1->idim;  
  int kcopy = 1;             
  int knlr = 0;              
  int knrc = 0;              
  int kopen = SISL_CRV_OPEN; 
  int *lder = 0;             
  double tpar;         
  double tmin1,tmax1;  
  double tmin2,tmax2;  
  double tval1,tval2;  
  double *stunion = SISL_NULL;  
  double *stcross = SISL_NULL;  
  double *spar = SISL_NULL;     
  double *sbcoef = SISL_NULL;   
  double *spoint = SISL_NULL;   
  double *sder1 = SISL_NULL;
  double *sder2 = SISL_NULL;    

  

  if (pcurve2->idim != kdim) goto err106;
  
  

  s1363(pcurve1,&tmin1,&tmax1,&kstat);
  if (kstat < 0) goto error;
  
  s1363(pcurve2,&tmin2,&tmax2,&kstat);
  if (kstat < 0) goto error;
  
  

  if (tmin1 != tmin2 || tmax1 != tmax2) goto err121;
  
  

  korder = MAX(pcurve1->ik+ik1-1,pcurve2->ik+ik2-1);
  
  

  s6takeunion(pcurve1->et,pcurve1->ik+pcurve1->in,pcurve2->et,
	  pcurve2->ik+pcurve2->in,&stunion,&knunion,&kstat);
  if (kstat < 0) goto error;
  
  

  kk = MAX(ik1,ik2) - 1;
  knvert = knunion - korder + kk + kk;
  
  

  if ((stcross = newarray(korder+knvert,DOUBLE)) == SISL_NULL) goto err101;
  if ((spar = newarray(knvert,DOUBLE)) == SISL_NULL) goto err101;
  if ((spoint = newarray(kdim*knvert,DOUBLE)) == SISL_NULL) goto err101;
  if ((sder1 = newarray(kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((sder2 = newarray(kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((lder = new0array(knvert,INT)) == SISL_NULL) goto err101;
  
  

  memcopy(stcross+kk,stunion,knunion,DOUBLE);
  for (ki=1; ki<=kk; ki++)
    {
      stcross[kk-ki] = stcross[kk];
      stcross[knunion+kk+ki-1] = stcross[knunion+kk-1];
    }
  
  

  for (ki=0; ki<knvert; ki++)
    {
      

      for (tpar=(double)0.0,kj=1; kj<korder; kj++) tpar += stcross[ki+kj];
      tpar /= (korder - 1);
      
      spar[ki] = tpar;
      
      

      sh1261_s9evalbez(ecoef1,ik1,tmin1,tmax1,tpar,&tval1,&kstat);
      if (kstat < 0) goto error;
      
      
  
		     sh1261_s9evalbez(ecoef2,ik2,tmin2,tmax2,tpar,&tval2,&kstat);
      if (kstat < 0) goto error;

      

      s1221(pcurve1,kder,tpar,&kleft,sder1,&kstat);
      if (kstat < 0) goto error;

      

      s1221(pcurve2,kder,tpar,&kleft,sder2,&kstat);
      if (kstat < 0) goto error;
      
      

      for (kj=0; kj<kdim; kj++)
	spoint[ki*kdim+kj] = tval1*sder1[kj] + tval2*sder2[kj];
    }
  
  

  s1891(spar,spoint,kdim,knvert,1,lder,kopen,stcross,&sbcoef,&kn,korder,
	knlr,knrc,&kstat);
  if (kstat < 0) goto error;
  
  
    if ((*rcrtanc = newCurve(kn,korder,stcross,sbcoef,
			   kind,kdim,kcopy)) == SISL_NULL) goto err101;
    (*rcrtanc)->cuopen = MAX(pcurve1->cuopen,pcurve2->cuopen);
  
  

  *jstat = 0;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  err106 :
    *jstat = -106;
  goto out;

  

  err121 :
    *jstat = -121;
  goto out;
  

  

  error :
    *jstat = kstat;
  goto out;
  
  out :     
    

    if (stunion != SISL_NULL) freearray(stunion);
  if (stcross != SISL_NULL) freearray(stcross);
  if (sbcoef != SISL_NULL) freearray(sbcoef);
  if (spar != SISL_NULL) freearray(spar);
  if (spoint != SISL_NULL) freearray(spoint);
  if (sder1 != SISL_NULL) freearray(sder1);
  if (sder2 != SISL_NULL) freearray(sder2);
  if (lder != SISL_NULL) freearray(lder);
  
  return;
}


#if defined(SISLNEEDPROTOTYPES)
   static
      void
	    sh1261_s9evalbez(double ecoef[],int iorder,double amin,
			     double amax,double apar,double *cvalue,int *jstat)
#else	       
static void sh1261_s9evalbez(ecoef,iorder,amin,amax,apar,cvalue,jstat)
     int iorder,*jstat;
     double ecoef[],amin,amax,apar,*cvalue;
#endif     
{
  int ki,kj;
  double *scoef = SISL_NULL;
  double tdist = amax - amin;
  
  

  if ( DEQUAL(tdist,DZERO)  )
    {
      *cvalue = ecoef[0];
      *jstat = 1;
      goto out;
    }
  
  

  if ((scoef = newarray(iorder,DOUBLE)) == SISL_NULL) goto err101;
  memcopy(scoef,ecoef,iorder,DOUBLE);
  
  

  for (ki=1; ki<iorder; ki++)
    for (kj=0; kj<iorder-ki; kj++)
      scoef[kj] = (apar-amin)*scoef[kj+1]/tdist + (amax-apar)*scoef[kj]/tdist;
  
  

  *cvalue = scoef[0];
  *jstat = 0;
  goto out;
  

  

  err101 :
    *jstat = -101;
  goto out;
  
  out :
    

    if (scoef != SISL_NULL) freearray(scoef);

  return;
}


