/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1369

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
     sh1369(SISLSurf *ps,double ecentr[],double enorm[],double abigr,
	   double asmalr,int idim,double aepsco,double aepsge,
	int trackflag,int *jtrack,SISLTrack ***wtrack,
	int *jpt,double **gpar,int **pretop,int *jcrv,SISLIntcurve ***wcurve,int *jsurf,SISLIntsurf ***wsurf,int *jstat)
#else
void sh1369(ps,ecentr,enorm,abigr,asmalr,idim,aepsco,aepsge,
	trackflag,jtrack,wtrack,jpt,gpar,pretop,jcrv,wcurve,jsurf,wsurf,jstat)
     SISLSurf     *ps;
     double   ecentr[];
     double   enorm[];
     double   abigr;
     double   asmalr;
     int      idim;
     double   aepsco;
     double   aepsge;
     int       trackflag;
     int       *jtrack;
     SISLTrack ***wtrack;
     int      *jpt;
     double   **gpar;
     int      **pretop;
     int      *jcrv;
     SISLIntcurve ***wcurve;
     int      *jsurf;
     SISLIntsurf ***wsurf;
     int      *jstat;
#endif
{
  int kstat = 0;              
  int kpos = 0;               
  int kdim=1;                 
  int kdeg = 1001;            
  double simpli[16];           
  double snorm[3];            
  double spoint[1];           
  double *spar = SISL_NULL;        
  SISLSurf *qs = SISL_NULL;        
  SISLPoint *qp = SISL_NULL;       
  SISLObject *qo1 = SISL_NULL;     
  SISLObject *qo2 = SISL_NULL;     
  SISLIntdat *qintdat = SISL_NULL; 
  SISLObject *track_obj=SISL_NULL;
  SISLSurf *qkreg=SISL_NULL; 
  double aepsge2;             

  int ki;
  double nmax=(double)1.0;
  

  if (ps->cuopen_1 == SISL_SURF_PERIODIC ||
      ps->cuopen_2 == SISL_SURF_PERIODIC)
  {
     

     make_sf_kreg(ps,&qkreg,&kstat);
     if (kstat < 0) goto error;
   }
  else
    qkreg = ps;

  

  if (!(track_obj = newObject (SISLSURFACE)))
    goto err101;
  track_obj->s1 = ps;

  

  *jpt  = 0;
  *jcrv = 0;
  *jtrack = 0;

  if (idim != 3) goto err104;
  if (qkreg -> idim != idim) goto err106;

  

  (void)s6norm(enorm,idim,snorm,&kstat);
  if (kstat<0) goto error;

  

  memcopy(simpli,ecentr,3,DOUBLE);
  memcopy(simpli+3,snorm,3,DOUBLE);
  simpli[6] = abigr;
  simpli[7] = asmalr;

  

  s1378(qkreg,simpli,kdeg,idim,&qs,&kstat);
  if (kstat<0) goto error;

  

  if(!(qo1 = newObject(SISLSURFACE))) goto err101;
  qo1 -> s1 = qs;
  qo1 -> o1 = qo1;

  

  if(!(qo2 = newObject(SISLPOINT))) goto err101;
  spoint[0] = DZERO;
  if(!(qp = newPoint(spoint,kdim,1))) goto err101;
  qo2 -> p1 = qp;

  

  

  aepsge2 = (double)8.0 * aepsge * asmalr * abigr * abigr;

  
  for(ki=0; ki<qs->in1*qs->in2;ki++)
     nmax = max(fabs(qs->ecoef[ki]),nmax);

  
  
  nmax = MIN(nmax, aepsge2*1.0e11);
             
  if (nmax >(double)10.0)
  {
     for(ki=0; ki<qs->in1*qs->in2;ki++)
        qs->ecoef[ki] /= nmax;
     aepsge2 /= nmax;
  }

  

  sh1761(qo1,qo2,aepsge2,&qintdat,&kstat);
  if (kstat < 0) goto error;

  

  sh6degen(track_obj,track_obj,&qintdat,aepsge,&kstat);
  if (kstat < 0) goto error;


  
  if (trackflag && qintdat)
    {

      refine_all (&qintdat, track_obj, track_obj, simpli, kdeg, aepsge, &kstat);
      if (kstat < 0)
	goto error;
    }

  
  int_join_per( &qintdat,track_obj,track_obj,simpli,kdeg,aepsge,&kstat);
  if (kstat < 0)
    goto error;

  if (trackflag && qintdat)
    {
      make_tracks (track_obj, track_obj, kdeg, simpli,
		   qintdat->ilist, qintdat->vlist,
		   jtrack, wtrack, aepsge, &kstat);
      if (kstat < 0)
	goto error;

    }

  

  if (qintdat)
    {
      hp_s1880(track_obj, track_obj, kdeg,
	       2,0,qintdat,jpt,gpar,&spar,pretop,jcrv,wcurve,jsurf,wsurf,&kstat);
      if (kstat < 0) goto error;
    }
  else
  {
    *jcrv = 0;
    *jsurf = 0;
  }

  

  *jstat = 0;
  goto out;

  

 err101: *jstat = -101;
         s6err("sh1369",*jstat,kpos);
         goto out;

  

 err106: *jstat = -106;
         s6err("sh1369",*jstat,kpos);
         goto out;

  

 err104: *jstat = -104;
         s6err("sh1369",*jstat,kpos);
         goto out;

 

 error : *jstat = kstat;
         s6err("sh1369",*jstat,kpos);
         goto out;

 out:

  

  if (spar)    freearray(spar);
  if (qo1)     freeObject(qo1);
  if (qo2)     freeObject(qo2);
  if (qintdat) freeIntdat(qintdat);
  if (track_obj)
    {
       track_obj->s1 = SISL_NULL;
       freeObject(track_obj);
    }

  
    if (qkreg != SISL_NULL && qkreg != ps) freeSurf(qkreg);

return;
}

