/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1461

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
typedef void (*fshapeProc)(double [],double [],int,int,int *);
typedef void (*initProc)(fshapeProc,SISLCurve *[],int,double [],
			 double [],double [],int *);
#else
typedef void (*fshapeProc)();
typedef void (*initProc)();
#endif

#if defined(SISLNEEDPROTOTYPES)
static void sh1461_s9coef(double [],double [],double [],double [],int,
			  double *,double *,double *,double *,int *);
static void sh1461_s9hermit(double [],int,int,int *);
static void sh1461_s9chcoor(double [],int,double [],int,double [],
			    int,double [],double [],double [],int,
			    double [],int *,int *);
static void sh1461_s9mult(double [],double [],int,int,double [],int *);
static void sh1461_s9comder(int,int,double [],int,double,double,double,
			    double,double [],int *);
static double sh1461_s9ang(double [],double [],int);
#else
static void sh1461_s9coef();
static void sh1461_s9hermit(); 
static void sh1461_s9chcoor();
static void sh1461_s9mult(); 
static void sh1461_s9comder(); 
static double sh1461_s9ang();   
#endif

#if defined(SISLNEEDPROTOTYPES)
void
  sh1461(fshapeProc fshape,initProc f_initmid,
	 SISLCurve *vboundc[],int icurv,SISLSurf *vsurf[],int *jstat)
#else	 
void sh1461(fshape,f_initmid,vboundc,icurv,vsurf,jstat)
     fshapeProc  fshape;
     initProc    f_initmid;
     SISLCurve   *vboundc[];
     SISLSurf    *vsurf[];
     int         icurv;
     int         *jstat;
#endif     
{
  int kstat = 0;                
  int kdim = 3;                 
  int ki,kj,kk,kh;              
  int kder = 2;                 
  int knmb = 0;                 
  int kleft = 0;                
  int kkcrt2;                   
  int lder[4];
  double tpar;                  
  double tro00,tro01,tro10,tro11;  
  double *sder = SISL_NULL;          
  double *stwist = SISL_NULL;        
  double *stang = SISL_NULL;         
  double spos[15];              
  double scrt1[21];             
  double scrt2[12];             
  double stpos[10];             
  double stcrt1[14];            
  double stcrt2[8];             
  double sblend[4];             
  double sdum[6];               
  double sdum2[6];              
  double *stwist2 = SISL_NULL;       
  double *st,*st2;              
  double tstart;                
  SISLCurve *qc;                
  SISLCurve **qbound = SISL_NULL;    
  
  for (ki=0; ki<4; ki++) lder[ki] = 2;
  
  for (ki=0; ki<=kder; ki++) knmb += ki + 1;
  
  

  for (ki=0; ki<5; ki++)
    {
      stpos[ki] = (double)0.0;
      stpos[5+ki] = (double)1.0;
    }
  for (ki=0; ki<4; ki++)
    {
      stcrt2[ki] = (double)0.0;
      stcrt2[4+ki] = (double)1.0;
    }
  for (ki=0; ki<7; ki++)
    {
      stcrt1[ki] = (double)0.0;
      stcrt1[7+ki] = (double)1.0;
    }
      
      
  

  if ((sder = new0array(icurv*kdim*knmb,DOUBLE)) == SISL_NULL) goto err101;
  if ((stwist = newarray(icurv*kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((stwist2 = newarray(4*icurv*kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((stang = newarray(icurv*kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((qbound = newarray(8*icurv,SISLCurve*)) == SISL_NULL) goto err101;
  
  for (ki=0; ki<icurv; ki++)
    {
      

      if (vboundc[ki]->idim != kdim) goto err102;
      
      

      tpar = (double)0.0;
      s1221(vboundc[2*ki+1],1,tpar,&kleft,sdum,&kstat);
      if (kstat < 0) goto error;
      
      memcopy(stwist+ki*kdim,sdum+kdim,kdim,double);
    }
 
  

  for (ki=0; ki<kdim; ki++)
    for (kj=0; kj<icurv; kj++)
      {
	kk = (kj + 1) % icurv;
	stwist2[(kj*4+2)*kdim+ki] = stwist[kk*kdim+ki]/(double)4.0;
      }
  
      
  

  sblend[0] = (double)1.0;
  sblend[1] = sblend[2] = sblend[3] = (double)0.0;
  sh1461_s9hermit(sblend,4,1,&kstat);
  
  

  f_initmid(fshape,vboundc,icurv,stwist,stang,sder,&kstat);
  if (kstat < 0) goto error;
  
  for (ki=1; ki<icurv; ki++)
    {
      

      

      memcopy(sder+ki*knmb*kdim,sder,kdim,DOUBLE);
      
      

      memcopy(sder+(ki*knmb+1)*kdim,stang+(ki-1)*kdim,kdim,DOUBLE);
      memcopy(sder+(ki*knmb+2)*kdim,stang+ki*kdim,kdim,DOUBLE);
      
      

      memcopy(sder+(ki*knmb+3)*kdim,sder+((ki-1)*knmb+5)*kdim,kdim,DOUBLE);
      
      

      sh1461_s9coef(stang+(icurv-1)*kdim,stang,stang+(ki-1)*kdim,
	     stang+ki*kdim,kdim,&tro00,&tro01,&tro10,&tro11,&kstat);
      if (kstat < 0) goto error;
      
      

		     sh1461_s9comder(1,1,sder+3*kdim,kdim,tro00,tro01,tro10,tro11,
	       sder+(ki*knmb+4)*kdim,&kstat);
	  
      

		     sh1461_s9comder(0,2,sder+3*kdim,kdim,tro00,tro01,tro10,tro11,
	       sder+(ki*knmb+5)*kdim,&kstat);
    }  

  

  for (ki=0; ki<icurv; ki++)
    {
      

      memcopy(spos,sder+ki*knmb*kdim,kdim,DOUBLE);
      memcopy(spos+kdim,sder+(ki*knmb+2)*kdim,kdim,DOUBLE);
      memcopy(spos+2*kdim,sder+(ki*knmb+5)*kdim,kdim,DOUBLE);
      memcopy(scrt2,sder+(ki*knmb+1)*kdim,kdim,DOUBLE);
      memcopy(scrt2+kdim,sder+(ki*knmb+4)*kdim,kdim,DOUBLE);
      memcopy(stwist2+4*ki*kdim,sder+(ki*knmb+4)*kdim,kdim,DOUBLE);
      
      

      kj = (ki < icurv-1) ? ki+1 : 0;
      
      qc = vboundc[2*kj];
      tpar = (*(qc->et + qc->ik - 1) + *(qc->et + qc->in))/(double)2.0;
      
      s1221(qc,1,tpar,&kleft,sdum,&kstat);
      if (kstat < 0) goto error;
      
      s1221(vboundc[2*kj+1],1,tpar,&kleft,sdum2,&kstat);
      if (kstat < 0) goto error;
      
      

      memcopy(spos+4*kdim,sdum,kdim,DOUBLE);
      for (kh=0; kh<kdim; kh++)
	{
	  spos[3*kdim+kh] = sdum2[kh]/(double)2.0;
	  scrt2[3*kdim+kh] = -sdum[kdim+kh]/(double)2.0;
	  stwist2[(ki*4+3)*kdim+kh] = scrt2[2*kdim+kh] = -sdum2[kdim+kh]/(double)4.0;
	  stwist2[(kj*4+1)*kdim+kh] = -stwist2[(ki*4+3)*kdim+kh];
	}
      
      

      sh1461_s9hermit(spos,5,kdim,&kstat);
      if (kstat < 0) goto error;
      
		     sh1461_s9hermit(scrt2,4,kdim,&kstat);
      if (kstat < 0) goto error;
      
      

      sh1461_s9chcoor(sblend,4,spos,5,scrt2,4,sder+(knmb*ki+1)*kdim,sder+(knmb*ki+2)*kdim,
	       sder+(kj*knmb+2)*kdim,kdim,scrt1,&kkcrt2,&kstat);
      if (kstat < 0) goto error;
      
      

      if ((qbound[8*kj] = newCurve(5,5,stpos,spos,1,kdim,1)) == SISL_NULL) 
	goto err101;
      if ((qbound[8*kj+1] = newCurve(kkcrt2,kkcrt2,stcrt1,scrt1,1,kdim,1)) == SISL_NULL)
	goto err101;
      if ((qbound[8*ki+6] = newCurve(5,5,stpos,spos,1,kdim,1)) == SISL_NULL) 
	goto err101;
      if ((qbound[8*ki+7] = newCurve(4,4,stcrt2,scrt2,1,kdim,1)) == SISL_NULL)
	goto err101;
      
      

      s1710(qc,tpar,qbound+8*ki+4,qbound+8*kj+2,&kstat);
      if (kstat < 0) goto error;
      
      s1710(vboundc[2*kj+1],tpar,qbound+8*ki+5,qbound+8*kj+3,&kstat);
      if (kstat < 0) goto error;

      

      s1706(qbound[8*ki+4]);
      s1706(qbound[8*ki+5]);

      

      for (kk=0; kk<kdim*(qbound[8*ki+5]->in); kk++)
	*(qbound[8*ki+5]->ecoef+kk) *= (double)0.5;
      
      for (kk=0; kk<kdim*(qbound[8*kj+3]->in); kk++)
	*(qbound[8*kj+3]->ecoef+kk) *= (double)0.5;

      

      for (kh=2; kh<6; kh++)
	{
	  kk = (kh > 3) ? 8*ki : 8*kj;
	  kk += kh;
	  
	  for (st=qbound[kk]->et,tstart=st[0],
	       st2=st+qbound[kk]->in+qbound[kk]->ik;
	       st<st2; st++)
	    *st = (double)2.0*(*st - tstart);
	}
      
    }
  
  for (ki=0; ki<icurv; ki++)
    {
      

       s1401(qbound+8*ki,stwist2+4*ki*kdim,vsurf+ki,&kstat);  
       
      if (kstat < 0) goto error;
    }
  
  

  *jstat = 0;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  

  err102 :
    *jstat = -102;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :

    

    if (sder) freearray(sder);
  if (stwist) freearray(stwist);
  if (stwist2) freearray(stwist2);
  if (stang) freearray(stang);
  if (qbound)
    {
      for (ki=0; ki<8*icurv; ki++)
	if (qbound[ki]) freeCurve(qbound[ki]);
      freearray(qbound);      
    }
  
    return;
}

#if defined(SISLNEEDPROTOTYPES)
static void
  sh1461_s9hermit(double econd[],int icond,int idim,int *jstat)
#else	       
static void sh1461_s9hermit(econd,icond,idim,jstat)
     int icond,idim,*jstat;
     double econd[];
#endif     
{
  int ki;    

  

  if (icond != 4 && icond != 5) goto err001;
  
  if (icond == 4)
    {
      

      for (ki=0; ki<idim; ki++)
	{
	  econd[idim+ki] = ONE_THIRD*econd[idim+ki] + econd[ki];
	  econd[2*idim+ki] = ONE_THIRD*econd[2*idim+ki] + econd[3*idim+ki];
	}
    }
  
  if (icond == 5)
    {
      

      for (ki=0; ki<idim; ki++)
	{
	  econd[idim+ki] = ONE_FOURTH*econd[idim+ki] + econd[ki];
	  econd[2*idim+ki] = econd[2*idim+ki]/(double)12.0 
	    + (double)2.0*econd[idim+ki] - econd[ki];
	  econd[3*idim+ki] = ONE_FOURTH*econd[3*idim+ki] + econd[4*idim+ki];	  
	}
    }
   
  *jstat = 0;
  goto out;
  
  

  err001 :
    *jstat = -1;
  goto out;
  
  out :
    return;
}

#if defined(SISLNEEDPROTOTYPES)
static void
  sh1461_s9coef(double evec1[],double evec2[],double evec3[],
		double evec4[],int idim,double *cro00,double *cro01,
		double *cro10,double *cro11,int *jstat)
#else
static void sh1461_s9coef(evec1,evec2,evec3,evec4,idim,cro00,cro01,
			  cro10,cro11,jstat)
     int idim,*jstat;
     double evec1[],evec2[],evec3[],evec4[];
     double *cro00,*cro01,*cro10,*cro11;
#endif     
{
  int kstat = 0;                       
  int ksin,ksin1,ksin2,ksin3,ksin4;    
  double tang,tang1,tang2,tang3,tang4; 
  double tl1,tl2,tl3,tl4;              
  double tsin,tsin1,tsin2,tsin3,tsin4; 
  double snorm[3];                     
  double svec[3];                      
  double tlvec;                        
  
  

  s6crss(evec1,evec2,snorm);
  
  

  tl1 = s6length(evec1,idim,&kstat);
  tl2 = s6length(evec2,idim,&kstat);
  tl3 = s6length(evec3,idim,&kstat);
  tl4 = s6length(evec4,idim,&kstat);
  
  

  s6crss(snorm,evec1,svec);
  tlvec = s6length(svec,idim,&kstat);
  
  

  tsin = s6scpr(svec,evec2,idim)/(tlvec*tl2);
  tsin1 = s6scpr(svec,evec3,idim)/(tlvec*tl3);
  tsin2 = s6scpr(svec,evec4,idim)/(tlvec*tl4);
  
  

  s6crss(snorm,evec2,svec);
  tlvec = s6length(svec,idim,&kstat);

  

  tsin3 = s6scpr(svec,evec3,idim)/(tlvec*tl3);
  tsin4 = s6scpr(svec,evec4,idim)/(tlvec*tl4); 
  
  

  ksin = (tsin < DZERO) ? -1 : 1;
  ksin1 = (tsin1 < DZERO) ? -1 : 1;
  ksin2 = (tsin2 < DZERO) ? -1 : 1;
  ksin3 = (tsin3 < DZERO) ? -1 : 1;
  ksin4 = (tsin4 < DZERO) ? -1 : 1;
  
  

  tang = sh1461_s9ang(evec1,evec2,idim);
  tang1 = sh1461_s9ang(evec1,evec3,idim);
  tang2 = sh1461_s9ang(evec1,evec4,idim);
  tang3 = sh1461_s9ang(evec2,evec3,idim);
  tang4 = sh1461_s9ang(evec2,evec4,idim);
  
  

  tsin = ksin*sin(tang);
  tsin1 = ksin1*sin(tang1);
  tsin2 = ksin2*sin(tang2);
  tsin3 = ksin3*sin(tang3);
  tsin4 = ksin4*sin(tang4);
  
  

  *cro00 = (tl3*tsin1)/(tl2*tsin);
  *cro01 = (tl4*tsin2)/(tl2*tsin);
  *cro10 = -(tl3*tsin3)/(tl1*tsin);
  *cro11 = -(tl4*tsin4)/(tl1*tsin);
  
  *jstat = 0;
  
  return;
}

#if defined(SISLNEEDPROTOTYPES)
static void
  sh1461_s9chcoor(double eblend[],int iordblend,double epos[],
		  int iordpos,double ecrt1[],int iordcrt1,
		  double evec1[],double evec2[],double evec3[],
		  int idim,double ecrt2[],int *jordcrt2,int *jstat)
#else	       
static void sh1461_s9chcoor(eblend,iordblend,epos,iordpos,ecrt1,iordcrt1,
			    evec1,evec2,evec3,idim,ecrt2,jordcrt2,jstat)
     int iordblend,iordpos,iordcrt1,idim,*jordcrt2,*jstat;
     double evec1[],evec2[],evec3[];
     double eblend[],epos[],ecrt1[],ecrt2[];
#endif     
{
  int kstat;                       
  int ki;                          
  double tl1,tl2,tl3;              
  double tang1,tang2;              
  double tsin1,tsin2,tsin3;        
  double tmy,tmy1;                 
  double tlambda;                  
  double scoef[12];                
  double sc1[21],sc2[21];          
  double sblend2[4];               
  
  if (iordblend != 4 || iordcrt1 != 4) goto err002;   

  *jordcrt2 = iordblend + iordcrt1 - 1;
  
  

  tl1 = s6length(evec1,idim,&kstat);
  tl2 = s6length(evec2,idim,&kstat);
  tl3 = s6length(evec3,idim,&kstat);
  tang1 = sh1461_s9ang(evec1,evec2,idim);
  tang2 = sh1461_s9ang(evec2,evec3,idim);
  tsin1 = sin(tang1);
  tsin2 = sin(tang2);
  tsin3 = sin(tang1+tang2);
  tmy = - (tl3*tsin2)/(tl1*tsin1);
  tlambda = (tl3*tsin3)/(tl2*tsin1);
  tmy1 = tmy + (double)1.0;
  
  

  for (ki=0; ki<4; ki++) sblend2[ki] = tmy1*eblend[ki] - (double)1.0;
  
  

  for (ki=0; ki<4*idim; ki++)
    scoef[ki] = (double)4.0*(epos[idim+ki] - epos[ki]);

   

  sh1461_s9mult(eblend,scoef,4,idim,sc1,&kstat);

   

  sh1461_s9mult(sblend2,ecrt1,4,idim,sc2,&kstat);
  
  

  for (ki=0; ki<7*idim; ki++)
    ecrt2[ki] = (tlambda*sc1[ki] + sc2[ki]);
  
  *jstat = 0;
  goto out;
  
  

  err002 :
    *jstat = -2;
  goto out;
  
  out :
    return;
}


#if defined(SISLNEEDPROTOTYPES)
static void
  sh1461_s9mult(double eblend[],double ecoef[],int iord,
		int idim,double ecoefnew[],int *jstat)
#else	       
static void sh1461_s9mult(eblend,ecoef,iord,idim,ecoefnew,jstat)
     int iord,idim,*jstat;
     double eblend[],ecoef[],ecoefnew[];
#endif     
{
  int ki;   
  
  

  if (iord != 4) goto err001;
  
  for (ki=0; ki<idim; ki++)
    {
      

      ecoefnew[ki] = eblend[0]*ecoef[ki];
      ecoefnew[idim+ki] = (eblend[0]*ecoef[idim+ki] 
			   + eblend[1]*ecoef[ki])/(double)2.0;
      ecoefnew[2*idim+ki] = (eblend[0]*ecoef[2*idim+ki]
			     + (double)3.0*eblend[1]*ecoef[idim+ki]
			     + eblend[2]*ecoef[ki])/(double)5.0;
      ecoefnew[3*idim+ki] = (eblend[0]*ecoef[3*idim+ki] + eblend[3]*ecoef[ki]
			     + (double)9.0*(eblend[1]*ecoef[2*idim+ki] + 
					    eblend[2]*ecoef[idim+ki]))/(double)20.0;
      ecoefnew[4*idim+ki] = (eblend[1]*ecoef[3*idim+ki]
			     + (double)3.0*eblend[2]*ecoef[2*idim+ki]
			     + eblend[3]*ecoef[idim+ki])/(double)5.0;      
      ecoefnew[5*idim+ki] = (eblend[2]*ecoef[3*idim+ki] 
			     + eblend[3]*ecoef[2*idim+ki])/(double)2.0;
      ecoefnew[6*idim+ki] = eblend[3]*ecoef[3*idim+ki];
    }

  *jstat = 0;
  goto out;
  
  

  err001 :
    *jstat = -1;
  goto out;
  
  out :
    return;
}

#if defined(SISLNEEDPROTOTYPES)
static double
  sh1461_s9ang(double evec1[],double evec2[],int idim)
#else	       
static double sh1461_s9ang(evec1,evec2,idim)
     double evec1[];
     double evec2[];
     int    idim;
#endif     
                                     
{
  double tscpr,tang,tlength1,tlength2,tcos;
  int    kstat1,kstat2;
  
  tscpr = s6scpr(evec1,evec2,idim);
  
  tlength1 = s6length(evec1,idim,&kstat1);
  tlength2 = s6length(evec2,idim,&kstat2);
  
  if (!kstat1 || !kstat2)
    tang = DZERO;
  else
    {
      tcos = tscpr/(tlength1*tlength2);
      tcos = MIN((double)1.0,tcos);
      tcos = MAX(-(double)1.0,tcos);
      tang = acos(tcos);
    }
      
  return(tang);
}

#if defined(SISLNEEDPROTOTYPES)
static  void
  sh1461_s9comder(int ider1,int ider2,double ederprev[],int idim,
		  double aro00,double aro01,double aro10,
		  double aro11,double eder[],int *jstat)
#else	    
static void sh1461_s9comder(ider1,ider2,ederprev,idim,aro00,aro01,aro10,
			    aro11,eder,jstat)
     int ider1,ider2,idim,*jstat;
     double ederprev[],aro00,aro01,aro10,aro11,eder[];
#endif     
{
  int kj,kk,kh;            
  double t00,t01,t10,t11;  
  double tfac1,tfac2;      
  
  

  if (ider1 + ider2 != 2) goto err001;
  
  

  t00 = t10 = (double)1.0;
  for (kj=0; kj<ider1; kj++) t00 *= aro00;

  for (kj=0; kj<=ider1; kj++)
    {
      tfac1 = (ider1 > 0) ? (double)((kj % ider1) + 1) : (double)1.0;

      t01 = t11 = (double)1.0;
      for (kk=0; kk<ider2; kk++) t01*=aro01;

      for (kk=0; kk<=ider2; kk++)
	{
	  tfac2 = (ider2 > 0) ? (double)((kk % ider2) + 1) : (double)1.0;

	  for (kh=0; kh<idim; kh++)
	    eder[kh] += tfac1*tfac2*t00*t01*t10*t11*ederprev[(2-kj-kk)*idim+kh];
	      
	  if (aro01 != DZERO) t01 /= aro01;
	  else if (kk == ider2-1) t01 = (double)1.0;
	  t11 *= aro11;
	}
      if (aro00 != DZERO) t00 /= aro00;
      else if (kj == ider1-1) t00 = (double)1.0;
      t10 *= aro10;	  
    }
  
  *jstat = 0;
  goto out;
  
  

  err001 :
    *jstat = -1;
  goto out;
  
  out :
    return;
}

