/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1464

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
typedef void (*fshapeProc)(double [],double [],int,int,int *);
#else
typedef void (*fshapeProc)();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
  sh1464(fshapeProc fshape,SISLCurve *vboundc[],int icurv,
	 double etwist[],double etang[],double eder[],int *jstat)
#else      
void sh1464(fshape,vboundc,icurv,etwist,etang,eder,jstat)
     fshapeProc    fshape;
     double        etwist[],etang[],eder[];
     SISLCurve     *vboundc[];
     int           icurv,*jstat;
#endif     
{
  int kstat = 0;         
  int kder = 2;          
  int ki;                
  int kdim = 3;          
  double tlambda = (double)1.0/sqrt((double)5.0);           
  double tl1 = (double)2.0*tlambda*tan(PI/(double)5.0);     
  double tl2 = sin((double)0.3*PI);                         
  double tconst1 = tl1/(double)2.0 - tlambda;               
  double tconst2 = tl2 - tlambda;                           
  double sbar[5];        
  double sder[18];       

  

  sbar[0] = sbar[1] = sbar[2] = sbar[3] = sbar[4] = tlambda;
  
  

  sh1467(vboundc,etwist,kder,sbar,sder,&kstat);
  if (kstat < 0) goto error;
   
  

  for (ki=0; ki<kdim; ki++)
    {
      etang[ki] = sder[kdim+ki]*tconst1 + sder[2*kdim+ki]*tconst2;
      etang[kdim+ki] = -sder[kdim+ki]*tlambda + sder[2*kdim+ki]*tconst1;
      etang[2*kdim+ki] = sder[kdim+ki]*tconst1 - sder[2*kdim+ki]*tlambda;
      etang[3*kdim+ki] = sder[kdim+ki]*tconst2 + sder[2*kdim+ki]*tconst1;
      etang[4*kdim+ki] = sder[kdim+ki]*tconst2 + sder[2*kdim+ki]*tconst2;
    }
  
  

  fshape(sder,etang,kdim,icurv,&kstat);
  if (kstat < 0) goto error;
  
  

  memcopy(eder,sder,kdim,DOUBLE);
  memcopy(eder+kdim,etang+4*kdim,kdim,DOUBLE);
  memcopy(eder+2*kdim,etang,kdim,DOUBLE);
  
  

  for (ki=0; ki<kdim; ki++)
    {
      eder[3*kdim+ki] = sder[3*kdim+ki]*tconst2*tconst2 
	+ (double)2.0*sder[4*kdim+ki]*tconst2*tconst2 
	  + sder[5*kdim+ki]*tconst2*tconst2;
      eder[4*kdim+ki] = sder[3*kdim+ki]*tconst1*tconst2 
	+ sder[4*kdim+ki]*tconst2*(tconst1+tconst2)
	  + sder[5*kdim+ki]*tconst2*tconst2;
      eder[5*kdim+ki] = sder[3*kdim+ki]*tconst1*tconst1 
	- (double)2.0*sder[4*kdim+ki]*tconst1*tconst2 
	  + sder[5*kdim+ki]*tconst2*tconst2;
    }
  
  *jstat = 0;
  goto out;
  
  

 error:
  *jstat = kstat;
  goto out;
  
  out :
    return;
}

