/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1466

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
      sh1466(SISLCurve *ecurve[],double etwist[],int ider,double ebar[],
	     double eval[],int *jstat)
#else	 
void sh1466(ecurve,etwist,ider,ebar,eval,jstat)
     double etwist[],ebar[],eval[];
     SISLCurve *ecurve[];
     int ider,*jstat;
#endif     
{
  int kstat=0;         
  int ki,kj,kk,kh;     
  int kder;            
  int kleft = 0;       
  int kdim = 3;        
  int kwarn = 0;       
  int knmb;            
  int kl = 0;          
  double tpar1;        
  double tpar2;        
  double tlambi;       
  double tlambj;       
  double tlambk;       
  double salpha[3];    
  double sp[9];        
  double sstart[3];    
  double send[3];      
  double sint[3];      
  double spos1[27];    
  double sder1[27];    
  double spos2[27];    
  double sder2[27];    
  double scorn[9];     
  double scornder1[9]; 
  double scornder2[9]; 

  

  if (ider > 2) kwarn = 1;
  
  

  kder = ider;
  knmb = kdim*(ider+1);
  for (ki=0; ki<ider; ki++) kl += ider + 1;
  
  

  for (kh=0; kh<kl*kdim; kh++) eval[kh] = (double)0.0;
        
  

  for (ki=0; ki<3; ki++)
    {
      sstart[ki] = *(ecurve[2*ki] -> et + ecurve[2*ki] -> ik - 1);
      send[ki] = *(ecurve[2*ki] -> et + ecurve[2*ki] -> in);
      sint[ki] = send[ki] - sstart[ki];
    }

  
  
  for (ki=0; ki<3; ki++)
    {
      kj = (ki+1) % 3;
      kk = (ki+2) % 3;

      

      tlambi = ebar[ki];      
      tlambj = ebar[kj];
      tlambk = ebar[kk];
      
      

      tpar1 = ((double)1.0 - tlambj)*sstart[ki] + tlambj*send[ki];
      tpar2 = tlambk*sstart[kk] + ((double)1.0 - tlambk)*send[kk];
 
      

      s1221(ecurve[2*ki],kder,tpar1,&kleft,spos1+knmb*ki,&kstat);
      if (kstat < 0) goto error;
      
      s1221(ecurve[2*ki+1],kder,tpar1,&kleft,sder1+knmb*ki,&kstat);
      if (kstat < 0) goto error;

      
      
      s1221(ecurve[2*kk],kder,tpar2,&kleft,spos2+knmb*ki,&kstat);
      if (kstat < 0) goto error;
      
      s1221(ecurve[2*kk+1],kder,tpar2,&kleft,sder2+knmb*ki,&kstat);
      if (kstat < 0) goto error;

      

      s1221(ecurve[2*ki],0,sstart[ki],&kleft,scorn+ki*kdim,&kstat);
      if (kstat < 0) goto error;

      s1221(ecurve[2*kk+1],0,send[kk],&kleft,scornder1+ki*kdim,&kstat);
      if (kstat < 0) goto error;

      s1221(ecurve[2*ki+1],0,sstart[ki],&kleft,scornder2+ki*kdim,&kstat);
      if (kstat < 0) goto error;

      

      salpha[ki] = tlambi*tlambi*((double)3.0 - (double)2.0*tlambi +
				  (double)6.0*tlambj*tlambk);
      
      

      for (kh=0; kh<kdim; kh++)
	{
	  sp[ki*kdim+kh] = spos1[knmb*ki+kh] + tlambk*sder1[knmb*ki+kh] +
	    spos2[knmb*ki+kh] + tlambj*sder2[knmb*ki+kh] -
	      scorn[ki*kdim+kh] - tlambj*scornder1[ki*kdim+kh] 
		- tlambk*scornder2[ki*kdim+kh] -
		  tlambj*tlambk*etwist[ki*kdim+kh];
	  
	  eval[kh] += salpha[ki]*sp[ki*kdim+kh];
	}
    }
  
  if (ider >= 1)
    {
      

      double tl1,tl2,tl3;            
      double sd1alpha[3],sd2alpha[3];  
      double sd1p[9],sd2p[9];        
      
      

      tl1 = ebar[0];
      tl2 = ebar[1];
      tl3 = ebar[2];
      
      

      sd1alpha[0] = (double)6.0*tl1*((double)1.0 - tl1 - tl1*tl2 + 
				  (double)2.0*tl2*tl3);
      sd2alpha[0] = (double)6.0*tl1*tl1*(tl3 - tl2);

      sd1alpha[1] = (double)6.0*tl2*tl2*(tl3 - tl1);
      sd2alpha[1] = (double)6.0*tl2*((double)1.0 - tl2 - tl1*tl2 + 
				  (double)2.0*tl1*tl3);

      sd1alpha[2] = (double)6.0*tl3*(-(double)1.0 + tl3 + tl2*tl3 - 
				  (double)2.0*tl1*tl2);
      sd2alpha[2] = (double)6.0*tl3*(-(double)1.0 + tl3 + tl1*tl3 - 
				  (double)2.0*tl1*tl2);
      
      

      for (kh=0; kh<kdim; kh++)
	{
	  sd1p[kh] = (spos2[kdim+kh] + tl2*sder2[kdim+kh])*sint[2]
	    - sder1[kh] + scornder2[kh] + tl2*etwist[kh];
	  sd2p[kh] = (spos2[kdim+kh] + tl2*sder2[kdim+kh])*sint[2] 
	    + (spos1[kdim+kh] + tl3*sder1[kdim+kh])*sint[0]
	      + sder2[kh] - sder1[kh] - scornder1[kh]
		+ scornder2[kh] + (tl2 - tl3)*etwist[kh];

	  sd1p[kdim+kh] = -sder2[knmb+kh] + sder1[knmb+kh]
	    - (spos2[knmb+kdim+kh] + tl3*sder2[knmb+kdim+kh])*sint[0] 
	      - (spos1[knmb+kdim+kh] + tl1*sder1[knmb+kdim+kh])*sint[1] 
		+ scornder1[kdim+kh] - scornder2[kdim+kh]
		  + (tl1 - tl3)*etwist[kdim+kh];
	  sd2p[kdim+kh] = - sder2[knmb+kh]
	    - (spos1[knmb+kdim+kh] + tl1*sder1[knmb+kdim+kh])*sint[1]
	      + scornder1[kdim+kh] + tl1*etwist[kdim+kh];

	  sd1p[2*kdim+kh] = sder2[2*knmb+kh]
	    + (spos1[2*knmb+kdim+kh] + tl2*sder1[2*knmb+kdim+kh])*sint[2]
	      - scornder1[2*kdim+kh] - tl2*etwist[2*kdim+kh];
	  sd2p[2*kdim+kh] = sder1[2*knmb+kh]
	     - (spos2[2*knmb+kdim+kh] + tl1*sder2[2*knmb+kdim+kh])*sint[1]
	      - scornder2[2*kdim+kh] - tl1*etwist[2*kdim+kh];
	  
	  

	  for (ki=0; ki<3; ki++)
	    {
	      eval[kdim+kh] += sd1alpha[ki]*sp[ki*kdim+kh]
		+ salpha[ki]*sd1p[ki*kdim+kh];

	      eval[2*kdim+kh] += sd2alpha[ki]*sp[ki*kdim+kh]
		+ salpha[ki]*sd2p[ki*kdim+kh];
	    }
	}
      
      if (ider >= 2)
	{
	  double sd11alpha[3],sd12alpha[3],sd22alpha[3];  
	  double sd11p[9],sd12p[9],sd22p[9];  
	  
	  

	  

	  sd11alpha[0] = (double)6.0 - (double)12.0*tl1 
	    - (double)24.0*tl1*tl2 + (double)12.0*tl2*tl3;
	  sd12alpha[0] = tl1*((double)12.0 - (double)18.0*tl1
			      - (double)24.0*tl2);
	  sd22alpha[0] = -(double)12.0*tl1*tl1;
	  
	  sd11alpha[1] = -(double)12.0*tl2*tl2; 
	  sd12alpha[1] = tl2*((double)12.0 - (double)18.0*tl2
			      - (double)24.0*tl1);
	  sd22alpha[1] = (double)6.0 - (double)12.0*tl2 
	    - (double)24.0*tl1*tl2 + (double)12.0*tl1*tl3;

	  sd11alpha[2] = (double)6.0 - (double)12.0*tl3 
	    - (double)24.0*tl2*tl3 + (double)12.0*tl1*tl2;
	  sd12alpha[2] = (double)6.0 + (double)6.0*tl3*tl3
	    + (double)12.0*(-tl3 + tl1*tl2 - tl1*tl3 - tl2*tl3);
	  sd22alpha[2] = (double)6.0 - (double)12.0*tl3 
	    - (double)24.0*tl1*tl3 + (double)12.0*tl1*tl2;

	  

	  for (kh=0; kh<kdim; kh++)
	    {
	      sd11p[kh] = (spos2[2*kdim+kh]+ tl2*sder2[2*kdim+kh])*sint[2]*sint[2];
	      sd12p[kh] = ((spos2[2*kdim+kh] + tl2*sder2[2*kdim+kh])*sint[2]
			   + sder2[kdim+kh]*sint[2] - sder1[kdim+kh])*sint[0] 
			      + etwist[kh];
	      sd22p[kh] = ((spos2[2*kdim+kh] + tl2*sder2[2*kdim+kh])*sint[2]
			   + (double)2.0*sder2[kdim+kh])*sint[2]
			     + ((spos1[2*kdim+kh] + tl3*sder1[2*kdim+kh])*sint[0]
				- (double)2.0*sder1[kdim+kh])*sint[0]
				  + (double)2.0*etwist[kh];
	      
	      sd11p[kdim+kh] = ((spos2[knmb+2*kdim+kh] 
				 + tl3*sder2[knmb+2*kdim+kh])*sint[0]
				+ (double)2.0*sder2[knmb+kdim+kh])*sint[0]
				  + ((spos1[knmb+2*kdim+kh] 
				      + tl1*sder1[knmb+2*kdim+kh])*sint[1]
				     - (double)2.0*sder1[knmb+kdim+kh])*sint[1]
				       + (double)2.0*etwist[kdim+kh];
	      sd12p[kdim+kh] = ((spos1[knmb+2*kdim+kh] 
				 + tl1*sder1[knmb+2*kdim+kh])*sint[1]
				- sder1[knmb+kdim+kh])*sint[1]
				  + sder2[knmb+kdim+kh]*sint[0]
				  + etwist[kdim+kh];
	      sd22p[kdim+kh] = (spos1[knmb+2*kdim+kh]
				+ tl1*sder1[knmb+2*kdim+kh])*sint[1]*sint[1];
	      
	      sd11p[2*kdim+kh] = (spos1[2*knmb+2*kdim+kh]
				  + tl2*sder1[2*knmb+2*kdim+kh])*sint[2]*sint[2];
	      sd12p[2*kdim+kh] = -sder2[2*knmb+kdim+kh]*sint[1] 
		+ sder1[2*knmb+kdim+kh]*sint[2] - etwist[2*kdim+kh];
	      sd22p[2*kdim+kh] = (spos2[2*knmb+2*kdim+kh]
				+ tl1*sder2[2*knmb+2*kdim+kh])*sint[1]*sint[1];

	      

	      for (ki=0; ki<3; ki++)
		{
		  eval[3*kdim+kh] += sd11alpha[ki]*sp[ki*kdim+kh]
		    + (double)2.0*sd1alpha[ki]*sd1p[ki*kdim+kh]
		      + salpha[ki]*sd11p[ki*kdim+kh];
		  
		  eval[4*kdim+kh] += sd12alpha[ki]*sp[ki*kdim+kh]
		    + sd1alpha[ki]*sd2p[ki*kdim+kh] 
		      + sd2alpha[ki]*sd1p[ki*kdim+kh]
			+ salpha[ki]*sd12p[ki*kdim+kh];
		  
		  eval[5*kdim+kh] += sd22alpha[ki]*sp[ki*kdim+kh]
		    + (double)2.0*sd2alpha[ki]*sd2p[ki*kdim+kh]
		      + salpha[ki]*sd22p[ki*kdim+kh];
		}
	    }
	}
    }
  
  

  *jstat = kwarn;
  goto out;


  

  error :
    *jstat = kstat;
  goto out;

  out :
    return;
}


			      
	     



