/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1856

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void sh1856(SISLSurf *ps1,double epoint[],double edir[],int idim,
	    double aepsco,double aepsge,
	    int trackflag, int *jtrack, SISLTrack *** wtrack,
	    int *jpt,double **gpar,int **pretop,int *jcrv,
	    SISLIntcurve ***wcurve,int *jstat)
#else
void sh1856(ps1,epoint,edir,idim,aepsco,aepsge,
	    trackflag,jtrack,wtrack,jpt,gpar,pretop,jcrv,wcurve,jstat)
     SISLSurf     *ps1;
     double   epoint[];
     double   edir[];
     int      idim;
     double   aepsco;
     double   aepsge;
     int       trackflag;
     int       *jtrack;
     SISLTrack ***wtrack;
     int      *jpt;
     double   **gpar;
     int      **pretop;
     int      *jcrv;
     SISLIntcurve ***wcurve;
     int      *jstat;
#endif
{            
  double *nullp = SISL_NULL;
  int kstat = 0;           
  int kpos = 0;            
  int kdim;                
  double *spar = SISL_NULL;     
  double spoint[2];        
  double *snorm1 = SISL_NULL;   
  double *snorm2 = SISL_NULL;   
  SISLSurf *qs = SISL_NULL;         
  SISLPoint *qp = SISL_NULL;        
  SISLObject *qo1 = SISL_NULL;      
  SISLObject *qo2 = SISL_NULL;      
  SISLIntdat *qintdat = SISL_NULL;  
  int      ksurf=0;         
  SISLIntsurf **wsurf=SISL_NULL;    
  int      kdeg=2000;       
  SISLObject *track_obj=SISL_NULL;
  SISLSurf *qkreg=SISL_NULL; 

    

  if (ps1->cuopen_1 == SISL_SURF_PERIODIC ||
      ps1->cuopen_2 == SISL_SURF_PERIODIC)
  {
     

     make_sf_kreg(ps1,&qkreg,&kstat);
     if (kstat < 0) goto error;
   }
  else
    qkreg = ps1;
  
  
  
  if (!(track_obj = newObject (SISLSURFACE)))
    goto err101;
  track_obj->s1 = ps1;

  

  *jpt  = 0;
  *jcrv = 0;
  *jtrack = 0;

  if (idim != 2 && idim != 3) goto err105;
  if (idim != qkreg -> idim) goto err106;

  

  snorm1 = newarray(idim,double);
  snorm2 = newarray(idim,double);
  if (snorm1 == SISL_NULL || snorm2 == SISL_NULL) goto err101;

  if (idim == 3)
    {

      

      s6twonorm(edir,snorm1,snorm2,&kstat);
      if (kstat < 0) goto error;

      

      s1328(qkreg,epoint,snorm1,snorm2,idim,&qs,&kstat);
      if (kstat < 0) goto error;

      

      kdim      = 2;
      spoint[0] = spoint[1] = DZERO;
      if (!(qo2  = newObject(SISLPOINT))) goto err101;
      if (!(qp   = newPoint(spoint,kdim,1))) goto err101;
      qo2 -> p1 = qp;
    }
  else if (idim == 2)
    {

      

      snorm1[0] = edir[1];
      snorm1[1] = (-1)*edir[0];

      

      s1329(qkreg,epoint,snorm1,idim,&qs,&kstat);
      if (kstat < 0) goto error;

      

      kdim      = 1;
      spoint[0] = DZERO;
      if (!(qo2  = newObject(SISLPOINT))) goto err101;
      if (!(qp   = newPoint(spoint,kdim,1))) goto err101;
      qo2 -> p1 = qp;
    }

  

  if(!(qo1 = newObject(SISLSURFACE))) goto err101;
  qo1 -> s1 = qs;
  qo1 -> o1 = qo1;

  

  sh1761(qo1,qo2,aepsge,&qintdat,&kstat);
  if (kstat < 0) goto error;

  

  sh6degen(track_obj,track_obj,&qintdat,aepsge,&kstat);
  if (kstat < 0) goto error;

  
  int_join_per( &qintdat,track_obj,track_obj,nullp,kdeg,aepsge,&kstat);
  if (kstat < 0)
    goto error;

  
  if (trackflag && qintdat)
    {
      make_tracks (qo1, qo2, 0, nullp,
		   qintdat->ilist, qintdat->vlist,
		   jtrack, wtrack, aepsge, &kstat);
      if (kstat < 0)
	goto error;
    }

  

  if (qintdat)
    {
      hp_s1880(track_obj, track_obj, kdeg,
	       2,0,qintdat,jpt,gpar,&spar,pretop,jcrv,wcurve,&ksurf,&wsurf,&kstat);
      if (kstat < 0) goto error;
    }
  
  

  *jstat = 0;
  goto out;

  

 err101: *jstat = -101;
        s6err("sh1856",*jstat,kpos);
        goto out;

  

 err105: *jstat = -105;
        s6err("sh1856",*jstat,kpos);
        goto out;

  

 err106: *jstat = -106;
        s6err("sh1856",*jstat,kpos);
        goto out;

  

  error : *jstat = kstat;
        s6err("sh1856",*jstat,kpos);
        goto out;

 out:

  

  if (snorm1)  freearray(snorm1);
  if (snorm2)  freearray(snorm2);
  if (spar)    freearray(spar);
  if (qo1)     freeObject(qo1);
  if (qo2)     freeObject(qo2);
  if (qintdat) freeIntdat(qintdat);
  if (track_obj)
    {
       track_obj->s1 = SISL_NULL;
       freeObject(track_obj);
    }

  
    if (qkreg != SISL_NULL && qkreg != ps1) freeSurf(qkreg);

return;
}                                               


