/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1860

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
sh1860(SISLSurf *ps,double eview[],int idim,double aepsco,double aepsge,
	int trackflag, int *jtrack, SISLTrack *** wtrack,
	   int *jpt,double **gpar,int **pretop,int *jcrv,SISLIntcurve ***wcurve,int *jsurf,SISLIntsurf ***wsurf,int *jstat)
#else
void sh1860(ps,eview,idim,aepsco,aepsge,
	trackflag,jtrack,wtrack,jpt,gpar,pretop,jcrv,wcurve,jsurf,wsurf,jstat)
     SISLSurf     *ps;
     double   eview[];
     int      idim;
     double   aepsco;
     double   aepsge;
     int       trackflag;
     int       *jtrack;
     SISLTrack ***wtrack;
     int      *jpt;
     double   **gpar;
     int      **pretop;
     int      *jcrv;
     SISLIntcurve ***wcurve;
     int      *jsurf;
     SISLIntsurf ***wsurf;
     int      *jstat;
#endif
{            
  int kstat = 0;             
  int kpos = 0;              
  int kdim = 1;              
  int ki;                    
  double nmax=1.0;           
  double *spar = SISL_NULL;       
  double spoint[1];          
  SISLSurf *qs = SISL_NULL;           
  SISLPoint *qp = SISL_NULL;          
  SISLObject *qo1 = SISL_NULL;        
  SISLObject *qo2 = SISL_NULL;        
  SISLIntdat *qintdat = SISL_NULL;    
  int kdeg=1003;      
  double simpli[16];  
  double snorm[3];
  SISLObject *track_obj=SISL_NULL;
  SISLSurf *qkreg=SISL_NULL; 

    
  
  
  aepsge = (double)1.0e-09;

  if (ps->cuopen_1 == SISL_SURF_PERIODIC ||
      ps->cuopen_2 == SISL_SURF_PERIODIC)
  {
     

     make_sf_kreg(ps,&qkreg,&kstat);
     if (kstat < 0) goto error;
   }
  else
    qkreg = ps;
  
  
  
  if (!(track_obj = newObject (SISLSURFACE)))
    goto err101;
  track_obj->s1 = ps;
  
  (void)s6norm(eview,3,snorm,&kstat);
  
  simpli[0] = snorm[0];
  simpli[1] = snorm[1];
  simpli[2] = snorm[2];
  

  

  *jpt  = 0;
  *jcrv = 0;
  *jtrack = 0;

  if (idim != qkreg -> idim) goto err106;

  

  s1382(qkreg,eview,idim,&qs,&kstat);
  if (kstat < 0) goto error;

  

  
  
  {
     double min1,min2,max1,max2;
     int kleft=0;
     double qview[3];
     int kj;
     SISLSurf *qstmp=SISL_NULL;
     SISLSurf *qstmp_1d=SISL_NULL;
     
     
     min1 = s1792(qkreg->et1,qkreg->ik1,qkreg->in1);
     s1219(qkreg->et1,qkreg->ik1,qkreg->in1,&kleft,min1,&kstat);
     if (kstat < 0) goto error;
     max1 = qkreg->et1[kleft+1];
     min2 = s1792(qkreg->et2,qkreg->ik2,qkreg->in2);
     s1219(qkreg->et2,qkreg->ik2,qkreg->in2,&kleft,min2,&kstat);
     if (kstat < 0) goto error;
     max2 = qkreg->et2[kleft+1];
     
     s1001 (qkreg, min1, min2, max1, max2, &qstmp, &kstat);
     if (kstat < 0) goto error;
     for (ki=0;ki<3;ki++)  qview[ki] = DZERO;
     
     nmax = (double)1.0;
     for (ki=0;ki<3;ki++)
     {
	qview[ki] = (double)1.0;
	s1382(qstmp,qview,idim,&qstmp_1d,&kstat);
	if (kstat < 0) goto error;
	for(kj=0; kj<qstmp_1d->in1*qstmp_1d->in2;kj++)
	   nmax = max(fabs(qstmp_1d->ecoef[kj]),nmax);
	
	if (qstmp_1d) freeSurf(qstmp_1d);
	qstmp_1d  = SISL_NULL;
	qview[ki] = DZERO;
     }
     if (qstmp) freeSurf(qstmp);
     qstmp = SISL_NULL;
  }
  
     
  for(ki=0; ki<qs->in1*qs->in2;ki++)
     nmax = max(fabs(qs->ecoef[ki]),nmax);
  
  for(ki=0; ki<qs->in1*qs->in2;ki++)
     qs->ecoef[ki] /= nmax;
  

  if(!(qo1 = newObject(SISLSURFACE))) goto err101;
  qo1 -> s1 = qs;
  qo1 -> o1 = qo1;
  
  

  if(!(qo2 = newObject(SISLPOINT))) goto err101;
  spoint[0] = DZERO;
  if(!(qp = newPoint(spoint,kdim,1))) goto err101;
  qo2 -> p1 = qp;

  

  sh1761(qo1,qo2,aepsge,&qintdat,&kstat);
  if (kstat < 0) goto error;

  

  sh6degen(track_obj,track_obj,&qintdat,aepsge,&kstat);
  if (kstat < 0) goto error;

  
  
  spli_silh(&qintdat,track_obj,&kstat);
  if (kstat < 0) goto error;

  
  if (trackflag && qintdat)
  {

     
      refine_all (&qintdat, track_obj, track_obj, simpli, kdeg, aepsge, &kstat);
      if (kstat < 0)
	goto error;
  }
  
  
  int_join_per( &qintdat,track_obj, track_obj, simpli, kdeg,aepsge,&kstat);
  if (kstat < 0)
    goto error;

  
  
  if (trackflag && qintdat)
  {
      make_tracks (track_obj, track_obj, kdeg, simpli,
		   qintdat->ilist, qintdat->vlist, 
		   jtrack, wtrack, aepsge, &kstat);
      if (kstat < 0)
	goto error;
    }

  

  if (qintdat)
    {
      hp_s1880(track_obj, track_obj, kdeg,
	       2,0,qintdat,jpt,gpar,&spar,pretop,jcrv,wcurve,jsurf,wsurf,&kstat);
      if (kstat < 0) goto error;
    }
  

  

  *jstat = 0;
  goto out;

  

 err101: *jstat = -101;                
        s6err("sh1860",*jstat,kpos);
        goto out;

  

 err106: *jstat = -106;
        s6err("sh1860",*jstat,kpos);
        goto out;

  

  error : *jstat = kstat;
        s6err("sh1860",*jstat,kpos);
        goto out;

 out:

  

  if (spar)    freearray(spar);
  if (qo1)     freeObject(qo1);
  if (qo2)     freeObject(qo2);
  if (qintdat) freeIntdat(qintdat);
  if (track_obj)
    {
       track_obj->s1 = SISL_NULL;
       freeObject(track_obj);
    }

  
    if (qkreg != SISL_NULL && qkreg != ps) freeSurf(qkreg);
                                         
return;
}                                               


