/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH6IDFCROSS

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void 
      sh6idfcross(SISLIntdat *pintdat, SISLIntpt *vcross[], int *jncross,
		  int ipar1, int ipar2, int *jstat)
#else
void sh6idfcross(pintdat, vcross, jncross, ipar1, ipar2, jstat)
   SISLIntdat *pintdat;
   SISLIntpt  *vcross[];
   int        *jncross;
   int        ipar1;
   int        ipar2;
   int       *jstat;
#endif   
{ 
   int ki,kj;       
   int kpt;         
   int kpar1;       
   int kpar2;       
   double tdist;    
   SISLIntpt *pt;   
   SISLIntpt *qnext; 
   
 
   
   
   if (pintdat->ipoint < 4)
   {
      
      
      *jstat = 0;
      return;
   }
   
   
   
   if (*jncross == 4)
   {
      
      
      tdist = s6dist(vcross[0]->epar+ipar1,vcross[3]->epar+ipar1,ipar2);
      if (DEQUAL(tdist+(double)1.0,(double)1.0))
	 
	 
	 *jstat = 1;
      else
	 *jstat = 0;
      
      return;
   }
   
   
   
   kpt = (*jncross) - 1;
   pt = vcross[kpt];
   kpar1 = (kpt % 2 == 0) ? 0 : ipar1;
   kpar2 = (kpt % 2 == 0) ? ipar1 : ipar2;
   
   
   
   for (ki=0; ki<pintdat->ipoint; ki++)
   {
      qnext = pintdat->vpoint[ki];
      
      
      
      for (kj=0; kj<=kpt; kj++)
	 if (qnext == vcross[kj]) break;
      if (kj <= kpt) continue;
      
      
      
      tdist = s6dist(qnext->epar+kpar1,pt->epar+kpar1,kpar2);
      if (DEQUAL(tdist+(double)1.0,(double)1.0))
      {
	 
	 
	 kpt++;
	 vcross[kpt] = qnext;
	 (*jncross)++;
	 
	 
	 
	 sh6idfcross(pintdat,vcross,jncross,ipar1,ipar2,jstat);
	 if (*jstat == 1) return;  
	 
	 (*jncross)--;
	 kpt--;
      }
   }
   
   
   
   *jstat = 0;
   return;
}

