/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SHCHECKPUT

#include "sislP.h"                            


#if defined (SISLNEEDPROTOTYPES)
void
     shcheckput(SISLObject *po1,SISLIntdat **rintdat,
		SISLIntdat *pintdat,int inr,double apar,int *jstat)
#else
void shcheckput(po1,rintdat,pintdat,inr,apar,jstat)
     SISLObject *po1;
     SISLIntdat **rintdat;
     SISLIntdat *pintdat;
     int    inr;
     double apar;
     int    *jstat;
#endif
                                     
{

  int ktype;                    
  int kstat;                    
                  
  int kpos=0;                   
  int ki,kj;                    
  int kant;                     
  int knum;                     
  int *perm_arr = SISL_NULL;         
  double *spar   = SISL_NULL;        
  SISLIntpt **uintpt = SISL_NULL;    
  
  
  
  *jstat = 0;

  
    if (po1->iobj != SISLSURFACE) goto err192;
			       
  
  if (pintdat == SISL_NULL) goto out;
   
  
  
  kant = pintdat->vpoint[0]->ipar + 1;
  
  
  if (inr<0 || inr>=kant) goto err191;
  
  
  
  
  if ((uintpt = newarray(pintdat->ipoint,SISLIntpt *)) == SISL_NULL)
    goto err101;
  
  
  
  if ((perm_arr = newarray(pintdat->ipoint,int)) == SISL_NULL)
    goto err101;
  
  
  
  if ((spar = newarray(kant,double)) == SISL_NULL)
    goto err101;
  
  
  
  
  for (ki=0,knum=0; ki<pintdat->ipoint; ki++)
  {
     
     
     for(kj=0; kj<inr; kj++) spar[kj] = pintdat->vpoint[ki]->epar[kj];
     spar[kj] = apar;
     for(kj++; kj<kant; kj++) spar[kj] = pintdat->vpoint[ki]->epar[kj-1];
     
     
     ktype = shchecktype(po1,spar);
     if (ktype < 0) goto error;
     if (ktype == 1 || ktype == 3 || ktype == 4) 
     {
	
	uintpt[knum] = newIntpt(kant,spar,pintdat->vpoint[ki]->adist);
 	perm_arr[ki] = knum;
 	knum++;
     }
     else perm_arr[ki] = -1;
  }
  
  if (knum == 0) goto out;
		 
  
  
  for (ki=0; ki<knum; ki++)
    {
      s6idnpt(rintdat,&uintpt[ki],1,&kstat);
      if (kstat < 0) goto error;
    }
  
  
  
  for (ki=0; ki<pintdat->ipoint; ki++)
    if (pintdat->vpoint[ki]->pcurve != SISL_NULL)
      {
	for (kj=0;kj<pintdat->ipoint;kj++)
	  if (pintdat->vpoint[ki]->pcurve == pintdat->vpoint[kj])
	    break;
	
	if (kj == pintdat->ipoint) goto err190;
	
	if (perm_arr[ki] >=0 && perm_arr[kj] >= 0)
	   {
	   s6idcon(rintdat,&uintpt[perm_arr[ki]],&uintpt[perm_arr[kj]],&kstat);
	   if (kstat < 0) goto error;
           }
      }
  
  
  *jstat = knum;
  goto out;
  


err192: *jstat = -192;
        s6err("shcheckput",*jstat,kpos);
        goto out;


err191: *jstat = -191;
        s6err("shcheckput",*jstat,kpos);
        goto out;

err190: *jstat = -190;
        s6err("shcheckput",*jstat,kpos);
        goto out;


err101: *jstat = -101;
        s6err("shcheckput",*jstat,kpos);
        goto out;


error: *jstat = kstat;
        s6err("shcheckput",*jstat,kpos);
        goto out;

 out: if (uintpt     != SISL_NULL) freearray(uintpt);
      if (spar       != SISL_NULL) freearray(spar);
      if (perm_arr   != SISL_NULL) freearray(perm_arr);
}

