#include <synaps/arithm/QQ.h>
#include <synaps/mpol/xml.h>
#include <synaps/topology/TopSweep2d.h>
#include <synaps/base/io/geomview.h>

void usage (char* cmd)
{
  std::cout<<"usage:\n"<<cmd<<" file [-bZa] ..."<<std::endl;
  std::cout<<"\n  The file should contain the equation of an implicit curve, with (big) integer,"<<std::endl;
  std::cout<<"  in the format <polynomial>...</polynomial> (example: <polynomial>x^2+2*y^2-1</polynomial>)"<<std::endl;
  std::cout<<"options:\n";
  std::cout<<"   -b x0 x1 y0 y1   box where the topology is computed (default -10 10 -10 10)"<<std::endl;
  std::cout<<"   -N n             insert n regular sections between two critical values"<<std::endl;
  std::cout<<"   -a eps           maximal size of box at the end of the subdivision"<<std::endl;
}

int main(int argc, char **argv)
{
  typedef MPol<QQ>      mpol_t;
  typedef std::string   str_t;

  using std::cout;
  using std::endl;
  if(argc<2)
    {
      usage(argv[0]);
      return 0;
    }
  std::ifstream in(argv[1],std::ios::in);
  if(!in.good())
    {
      if(str_t(argv[1])!=str_t("-h"))
	{
	  std::cerr<<argv[0]<<": file not found"<<std::endl;
	  return 1;
	}
      usage(argv[0]);
      return 0;
    }
  Seq<mpol_t> lp;
  mpol_t p;
  while(xml::read(in,p,Variables("x y")))
    {
      std::cout<<"polynomial: "<<p<<"\n  "<<degree(p)<<" ["<<degree(p,0)<<","<<degree(p,1)<<"] "<<p.size()<<std::endl;     
      lp.push_back(p);
    }

  double eps=1e-3,a=1,a0=-10,b0=10,a1=-10,b1=10;
  int N=1;
  unsigned Z=1;
  for(unsigned c=2;c<argc;c++)
    {
      if(str_t(argv[c])==str_t("-N"))
	{
	  c++; N=atoi(argv[c]);
	  std::cout<<"N "<<N<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-Z"))
	{
	  c++; Z=atoi(argv[c]);
	  std::cout<<"Z "<<Z<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-a"))
	{
	  c++; a=atof(argv[c]);
	  std::cout<<"A "<<a<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-e"))
	{
	  c++; eps=atof(argv[c]);
	  std::cout<<"E "<<eps<<std::endl;
	}
      else if(str_t(argv[c])==str_t("-b"))
	{
	  c++; if(c<argc) a0=atof(argv[c]);
	  c++; if(c<argc) b0=atof(argv[c]);
	  a1=a0; b1=b0;
	  c++; if(c<argc) a1=atof(argv[c]);
	  c++; if(c<argc) b1=atof(argv[c]);

	}
      else if(str_t(argv[c])==str_t("-h"))
	{
	  usage(argv[0]);
	  return 0;
	}
    }


  TopSweep2d<double> mth(N);
  topology::point_graph<double> g;
  geomview::ostream os("tmp.off");
  os<<Color(0,0,1);
  for(unsigned i=0;i<lp.size();i++)
    {
      topology::assign(g, lp[i], mth);  os<<g;
    }
  if(lp.size()>1)
    {
      topology::assign(g, lp[0], lp[1], mth);
      os<<Color(1,0,0);
      os<<Width(0.1);
      os<<g;
    }
  os.view();


}

