<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Subdivision methods for implicit curves in 2D>

  In this section, we describe a subdivision method, for computing the
  topology of a curve <with|mode|math|\<cal-C\>>, defined by an implicit
  equation <with|mode|math|f(x,y)=0>, where
  <with|mode|math|f\<in\>\<bbb-Q\>[x,y]>. It uses the representation of
  polynomials in the Bernstein basis of a domain
  <with|mode|math|[a,b]\<times\>[c,d]>:

  <\equation*>
    f(x,y)=<big|sum><rsub|0\<leqslant\>i\<leqslant\>d<rsub|1>,0\<leqslant\>j\<leqslant\>d<rsub|2>>
    c<rsub|i,j> \ B<rsup|i><rsub|d<rsub|1>>(x;a,b)B<rsup|j><rsub|d<rsub|2>>(y;c,d),
  </equation*>

  where <with|mode|math|(B<rsup|i><rsub|d>(x;a,b))<rsub|i=0,\<ldots\>,d>> is
  the Bernstein basis in degree <with|mode|math|d>, on the interval
  <with|mode|math|[a,b]>. The polynomial <with|mode|math|f(x,y)> will be
  represented by its control coefficients
  <with|mode|math|\<b-c\>=(c<rsub|i,j>)<rsub|0\<leqslant\>i\<leqslant\>d<rsub|1>,0\<leqslant\>j\<leqslant\>d<rsub|2>><rsub|>>
  and the domain <with|mode|math|[a,b]\<times\>[c,d]>. It can be subdivided
  by applying the de Casteljau algorithm, in the <with|mode|math|x> and
  <with|mode|math|y> direction.

  The subdivision algorithm is based on a criterion for detecting the
  regularity of the curve <with|mode|math|\<cal-C\>>, in the domain
  <with|mode|math|[a,b]\<times\>[c,d]>.

  <\definition>
    We say that a \ domain <with|mode|math|D=[a,b]\<times\>[c,d]> is regular
    for <with|mode|math|\<cal-C\>>, if its topology in <with|mode|math|D> is
    uniquely determined from the intersection points of
    <with|mode|math|\<cal-C\>>, with the boundary
    <with|mode|math|\<partial\>D> of <with|mode|math|D>.
  </definition>

  Here is a property that we use to detect regular domains:

  <\proposition>
    If <with|mode|math|\<partial\><rsub|y>f(x,y)\<neq\>0> (resp.
    <with|mode|math|\<partial\><rsub|x>f(x,y)\<neq\>0>) in
    <with|mode|math|D=[a,b]\<times\>[c,d]>, then <with|mode|math|D> is
    regular for <with|mode|math|\<cal-C\>> .
  </proposition>

  The test is implemented by checking that the control coefficients of
  <with|mode|math|\<partial\><rsub|y>f> or
  <with|mode|math|\<partial\><rsub|y>f> on D have a constant sign, which
  implies the previous proposition.

  \ 

  <\notation>
    <name|Subdivision algorithm for implicit curves.>

    <name|input:> <with|mode|math|f(x,y)\<in\>\<bbb-Q\>[x,y]> defining the
    implicit curves <with|mode|math|C> and a domain
    <with|mode|math|D=[a,b]\<times\>[c,d]\<subset\>\<bbb-R\><rsup|2>>.

    <\itemize>
      <item>Initialize <with|mode|math|\<cal-L\>> with <with|mode|math|D>;

      <item>while <with|mode|math|\<cal-L\>\<neq\>\<emptyset\>>;

      <\itemize>
        <item>pick a domain <with|mode|math|D> in <with|mode|math|\<cal-L\>>
        and test if it is regular;

        <item>if <with|mode|math|D> is not regular not subdivide it and add
        the subdomains to <with|mode|math|\<cal-L\>>;

        <item>otherwise, connect the points on
        <with|mode|math|\<partial\>D\<cap\>\<cal-C\>> to get the topology of
        <with|mode|math|\<cal-C\>> in <with|mode|math|D>;
      </itemize>
    </itemize>

    <name|output:> a graph of points connected by segments, isotopic to
    <with|mode|math|\<cal-C\>>.
  </notation>

  <subsection|Implementation>

  <\code>
    void topology::assign(point_graph\<less\>C\<gtr\> & g,
    MPol\<less\>QQ\<gtr\>& p,\ 

    \ \ \ \ \ \ \ \ \ \ \ TopSbdBdg2d\<less\>C\<gtr\> mth,\ 

    \ \ \ \ \ \ \ \ \ \ \ C x0, C x1, C y0, C y1);
  </code>

  <\itemize>
    <item>This function computes a graph of points <verbatim|g> of type
    <verbatim|topology::point_graph\<less\>C\<gtr\>>, which is isotopic to
    the curve defined by the polynomial <verbatim|p>, in the box
    <with|mode|math|B=[x<rsub|0>,x<rsub|1>]\<times\>[y<rsub|0>,y<rsub|1>]>.
    The coordinates of the points in <verbatim|g> are of type <verbatim|C>.
    The implementation is designed for <verbatim|C=double>.

    <item>The input coefficients of <verbatim|p> are rational numbers. During
    the computation, they are rounded to <verbatim|double> number types. If
    the topology in the box can be certified from this approximation, a graph
    of points connecting the intersection of the curve with the boundary of
    <with|mode|math|B> is computed. Otherwise the box is subdivided into
    <with|mode|math|4> subboxes and the test is applied recursively.

    <item>If the curve has singular points, the subdivision will continue
    until the size of the box is smaller than the precision
    <with|mode|math|\<varepsilon\>> specified by the class
    <verbatim|TopSbdBdg2d\<less\>C\<gtr\>>. The default value is
    <with|mode|math|10<rsup|-3>>.
  </itemize>

  See: <verbatim|synaps/topology/TopSbdBdg2d.h>, <verbatim|TopSbdBdg>.

  <subsection|Example>

  <action|<with|mode|math|\<boxdot\>>|TopSbdBdg2d.cc>

  Here is a picture of such output:

  <\with|par-mode|center>
    <postscript|sbdv_implicit2d-fig1.eps|*2/8|||||>\ 
  </with>

  \;
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|1>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Subdivision
      methods for implicit curves> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>
    </associate>
  </collection>
</auxiliary>