/*********************************************************************
*      This file is part of the source code of SYNAPS library          *
*   Author(s):                                                       *
*              G.  Gatellier                                         *
*              B.  Mourrain                                          *
*              JP. Tecourt, GALAAD, INRIA                            *
**********************************************************************/
#ifndef synaps_topology_triangle_set_H
#define synaps_topology_triangle_set_H
/*********************************************************************/
#include <iostream>
#include <vector>
#include <synaps/init.h>
#include "synaps/topology/point.h"
#include <synaps/topology/triangle.h>
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace topology
{

template <class Point>
struct triangle_set
{
  typedef Point                                       point_t;
  typedef topology::triangle<point_t>                   triangle_t;
  typedef typename Point::coeff_t                     coeff_t;
  typedef std::vector<triangle_t>                     rep_t;
  typedef typename rep_t::iterator                    iterator;
  typedef typename rep_t::const_iterator              const_iterator;


  rep_t rep_;
  
  triangle_set():rep_() {}
  
  /*! Copy constructor. */
  triangle_set( const triangle_set & T ): rep_(T.rep_) {}
  
  /*! Constructor from a triangle. */
  triangle_set( const triangle_t & t1): rep_() 
    {
      rep_.push_back(t1);
    }

  /*! Insert a triangle in the triangle set. */
  triangle_set<point_t> & insert( const triangle_t & t1)
    {
      rep_.push_back(t1); return *this;
    }  

  iterator       begin()        {return rep_.begin();}
  const_iterator begin() const  {return rep_.begin();}
  iterator       end()          {return rep_.end();}
  const_iterator end()   const  {return rep_.end();}

  int size() {return (int)rep_.size();}

  void resize( int n ) {return rep_.resize(n);}

  triangle_t & operator[] (unsigned i)       {return rep_[i];}
  triangle_t   operator[] (unsigned i) const {return rep_[i];}

};


/*! Output stream. */
template <class Point>
std::ostream & 
operator<<( std::ostream & os, const topology::triangle_set<Point> & t )
{
  os << "TriangleSet("; topology::print(os,t);os << ")";
  return os;
}

template <class Point>
std::ostream & 
print( std::ostream& os, const topology::triangle_set<Point> & t ) 
{
  typedef typename topology::triangle_set<Point>::const_iterator const_iterator; 
  const_iterator it=t.begin();
  os <<(*it); ++it;
  for( ; it !=s.end(); it++) os<<";"<<*it;
  return os; 
}

} // namespace topology
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/*********************************************************************/
#endif // synaps_topology_triangle_set_H
