<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Sturm sequences>

  Sturm sequences for two polynomials <with|mode|math|f(x), g(x)\<in\>
  \<bbb-K\>[x]> are sequences of polynomials defined as follows

  <\equation*>
    R<rsub|0>=f, R<rsub|1>=g, R<rsub|2>, \ \<ldots\> \<ldots\> ,R<rsub|N>
  </equation*>

  such that <with|mode|math|R<rsub|i>(x)=0> implies that for the first term
  such that<with|mode|math| R<rsub|i-k>(x)\<neq\>0> and
  <with|mode|math|R<rsub|i+l>(x)\<neq\>0>, we have
  <with|mode|math|R<rsub|i-k>(x)R<rsub|i+l>(x) \<less\>0>, we. One can chose
  for instance

  <\eqnarray*>
    <tformat|<table|<row|<cell| R<rsub|i+1> = - \<alpha\><rsub|i+1>
    rem*(R<rsub|i-1>,R<rsub|i>)>|<cell|>|<cell|>>>>
  </eqnarray*>

  where <with|mode|math|rem(p,q)> is the remainder in the Euclidean division
  of a polynomial <with|mode|math|p> by <with|mode|math|q>, and
  <with|mode|math|\<alpha\><rsub|i>> is a positive number.\ 

  <subsection|The class computing the Sturm sequence>

  The corresponding class used to store the result is\ 

  <\code>
    template\<less\>class C, class UPOL=UPolDse\<less\>C\<gtr\>, class
    SEQ=std::vector\<less\>UPOL\<gtr\> \<gtr\>\ 

    \ \ \ SturmSeq
  </code>

  which is parameterised by

  <\itemize>
    <item><verbatim|C>, the type of coefficients of the polynomials

    <item><verbatim|UPOL>, the type of univariate polynomials. The default
    value is <verbatim|UPolDse\<less\>C\<gtr\>>.

    <item><verbatim|SEQ>, the type of the container used to store the
    sequence. The default value is <verbatim|std::vector\<less\>UPOL\<gtr\>>.
  </itemize>

  \ Several scheme of construction of such type of sequences are available:

  <\itemize-dot>
    <item><verbatim|EUCLIDEAN> is the usual Sturm sequence where
    <with|mode|math|\<alpha\><rsub|i>=1>,

    <item><verbatim|HABICHT> is the Sturm Habicht sequence, which compute
    sub-resultants.
  </itemize-dot>

  Here is an example of construction of a Sturm Habicht sequence of two
  polynomials:

  <\code>
    UPolDse\<less\>ZZ\<gtr\> p,q; SturmSeq\<less\>ZZ\<gtr\> s(p,q,HABICHT());
  </code>

  <\itemize>
    <item>The euclidean construction requires coefficients coefficient in a
    field.

    <item>The Sturm-Habicht construction can be applied with coefficients in
    a ring (for instance integers, univariate polynomials,...)
  </itemize>

  See: <verbatim|synaps/upol/SturmSeq.h>

  <subsection|Sign variations>

  <\code>
    int variation(const SturmSeq\<less\>C,UPOL,S\<gtr\>& s, const R& x)
  </code>

  <\itemize>
    <item>This function computes the number of sign variation of the sequence
    <verbatim|s> at the value <verbatim|x>.

    <item>The number type used during this computation is <verbatim|R>.

    <item>It is based on the function <verbatim|UPOLDAR::sign_at>.
  </itemize>

  <subsection|Example>

  File: <verbatim|SturmSeq.cc>
</body>

<\initial>
  <\collection>
    <associate|language|french>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|1>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Sturm
      sequences> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>The class
      <with|font-family|<quote|tt>|language|<quote|verbatim>|SturmSeq>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Sign variations
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>