<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Quotient modulo a univariate polynomial>

  <index|Multivariate polynomial>The quotient modulo a polynomial is
  implemented as a class <verbatim|upoly>, which provides the operations in
  this quotient algebra:

  <\quote-env>
    <\code>
      namespace quotient {

      \ \ \ template\<less\>class C, class P=UPolDse\<less\>C\<gtr\>, class
      R=upol::horner\<less\>C\<gtr\> \<gtr\>

      \ \ \ struct upoly;

      }
    </code>
  </quote-env>

  <\itemize>
    <item>The type of coefficients is <verbatim|C>.

    <item>The interface type of polynomials is <verbatim|P>. The default is
    the type <verbatim|UPolDse\<less\>C\<gtr\>>.

    <item>The type for the internal representation is <verbatim|R>. By
    default, it is <verbatim|<verbatim|>upol::horner\<less\>C\<gtr\>>.
  </itemize>

  This class provides the following functions

  <\code>
    \ void assign(R& r, const R & a);\ 

    \ void assign(R& r, const P & a);

    \;

    \ void add(R& r, const R & a, const R& b);\ 

    \ void add(R& r, const R & a);\ 

    \ void sub(R& r, const R & a, const R& b);\ 

    \ void sub(R& r, const R & a);\ 

    \ void mul(R& r, const R & a, const R& b);\ 

    \ void mul(R& r, const R & a, const C& b);\ 

    \ void div(R& r, const R & a, const C& b);

    \ void square(R& r, const R & a);

    \;

    \ void print(std::ostream & os, const R & a) \ 
  </code>

  which implements the arithmetic operations of the polynomials of type
  <verbatim|R> modulo a given polynomial of type <verbatim|P>.\ 

  In order to define the polynomial modulo which we compute, we use the
  constructor

  \;

  <code| \ \ upoly(const P & p); >

  <subsection|Example>

  <\code>
    #include \<less\>synaps/upol/ModUPol.h\<gtr\>\n
    \ \ \ \ \ \ \ \ \ \ \ \ \ 

    typedef UPolDse\<less\>double\<gtr\> pol_t;

    int main()\ 

    {\ 

    \ \ using namespace std;\ 

    \ \ quotient::upoly\<less\>double, pol_t\<gtr\>
    modpol(pol_t("x0^3-2*x0^2"));\ 

    \ \ upol::horner\<less\>double\<gtr\> p,q,r; \ \ 

    \ \ modpol.assign(p,"x0^2-2*x0+1");\ 

    \ \ modpol.assign(q,"x0-1");

    \ \ cout \<less\>\<less\>"p="; modpol.print(cout,p);cout \<less\>\<less\>
    endl;

    \ \ cout \<less\>\<less\>"q="; modpol.print(cout,q);cout \<less\>\<less\>
    endl; \ 
  </code>

  <\equation*>
    p=x<rsup|2>-2 x +1; q= x -1;
  </equation*>

  <\code>
    \ \ modpol.add(r,p,q); modpol.print(cout,r);cout \<less\>\<less\> endl;
  </code>

  <\equation*>
    p+q\<equiv\>x<rsup|2>-x-1
  </equation*>

  <\code>
    \ \ modpol.mul(r,p,q); modpol.print(cout,r);cout \<less\>\<less\> endl;
  </code>

  <\equation*>
    p q \<equiv\> -x<rsup|2>+3 x -1
  </equation*>

  <\code>
    }
  </code>

  <subsection|Implementation>

  See: <verbatim|synaps/upol/ModUPol.h>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1|?>>
    <associate|auto-3|<tuple|1.1|?>>
    <associate|auto-4|<tuple|1.2|?>>
    <associate|auto-5|<tuple|1.3|?>>
    <associate|auto-6|<tuple|1.4|?>>
    <associate|auto-7|<tuple|1.5|?>>
    <associate|idx-1|<tuple|1|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-3|<tuple|1.2|?>>
    <associate|toc-4|<tuple|1.3|?>>
    <associate|toc-5|<tuple|1.4|?>>
    <associate|toc-6|<tuple|1.5|?>>
    <associate|toc-7|<tuple|1.6|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|idx>
      <tuple|<tuple|Multivariate polynomial>|<pageref|auto-2>>
    </associate>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Quotient
      modulo a univariate polynomial> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>