/*********************************************************************
*    This file is part of the source code of SYNAPS library.         *
*    Author(s): B. Mourrain, GALAAD, INRIA                           *
**********************************************************************/
#ifndef synaps_upol_xml_H_
#define synaps_upol_xml_H_
//--------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/base/xml.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
template<typename C,typename R> struct UPolDse;
//--------------------------------------------------------------------
namespace xml 
{
  //  using namespace SYNAPS;

  /// Read a polynomial, enclosed in @polynomial@ tags.
  template<class ISTREAM, typename T>
  bool read_upol(ISTREAM& is, T& p)
  {
    if(skip(is,begin_polynomial))
      {
        std::string s; char c;
	while((c=is.get()) != EOF && c != '<') 
	  {
	    if(c != ' ' && c != '\n') s+=c;
	  }
	s+='\0';
	p = T(s.data());
	return true;
      }
    else 
      {
	p=T(0); return false;
      }
  }

  /// Read a univariate polynomial, enclosed in @polynomial@ tags.
  template<class ISTREAM, typename C,typename R>
  bool read(ISTREAM& is, UPolDse<C,R>& p)
  {
    return read_upol(is,p);
  }

  /// Print a univariate polynomial, with its @polynomial@ tags.
  template<class OSTREAM, typename C,typename R>
  OSTREAM& print(OSTREAM& os, const UPolDse<C,R>& p)
  {
    xml::print(os,p,"polynomial");
    return os;
  }

}
//----------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
#endif //synaps_upol_xml_H_
