
#ifndef _SYNAPS_SOLVE_STURMABERTH_H_
#define _SYNAPS_SOLVE_STURMABERTH_H_


#include <synaps/init.h>
#include <synaps/arithm/RealOf.h>
#include <synaps/usolve/solver_base.h>
#include <synaps/usolve.h>
#include <synaps/usolve/sturm/solve_sturm_aberth.h>
#include <synaps/usolve/sturm/common.h>


__BEGIN_NAMESPACE_SYNAPS

template < typename NT >
struct SturmAberth : public solver_base
{
  
  typedef NumberTraits<NT>             NTR;
  
  typedef typename NTR::RT             RT;
  typedef typename NTR::FT             FT;
  typedef typename NTR::XT             XT;
  typedef RR                           RL;
  
  typedef typename NTR::FIT            FIT;
  typedef typename NTR::XIT            XIT;
  typedef typename NTR::XIT            RIT;
  //  typedef typename NTR::RIT            RIT;
  

  typedef UPolDse<RT>    Poly;
  typedef UPolDse<RT>    RT_Poly;
  typedef UPolDse<FT>    FT_Poly;

  typedef UPolDse<XT>    FT_Poly;
  typedef UPolDse<RL>    RL_Poly;

  typedef ALGEBRAIC::root_of<RT, Poly>   RO_t;
  typedef ALGEBRAIC::root_of<RT, Poly>   solution_type;


  typedef SturmAberth    self_t;

};


template < typename NT >
Seq
< 
  typename SturmAberth<NT>::RO_t
> 
solve( const typename SturmAberth<NT>::Poly& f,
       SturmAberth<NT> )
{
  typedef SturmAberth<NT>    K;
  typedef typename K::RO_t          RO_t;

  Seq<RO_t> sol;
  ALGEBRAIC::solve_sturm_aberth(f, std::back_inserter(sol.rep()), K());
  
  std::stable_sort( sol.begin(), sol.end(), ALGEBRAIC::Refine_compare());
    
  return sol;
}

__END_NAMESPACE_SYNAPS

#endif // _SYNAPS_SOLVE_STURMABERTH_H_

    
