
#ifndef SYNAPS_ARITHM_ALGEBRAIC_UTIL_H
#define SYNAPS_ARITHM_ALGEBRAIC_UTIL_H


#include <ext/algorithm>

#include <synaps/init.h>
#include <synaps/arithm/Abs.h>


__ALGEBRAIC_BEGIN_NAMESPACE


template < typename POLY >  inline
void
make_lcoeff_positive(POLY& p)
{
	typedef typename POLY::value_type    RT;
	if (UPOLDAR::lcoeff(p) < RT(0))
	  std::transform(p.begin(), p.end(), p.begin(), std::negate<RT>());
	return;
}


template < typename POLY >
struct Construct_poly_func {

  typedef typename POLY::value_type   RT;

  POLY operator()(const RT& a0, const RT& a1)
  {
	POLY f(2, AsSize());
	f[0] = a0;
	f[1] = a1;
	return f;
  }

  POLY operator()(const RT& a0, const RT& a1, const RT& a2)
  {
	POLY f(3, AsSize());
	f[0] = a0;
	f[1] = a1;
	f[2] = a2;
	return f;
  }

  POLY operator()(const RT& a0, const RT& a1, const RT& a2, const RT& a3)
  {
	POLY f(4, AsSize());
	f[0] = a0;
	f[1] = a1;
	f[2] = a2;
	f[3] = a3;
	return f;
  }

  POLY operator()(const RT& a0, const RT& a1, const RT& a2, const RT& a3, const RT& a4)
  {
	POLY f(5, AsSize());
	f[0] = a0;
	f[1] = a1;
	f[2] = a2;
	f[3] = a3;
	f[4] = a4;
	return f;
  }
};


template < typename POLY > inline
void construct_poly( const typename POLY::value_type& a0,
	                 const typename POLY::value_type& a1,
					 POLY& f)
{ f = Construct_poly_func<POLY>()(a0, a1); }



template < typename POLY > inline
void construct_poly( const typename POLY::value_type& a0,
	                 const typename POLY::value_type& a1,
	                 const typename POLY::value_type& a2,
					 POLY& f)
{ f = Construct_poly_func<POLY>()(a0, a1, a2); }


template < typename POLY > inline
void construct_poly( const typename POLY::value_type& a0,
	                 const typename POLY::value_type& a1,
	                 const typename POLY::value_type& a2,
	                 const typename POLY::value_type& a3,
					 POLY& f)
{ f = Construct_poly_func<POLY>()(a0, a1, a2, a3); }


template < typename POLY > inline
void construct_poly( const typename POLY::value_type& a0,
	                 const typename POLY::value_type& a1,
	                 const typename POLY::value_type& a2,
	                 const typename POLY::value_type& a3,
	                 const typename POLY::value_type& a4,
					 POLY& f)
{ f = Construct_poly_func<POLY>()(a0, a1, a2, a3, a4); }



__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_ARITHM_ALGEBRAIC_UTIL_H

