

#ifndef SYNAPS_ALGEBRAIC_KERNEL_4_H
#define SYNAPS_ALGEBRAIC_KERNEL_4_H


#include <synaps/config.h>
#include <synaps/init.h>
#include <synaps/arithm/root_of.h>
#include <synaps/bignum.h>


#include <synaps/arithm/algebraic/AK/global_functions_4.h>
#include <synaps/arithm/algebraic/AK/function_objects_4.h>


__BEGIN_NAMESPACE_SYNAPS


template < typename NT,
	   typename SolverType = Small_degree<NT> >
struct Algebraic_kernel_4 
{
  typedef Algebraic_kernel_4<NT>             Self;
  typedef SolverType                         Solver_type;
  


  typedef typename Solver_type::RT           RT;
  typedef typename Solver_type::FT           FT;
  typedef typename Solver_type::RO_t         Root_of_4;

  typedef typename Solver_type::MPoly_2_2    Polynomial_2_2;
   

  typedef SYNAPSi::Construct_polynomial_2_2<Self>   Construct_polynomial_2_2;
  typedef SYNAPSi::Solve<Self>                      Solve;
  typedef SYNAPSi::Sign_at<Self>                    Sign_at;
  
  

  Construct_polynomial_2_2
  construct_polynomial_2_2_object() const
  {
    return Construct_polynomial_2_2();
  }


  Solve
  solve_object() const 
  {
    return Solve();
  }
  
Sign_at
sign_at_object() const 
  {
    return Sign_at();
  }
  

};


__END_NAMESPACE_SYNAPS




namespace CGAL
{
  template < typename RT >
  double 
  to_double( const SYNAPS::ALGEBRAIC::root_of<RT>& r)
  {
    return SYNAPS::ALGEBRAIC::to_double(r);
  }


 
  template < typename RT >
  bool
  is_valid( const SYNAPS::ALGEBRAIC::root_of<RT>& r)
  {
    return SYNAPS::ALGEBRAIC::is_valid(r);
  }
 
  
 
  template < typename RT >
  typename SYNAPS::ALGEBRAIC::root_of<RT>
  square( const SYNAPS::ALGEBRAIC::root_of<RT>& r)
  {
    return SYNAPS::ALGEBRAIC::square<RT>(r);
  }

} // namespace CGAL

#endif // SYNAPS_ALGEBRAIC_KERNEL_4_H
