
#ifndef SYNAPS_ARITHM_ALGEBRAIC_ROOT_H
#define SYNAPS_ARITHM_ALGEBRAIC_ROOT_H

#include <synaps/arithm/root_of.h>
#include <synaps/arithm/algebraic/Root.h>


__ALGEBRAIC_BEGIN_NAMESPACE


template < typename K >
struct root{
  typedef typename K::RO_t       exact_RO_t;
  typedef typename K::apx_RO_t   approx_RO_t;


  exact_RO_t    exact_;
  approx_RO_t   approx_;

  root(const exact_RO_t& exact, const approx_RO_t& approx)
	: exact_(exact), approx_(approx)
	{ }

  const exact_RO_t&  exact()  const { return exact_; }
  const approx_RO_t& approx() const { return approx_; }

};


template < typename K >  inline
int compare(const root<K>& a, const root<K>& b)
{
  int result = compare(a.approx(), b.approx());
  if (result != UNKNOWN) {
	// cout << "SUCCESS\n";
	return result;
  }
  return compare(a.exact(), b.exact());
}

template < typename K >  inline
int compare(const root<K>& a, const typename K::RT& x)
{
  int result = compare(a.approx(), x);
  if (result != UNKNOWN) {
	// cout << "SUCCESS\n";
	return result;
  }
  return compare(a.exact(), x);
}


template < typename K >  inline
int compare(const root<K>& a, const typename K::FT& x)
{
  int result = compare(a.approx(), x);
  if (result != UNKNOWN) {
	// cout << "SUCCESS\n";
	return result;
  }
  return compare(a.exact(), x);
}

__ALGEBRAIC_END_NAMESPACE




__BEGIN_NAMESPACE_SYNAPS


template < typename K >
std::ostream& operator<<(std::ostream& output, const ALGEBRAIC::root<K>& r)
{
  ALGEBRAIC::print(output, r.exact());
  output << ", approx = " << ALGEBRAIC::to_double(r.exact());
  return output;
}


template < typename K >
bool
operator>(const ALGEBRAIC::root<K>& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == LARGER; }

template < typename K >
bool
operator>=(const ALGEBRAIC::root<K>& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != SMALLER; }

template < typename K >
bool
operator<(const ALGEBRAIC::root<K>& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == SMALLER; }

template < typename K >
bool
operator<=(const ALGEBRAIC::root<K>& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != LARGER; }

template < typename K >
bool
operator==(const ALGEBRAIC::root<K>& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == EQUAL; }

template < typename K >
bool
operator!=(const ALGEBRAIC::root<K>& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != EQUAL; }


//-------------------------------------------------------------------

template < typename K >
bool
operator>(const ALGEBRAIC::root<K>& a, const typename K::RT b)
{ return ALGEBRAIC::compare(a, b) == LARGER; }

template < typename K >
bool
operator>=(const ALGEBRAIC::root<K>& a, const typename K::RT b)
{ return ALGEBRAIC::compare(a, b) != SMALLER; }

template < typename K >
bool
operator<(const ALGEBRAIC::root<K>& a, const typename K::RT b)
{ return ALGEBRAIC::compare(a, b) == SMALLER; }

template < typename K >
bool
operator<=(const ALGEBRAIC::root<K>& a, const typename K::RT b)
{ return ALGEBRAIC::compare(a, b) != LARGER; }

template < typename K >
bool
operator==(const ALGEBRAIC::root<K>& a, const typename K::RT b)
{ return ALGEBRAIC::compare(a, b) == EQUAL; }

template < typename K >
bool
operator!=(const ALGEBRAIC::root<K>& a, const typename K::RT b)
{ return ALGEBRAIC::compare(a, b) != EQUAL; }


template < typename K >
bool
operator>(const typename K::RT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == LARGER; }

template < typename K >
bool
operator>=(const typename K::RT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != SMALLER; }

template < typename K >
bool
operator<(const typename K::RT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == SMALLER; }

template < typename K >
bool
operator<=(const typename K::RT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != LARGER; }

template < typename K >
bool
operator==(const typename K::RT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == EQUAL; }

template < typename K >
bool
operator!=(const typename K::RT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != EQUAL; }

//-------------------------------------------------------------------


template < typename K >
bool
operator>(const ALGEBRAIC::root<K>& a, const typename K::FT b)
{ return ALGEBRAIC::compare(a, b) == LARGER; }

template < typename K >
bool
operator>=(const ALGEBRAIC::root<K>& a, const typename K::FT b)
{ return ALGEBRAIC::compare(a, b) != SMALLER; }

template < typename K >
bool
operator<(const ALGEBRAIC::root<K>& a, const typename K::FT b)
{ return ALGEBRAIC::compare(a, b) == SMALLER; }

template < typename K >
bool
operator<=(const ALGEBRAIC::root<K>& a, const typename K::FT b)
{ return ALGEBRAIC::compare(a, b) != LARGER; }

template < typename K >
bool
operator==(const ALGEBRAIC::root<K>& a, const typename K::FT b)
{ return ALGEBRAIC::compare(a, b) == EQUAL; }

template < typename K >
bool
operator!=(const ALGEBRAIC::root<K>& a, const typename K::FT b)
{ return ALGEBRAIC::compare(a, b) != EQUAL; }


template < typename K >
bool
operator>(const typename K::FT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == LARGER; }

template < typename K >
bool
operator>=(const typename K::FT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != SMALLER; }

template < typename K >
bool
operator<(const typename K::FT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == SMALLER; }

template < typename K >
bool
operator<=(const typename K::FT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != LARGER; }

template < typename K >
bool
operator==(const typename K::FT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) == EQUAL; }

template < typename K >
bool
operator!=(const typename K::FT& a, const ALGEBRAIC::root<K>& b)
{ return ALGEBRAIC::compare(a, b) != EQUAL; }

//-------------------------------------------------------------------


__END_NAMESPACE_SYNAPS



#endif // SYNAPS_ARITHM_ALGEBRAIC_ROOT_H

