// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)


#ifndef SYNAPS_USOLVE_ALGEBRAIC_COMPARISON_COMPARE_FT_OPERATORS_HH
#define SYNAPS_USOLVE_ALGEBRAIC_COMPARISON_COMPARE_FT_OPERATORS_HH


#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/usolve/algebraic/root_of.h>
#include <synaps/usolve/algebraic/Comparison.h>

//__BEGIN_NAMESPACE_::SYNAPS


// RT a, Root_of b
template < typename RT,
	       typename P >  inline
bool operator<(const typename ::SYNAPS::NumberTraits<RT>::FT& lhs,
	           const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) == ::SYNAPS::SMALLER; }

template < typename RT,
	       typename P >  inline
bool operator<=(const typename ::SYNAPS::NumberTraits<RT>::FT& lhs,
	            const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) != ::SYNAPS::LARGER; }

template < typename RT,
	       typename P >  inline
bool operator>(const typename ::SYNAPS::NumberTraits<RT>::FT& lhs,
	           const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) == ::SYNAPS::LARGER; }

template < typename RT,
	       typename P >  inline
bool operator>=(const typename ::SYNAPS::NumberTraits<RT>::FT& lhs,
	            const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) != ::SYNAPS::SMALLER; }


template < typename RT,
	       typename P >  inline
bool operator==(const typename ::SYNAPS::NumberTraits<RT>::FT& lhs,
	            const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) == ::SYNAPS::EQUAL; }

template < typename RT,
	       typename P >  inline
bool operator!=(const typename ::SYNAPS::NumberTraits<RT>::FT& lhs,
	            const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) != ::SYNAPS::EQUAL; }



// Root_of a, RT b
template < typename RT,
	       typename P >  inline
bool operator<(const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& lhs,
	           const typename ::SYNAPS::NumberTraits<RT>::FT& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) == ::SYNAPS::SMALLER; }

template < typename RT,
	       typename P >  inline
bool operator<=(const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& lhs,
	            const typename ::SYNAPS::NumberTraits<RT>::FT& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) != ::SYNAPS::LARGER; }

template < typename RT,
	       typename P >  inline
bool operator>(const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& lhs,
	           const typename ::SYNAPS::NumberTraits<RT>::FT& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) == ::SYNAPS::LARGER; }

template < typename RT,
	       typename P >  inline
bool operator>=(const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& lhs,
	            const typename ::SYNAPS::NumberTraits<RT>::FT& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) != ::SYNAPS::SMALLER; }

template < typename RT,
	       typename P >  inline
bool operator==(const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& lhs,
	            const typename ::SYNAPS::NumberTraits<RT>::FT& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) == ::SYNAPS::EQUAL; }

template < typename RT,
	       typename P >  inline
bool operator!=(const ::SYNAPS::ALGEBRAIC::root_of<RT, P>& lhs,
	            const typename ::SYNAPS::NumberTraits<RT>::FT& rhs)
{ return ::SYNAPS::ALGEBRAIC::compare(lhs, rhs) != ::SYNAPS::EQUAL; }


//__END_NAMESPACE_::SYNAPS

#endif // SYNAPS_USOLVE_ALGEBRAIC_COMPARISON_COMPARE_FT_OPERATORS_HH

