
#ifndef SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_STURM_COMPARISON_H
#define SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_STURM_COMPARISON_H

#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/upol.h>
#include <synaps/upol/SturmSeq.h>


__ALGEBRAIC_BEGIN_NAMESPACE


template < typename RO_t >  inline
int
sturmcompare(const RO_t& a, const RO_t& b)
{
  typedef typename RO_t::NTR    NTR;
  typedef typename RO_t::RT     RT;
  typedef typename RO_t::FT     FT;
  typedef typename RO_t::FIT    FIT;
  typedef typename RO_t::POLY   POLY;

  int where = 0;
  FIT I;
  int result = decide_common_interval(a, b, I, where);
  if (result != UNKNOWN)
    return result;

  SturmSeq<RT> s(a.poly(), b.poly(), SUBRES());

  int v0 = variation(s, b.left()) - variation(s, b.right());
  return sign( v0)  * sign(a.sign_up() - a.sign_low()) * sign(b.sign_up() - b.sign_low());
  //  return v0 * b.sign_up();
}
//-------------------------------------------------------------------------------


__ALGEBRAIC_END_NAMESPACE


#endif // SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_STURM_COMPARISON_H

