// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
 //                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>
//           Athanasios Kakargias <grad0460@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)
//


#ifndef SYNAPS_ARITHM_ALGEBRAIC_ROOT_OF_C
#define SYNAPS_ARITHM_ALGEBRAIC_ROOT_OF_C

#include <synaps/arithm/algebraic/root_of.h>

__ALGEBRAIC_BEGIN_NAMESPACE

#if 0
template< typename RT_,
          typename POLY_ = UPolDse<RT_>,
		  typename NTR_  = Number_traits<RT_> >
struct ALG
{
  typedef typename NTR_       NTR;
  typedef typename NTR::RT    RT;
  typedef typename NTR::FT    FT;

  typedef POLY_               POLY;

  typedef root_of<RT, POLY, NTR>    RO_t
};


template < typename RT,
	       typename P,
		   typename NTR >
root_of<RT, P, NTR>
make_root_of_1(const RT& a, ALG<RT, P, NTR> mth)
{
  typedef typename NTR::FT  FT;

  P f(2, AsSize);
  f[0] = -a;
  f[1] = 1;
  FT bound = Cauchy_bound<NTR>(f);
  return root_of<RT, P, NTR>(f, 0, ROOT_ONE, 1, RT(1), -bound, bound, true, POS_FIRST_LAST);
}



/** Functions for constructions of root_of classes
 */


/** Construction of an algebraic number  from a number @a@.
 * The number belongs to a ring.
 **/
template < typename RT,
	       typename P,
		   typename NTR >
root_of<RT, P, NTR>
make_root_of_1(const RT& a)
{
  typedef typename root_of<RT>::UPoly UPoly;
  typedef typename root_of<RT>::FT    FT;

  UPoly f(2, AsSize());
  f[0] = RT(-a);
  f[1] = RT(1);

  FT bound = Bound(f);
  return root_of<RT>(f, 0, ROOT_ONE, 1, RT(1), -bound, bound, true, POS_FIRST_LAST);
}


/** Construction of an algebraic number from a number @a@.
 * The number belongs to a fraction field.
 **/
template< class FT > inline
root_of< typename Rational_traits<FT>::RT >
make_rat_root_of_1(const FT& a)
{
  typedef typename Rational_traits<FT>::RT  RT;
  typedef typename root_of<RT>::UPoly       UPoly;
  typedef Rational_traits<FT>               Rat_traits;

  UPoly f(2, AsSize());
  f[0] = -Rat_traits::numerator(a);
  f[1] =  Rat_traits::denominator(a);

  FT bound = Bound(f);
  return root_of<RT>(f, 0, ROOT_ONE, 1, RT(1), -bound, bound, true, POS_FIRST_LAST);
}


/** Construction of an algebraic number from two numbers.
 * The numbers belong to a ring. The algebraic  number is actually
 * a number that belongs to the fraction field of the ring.
 * Optionally, we can set the multiplicity and the root position
 * (in the original equation). This function is useful for the @adjust_poly()@ functions.
 **/
template< class RT > inline
root_of<RT>
make_root_of_1( const RT&  c1,
                const RT&  c0,
                const int& pmu = 1,
                const int& pos = POS_FIRST_LAST)
{
  typedef typename root_of<RT>::UPoly UPoly;
  typedef typename root_of<RT>::FT    FT;

  assert(c1 != 0);

  UPoly f(2, AsSize());
  f[0] = c0;
  f[1] = c1;
  adjust_sign_coeff(f);

  FT bound = Bound(f);
  return root_of<RT>(f, 0, ROOT_ONE, pmu, RT(1), -bound, bound, true, pos);
}


/** Construction of a quadratic algebraic number.
 * The coefficients of the corresponding polynomial belong to a ring.
 **/
template< class RT > inline
root_of<RT>
make_root_of_2( const RT&  c2,
                const RT&  c1,
                const RT&  c0,
				const int& idx)
{
  typedef typename root_of<RT>::UPoly UPoly;

  assert(c2 != RT(0));

  UPoly f(3, AsSize());
  f[0] = c0;
  f[1] = c1;
  f[2] = c2;
  adjust_sign_coeff(f);

  return root_of<RT>(f, idx);
}


/** Construction of cubic algebraic number.
 * The coefficients of the corresponding polynomial belong to a ring.
 **/
template< class RT > inline
root_of<RT>
make_root_of_3( const RT&  c3,
                const RT&  c2,
                const RT&  c1,
                const RT&  c0,
				const int& idx)
{
  typedef typename root_of<RT>::UPoly UPoly;

  assert(c3 != RT(0));

  UPoly f(4, AsSize());
  f[0] = c0;
  f[1] = c1;
  f[2] = c2;
  f[3] = c3;
  adjust_sign_coeff(f);

  return root_of<RT>(f, idx);
}


/** Construction of a quartic algebraic number.
 * The coefficients of the corresponding polynomial belong to a ring.
 **/
template< class RT > inline
root_of<RT>
make_root_of_4( const RT&  c4,
                const RT&  c3,
                const RT&  c2,
                const RT&  c1,
                const RT&  c0,
				const int& idx)
{
  typedef typename root_of<RT>::UPoly UPoly;

  assert(c4 != RT(0));

  UPoly f(5, AsSize());
  f[0] = c0;
  f[1] = c1;
  f[2] = c2;
  f[3] = c3;
  f[4] = c4;
  adjust_sign_coeff(f);

  return root_of<RT>(f, idx);
}


/** Generic constructor of an algebraic number as a root of a polynomial.
 * The coefficients of the corresponding polynomial belong to a ring.
 **/
template< class UPoly > inline
root_of< typename UPoly::value_type, UPoly >
make_root_of_x(const UPoly& p,
	           const int&   idx = 0)
{
  typedef typename UPoly::value_type  RT;

  assert(idx < UPOLDAR::degree(p));

  UPoly f(p);
  adjust_sign_coeff(f);
  return root_of<RT, UPoly>(f, idx);
}


/** Construction of an algebraic number that is the square root of a number @a@
 * that belongs to a ring.
 * The algebraic number is of the form $\sqrt{a}$.
 **/
template< class RT > inline
root_of<RT>
sqrt_of(const RT& a)
{
  typedef typename root_of<RT>::UPoly UPoly;
  typedef typename root_of<RT>::FT    FT;
  typedef Rational_traits<FT>         Rat_traits;

  assert(a >= RT(0));

  if (a == RT(0)) {
	UPoly f(2, AsSize());
	f[0] = RT(0);
	f[1] = RT(1);
	FT left  = Rat_traits::make_rational(RT(-1), RT(1));
	FT right = Rat_traits::make_rational(RT( 1), RT(1));
	return root_of<RT>(f, 0, ROOT_ONE, 2, RT(1), left, right, true, POS_FIRST_LAST);
  }

  UPoly f(3, AsSize());
  f[0] = -a;
  f[1] = RT(0);
  f[2] = RT(1);

  FT left  = Rat_traits::make_rational(RT(0), RT(1));
  FT right = Rat_traits::make_rational(a, RT(1));
  return root_of<RT>(f, 1, ROOT_ONE_ONE, 1, MUL(RT(4), num), left, right, true, POS_LAST);
}



/** Construction of an algebraic number that is the square root of a number @a@
 * that belongs to a fraction field.
 * The algebraic number is of the form $\sqrt{a} = \sqrt{\frac{n}{d}}$.
 **/
template< class FT > inline
root_of< typename Rational_traits<FT>::RT >
rat_sqrt_of(const FT& a)
{
  typedef typename Rational_traits<FT>::RT  RT;
  typedef typename root_of<RT>::UPoly       UPoly;
  typedef Rational_traits<FT>               Rat_traits;

  assert(num >= FT(0));

  if (num == FT(0)) {
	UPoly f(2, AsSize());
	f[0] = RT(0);
	f[1] = RT(1);

	FT left  = Rat_traits::make_rational(RT(-1), RT(1));
	FT right = Rat_traits::make_rational(RT( 1), RT(1));
	return root_of<RT>(f, 0, ROOT_ONE, 2, RT(1), left, right, true, POS_FIRST_LAST);
  }

  UPoly f(3, AsSize());
  f[0] = -Rat_traits::numerator(a);
  f[1] = RT(0);
  f[2] =  Rat_traits::denominator(a);
  RT D = RT(2) * f[2] * (-f[0]);
  FT left  = Rat_traits::make_rational(RT(0), RT(1));
  FT right = Rat_traits::make_rational(-f[0], f[2]);
  return root_of<RT>(f, 1, ROOT_ONE_ONE, 1, D, left, right, true, POS_LAST);
}


/** Construction of an algebraic number that is the  n-th root of a number @a@
 * that belongs to a ring.
 * The algebraic number is of the form $\sqrt[n]{a}$.
 **/
template< class RT > inline
root_of<RT>
sqrt_of_nth(const int& nth, const RT& a)
{
  typedef typename root_of<RT>::UPoly UPoly;
  typedef typename root_of<RT>::FT    FT;
  typedef Rational_traits<FT>         Rat_traits;

  assert(nth >= 0);

  if (nth == 0) return make_root_of_1(RT(1));
  if (nth == 1) return make_root_of_1(a);
  if (nth == 2) return sqrt(a);

  if (a == RT(0)) {
	UPoly f(2, AsSize());
	f[0] = RT(0);
	f[1] = RT(1);
	FT left  = Rat_traits::make_rational(RT(-1), RT(1));
	FT right = Rat_traits::make_rational(RT( 1), RT(1));
	return root_of<RT>(f, 0, ROOT_ONE, nth, RT(1), left, right, true, POS_FIRST_LAST);
  }

  RT D = pow(MUL(a * (-nth)), nth-1) * (-nth);
  if ( nth % 2 == 0 ){
	assert(a > 0);

	UPoly f(nth + 1, AsSize());
	f[0] = -a;
	f[nth] = RT(1);

	FT left  = Rat_traits::make_rational(RT(0), RT(1));
	FT right = Rat_traits::make_rational( a,  RT(1));
	return root_of<RT>(f, 1, ROOT_UNKNOWN, nth, D, RT(0), a, true, POS_LAST);
  }

  UPoly f(nth + 1, AsSize());
  f[0] = -a;
  f[nth] = RT(1);

  if (a > RT(0)) {
	FT left  = Rat_traits::make_rational(RT(0), RT(1));
	FT right = Rat_traits::make_rational( a,  RT(1));
	return root_of<RT>(f, 0, ROOT_UNKNOWN, 1, D, left, right, true, POS_FIRST_LAST);
  }

  FT left  = Rat_traits::make_rational(-a, RT(1));
  FT right = Rat_traits::make_rational(RT(0),  RT(1));
  return root_of<RT>(f, 0, ROOT_UNKNOWN, 1, D, left, right, true, POS_FIRST_LAST);
}


//TODO
/* Construction of an algebraic number that is the  n-th root of a number
 * that belongs to a fraction field.
 * The algebraic number is of the form $\sqrt[n]{a} = \sqrt[n]{\frac{n}{d}}$.
 */

#endif

__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_ARITHM_ALGEBRAIC_ROOT_OF_C


