// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)


#ifndef SYNAPS_USOLVE_ALGEBRAIC_SIGN_22_H
#define SYNAPS_USOLVE_ALGEBRAIC_SIGN_22_H

#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>


__ALGEBRAIC_BEGIN_NAMESPACE


template < typename POLY,
           typename RT,
		   typename FT >  inline
int
static_full_sturm_22(
	const RT& a1, const RT& b1, const RT& c1,
	const RT& a2, const RT& b2, const RT& c2,
	const FT& a,  const FT& b,
	int& where
	)
{
  typedef NumberTraits<RT>    NTR;

  print(__FUNCTION__);
  int result;

  RT J1 = b2 * a1 - a2 * b1;
  RT J2 = c2 * a1 - a2 * c1;
  if (J1 == RT(0)) {
	if (J2 == RT(0)) return Result_deg_2::on_end(where);
	if (J2 > RT(0)) { where ^= 16; where ^= 32; }
	return Result_deg_3::on_end(where);
  }
  POLY S2(2, AsSize());
  S2[1] = J1;
  S2[0] = J2;

  if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
  result = Static_Result::Result_4.on_S2_p2q2(where);
  if (result != UNKNOWN) return result;
  if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
  result = Static_Result::Result_4.on_S2_p1q1(where);
  if (result != UNKNOWN) return result;

  RT S3 = J1 * (c2 * b1 - b2 * c1) - J2 * J2;
  if (S3 == RT(0)) return EQUAL;
  if (S3 > RT(0)) where ^= 64;
  result = Static_Result::Result_4.on_S3(where);
  if (result != UNKNOWN) return result;
  return EQUAL;
}


template < typename POLY,
           typename RO_t > inline
int sign_at_22(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::NTR   NTR;
  typedef typename NTR::RT     RT;
  typedef typename NTR::FT     FT;

  print(__FUNCTION__);
  int where = 0;
  in_isolating_interval(f, x, where);
  return static_full_sturm_22<POLY>(
	  x[2], x[1], x[0],
	  f[2], f[1], f[0],
	  x.left(),
	  x.right(),
	  where);
}


__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_USOLVE_ALGEBRAIC_SIGN_22_H

