// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)


#ifndef SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_34_H
#define SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_34_H

#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>

__ALGEBRAIC_BEGIN_NAMESPACE


template < typename POLY,
		   typename RT,
		   typename FT >  inline
Comparison_result
static_degJ1_sturm_43(
	const RT& a1, const RT& b1, const RT& c1, const RT& d1, const RT& e1,
	const RT& a2, const RT& b2, const RT& c2, const RT& d2,
	const FT& a,  const FT& b,
	int& where)
{
  print(__FUNCTION__);
  Comparison_result result;

  RT t26 = a1 * c2;
  RT J1 = -a1 * a2;
  RT J5 = c1 * a2 - b1 * b2 - t26;
  RT t25 = a1 * d2;
  RT J6 = d1 * a2 - b1 * c2 - t25;
  RT t24 = e1 * a2 - b1 * d2;
  RT J7 = t24;
  RT t21 = J1 * J7;
  RT J4 = -t25;
  RT J2 = -a1 * b2;
  RT t14 = J2 * J2;
  RT J3 = -t26;
  RT t22 = J2 * J3;
  POLY S2(2, AsSize());
  S2[1] =  J1 * J6 - t22 ;
  if (S2[1] == RT(0)) {
	S2[0] = t21 - J4 * J2;
	if (S2[0] == RT(0)) return Result_deg_2::on_end(where);
	if (S2[0] > RT(0)) { where ^= 16; where ^= 32; }
	return Result_deg_3::on_end(where);
  }
  S2[0] = t21 - J4 * J2;

//  cout << "S2 = "; UPOLDAR::print(cout, S2); cout << " d = " << UPOLDAR::degree(S2) << endl;

  if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
  print("S2 on b",  sign(UPOLDAR::eval_horner(S2, b)));
  result = Static_Result::Result_4.on_S2_p2q2(where);
  if (result != UNKNOWN) return result;
  if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
  print("S2 on a",  sign(UPOLDAR::eval_horner(S2, a)));
  result = Static_Result::Result_4.on_S2_p1q1(where);
  if (result != UNKNOWN) return result;

  RT t23 = J3 * J4;
  RT J9 = e1 * b2 - c1 * d2;
  RT t20 = J9 * J6;
  RT J10 = e1 * c2 - d1 * d2;
  RT J8 = d1 * b2 - c1 * c2 + t24;
  RT t19 = -J8 * J10 + J9 * J9;
  RT t18 = J4 * J4;
  RT t17 = J6 * J6;
  RT t16 = J3 * J3;
  RT t13 = J7 * J7;

  RT S3 = t16 * t13 - 2 * J7 * J6 * t23 + t17 * t18 + t19 * t14
	      + (-J8 * t13 + 2 * J7 * t20 - t17 * J10) * J1 +
		  RT(2) * ((-J7 * J9 + J6 * J10) * J3 + (J7 * J8 - t20) * J4) * J2
		  + (RT(2) * J9 * t23 - J8 * t18 - t16 * J10 - t19 * J1) * J5;

  print("S3", S3);
//  S3 = -S3;
  print("sign(S2[1])", sign(S2[1]));
  S3 = -sign(S2[1]) * S3;
  print("S3", S3);
  if (S3 ==  RT(0)) return EQUAL;
  if (S3 > RT(0)) where ^= 64;
  result = Static_Result::Result_4.on_S3(where);
  if (result != UNKNOWN) return result;
  return EQUAL;


}


template < typename POLY,
		   typename RT,
		   typename FT >  inline
Comparison_result
static_full_sturm_43(
	const RT& a1, const RT& b1, const RT& c1, const RT& d1, const RT& e1,
	const RT& a2, const RT& b2, const RT& c2, const RT& d2,
	const FT& a,  const FT& b,
	int& where)
{
  print(__FUNCTION__);
  Comparison_result result;

  RT t26 = a1 * c2;
  RT J1 = -a1 * a2;
  RT J5 = c1 * a2 - b1 * b2 - t26;
  RT t25 = a1 * d2;
  RT J6 = d1 * a2 - b1 * c2 - t25;
  RT t24 = e1 * a2 - b1 * d2;
  RT J7 = t24;
  RT t21 = J1 * J7;
  RT J4 = -t25;
  RT J2 = -a1 * b2;
  RT t14 = J2 * J2;
  RT J3 = -t26;
  RT t22 = J2 * J3;
  POLY S2(3, AsSize());
  S2[2] = J1 * J5 - t14;

  if (S2[2] == RT(0)) {
		return static_degJ1_sturm_43<POLY>(
				a1, b1, c1, d1, e1,
				a2, b2, c2, d2,
				a, b,
				where);
  }

  S2[1] = J1 * J6 - t22;
  S2[0] = t21 - J4 * J2;

//  cout << "S2 = "; UPOLDAR::print(cout, S2); cout << " d = " << UPOLDAR::degree(S2) << endl;

  if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
  print("S2 on b", sign(UPOLDAR::eval_horner(S2, b)));
  result = Static_Result::Result_5.on_S2_p2q2(where);
  if (result != UNKNOWN) return result;
  if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
  print("S2 on a", sign(UPOLDAR::eval_horner(S2, a)));
  result = Static_Result::Result_5.on_S2_p1q1(where);
  if (result != UNKNOWN) return result;

  RT t23 = J3 * J4;
  RT J9 = e1 * b2 - c1 * d2;
  RT t20 = J9 * J6;
  RT J10 = e1 * c2 - d1 * d2;
  RT J8 = d1 * b2 - c1 * c2 + t24;
  RT t19 = -J8 * J10 + J9 * J9;
  RT t18 = J4 * J4;
  RT t17 = J6 * J6;
  RT t16 = J3 * J3;
  RT t13 = J7 * J7;

  POLY S3(2, AsSize());
  S3[1] = -( RT(2) * J6 * t22 - t14 * J8 - J1 * t17 + (J1 * J8 - t16) * J5 );
  S3[0] = -( -J6 * t21 - t14 * J9 + (J1 * J9 - t23) * J5 + (J6 * J4 + J3 * J7) * J2 );

// cout << "S3 = "; UPOLDAR::print(cout, S3); cout << " d = " << UPOLDAR::degree(S3) << endl;

  if ((UPOLDAR::degree(S3) <= 0) && S3[0] == RT(0))
  {
//	cout << "Here " << where << endl;
	return Result_deg_3::on_end(where);
  }

  if (sign(UPOLDAR::eval_horner(S3, b)) >= 0) where ^= 64;
  print("S3 on b", sign(UPOLDAR::eval_horner(S3, b)));
  result = Static_Result::Result_5.on_S3_p2q2(where);
  if (result != UNKNOWN) return result;
  if (sign(UPOLDAR::eval_horner(S3, a)) >= 0) where ^= 128;
  print("S3 on a", sign(UPOLDAR::eval_horner(S3, a)));
  result = Static_Result::Result_5.on_S3_p1q1(where);
  if (result != UNKNOWN) return result;

  RT S4 = t16 * t13 - 2 * J7 * J6 * t23 + t17 * t18 + t19 * t14
	      + (-J8 * t13 + 2 * J7 * t20 - t17 * J10) * J1 +
		  RT(2) * ((-J7 * J9 + J6 * J10) * J3 + (J7 * J8 - t20) * J4) * J2
		  + (RT(2) * J9 * t23 - J8 * t18 - t16 * J10 - t19 * J1) * J5;

  print("S4", S4);
  if (S4 ==  RT(0)) return EQUAL;
  if (S4 > RT(0)) where ^= 256;
  result = Static_Result::Result_5.on_S4(where);
  if (result != UNKNOWN) return result;
  return EQUAL;

}


template < typename POLY,
           typename RO_t >  inline
int
sign_at_34(const POLY& f, const RO_t& x)
{
  typedef typename RO_t::NTR    NTR;
  typedef typename NTR::RT      RT;
  typedef typename NTR::FT      FT;

  print(__FUNCTION__);
  int where = 0;
  in_isolating_interval(f, x, where);
#if 0
  FT I2 = x.right_isolating_point();
  FT I1 = x.left_isolating_point();
  int sgn_x_I2 = ((x.degree()-1-x.which()) % 2 ? NEGATIVE : POSITIVE);
  int sgn_x_I1 = opposite(sgn_x_I2);
  if (sgn_x_I2 == POSITIVE) where ^= 1;
  if (sgn_x_I1 == POSITIVE) where ^= 2;
  cout << sgn_x_I2 << " " << sgn_x_I1 << endl;
#endif
  return static_full_sturm_43<POLY>(
	  x[4], x[3], x[2], x[1], x[0],
	  f[3], f[2], f[1], f[0],
	  x.left(),
	  x.right(),
	  where);
}

__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_USOLVE_ALGEBRAIC_SIGN_SIGN_34_H

