/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: Integer.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_IslBzInteger_H
#define synaps_solve_bezier_IslBzInteger_H
/********************************************************************/
#include <synaps/init.h>
#include <synaps/upol/bound.h>
#include <synaps/usolve/bezier/integer.h>
#include <synaps/arithm/Interval.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS
//====================================================================
template<class O, class X=double>
struct IslBzInteger
{
  typedef Interval<O> solution_t;
  X eps;
  IslBzInteger():eps(1.e-6) {}
  IslBzInteger(const X& e):eps(e) {}
  IslBzInteger(const IslBzInteger & mth): eps(mth.eps) {}

};
//====================================================================
/* UPOL: view on an univariate polynom                 */
/*    C: number type used in computation               */
/*    O: type of result                                */
template< class UPOL, class O >
Seq< Interval<O> > solve( const UPOL& p, const IslBzInteger<O>& mth )
{
  typedef Interval<O> sol_t;
  Seq<O> sol;
  bezier::integer_appendsol( sol, p, mth.eps, true);
  Seq<Interval<O> > res;
  if(sol.size())
    for(int i =0; i< sol.size()-1; i+=2)
      res.push_back(sol_t(sol[i],sol[i+1]));
  return  res;
};
//====================================================================
template< class UPOL, class O >
Seq< Interval<O> > solve( const UPOL& p, const IslBzInteger<O>& mth, const O& a, const O& b)
{
  typedef Interval<O> sol_t;
  Seq<O> sol;
  bezier::integer_appendsol( sol, p, mth.eps, a, b, true);
  Seq<Interval<O> > res;
  if(sol.size())
    for(int i =0; i< sol.size()-1; i+=2)
      res.push_back(sol_t(sol[i],sol[i+1]));
  return  res;
};
//====================================================================
__END_NAMESPACE_SYNAPS 
/********************************************************************/
#endif // synaps_solve_bezier_IslBzInteger_H

