/*******************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *              B. Mourrain, GALAAD, INRIA
 *   $Id: Sleever.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_IslBzStd_H
#define synaps_solve_bezier_IslBzStd_H
/********************************************************************/
#include <synaps/init.h>
#include <synaps/usolve/bezier/SlvBzStd.h>
#include <synaps/arithm/Interval.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
template<class O, class C=double, class X=double>
struct IslBzStd 
{
  typedef Interval<O> solution_t;
  typedef X precision_t;
  X eps;
  IslBzStd(): eps(1.e-6) {}
  IslBzStd(const X& e):eps(e) {}
  IslBzStd(const IslBzStd & mth): eps(mth.eps) {}

};
//--------------------------------------------------------------------
/**
This solver isolate the real roots of the polynomial @p@.
If @p@ has mutiple roots,  a cluster of roots,  
within an interval of size $\epsilon$
where $\epsilon$ is defined in @mth@,
an interval of size $\epsilon$ containing these roots 
is computed.

The class @C@ specify the coefficient used during the subdivision process. It is not
necessarly the type of the coefficients of @p@, but it should be a field type. 
**/
template< class UPOL, class O, class C >
Seq<Interval<O> > solve( const UPOL& p, const IslBzStd<O,C>& mth )
{ 
  typedef Interval<O> sol_t;
  Seq<O> sol;
  bezier::Isole<C> slv;
  unsigned v=0;
  while(p[v]==0 && v< p.size()) v++;
  bezier::rep1d<C> bp(p.size()-v); // TC->C
  let::assign(bp,p,v);
  for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;
  slv.Run( bp, mth.eps, false);
  slv.GetIntervals( sol );


  unsigned s = sol.size();

  if( v != 0 ) 
    {
      sol.push_back(O(0));
      sol.push_back(O(0));
    }

  for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;    
  slv.Run( bp, mth.eps, false);
  slv.GetIntervals(sol);
  RConvert(sol, s, bound<O>(p,Cauchy()) );
  Seq<sol_t> res;
  for(unsigned i=0;i<sol.size(); i+=2)
    res.push_back(sol_t(sol[i],sol[i+1]));
  return res;
};
//--------------------------------------------------------------------
template< class UPOL, class O, class C >
Seq<Interval<O> > solve( const UPOL& p, const IslBzStd<O,C>& mth, 
	      const O& a, const O& b)
{ 
  typedef Interval<O> sol_t;
  Seq<O> sol;
  bezier::Isole<C> slv;
  appendsol( sol, p, mth.eps, slv, a,b,  true  );
  Seq<sol_t> res;
  for(unsigned i=0;i<sol.size(); i+=2)
    res.push_back(sol_t(sol[i],sol[i+1]));
  return res;
};
//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_bezier_IslBzStd_H
