/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: Integer.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bezier_integer_H
#define synaps_solve_bezier_integer_H
/********************************************************************/
#include <synaps/init.h>
#include <synaps/arithm/double.h>
#include <synaps/upol/bound.h>

#include <synaps/upol/bezier/rep1d.h>
#include <synaps/upol/bezier/BEZIER.m>
#include <synaps/usolve/bezier/sbd1d.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS
//====================================================================
namespace bezier
{

  template < class C >
  struct IsoleInteger : IsoleSimple<C>
  {
    typedef C creal_t;
    typedef unsigned sz_t; 
    typedef IsoleSimple<C>  Base_t;
    typedef typename Base_t::unsigned_t unsigned_t;
    typedef typename Base_t::res_t      Domain_t;
    typedef typename Base_t::bound      bound;
    
    
    void barre( char c, unsigned n )
    {
      char  _bar[ n+1 ];
      std::fill(_bar,_bar+n, c);
      _bar[n] = 0;
      std::cout << _bar << std::endl;
    };
    
    
    inline
    static void split( creal_t * r, creal_t * l, sz_t sz )
    {
      BEZIER::subdivide_integer(r,l,sz);
    };

    
    void Loop( bool isole = true )
    {
      //      std::cout <<"Starting simple solver loop"<<std::endl;
      creal_t * pup, * pdw;
      int d;
      sz_t h,a,s,cup,cdw;
      unsigned_t v;
      s = this->m_s;
      pup = this->m_data;
      this->m_dmn[0].m = 0;
      this->m_dmn[0].e = 0;
      
      for( a = 0, d = 0; d >= 0; d--, a -= s   ) 
	{
	  cup = this->sgncnt(pup+a,s);
	  if ( cup ) 
	    {
	      if ( isole && cup == 1 ) 
		{ 
		  if ( *(pup+a) != 0 )
		    {
		      if( *(pup+a+s-1) == 0)
			this->estore(d);
		      else
			this->sstore(d); 
		    }
		  continue; 
		};
	      
	      if ( this->m_dmn[d].e == this->m_limit-1 )
		{
		  if ( cup == 1 ) 
		    {
		      if ( *(pup+a) != 0 )
			{
			  if ( *(pup+a+s-1) == 0 )
			    this->sstore(d,0);
			  else 
			    this->sstore(d);
			};
		    }
		  else  this->mstore(d);
		  // std::cout<<"s "<<cup<<" "<<d<<" "<<m_limit<<std::endl;
		  continue;
		};
	      split(pup+a,pup+a+s,s);
	      Base_t::split(this->m_dmn[d],this->m_dmn[d+1]);
	      d += 2;
	      a += 2*s;
	    };
	};
      //std::cout <<"End of simple solver loop"<<std::endl;
    }
    
    
    template<class input,class prec_t> inline
    void Run( const input& in, const prec_t& eps, bool isole )
    { 
      sz_t prec = numerics::bitprec(eps);
      alloc(in.size(),prec);
      std::copy(in.begin(),in.end(),this->m_data);
      Loop(isole);
    };
    
    template<class input> inline
    void RootsRun( const input& in, const creal_t& eps )
    {
      Run(in,eps,false);    
    }
    
    template<class input> inline
    void RootsRun( const input& up, const input& dw, const creal_t& eps )
    { 
      Run(up,dw,eps,false); 
    }
    
    template<class input> inline
    void IsoleRun( const input& in, const creal_t& eps )
    { 
      Run(in,eps,true); 
    };

    template<class input>
    void IsoleRun( const input& up, const input& dw, const creal_t& eps )
    { 
      Run(up,dw,eps,true); 
    };
    
    const Domain_t& operator[]( int i ) const 
    { 
      return this->m_res[i]; 
    };
    
    Domain_t& operator[]( int i ) 
    { 
      return this->m_res[i]; 
    };
    
    sz_t size() const 
    { 
      return this->m_res.size(); 
    };
    
    template<class real>
    void Convert( real& a, const bound& b )
    {
      unsigned_t m(b.m);
      numerics::hdwi<unsigned_t>::reverse(b.e+1,m);
      a = 0;
      real x = 1;
      //      std::cout<<"idx "<<m<<std::endl;
      for ( unsigned i = 0; i < b.e + 1; x /= 2, i ++ )
	{
	  if ( m & 1 ) a += x;
	  m >>= 1;
	};
      //      std::cout<<"sol "<<a<<std::endl;
    };
    
    template<class output>
    void GetIntervals( output& o )
    {
      unsigned s = o.size();
      //      typename output::value_type a,b;
      o.resize(s+2*size());
      for ( sz_t i = 0; i < this->m_res.size(); i ++ )
	{
	  Convert(o[s+2*i],this->m_res[i].a);
	  Convert(o[s+2*i+1],this->m_res[i].b);
	};
    };
  
    template<class output>
    void GetFlags( output& o )
    {
      unsigned s = o.size();
      o.resize( s + o.size() );
      for ( sz_t i = 0; i < this->m_res.size(); o[s+i] = this->m_res[i++].t );
    };
    
    template<class output>
    void GetMidPoints( output& o )
    {
      typename output::value_type a,b;
      unsigned s = o.size();
      //      std::cout << " osize  = ";
      //      std::cout << o.size() << std::endl;
      //      std::cout << m_res.size() << std::endl;
      o.resize(this->m_res.size()+o.size());
      for ( sz_t i = 0; i < this->m_res.size(); i ++ )
	{
	  Convert(a,this->m_res[i].a);
	  Convert(b,this->m_res[i].b);
	  o[s+i] = (a+b)/2.0;
	};

    };
    
  };

  //====================================================================
  template<class O, class R, class X > inline
  void integer_appendsol(Seq<O>& sol, 
			 const R& r, 
			 const X& eps, bool isole_)
  {
    using let::assign;
    typedef typename R::coeff_t coeff_t;
    typedef typename rationalof<coeff_t>::T rational_t;

    bezier::rep1d<rational_t> bpr(r.size()); // TC->C
    assign(bpr,r);
    
    coeff_t denom=1;
    for ( unsigned i = 0; i < bpr.size(); i++ ) 
      assign(denom,lcm(denom,denominator(bpr[i]))) ;
    
    bezier::rep1d<coeff_t> bp(r.size()); 
    for ( unsigned i = 0; i < bpr.size(); i++ ) 
      assign(bp[i],numerator(rational_t(bpr[i]*denom)));
    
    for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;

    bezier::IsoleInteger<coeff_t> isole;
    isole.Run( bp,eps, isole_ );
    if(isole_)
      isole.GetIntervals( sol );
    else
      isole.GetMidPoints( sol );
    if( bp[0] == 0 ) { sol.push_back(0); if ( isole_ ) sol.push_back(0); };
    unsigned s = sol.size();
    for ( unsigned i = 1; i < bp.size(); bp[i] = -bp[i], i += 2 ) ;    
    isole.Run( bp, eps, isole_ );
    if(isole_)
      isole.GetIntervals( sol );
    else
      isole.GetMidPoints(sol);
    RConvert( sol, s, bound<rational_t>(r,Cauchy()) );//, isole_ );
  }

  //--------------------------------------------------------------------
  template<class O, class R, class X > inline
  void integer_appendsol(Seq<O>& sol, 
			 const R& r, 
			 const X& eps, const O& a, const O& b, bool isole_)
  {
    using let::assign;
    typedef typename R::coeff_t coeff_t;
    typedef typename rationalof<coeff_t>::T rational_t;
    //    typedef O rational_t;
    bezier::rep1d<rational_t> bpr(r.size()); // TC->C
    assign(bpr,r,a,b);
    
    coeff_t denom=1;
    for ( unsigned i = 0; i < bpr.size(); i++ ) 
      assign(denom,lcm(denom,denominator(bpr[i]))) ;
    
    bezier::rep1d<coeff_t> bp(r.size()); 
    for ( unsigned i = 0; i < bpr.size(); i++ ) 
      assign(bp[i],numerator(rational_t(bpr[i]*denom)));

    
    bezier::IsoleInteger<coeff_t> isole;
    isole.Run( bp,eps, isole_ );
    if(isole_)
      isole.GetIntervals( sol );
    else
      isole.GetMidPoints( sol );
    RConvert( sol, a, b );//, isole_ );
  }
  //--------------------------------------------------------------------

} // bezier
//====================================================================
__END_NAMESPACE_SYNAPS 
/********************************************************************/
#endif // synaps_solve_IsoleInteger_H

