#ifndef synaps_usolve_contfrac_IntervalData_h
#define synaps_usolve_contfrac_IntervalData_h


template < typename RT, typename Poly >
struct IntervalData 
{
  RT a, b, c, d;
  Poly  p;
  unsigned long s;
  
  IntervalData( const RT& a_, const RT& b_, const RT& c_, const RT& d_,
		const Poly& p_, unsigned long s_)
    : a(a_), b(b_), c(c_), d(d_), p(p_), s(s_) {}
  
  IntervalData() {}
  
};


template < typename FT >
Interval< FT > inline
to_FIT( const FT& a, const FT& b)
{
  if ( a <= b ) return Interval<FT>( a, b);
  return Interval<FT>( b, a);
}


template < typename K >
typename K::FIT
to_FIT( const typename K::RT& a, const typename K::RT& b, 
	const typename K::RT& c, const typename K::RT& d, 
	const typename K::FT& B)
{
  typedef typename K::RT    RT;
  typedef typename K::FT    FT;
  typedef typename K::FIT   FIT;
  
  
  FT left;
  FT right;
  
  if ( c == RT( 0)) left = sign( a) * B;
  else left = to_FT( a, c);
  
  if ( d == RT( 0)) right = sign( b) * B;
  else right = to_FT( b, d);
  
  return to_FIT( left, right);
}



template < typename Poly> inline 
void 
reverse(Poly & R, const Poly& P)
{ 
  int deg = degree( P);
  R.resize( deg + 1);
  for (int i = 0; i <= deg; ++i)
    R[i] = P[deg-i];
}

/* Replace R by its reverse */
template < typename Poly> inline 
void 
reverse(Poly & R)
{ 
  int deg = degree(R);
  Poly temp(R);
  for (int i = 0; i <= deg; ++i)
    R[i] = temp[deg-i];
}

/* Replaces P by the polynomial P(X+1) */
template < typename RT, typename Poly > inline
void reverse( IntervalData<RT,Poly>& I1)
{
  reverse(I1.p);
  std::swap(I1.a,I1.b);
  std::swap(I1.c,I1.d);
  return; 
}



/* Replaces P by the polynomial P(X+1) */
template < typename Poly > inline
void translate_by_1( Poly& R, const Poly& P)
{
  long deg = degree( P);
  R.resize( deg + 1);

  for (int i = 0; i <= deg; ++i)
    R[i] = P[i];

  for (int j = deg-1; j >= 0; j--) {
    for (int i = j; i < deg; i++) {
      R[i] +=  R[i+1];
    }

  }

  return;
}

/* Replaces P by the polynomial P(X+1) */
template < typename RT, typename Poly > inline
void translate_by_1( IntervalData<RT,Poly>& I1, const IntervalData<RT,Poly>& I2)
{
  translate_by_1(I1.p,I2.p);
  I1.a = I2.a;
  I1.b = I2.a + I2.b;
  I1.c = I2.c;
  I1.d = I2.c + I2.d;
  return; 
}

/* Replaces P by the polynomial P(X+1) */
template < typename RT, typename Poly > inline
void translate_by_1( IntervalData<RT,Poly>& I1)
{
  translate_by_1(I1.p);
  I1.b += I1.a;
  I1.d += I1.c;
}

/* Replaces P by the polynomial P(X+1) */
template < typename Poly > inline
void translate_by_1( Poly& R)

{
  long deg = degree( R);
  for (int j = deg-1; j >= 0; j--) {
    for (int i = j; i < deg; i++) {
      R[i] +=  R[i+1]; 
    }
  }
  return; 
}


template < typename RT, typename Poly > inline
void scale( IntervalData<RT,Poly>& ID, const RT& a)
{
  UPOLDAR::scale( ID.p, ID.p, a);
  ID.a *= a;
  ID.c *= a;
}

template < typename RT, typename Poly > inline
void inv_scale( IntervalData<RT,Poly>& ID, const RT& a)
{
  UPOLDAR::inv_scale( ID.p, ID.p, a);
  ID.b *= a;
  ID.d *= a;
}


template < typename RT, typename Poly > inline
void shift( IntervalData<RT,Poly>& ID, const RT& a)
{
   UPOLDAR::shift( ID.p, ID.p, a);
   ID.b += a * ID.a;
   ID.d += ID.c * a;
}


#endif //synaps_usolve_contfrac_IntervalData_h
