/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef _synaps_usolve_contfrac_lower_bound_slv_h_
#define _synaps_usolve_contfrac_lower_bound_slv_h_
//===================================================================
#include <synaps/arithm/sign.h>
#include <synaps/usolve/bezier/IslBzBdg.h>
#include <synaps/usolve/bezier/SlvBzBdg.h>
#include <synaps/usolve/bezier/IslBzFlt.h>
#include <synaps/usolve/Aberth.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================
struct SlvBzBdgBound : public bezier::Isole<double>
{

  template < typename Poly >
  typename Poly::value_type lower( const Poly& r)
  {
    typedef typename Poly::value_type    RT;
    typedef typename arithm::kernelof<RT>::T kernel_t;
    typedef typename kernel_t::integer  integer_t;
    typedef typename kernel_t::rational rational_t;
    using let::assign;

    RT res = AkritasBound().lower(r);
//     {
//       std::cout<<"Cauchy:  "<<CauchyBound().lower(r)<<std::endl;
//       std::cout<<"Hong:    "<<HongBound().lower(r)<<std::endl;
//       std::cout<<"Bezier:  "<<BezierBound().lower(r)<<std::endl;
//       std::cout<<"Akritas: "<<res<<std::endl;
//     }

    double eps=0.01;
    if(res>integer_t(255)) 
      {
	std::cout<<"\tLower bound:" <<res<<std::endl;
	return res;
      }

    Seq<Interval<double> > sol;
    IslBzFlt<double>(eps).solve(sol,r);
    std::cout<<"Sol_bz: "<<sol<<std::endl;    
    //    std::cout<<"Aberth: "<<solve(r,IslAberth<RR>(10))<<std::endl;    
    if(sol.size())
      assign(res, sol[0].lower());

    std::cout<<"\tLower bound:" <<res<<std::endl;
    return res;
  }


  template < typename Poly >
  Interval< typename rationalof<typename Poly::value_type>::T>  range( const Poly& r)
  {
    typedef typename Poly::value_type    RT;
    typedef typename arithm::kernelof<RT>::T kernel_t;
    typedef typename kernel_t::integer  integer_t;
    typedef typename kernel_t::rational rational_t;
    typedef Interval<rational_t> interval_t;
    using let::assign;

    Seq<Interval<rational_t> > sol;
    IslBzFlt<rational_t>().solve(sol,r);
    std::cout<<"Sign var: "<<sign_variation(r)<<std::endl;
    std::cout<<"Sol ("<<sol.size()<<"): "<< sol<<std::endl;


    double s;
    for(unsigned i =0;i<sol.size();i++)
      {
	assign(s,sol[i].lower()); std::cout <<"["<<s<<",";
	assign(s,sol[i].upper()); std::cout <<s<<"] (";
	assign(s,rational_t(sol[i].upper()-sol[i].lower())); std::cout <<s<<") ";
      }
    std::cout<<std::endl;
    if(sign_variation(r)==sol.size())
      return interval_t(-1,-1);
    if(sol.size())
      {
	return interval_t(sol[0].lower(),sol[sol.size()-1].upper());
      }
    else
      return interval_t(-1,-1);
  }

};

//===================================================================
__END_NAMESPACE_SYNAPS
#endif //_synaps_usolve_contfrac_lower_bound_slv_h_
