#ifndef _SYNAPS_SOLVE_STURMABERTH_H_
#define _SYNAPS_SOLVE_STURMABERTH_H_


#include <synaps/init.h>
#include <synaps/arithm/RealOf.h>
#include <synaps/usolve/solver_base.h>
#include <synaps/usolve.h>
#include <synaps/usolve/sturm/common.h>


__BEGIN_NAMESPACE_SYNAPS

template <class O, class C=QQ, class X=double>
struct SlvSturm 
{
  typedef O solution_t;
  X eps;
  SlvSturm():eps(1.e-6) {}
  SlvSturm(const X& e):eps(e) {}
  SlvSturm(const SlvSturm & mth): eps(mth.eps) {}
};


template <class O, class C=QQ>
Seq<O> 
solve( const UPOL& f,  SlvSturm<O,C> )
{
  Seq<O> sol;
  ALGEBRAIC::solve_sturm(f, std::back_inserter(sol.rep()), K());
  std::stable_sort( sol.begin(), sol.end(), ALGEBRAIC::Refine_compare());
  return sol;
}

__END_NAMESPACE_SYNAPS
#endif // _SYNAPS_SOLVE_STURMABERTH_H_

    
