
#ifndef __SYNAPS_SOLVE_STURM_SOLVE_D2_H__
#define __SYNAPS_SOLVE_STURM_SOLVE_D2_H__


#include <synaps/init.h>
#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/usolve/Util.h>
#include <synaps/upol/bound.h>
#include <synaps/usolve/sturm/construct_root_of.h>


__ALGEBRAIC_BEGIN_NAMESPACE


//! Computes the roots of a polynomial of degree 2
template < typename K_,
	   typename OutputIterator >
void
solve_adjust_poly_2( const typename K_::Poly& f,
		     OutputIterator sol,
		     unsigned mult,
		     K_)
{
    typedef K_                 K;
    typedef typename K::RT     RT;
    typedef typename K::FT     FT;
    typedef typename K::Poly   Poly;
    typedef typename K::RO_t   RO_t;
    
    RT D = SQR(f[1]) - RT(4) * f[2] * f[0];
    FT ub = bound<FT>(f, Cauchy());
    FT lb = -ub;

    // There are two algebraic numbers
    if (D > RT(0)) {
	FT mid = to_FT( RT(-f[1]), RT( RT(2)*f[2] ) );
	*sol++ = RO_t(f, 0, D, lb, mid, 1*mult);
	*sol++ = RO_t(f, 1, D, mid, ub, 1*mult);
    } else if (D == RT(0)) {
      // There is one algebraic number. A rational one
	Poly p(2, AsSize());
	p[0] = f[1];
	p[1] = 2 * f[2];
	*sol++ = construct_root_of( p, 2*mult, K());
    }
    return;
}

__ALGEBRAIC_END_NAMESPACE


#endif // __SYNAPS_SOLVE_STURM_SOLVE_D2_H__
