
#ifndef __SYNAPS_SOLVE_STURM_SOLVE_D4_H__
#define __SYNAPS_SOLVE_STURM_SOLVE_D4_H__


#include <synaps/init.h>
#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/usolve/Util.h>
#include <synaps/upol/bound.h>
#include <synaps/usolve/sturm/construct_root_of.h>


__ALGEBRAIC_BEGIN_NAMESPACE

//! Computes the roots of a polynomial of degree 4
template < typename K_,
	   typename OutputIterator >
void
solve_adjust_poly_4(const typename K_::Poly& f,
		    OutputIterator  sol,
		    unsigned m,
		    K_)
{
    typedef K_                 K;
    typedef typename K::RT     RT;
    typedef typename K::FT     FT;
    typedef typename K::Poly   Poly;
    typedef typename K::RO_t   RO_t;

    RT a = RT(12) * f[4];
    RT b = RT(-3) * f[3];
    RT c = RT( 2) * f[2];
    RT d = RT(-3) * f[1];
    RT e = RT(12) * f[0];

    RT D2 = calc_D2(a, b, c); //D2
    RT D3 = calc_D2(b, c, d); //D3
    RT W1 = calc_brackets(a, d, b, c); //W1
    RT W2 = calc_brackets(b, e, c, d); //W2
    RT W3 = calc_brackets(a, e, b, d); //W3

    RT D2_x2 = MUL( RT(2), D2 );
    RT W1_neg  = MUL( RT(-1), W1 );
    RT A  = calc_A(W3, D3);
    RT B  = calc_B(d, W1, e, D2, c, D3);
    RT T1 = calc_T1(W1, D2, D3, W3);
    // calc the discriminant
    RT D = calc_D4(A, B);

    if (D > RT(0)) {
	if ((T1 <= 0) || (D2 <= 0)) {
	    //r.mu_ = ROOT_ALL_COMPLEX;
	    return;
	}
	solve_adjust_poly_4_ONE(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B,
	    sol,
	    m,
	    K()
	    );
	return;
    }
    if (D < RT(0)) {
	//	r.mu_ = ROOT_ONE_ONE_COMPLEX_COMPLEX;
	adjust_poly_2_ONE_2_COMPLEX(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B,
	    sol,
	    m,
	    K()
	    );
	return;
    }
    //  From now on (r.discriminant() == RT(0))
    if (T1 > RT(0)) {
	adjust_poly_DOUBLE_2_ONE(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, T1,
	    sol,
	    m,
	    K()
	    );
	return;
    }
    if (T1 < RT(0)) {
	//X		mu_ = ROOT_DOUBLE_COMPLEX_COMPLEX;
	// We can compute the root exactly from PRS
	RT p1, q1;
	print("D2", D2);
	if (D2 == RT(0)) {
	    p1 = W3;
	    q1 = MUL( RT(3), W1 );
	} else {
	    RT T2 = calc_T2(A, W1, b, B);
	    p1 = MUL( RT(-1), T2 );
	    q1 = RT(3) * T1;
	}
	Poly p(2, AsSize());
	p[0] = -p1;
	p[1] = q1;
	FT isol = to_FT(-p[0], p[1]);
	*sol++ = RO_t(p, 0, RT(1), isol, isol, 2*m);
	return;
    }
    // From now on r.D_ == T1 == 0
    //	if (T1 == RT(0))
    if (D2 > RT(0)) {
	RT R = calc_brackets(b, D2_x2, W1_neg, a);
	if (R == RT(0)) {
	    //	  r.mu_ = ROOT_DOUBLE_DOUBLE;
	    Poly P(3, AsSize());
	    P[2] = RT(3) * D2;
	    P[1] = RT(3) * W1;
	    P[0] = -W3;
	    RT PD = SQR(P[1]) - RT(4)*P[2]*P[0];
	    FT isol2 = bound<FT>(P, Cauchy());
	    FT isol0 = -isol2;
	    FT isol1 = to_FT(RT(-P[1]), RT(RT(2)*P[2]));
	    *sol++ = RO_t(P, 0, PD, isol0, isol1, 2*m);
	    *sol++ = RO_t(P, 1, PD, isol1, isol2, 2*m);
	    return;
	}
	// else R != 0
	// We know the roots exactly
	// DECIDE if
	//  1  3
	//  3  1
	RT tp1 = W1_neg;
	RT tq1 = D2_x2;
	RT tp2 = calc_triple_p(a, W1, b, D2);
	RT tq2 = calc_triple_q(a, D2);
	RT Dp  = calc_brackets(tp1, tq2, tp2, tq1);
	if (Dp > RT(0)) { //3 1
	    //	  r.mu_ = ROOT_ONE_TRIPLE;
	    Poly p(2, AsSize());
	    p[0] = -tp2;
	    p[1] = tq2;
	    FT isol = to_FT(-p[0], p[1]);
	    *sol++ = RO_t(p, 0, RT(1), isol, isol, 1*m);

	    p[0] = -tp1;
	    p[1] = tq1;
	    isol = to_FT(-p[0], p[1]);
	    *sol++ = RO_t(p, 0, RT(1), isol, isol, 3*m);
	    return;
	}
	// else  // 1 3
	//	r.mu_  = ROOT_TRIPLE_ONE;
	Poly p(2, AsSize());
	p[0] = -tp1;
	p[1] = tq1;
	FT isol = to_FT(-p[0], p[1]);
	*sol++ = RO_t(p, 0, RT(1), isol, isol, 3*m);

	p[0] = -tp2;
	p[1] = tq2;
	isol = to_FT(-p[0], p[1]);
	*sol++ = RO_t(p, 0, RT(1), isol, isol, 1*m);
	return;
    }
    if (D2 < RT(0)) {
	//	assert(D2 >= RT(0));
	//	r.mu_ = ROOT_ALL_COMPLEX;
	return;
    }
    // D2 = 0
    // We know the root exactly, it is  b/a OR - C3_/(4*C4_)
    // r.mu_ = ROOT_QUADRUPLE;
    //  r.redefine(a, -b);
    Poly p(2, AsSize());
    p[0] = -b;
    p[1] = a;
    FT isol = to_FT(-p[0], p[1]);
    *sol++ = RO_t(p, 0, RT(1), isol, isol, 4*m);
    return;
}





// Adjust the poly when it has 4 distinct real roots
template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K>
void
solve_adjust_poly_4_ONE(
    const Poly& f,
    const RT& a,     const RT& b,      const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,     const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,  const RT& B,
    OutputIterator sol,
    unsigned m,
    K
    )
{

    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    //  r.mu_ = ROOT_ONE_ONE_ONE_ONE;
    RT f_ba = calc_F4_BA(a, A, D2);
    RT R = calc_R(a, W1, b, D2_x2);
    if (f_ba > RT(0)) {
	adj_4_ONE_fba_pos(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, R,
	    sol,
	    m,
	    K()
	    );
	return;
    }  // if (f_ba > RT(0))
    if (f_ba < RT(0)) {
	adj_4_ONE_fba_neg(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, R,
	    sol,
	    m,
	    K()
	    );
	return;
    } // if (f_ba < RT(0))
    //if (f_ba == RT(0)) {
    adj_4_ONE_fba_zero(
	f,
	a, b, c, d, e,
	D,
	D2, D3, W1, W2, W3,
	D2_x2, W1_neg,
	A, B, R,
	sol,
	m,
	K()
	);
    //  } // if (f_ba == RT(0))
    return;
}


template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_4_ONE_fba_pos(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    // b/a is in the middle of the 4 roots
    // we calculate the other 2 isolating points
    // using the roots of the isolator polynomial
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    FT is4 = bound<FT>(f, Cauchy());
    FT is0 = -is4;

    RT IB = RT(-3) * W1;
    RT IA = RT( 6) * D2;
    RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
    RT p1 = IB - ID;
    RT p3 = IB + ID;
    RT q = IA;

    FT is1 = to_FT(p1, q);
    FT is2 = to_FT(b, a);
    FT is3 = to_FT(p3, q);

    *sol++ = RO_t(f, 0, D, is0, is1, 1*m);
    *sol++ = RO_t(f, 1, D, is1, is2, 1*m);
    *sol++ = RO_t(f, 2, D, is2, is3, 1*m);
    *sol++ = RO_t(f, 3, D, is3, is4, 1*m);
    return;
}



template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_4_ONE_fba_neg(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    // b/a is either the 1st or the 3rd isolating point
    // decide its index and calculate the other 2 isolating points
    // We know for sure that one is -W1_/D2_x2
    // the remaining isolating points is derived as a root
    // of the isolator polynomial
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;


    RT f_WD = calc_F4_WD(a, B, D2, D3, W1);
    // Decide the order of b/a and -W1/D2_x2
    if (R > RT(0)) {    //  -W1/D2_x2 < b/a
	RT p3 = b;
	RT q3 = a;
	RT p1, q1, p2, q2;
	if (f_WD > RT(0)) {
	    RT IB = RT(-3) * W1;
	    RT IA = RT( 6) * D2;
	    RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	    p1 = IB - ID;
	    q1 = IA;
	    p2 = W1_neg;
	    q2 = D2_x2;
	} else if (f_WD < RT(0)) {
	    p1 = W1_neg;
	    q1 = D2_x2;
	    RT IB = RT(-3) * W1;
	    RT IA = RT( 6) * D2;
	    RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	    p2 = IB + ID;
	    q2 = IA;
	} else {
	    // TODO Test for Collins
	    RT L = std::max( std::max( std::max(abs(a), abs(b)),
				       std::max(abs(c), abs(d)) ),
			     abs(e) );
	    RT L_2x8 = SQR(L);
	    L_2x8 = MUL( RT(8), L_2x8);
	    p1 = calc_brackets(W1_neg, L_2x8, RT(1), D2_x2);
	    q1 = MUL( L_2x8, D2_x2 );
	    p2 = calc_brackets(W1_neg, L_2x8, RT(-1), D2_x2);
	    q2 = MUL( L_2x8, D2_x2 );
	}
	FT is4 = bound<FT>(f, Cauchy());
	FT is0 = -is4;
	FT is1 = to_FT(p1, q1);
	FT is2 = to_FT(p2, q2);
	FT is3 = to_FT(p3, q3);

	*sol++ = RO_t(f, 0, D, is0, is1, 1*m);
	if (f_WD == RT(0)) {
	    RO_t r = construct_root_of( RT(-W1_neg), D2_x2, K());
	    r.set_multiplicity(m);
	    // r.set_left_isolating_point( is1 );
	    // r.set_right_isolating_point( is2 );
	    *sol++ = r;
	} else {
	    *sol++ = RO_t(f, 1, D, is1, is2, 1*m);
	}
	*sol++ = RO_t(f, 2, D, is2, is3, 1*m);
	*sol++ = RO_t(f, 3, D, is3, is4, 1*m);
	return;
    } // if (R > RT(0))

    // if (R < RT(0))
    RT p1 = b;
    RT q1 = a;
    RT p2, q2, p3, q3;
    if (f_WD > RT(0)) {
	RT IB = RT(-3) * W1;
	RT IA = RT( 6) * D2;
	RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	p3 = IB + ID;
	q3 = IA;
	p2 = W1_neg;
	q2 = D2_x2;
    } else if (f_WD < RT(0)) {
	p3 = W1_neg;
	q3 = D2_x2;
	RT IB = RT(-3) * W1;
	RT IA = RT( 6) * D2;
	RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	p2 = IB - ID;
	q2 = IA;
    } else {
	// TODO Test for Collins
	RT L = std::max( std::max( std::max(abs(a), abs(b)),
				   std::max(abs(c), abs(d)) ),
			 abs(e) );
	RT L_2x8 = SQR(L);
	L_2x8 = MUL( RT(8), L_2x8);
	p2 = calc_brackets(W1_neg, L_2x8, RT(1), D2_x2);
	q2 = MUL( L_2x8, D2_x2 );
	p3 = calc_brackets(W1_neg, L_2x8, RT(-1), D2_x2);
	q3 = MUL( L_2x8, D2_x2 );
    }

    FT is4 = bound<FT>(f, Cauchy());
    FT is0 = -is4;
    FT is1 = to_FT(p1, q1);
    FT is2 = to_FT(p2, q2);
    FT is3 = to_FT(p3, q3);

    *sol++ = RO_t(f, 0, D, is0, is1, 1*m);
    *sol++ = RO_t(f, 1, D, is1, is2, 1*m);
    if (f_WD == RT(0)) {
	RO_t r = construct_root_of( RT(-W1_neg), D2_x2, K());
	r.set_multiplicity(m);
	// r.set_left_isolating_point( is2 );
	// r.set_right_isolating_point( is3 );
	*sol++ = r;
    } else
	*sol++ = RO_t(f, 2, D, is2, is3, m);
    *sol++ = RO_t(f, 3, D, is3, is4, m);
    return;
}




template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_4_ONE_fba_zero(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    // b/a is a root of the quartic
    // We decide its index
    // So we can have 2 more isolating points from the separation bound
    // We use Collins conjecture for this
    // -W1_/D2_x2 is for sure an isolating point
    // When we need one more isolating point we compute as
    // a root of the isolator polynomial
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    RT f_WD = calc_F4_WD(a, B, D2, D3, W1);
    RT df_ba = calc_cubic_normal(a, b, c, d, b, a);//C4_ * W1_neg - RT(2)*C3_*D2_;
    print("df_ba", df_ba);
    print("f_WD", f_WD);
    RT L = std::max( std::max( std::max(abs(a), abs(b)),
			       std::max(abs(c), abs(d)) ),
		     abs(e) );
    // b/a is the second root then df(b/a) > 0
    RT p1, q1, p2, q2, p3, q3;
    if (df_ba > RT(0)) {
	//	if (r.rootnb_ == 1)
	//	  r.redefine(a, -b);
	if (f_WD > RT(0)) {
	    RT L_2x8 = SQR(L);
	    L_2x8 = MUL( RT(8), L_2x8);
	    p1 = calc_brackets(b, L_2x8, RT(1), a);
	    q1 = MUL(a, L_2x8);
	    p2 = W1_neg;
	    q2 = D2_x2;
	    RT IB = RT(-3) * W1;
	    RT IA = RT( 6) * D2;
	    RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	    p3 = IB + ID;
	    q3 = IA;
	} else { //if (f_WD < RT(0))
	    // TODO Tet for Collins
	    RT L_2x8 = SQR(L);
	    L_2x8 = MUL( RT(8), L_2x8);
	    p1 = calc_brackets(b, L_2x8, RT(1), a);
	    q1 = MUL(a, L_2x8);
	    p2 = calc_brackets(b, L_2x8, RT(-1), a);
	    q2 = MUL(a, L_2x8);
	    p3 = W1_neg;
	    q3 = D2_x2;
	}
	FT is4 = bound<FT>(f, Cauchy());
	FT is0 = -is4;
	FT is1 = to_FT(p1, q1);
	FT is2 = to_FT(p2, q2);
	FT is3 = to_FT(p3, q3);

	*sol++ = RO_t(f, 0, D, is0, is1, m);
	RO_t r = construct_root_of( RT(-b), a, K());
	r.set_multiplicity(m);
	// r.set_left_isolating_point( is1 );
	// r.set_right_isolating_point( is2 );
	*sol++ = r;
	*sol++ = RO_t(f, 2, D, is2, is3, 1*m);
	*sol++ = RO_t(f, 3, D, is3, is4, 1*m);
	return;
    } // if (df_ba > RT(0))

    // b/a is the third root then df(b/a) < 0
    //	if (df_ba < RT(0))
    //  if (r.rootnb_ == 2) {
    //	r.redefine(a, -b);
    //	return;
    //  }
    if (f_WD > RT(0)) {
	RT IB = RT(-3) * W1;
	RT IA = RT( 6) * D2;
	RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	p1 = IB - ID;
	q1 = IA;
	p2 = W1_neg;
	q2 = D2_x2;
	RT L_2x8 = SQR(L);
	L_2x8 = MUL( RT(8), L_2x8);
	p3 = calc_brackets(b, L_2x8, RT(-1), a);
	q3 = MUL(L_2x8, a);
    } else { //if (f_WD < RT(0))
	// TODO Test for Collins
	p1 = W1_neg;
	q1 = D2_x2;
	RT L_2x8 = SQR(L);
	L_2x8 = MUL( RT(8), L_2x8);
	p2 = calc_brackets(b, L_2x8, RT(1), a);
	q2 = MUL(L_2x8, a);
	p3 = calc_brackets(b, L_2x8, RT(-1), a);
	q3 = MUL(L_2x8, a);
    }
    FT is4 = bound<FT>(f, Cauchy());
    FT is0 = -is4;
    FT is1 = to_FT(p1, q1);
    FT is2 = to_FT(p2, q2);
    FT is3 = to_FT(p3, q3);

    *sol++ = RO_t(f, 0, D, is0, is1, m);
    *sol++ = RO_t(f, 1, D, is1, is2, m);
    RO_t r = construct_root_of( RT(-b), a, K());
    r.set_multiplicity(m);
    // r.set_left_isolating_point( is2 );
    // r.set_right_isolating_point( is3 );
    *sol++ = r;
    *sol++ = RO_t(f, 3, D, is3, is4, m);
    return;
}




template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adjust_poly_DOUBLE_2_ONE(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& T1,
    OutputIterator  sol,
    unsigned m,
    K
    )
{
    // We know exactly the double root
    // decide if
    //  2 1 1
    //  1 2 1
    //  1 1 2
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    RT p, q;
    // find exactly the double root
    if (D2 == RT(0)) {
	p = W3;
	q = MUL(RT(3), W1);
    } else {
	RT T2 = calc_T2(A, W1, b, B);
	print("T2", T2);
	p = MUL( RT(-1), T2 );
	q = MUL( RT(+3), T1 );
    }
    RT Dr1 = calc_brackets(p, a, q, b);
    RT Dr2 = calc_brackets(p, D2_x2, W1_neg, q);
    RT R = calc_brackets(b, D2_x2, a, W1_neg);
    //	RT f_ba = calc_F4_BA(a, A, D2_);
    //case 2 1 1
    if ( (Dr1 < RT(0)) && (Dr2 < RT(0)) ) {
	return adj_Dr1_neg_Dr2_neg(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, R,
	    p, q,
	    sol,
	    m,
	    K());
    }
    // case 1 1 2
    if ((Dr1 > RT(0)) && (Dr2 > RT(0))) {
	return adj_Dr1_pos_Dr2_pos(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, R,
	    p, q,
	    sol,
	    m,
	    K());
    }
    // case 1 2 1
    if ( ((Dr1 < RT(0)) && (Dr2 > RT(0)))  ||
	 ((Dr1 > RT(0)) && (Dr2 < RT(0))) ) {
	return adj_Dr1_diff_Dr2(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, R,
	    p, q,
	    sol,
	    m,
	    K());
    }
// case 1 2 1
    if ((Dr1 == RT(0)) && (Dr2 == RT(0))) {
	return adj_Dr1_zero_Dr2_zero(
	    f,
	    a, b, c, d, e,
	    D,
	    D2, D3, W1, W2, W3,
	    D2_x2, W1_neg,
	    A, B, R,
	    p, q,
	    sol,
	    m,
	    K());
    }
}

template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_Dr1_neg_Dr2_neg(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    const RT& p,     const RT& q,
    OutputIterator& sol,
    unsigned m,
    K
    )
{
    // case 2 1 1
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    //  r.mu_ = ROOT_DOUBLE_ONE_ONE;
    //  if ((r.rootnb_ == 0) || (r.rootnb_ == 1))
    //	r.redefine(q, -p);
    //  RT p1 = p;
    //  RT q1 = q;
    RT p2, q2;
    RT f_ba = calc_F4_BA(a, A, D2);
    if (f_ba > RT(0)) {
	p2 = b;
	q2 = a;
    } else {
	p2 = W1_neg;
	q2 = D2_x2;
    }

    // TODO: Make them inline functions
    RT D5 = SQR(c) - a * e;
    RT p3 =   A * ( RT(11)* b * D2 + a * W1 )
	- RT(3) * b * SQR(W1)
	+ RT(2) * b * D2 * ( RT(5) * D5 - RT(14) * D3 );
    RT q3 =   SQR(a) * B
	- RT(8) * c * SQR(D2)
	- RT(2) * W1 * ( RT(4) * b * D2 + a * W1 );

    FT is4 = bound<FT>(f, Cauchy());
    FT is2 = to_FT(p2, q2);
    FT is3 = to_FT(p3, q3);

    RO_t r = construct_root_of (RT(-p), q, K());
    r.set_multiplicity(2*m);
    // r.set_left_isolating_point( -is4 );
    // r.set_right_isolating_point( is2 );
    *sol++ = r;

    //  *sol++ = ( RO_t(q, -p) );
    *sol++ = RO_t(f, 2, D, is2, is3, 1*m);
    *sol++ = RO_t(f, 3, D, is3, is4, 1*m);
    return;
}


template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_Dr1_pos_Dr2_pos(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    const RT& p,     const RT& q,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    // case 1 1 2
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    //  r.mu_ = ROOT_ONE_ONE_DOUBLE;
    //  if ((r.rootnb_ == 2) || (r.rootnb_ == 3))
    //	r.redefine(q, -p);
    // TODO: Make them inline functions
    RT D5 = SQR(c) - a * e;
    RT p1 =   A * ( RT(11)* b * D2 + a * W1 )
	- RT(3) * b * SQR(W1)
	+ RT(2) * b * D2 * ( RT(5) * D5 - RT(14) * D3 );
    RT q1 =   SQR(a) * B
	- RT(8) * c * SQR(D2)
	- RT(2) * W1 * ( RT(4) * b * D2 + a * W1 );

    RT f_ba = calc_F4_BA(a, A, D2);
    RT p2, q2;
    if (f_ba > RT(0)) {
	p2 = b;
	q2 = a;
    } else {
	p2 = W1_neg;
	q2 = D2_x2;
    }
    //  RT p3 = p;
    //  RT q3 = q;

    FT is0 = -bound<FT>(f, Cauchy());
    FT is1 = to_FT(p1, q1);
    FT is2 = to_FT(p2, q2);

    *sol++ = RO_t(f, 0, D, is0, is1, 1*m);
    *sol++ = RO_t(f, 1, D, is1, is2, 1*m);
    RO_t r = construct_root_of( RT(-p), q, K());
    r.set_multiplicity(2*m);
    // r.set_left_isolating_point( is2 );
    // r.set_right_isolating_point( -is0 );
    *sol++ = r;
    //  *sol++ = ( RO_t(q, -p, 2) );
    return;
}


template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_Dr1_diff_Dr2(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    const RT& p,     const RT& q,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    // case 1 2 1
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    //  r.mu_ = ROOT_ONE_DOUBLE_ONE;
    //  if ((r.rootnb_ == 1) || (r.rootnb_ == 2))
    //	r.redefine(q, -p);
    RT p2 = p;
    RT q2 = q;
    RT p1, q1, p3, q3;
    if (R > RT(0)) {
	p1 = W1_neg;
	q1 = D2_x2;
	p3 = b;
	q3 = a;
    }
    if (R < RT(0)) {
	p1 = b;
	q1 = a;
	p2 = p;
	q2 = q;
	p3 = W1_neg;
	q3 = D2_x2;
    }
    FT is4 = bound<FT>(f, Cauchy());
    FT is0 = -is4;
    FT is1 = to_FT(p1, q1);
    FT is3 = to_FT(p3, q3);

    *sol++ = RO_t(f, 0, D, is0, is1, 1*m);
    RO_t r = construct_root_of( RT(-p), q, K());
    r.set_multiplicity(2*m);
    // r.set_left_isolating_point( is1 );
    // r.set_right_isolating_point( is3 );
    *sol++ = r;
    //  *sol++ = ( RO_t(q, -p, 2) );
    *sol++ = RO_t(f, 2, D, is3, is4, 1*m);
    return;
}

template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adj_Dr1_zero_Dr2_zero(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,     const RT& R,
    const RT& p,     const RT& q,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    // case 1 2 1
    // ADDITIONALLY We can treat the other roots as
    // roots of
    //  a^2 X^2 - 2 a b X + 6 a c - 5 b^2
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    //  r.mu_ = ROOT_ONE_DOUBLE_ONE;
    //  if ((r.rootnb_ == 1) || (r.rootnb_ == 2))
    //	r.redefine(q, -p);
    //  RT p2 = p;
    //  RT q2 = q;
    RT L = std::max( std::max( std::max(abs(a), abs(b)),
			       std::max(abs(c), abs(d)) ),
		     abs(e) );
    RT L_2x8 = SQR(L);
    L_2x8 = MUL( RT(8), L_2x8);
    RT p1 = calc_brackets(p, L_2x8, RT(1), q);
    RT q1 = MUL( q, L_2x8 );
    RT p3 = calc_brackets(p, L_2x8, RT(-1), q);
    RT q3 = MUL( q, L_2x8 );

    FT is4 = bound<FT>(f, Cauchy());
    FT is0 = -is4;
    FT is1 = to_FT(p1, q1);
    FT is3 = to_FT(p3, q3);

    *sol++ = RO_t(f, 0, D, is0, is1, 1*m);
    RO_t r = construct_root_of( RT(-p), q, K());
    r.set_multiplicity(2*m);
    // r.set_left_isolating_point( is1 );
    // r.set_right_isolating_point( is3 );
    *sol++ = r;
    //  *sol++ = ( RO_t(q, -p, 2) );
    *sol++ = RO_t(f, 2, D, is3, is4, 1*m);
    return;
}



template < typename RT,
	   typename Poly,
	   typename OutputIterator,
	   typename K >
void
adjust_poly_2_ONE_2_COMPLEX(
    const Poly& f,
    const RT& a,     const RT& b,     const RT& c,  const RT& d,  const RT& e,
    const RT& D,
    const RT& D2,    const RT& D3,    const RT& W1, const RT& W2, const RT& W3,
    const RT& D2_x2, const RT& W1_neg,
    const RT& A,     const RT& B,
    OutputIterator sol,
    unsigned m,
    K
    )
{
    print(__FUNCTION__);
    typedef typename K::FT     FT;
    typedef typename K::RO_t   RO_t;

    FT is2 = bound<FT>(f, Cauchy());
    FT is0 = -is2;

    RT f_ba = calc_F4_BA(a, A, D2);
    print("f_ba", f_ba);

    bool first = false;
    bool exact = false;
    RT p, q;
    RT exact_p, exact_q;
    RT f_ed;

    // b/a is an isolating point
    if (f[0] < RT(0)) {
	p = RT(0);
	q = RT(1);
    } else if (f_ba < RT(0)) {
	p = b;
	q = a;
    } else if (f_ba == RT(0)) {
	exact = true;
	exact_p = b;
	exact_q = a;
	RT L = std::max( std::max( std::max(abs(a), abs(b)),
				   std::max(abs(c), abs(d)) ),
			 abs(e) );
	RT L_2x8 = SQR(L);
	L_2x8 = MUL( RT(8), L_2x8);
	p = calc_brackets(b, L_2x8, RT(1), a);
	q = MUL( a, L_2x8 );
	int sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
	if (sgn_on_isol == POSITIVE) {
	    p = calc_brackets(b, L_2x8, RT(-1), a);
	    first = true;
	}
    } else { //1
	f_ed = calc_quartic(f[4], f[3], f[2], f[1], f[0], e, d);
	print("f_ed", f_ed);
	if (f_ed < RT(0)) {
	    p = e;
	    q = d;
	} else if (f_ed == RT(0)) {
	    exact = true;
	    exact_p = e;
	    exact_q = d;
	    RT L = std::max( std::max( std::max(abs(a), abs(b)),
				       std::max(abs(c), abs(d)) ),
			     abs(e) );
	    RT L_2x8 = SQR(L);
	    L_2x8 = MUL( RT(8), L_2x8);
	    p = calc_brackets(e, L_2x8, RT(1), d);
	    q = MUL( d, L_2x8 );
	    int sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
	    if (sgn_on_isol == POSITIVE) {
		p = calc_brackets(e, L_2x8, RT(-1), d);
		first = true;
	    }
	} else if (D2 == RT(0)){
	    p = W3;
	    q = RT(3)*W1;
	} else if (W3 == RT(0)){
	    RT D4 = SQR(d) - c * e;
	    p = -D4;
	    q = W2;
	} else {
	    RT IB = RT(-3) * W1;
	    RT IA = RT( 6) * D2;
	    RT ID = isqrt( RT(SQR(IB) + RT(2) * IA * W3) );
	    p = IB - ID;
	    q = IA;
	    //cout << "1 = " << QQ(p, q).get_d() << endl;
	    int sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
	    if (sgn_on_isol == POSITIVE) {
		p = IB + ID;
		//cout << "2 = " << QQ(p, q).get_d() << endl;
		sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
		if (sgn_on_isol == POSITIVE) {
		    p = IB + ID - RT(1);
		    //cout << "3 = " << QQ(p, q).get_d() << endl;
		    sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
		    if (sgn_on_isol == POSITIVE) {
			p = IB - ID - RT(1);
			//cout << "4 = " << QQ(p, q).get_d() << endl;
			sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
			if (sgn_on_isol == POSITIVE) {
			    RT D4 = d * d - c *e;
			    IA = RT(3)*W2;
			    //cout << "W3 = " << W3 << endl;
			    //cout << "ID = " <<  RT( RT(9)*SQR(W2) + RT(12)*D4*W3 )<< endl;
			    ID = isqrt( RT( RT(9)*SQR(W2) + RT(12)*D4*W3 ) );
			    p = IA - ID;
			    q = RT(6)*W3;
			    // cout << "5 = " << QQ(p, q).get_d() << endl;
			    sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
			    if (sgn_on_isol == POSITIVE) {
				p = IA + ID;
				//		cout << "6 = " << QQ(p, q).get_d() << endl;
				sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
				if (sgn_on_isol == POSITIVE) {
				    p = p - RT(1);
				    //		  cout << "7 = " << QQ(p, q).get_d() << endl;
				    sgn_on_isol = ::SYNAPS::sign( calc_quartic(f[4], f[3], f[2], f[1], f[0], p, q) );
				    if (sgn_on_isol == POSITIVE) {
					p = IA - ID - RT(1);
					//				cout << "8 = " << QQ(p, q).get_d() << endl;
				    }
				}
			    }
			}
		    }
		}
	    }

	}
    } //1
    FT is1 = to_FT(p, q);

    if (exact) {
	if (first) {
	    RO_t r = construct_root_of( RT(-exact_p), exact_q, K());
	    r.set_multiplicity(m);
	    // r.set_left_isolating_point( is0 );
	    // r.set_right_isolating_point( is1 );
	    *sol++ = r;
	    *sol++ = RO_t(f, 1, D, is1, is2, 1*m);
	    return;
	}
	*sol++ = RO_t(f, 0, D, is0, is1, 1*m);
	RO_t r = construct_root_of( RT(-exact_p), exact_q, K());
	r.set_multiplicity(m);
	// r.set_left_isolating_point( is1 );
	// r.set_right_isolating_point( is2 );
	*sol++ = r;
	return;
    }
    *sol++ = RO_t(f, 0, D, is0, is1, 1*m);
    *sol++ = RO_t(f, 1, D, is1, is2, 1*m);
    return;
}



__ALGEBRAIC_END_NAMESPACE


#endif // __SYNAPS_SOLVE_STURM_SOLVE_D4_H__
