/*********************************************************************
*      This file is part of the source code of SYNAPS.               *
*   Author(s): O. Ruatta,  GALAAD, INRIA                             *
*              LACO - Universit de Limoges                          *
**********************************************************************
History:
$Id: WeierIter.H,v 1.1 2005/07/11 10:04:00 jppavone Exp $
**********************************************************************/

#ifndef __SYNAPS__WEIERITER__H
#define __SYNAPS__WEIERITER__H

#include "synaps/linalg.H"
#include "synaps/upol.H"
#include "synaps/linalg/norm.H"
#include "synaps/arithm/RealOf.H"
#include "WeierTools.H"

namespace weierstrass {

/** WeierIter : inline Weierstrass iteration **/
template<class VECT, class UPOL>
VECT WeierIter(const UPOL& P, VECT& V) {
  assert( P.degree() == V.size() );
  typedef typename VECT::size_type size_type;
  typedef typename VECT::value_type value_type;

  size_type d( V.size() );
  VECT TMP = WeierField(P,V);
  for (size_type i = 0; i < d; i++) V[i] += TMP[i];
}


/** Weierstrass : inline nb_step times the Weierstass iteration **/
template<class VECT, class UPOL>
void Weierstrass (const UPOL& P, VECT& V, unsigned int nb_step) {
  assert( P.degree() == V.size() );
  typedef typename VECT::size_type size_type;
  typedef typename VECT::value_type value_type; 
  
  for (unsigned int i = 0; i < nb_step; i++) V += WeierField(P,V);
}

/** WeierstrassSolver : Weierstrass solver       **/
/** It is a dynamic detection of the convergence **/
template<class VECT, class UPOL>
int WeierstrassSolver (const UPOL& P, VECT& Z, typename RealOf<typename VECT::value_type>::TYPE& prec, 
                       typename RealOf<typename VECT::value_type>::TYPE& back_err ) {
  assert ( P.degree() == Z.size() );
  typedef typename VECT::size_type size_type;
  typedef typename VECT::value_type value_type;
  typedef typename RealOf<value_type>::TYPE real_type;
  int cmpt(0);

  UPOL T;
  real_type err(prec + 1);
  VECT V = Z;
  while ( (err > prec) ) {
    V += WeierField(P,V);
    Z-=V;
    err = norme2(Z);
    Z = V;
    cmpt++;
  }
  PolByRoots(V,T);
  T -= P;
  back_err = pnorme2(T);
  prec=err;
  return cmpt;
}


}//namespace weierstrass

#endif //__SYNAPS__WEIERITER__H
