#include <iomanip>
#include <synaps/init.h>
#include <synaps/usolve/Algebraic.h>
#include <synaps/base/Rand.h>

using std::cout;
using std::cin;
using std::endl;

typedef double NT;
typedef Algebraic<NT> SOLVER;

typedef SOLVER::RT         RT;
typedef SOLVER::FT         FT;

typedef SOLVER::Poly       Poly;
//typedef SOLVER::MPoly_2_2  Poly_2;

typedef SOLVER::RO_t       RO_t;


int test_loops;

const int m = -10;
const int M =  10;

Rand<int> R;



int approx_sign(const double& x)
{
  if (abs(x) < 1e-3) return ZERO;
  return sign(x);
}

int compare(const double& a, const double& b)
{
  double d = a-b;
  if (abs(d) < 1e-6) return 0;
  if (d > 0) return 1;
  return -1;
}

// Generate random integral values.
template < class RT >
RT my_rand(int m_ = m, int M_ = M)
{
  RT t = R(m_, M_);
  return t;
}


// Generate random integral values.
int rand_bool()
{
  return R.get_rand_bool();
}

// Generate random root_of of degree 1
template < typename RT >
RO_t
my_rand_root_1()
{
  Poly f(2, AsSize());
  f[0] = my_rand<RT>();
  f[1] = RT(1);
  return solve(f, SOLVER(), 0);
}


// Generate random root_of of degree 2
template < typename RT >
RO_t
my_rand_root_2()
{
  Poly f(3, AsSize());
  RT a = my_rand<RT>();
  RT b = my_rand<RT>();
  f[2] = 1;
  f[1] = -a - b;
  f[0] = a * b;
  return solve(f, SOLVER(), rand_bool());
}



// Generate random root_of of degree 3
template < typename RT >
RO_t
my_rand_root_3()
{
  Poly f(4, AsSize());
  RT a = my_rand<RT>();
  RT b = my_rand<RT>();
  RT c = my_rand<RT>();
  int idx = my_rand<int>(0, 2);
  f[3] = 1;
  f[2] = -a - b - c;
  f[1] = a * b + a * c + b * c;
  f[0] =  -a * b * c;
  return solve(f, SOLVER(), idx);
}



// Generate random root_of of degree 4
template < typename RT >
RO_t
my_rand_root_4()
{
  RT a = my_rand<RT>();
  RT b = my_rand<RT>();
  RT c = my_rand<RT>();
  RT d = my_rand<RT>();
  int idx = my_rand<int>(0, 3);

  Poly f(5, AsSize());
  f[4] = 1;
  f[3] = -a - b - c - d;
  f[2] = a * b + a * c + a * d + b * c + b * d + c* d;
  f[1] = -a * b * c - a * b * d - a * c * d - b * c * d;
  f[0] = a * b * c * d;
  return solve(f, SOLVER(), idx);
}



template<class RT, class Poly>
Poly my_rand_poly(const int& dg)
{
  Poly f(dg+1, AsSize());

  for (int i = 0; i <= dg; ++i) {
    f[i] = my_rand<RT>();
  }
  while (f[dg] == RT(0)) {
    f[dg] = my_rand<RT>();
  }
  return f;
}



// Compares the comparison results of two roots with
// the comparison results of their double approximations.
template < typename Root_1, typename Root_2>
bool
test_compare(const Root_1 &r1, const Root_2 &r2)
{
  if (ALGEBRAIC::compare(r1, r2) == compare(to_double(r1), to_double(r2)))
    return true;

  std::cout << "ERROR in comparing :" << std::endl << " r1 = ";
  print(std::cout, r1);
  std::cout << " [approx = " << std::setprecision(10) << to_double(r1) << " ]\n" << " r2 = ";
  print(std::cout, r2);
  std::cout << " [approx = " << std::setprecision(10) << to_double(r2) << " ]\n"
	    << " comparison gives : " << ALGEBRAIC::compare(r1, r2) << " for exact, but "
	    << compare(to_double(r1), to_double(r2))
	    << " for double approximation" << std::endl;
  return false;
}



template< class RT, class Poly>
void test_sign_root_4()
{
  for (int i = 0; i < test_loops; ++i) {
    Poly f = my_rand_poly<RT, Poly>(3);
    cout << f << endl;
  }
}

#if 0

template < typename RT, typename Poly >
bool
test_root_of_44()
{
  std::cout << "Testing sign_at with root_of_4" << std::endl;
  using namespace ALGEBRAIC;
  for (int dg = 1; dg <= 8; ++dg) {
    std::cout << "  Testing f (" << dg << ")  on x (4)" << std::endl;
    for (int i = 0; i < test_loops; ++i) {
      Poly f = my_rand_poly<RT, Poly>(dg);

      Poly f1 = my_rand_poly<RT, Poly>(4);
      Seq<ALGEBRAIC::root_of<RT> > sol = Solve(f1, Sturm<RT>());
      if (sol.size() == 0) continue;
      int idx = my_rand<int>(0, sol.size()-1);

      ALGEBRAIC::root_of<RT> r = my_rand_root_4<RT>();
      //			root_of<RT> r = sol[idx];

      if (r.degree() != 4) continue;

      int exsign = sign_at(f, r);
      int apsign;
      QQ tmp = eval(f, QQ(to_double(r)));
      if (ALGEBRAIC::isZero(to_double(tmp)))
	apsign = ZERO;
      else
	apsign = sign(eval(f, QQ(to_double(r))));
      if (exsign != apsign) {
	cout << exsign << " " << apsign << endl;
	cout << "f :=" << f << endl;
	cout << "r :="; print(cout, r); cout << r << endl;
	return false;
      }
    }
  }
  return true;
}


#endif

template < typename RT >
bool
test_root_of_33()
{
  std::cout << "Testing sign_at" << std::endl;
  std::cout << "  Testing f (3)  on x (3)" << std::endl;
  using namespace ALGEBRAIC;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_3<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_3<RT>();
    //		if (r1.degree() != 3) continue;
    //		if (r2.degree() != 3) continue;
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << "exact_sgn = "<< exsign << ", approx_sgn = " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  std::cout << "  Testing f (3)  on x (2)" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_2<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_3<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << "exact_sgn = "<< exsign << ", approx_sgn = " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  std::cout << "  Testing f (3)  on x (1)" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_1<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_3<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << "exact_sgn = "<< exsign << ", approx_sgn = " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  std::cout << "  Testing f (2)  on x (3)" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_2<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_3<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << "exact_sgn = "<< exsign << ", approx_sgn = " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  std::cout << "  Testing f (1)  on x (3)" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_3<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_1<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << "exact_sgn = "<< exsign << ", approx_sgn = " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  return true;
}



template < typename RT >
bool
test_root_of_22()
{
  typedef typename ALGEBRAIC::root_of<RT>::NTR    NTR;

  std::cout << "Testing sign_at" << std::endl;
  std::cout << "  Testing f (2)  on x (2)" << std::endl;
  using namespace ALGEBRAIC;
  for (int i = 0; i < test_loops; ++i) {
    //		cout << "\n i = " << i << endl;
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_2<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_2<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << "exact_sgn = "<< exsign << ", approx_sgn = " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  std::cout << "  Testing f (1)  on x (2)" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    //		cout << "\n i = " << i << endl;
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_2<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_1<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << exsign << " " << apsign << endl;
      cout << "f =" << r2.poly() << endl;
      cout << "r =" << r1 << endl;
      return false;
    }
  }
  std::cout << "  Testing f (2)  on x (1)" << std::endl;
  for (int i = 0; i < test_loops; ++i) {
    //		cout << "\n i = " << i << endl;
    ALGEBRAIC::root_of<RT> r1 = my_rand_root_1<RT>();
    ALGEBRAIC::root_of<RT> r2 = my_rand_root_2<RT>();
    int exsign = ALGEBRAIC::sign_at(r2.poly(), r1);
    int apsign = approx_sign(UPOLDAR::eval_horner(r2.poly(), (to_double(r1))));
    if (exsign != apsign) {
      cout << exsign << " " << apsign << endl;
      cout << "r1 ="; print(cout, r1); cout << endl;
      cout << "r2 ="; print(cout, r2); cout << endl;
      return false;
    }
  }
  return true;
}


int main(int argc, char **argv)
{
  
  srand( (unsigned)time( NULL ) );

  test_loops = argc == 2 ? atoi(argv[1]) : 10;
  bool result = true;

  std::cout << "Iterations = " << test_loops << std::endl;
  std::cout << "The algebraic numbers are from " << m << " to " << M << std::endl;

#if 0
  result = true;
  std::cout << "Testing sign of root_of<ZZ>" << std::endl;
  result = result && test_root_of_22< RT >();
#endif

#if 1
  result = true;
  std::cout << "Testing sign of root_of<ZZ>" << std::endl;
  result = result && test_root_of_33< RT >();
#endif

#if 0
  result = true;
  std::cout << "Testing sign of root_of<ZZ>" << std::endl;
  result = result && test_root_of_44< RT, Poly >();
#endif

  if (result) {
    std::cout << "OK" << std::endl;
    return 0;
  } else {
    std::cout << "ERROR" << std::endl;
    return -1;
  }}

