#include <iostream>
#include <synaps/arithm/gmp.h>
#include <synaps/usolve/Algebraic.h>

using std::cout;
using std::cin;
using std::endl;

typedef UPolDse<ZZ>          Poly;
typedef Algebraic<ZZ>::RO_t  RO_t;

int main()
{
  Poly f1(4, AsSize());
  f1[3] = 1;
  f1[2] = 2;
  f1[1] = -65;
  f1[0] = 150;

  std::cout <<f1<<std::endl;
  std::cout <<solve(f1, Algebraic<ZZ>())<<std::endl;
  RO_t r1 = solve(f1, Algebraic<ZZ>(),1);
  
  cout << "r1 = " << r1 << endl;

  Poly f2(3, AsSize());
  f2[2] = 5;
  f2[1] = -4;
  f2[0] = -9;
  
  RO_t r2 = solve(f2, Algebraic<ZZ>(),0);
  cout << "r2 = " << r2 << endl;

  int result = ALGEBRAIC::compare(r1, r2);
  cout << "Result = " << result << endl;
  if (result == 1)
    cout << r1 << " > " << r2 << endl;
  else if (result == -1)
    cout << r1 << " < " << r2 << endl;
  else
    cout << r1 << " = " << r2 << endl;


  UPolDse<ZZ> 
    p1("2*x^7+3*x^6+2*x^5+1*x^4-1*x^3-2*x^2-3*x+1"),
    p2("x^4-3*x^3+2*x^3-2*x^2-5*x+10"),
    p3("x^4-3*x^3+2*x^3-2*x^2-5*x+9");

  Algebraic<ZZ>::root_t 
    alpha= solve(p1,Algebraic<ZZ>(),1),
    beta = solve(p2,Algebraic<ZZ>(),0);

  cout<<"alpha: "<<alpha<<endl;
  //| root_of( 2*x^7+3*x^6+2*x^5+1*x^4-1*x^3-2*x^2-3*x+1, 1)
  //|  in [ 0,5/8 ], approx = 0.278212

  cout<<"beta : "<<beta <<endl;
  //| root_of( 1*x^4-1*x^3-2*x^2-5*x+10, 0 )
  //|  in [ 0,559/342 ], approx = 1.43343

  assert(alpha<beta);
  assert(sign_at(p1,alpha)==0);
  assert(sign_at(p1,beta)==1);

  assert(sign_at(p2,alpha)==1);
  assert(sign_at(p2,beta)==0);

  assert(sign_at(p3,beta)==-1);

  return 0;

}


