<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Sturm Subdivision Solver>

  Another criterion to detect the number of roots in an interval, can be
  used. It is based on Sturm's theorem. Here is the simplifed version, that
  we exploit:

  <\theorem>
    Let <with|mode|math|f\<in\>\<bbb-R\>[x]> and
    <with|mode|math|S(x)=[f(x),f<rprime|'>(x), S<rsub|2>(x),\<ldots\>,
    S<rsub|N>(x)]> be the Sturm sequence of <with|mode|math|f> and
    <with|mode|math|f<rprime|'>>(x). Let <with|mode|math|V(a)> be the number
    of sign variation of <with|mode|math|S(a)> for
    <with|mode|math|a\<in\>\<bbb-R\>>. Then, for any <with|mode|math|a,b
    \<in\>\<bbb-R\>>, <with|mode|math|V(a)-V(b)> is the number of roots in
    ]a,b[.
  </theorem>

  \ \ This leads to the following schema of algorithms, (which applies event
  to polynomials with multiple roots):

  <\notation>
    <name|Isolation of the roots of <with|mode|math|f> on the interval
    <with|mode|math|[a,b]>.>

    INPUT: A polynomial <with|mode|math|f\<in\> \<bbb-R\>[x]>.\ 

    <\itemize>
      <item>Compute the Sturm sequence <with|mode|math|S(x)> of
      <with|mode|math|f> and <with|mode|math|f<rprime|'>>.

      <item>Compute the number of sign changes <with|mode|math|r=V(a)-V(b)>.

      <item>If <with|mode|math|r=0> remove the interval.\ 

      <item>If <with|mode|math|r\<gtr\>1> and compute
      <with|mode|math|V(<frac|a+b|2>)> \ and apply recursively the algorithm
      to the two halves of the input interval.
    </itemize>

    OUTPUT: a list of subintervals of <with|mode|math|[a, b]> containing
    exactly one real root of <with|mode|math|f>.
  </notation>

  <subsection|Sturm Approximation Solver>

  <\code>
    Seq\<less\>O\<gtr\> solve(const UPOL& p, SlvSturm\<less\>O,C\<gtr\> mth);
  </code>

  <\itemize>
    <item>This solver computes an approximation of the roots, within the
    precision <with|mode|math|\<varepsilon\>> specified by the solver
    <verbatim|SlvSturm>. The default value is
    <with|mode|math|\<varepsilon\>=10<rsup|-3>>. If we want a specific
    precision <with|mode|math|\<varepsilon\>=><verbatim|eps>, we can use
    \ the following construction: <verbatim|solve(p,
    SlvSturm\<less\>O,C\<gtr\>(eps))>.\ 

    <item>The solver outputs a sequence of numbers of type <verbatim|O>,
    corresponding to the first parameter <verbatim|O> of this class.\ 

    <item>The second parameter <verbatim|C> is the type of coefficients used
    during the computation. The default type is the type <verbatim|QQ> of
    rational numbers. \ 

    <item>The univariate polynomial <verbatim|p> of type <verbatim|UPOL>, is
    supposed to be given with exact coefficients. It is converted to a
    polynomial of type <verbatim|UPolDse\<less\>C\<gtr\>>, if needed.

    <item>See <verbatim|synaps/usolve/sturm/SlvSturm.h>.
  </itemize>

  <subsection|Sturm Isolation Solver>

  <\code>
    Seq\<less\> Interval\<less\>O\<gtr\> \<gtr\> solve(const UPOL& p,
    IslSturm\<less\>O,C\<gtr\> mth);
  </code>

  <\itemize>
    <item>This solver computes an isolation interval of the roots. It outputs
    a sequence of intervals of \ numbers of type <verbatim|O>, containing one
    and only one root of <verbatim|p.>

    <item>The second parameter <verbatim|C> is the type of coefficients used
    during the computation. The default type is the type <verbatim|QQ> of
    rational numbers. \ 

    <item>The univariate polynomial <verbatim|p> of type <verbatim|UPOL>, is
    supposed to be given with exact coefficients. It is converted to a
    polynomial of type <verbatim|UPolDse\<less\>C\<gtr\>>, if needed.

    <item>See <verbatim|synaps/usolve/sturm/IslSturm.h>.
  </itemize>

  <subsection|Example>

  <action|<with|mode|math|\<boxdot\>>|SturmSolver.cc>
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-10|<tuple|3.1|?>>
    <associate|auto-11|<tuple|3.2|?>>
    <associate|auto-12|<tuple|3.2|?>>
    <associate|auto-13|<tuple|4.1|?>>
    <associate|auto-14|<tuple|4.2|?>>
    <associate|auto-15|<tuple|4.2|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|3|?>>
    <associate|auto-6|<tuple|3.1|?>>
    <associate|auto-7|<tuple|3.2|?>>
    <associate|auto-8|<tuple|3.2|?>>
    <associate|auto-9|<tuple|3|?>>
    <associate|toc-1|<tuple|1|1>>
    <associate|toc-10|<tuple|1.4.1|?>>
    <associate|toc-11|<tuple|1.5|?>>
    <associate|toc-2|<tuple|1.1|1>>
    <associate|toc-3|<tuple|1.1.1|1>>
    <associate|toc-4|<tuple|1.2|1>>
    <associate|toc-5|<tuple|1|?>>
    <associate|toc-6|<tuple|1.2.1|?>>
    <associate|toc-7|<tuple|1.3|?>>
    <associate|toc-8|<tuple|1.3.1|?>>
    <associate|toc-9|<tuple|1.4|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Sturm
      Subdivision Solver> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Sturm Approximation Solver
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Sturm Isolation Solver
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>